@@ -1,10 +1,10 @@
-// Copyright 1999-2018 - Universit de Strasbourg/CNRS
+// Copyright 1999-2020 - Universit de Strasbourg/CNRS
 // The Aladin Desktop program is developped by the Centre de Donnes
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
 //
-//This file is part of Aladin.
+//This file is part of Aladin Desktop.
 //
 //    Aladin Desktop is free software: you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 //    GNU General Public License for more details.
 //
 //    The GNU General Public License is available in COPYING file
-//    along with Aladin.
+//    along with Aladin Desktop.
 //
 
 package cds.aladin;
@@ -384,7 +384,7 @@ public class Chaine {
                     "for an official distribution, please " +
                     "after installing and testing your work, attach the following file " +
                     "to a mail address to:\n \n" +
-                    "Email: language@aladin.u-strasbg.fr\n" + 
+                    "Email: cds-question@unistra.fr\n" + 
                     "File : "+getTranslationFullName()+"\n \n" +
                     "Please, do not forget to specify in this mail " +
                     "your name and quality\n" +
