@@ -1,10 +1,10 @@
-// Copyright 1999-2018 - Universit de Strasbourg/CNRS
+// Copyright 1999-2020 - Universit de Strasbourg/CNRS
 // The Aladin Desktop program is developped by the Centre de Donnes
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
 //
-//This file is part of Aladin.
+//This file is part of Aladin Desktop.
 //
 //    Aladin Desktop is free software: you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 //    GNU General Public License for more details.
 //
 //    The GNU General Public License is available in COPYING file
-//    along with Aladin.
+//    along with Aladin Desktop.
 //
 
 package cds.aladin;
@@ -90,12 +90,12 @@ public final class Console extends JFrame implements ActionListener,KeyListener,
    private JTextArea fieldPad;         // La zone du notepad
    private JButton exec,clear,delete,dump,clearPad;  // Qq boutons
 
-   private Vector<String> cmd = new Vector<String>();              // Les commandes non encore traites
-   private  Vector<Command> cmdHistory = new Vector<Command>();  // L'historique des commandes traites
+   private Vector<String> cmd = new Vector<>();              // Les commandes non encore traites
+   private  Vector<Command> cmdHistory = new Vector<>();  // L'historique des commandes traites
    
    private String currentCmd = null;  // La dernire commande en cours de saisie 
    
-   private ArrayList<Lot> lots = new ArrayList<Lot>(); // Liste des lots de commandes en attente de traitement
+   private ArrayList<Lot> lots = new ArrayList<>(); // Liste des lots de commandes en attente de traitement
    private Lot currentLot=null;                        // Lot courant, cd qui correspond  la commande 
                                                        // qui est entrain d'tre traite
 
@@ -219,7 +219,7 @@ public final class Console extends JFrame implements ActionListener,KeyListener,
                           // -2 est une valeur spciale pour indiquer l'attente du lancement imminent d'un thread
       
       // Fifo des commandes du lot
-      Queue<String> cmds=new LinkedList<String>();
+      Queue<String> cmds=new LinkedList<>();
       
       // Ajout d'une commande
       void add(String s) {
@@ -290,7 +290,7 @@ public final class Console extends JFrame implements ActionListener,KeyListener,
       }
       
       // Suppression des lots vides
-      ArrayList<Lot> lotsBis = new ArrayList<Console.Lot>();
+      ArrayList<Lot> lotsBis = new ArrayList<>();
       for( Lot lot : lots ) if( !lot.isEmpty() ) lotsBis.add(lot);
       lots=lotsBis;
       
@@ -539,7 +539,7 @@ public final class Console extends JFrame implements ActionListener,KeyListener,
    public ArrayList<String> getRecentHistory(int max) { return getRecentHistory(0,max); }
    public ArrayList<String> getRecentHistory(int indexInit, int max) {
       if( indexInit<=0 ) indexInit = 0;
-      ArrayList<String> v = new ArrayList<String>(max);
+      ArrayList<String> v = new ArrayList<>(max);
       int  i= cmdHistory.size()-1;
       for( int j=0; i>=0 && v.size()<max; i-- ) {
          Command cmd = cmdHistory.get(i);
