@@ -1,10 +1,10 @@
-// Copyright 1999-2018 - Universit de Strasbourg/CNRS
+// Copyright 1999-2020 - Universit de Strasbourg/CNRS
 // The Aladin Desktop program is developped by the Centre de Donnes
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
 //
-//This file is part of Aladin.
+//This file is part of Aladin Desktop.
 //
 //    Aladin Desktop is free software: you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 //    GNU General Public License for more details.
 //
 //    The GNU General Public License is available in COPYING file
-//    along with Aladin.
+//    along with Aladin Desktop.
 //
 
 /**
@@ -27,19 +27,21 @@ package cds.aladin;
 import static cds.aladin.Constants.ADDPOSCONSTRAINT;
 import static cds.aladin.Constants.CHANGESERVER;
 import static cds.aladin.Constants.CIRCLEORSQUARE;
-import static cds.aladin.Constants.DISCARDACTION;
+import static cds.aladin.Constants.EDITUPLOADTABLENAMEACTION;
 import static cds.aladin.Constants.EMPTYSTRING;
+import static cds.aladin.Constants.JOIN_TABLE;
 import static cds.aladin.Constants.OPEN_SET_RADEC;
 import static cds.aladin.Constants.RADECBUTTON;
+import static cds.aladin.Constants.REGEX_VALIDTABLEPREFIX;
 import static cds.aladin.Constants.RELOAD;
 import static cds.aladin.Constants.SHOWAYNCJOBS;
 import static cds.aladin.Constants.SYNC_ASYNC;
+import static cds.aladin.Constants.TABLEGUINAME;
+import static cds.aladin.Constants.TABLESLABEL;
 import static cds.aladin.Constants.TAPFORM_STATUS_ERROR;
 import static cds.aladin.Constants.TAPFORM_STATUS_LOADED;
 import static cds.aladin.Constants.TAPFORM_STATUS_LOADING;
 import static cds.aladin.Constants.TAPFORM_STATUS_NOTLOADED;
-import static cds.aladin.Constants.UPLOAD;
-import static cds.aladin.Constants.REGEX_VALIDTABLEPREFIX;
 import static cds.tools.CDSConstants.BOLD;
 
 import java.awt.BorderLayout;
@@ -47,12 +49,16 @@ import java.awt.Component;
 import java.awt.FlowLayout;
 import java.awt.GridBagConstraints;
 import java.awt.GridBagLayout;
+import java.awt.Image;
 import java.awt.Insets;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
+import java.beans.PropertyChangeEvent;
+import java.beans.PropertyChangeListener;
 import java.util.ArrayList;
 import java.util.Iterator;
 import java.util.List;
+import java.util.ListIterator;
 import java.util.Map;
 import java.util.Vector;
 
@@ -94,11 +100,13 @@ public abstract class DynamicTapForm extends Server implements FilterActionClass
 	 * 
 	 */
 	private static final long serialVersionUID = 8296179835574266057L;
+
+	private static final int TAPSCHEMACOUNT = 5;
 	
 	public static String TIPRETRY, TAPTABLEUPLOADTIP, TAPTABLENOUPLOADTIP, REFRESHQUERYTOOLTIP, CHECKQUERYTOOLTIP,
 			SYNCASYNCTOOLTIP, SHOWASYNCTOOLTIP, TAPTABLEJOINTIP, DISCARD, DISCARDTIP, SETRADECBUTTONTIP,
 			CHANGETARGETSETTINGSTOOLTIP, TIPCLICKTOADD, TAPEXDEFAULTMAXROWS, NORANGEERRORMESSAGE, TAPERRORSTATUSINFO, 
-			TAPLOADINGSTATUSINFO, MESSAGEUNKNOWNPARAMSINQUERY;
+			TAPLOADINGSTATUSINFO, MESSAGEUNKNOWNPARAMSINQUERY, TAPTABLEUPLOADLIMITTOOLTIP, NOSTACKTABLES;
 	
 	public String CLIENTINSTR;
 
@@ -157,24 +165,18 @@ public abstract class DynamicTapForm extends Server implements FilterActionClass
 			tc.gridx++;
 		}
 		
-		if (this.tapClient.mode != TapClientMode.UPLOAD) {
-			if (this instanceof DynamicTapForm) {
-				JButton reloadButton = TapClient.getReloadButton();
-				reloadButton.addActionListener(this);
-				addingPanel.add(reloadButton, tc);
-				tc.gridx++;
-			}
+		if (this instanceof DynamicTapForm) {
+			JButton reloadButton = TapClient.getReloadButton();
+			reloadButton.addActionListener(this);
+			addingPanel.add(reloadButton, tc);
+			tc.gridx++;
 		}
 		
 		JPanel titlePanel = new JPanel();
 		titlePanel.setBackground(this.tapClient.primaryColor);
 		titlePanel.setAlignmentY(SwingConstants.CENTER);
-		if (this.tapClient.mode != TapClientMode.UPLOAD) {
-			this.makeTitle(titlePanel, this.tapClient.getVisibleLabel());
-//			this.aladinLabel = this.name;
-		} else {
-			this.makeTitle(titlePanel, "Upload server");
-		}
+		this.makeTitle(titlePanel, this.tapClient.getVisibleLabel());
+//		this.aladinLabel = this.name;
 		tc.weightx = 0.88;
 		c.fill = GridBagConstraints.HORIZONTAL;
 	    addingPanel.add(titlePanel, tc);
@@ -213,13 +215,19 @@ public abstract class DynamicTapForm extends Server implements FilterActionClass
 	
 	/**
 	 * Creates the table selection panel with tables drop-down, upload button etc..
-	 * @param tableChoice
-	 * @param tables
+	 * @param labelText
+	 * @param tablesGui
+	 * @param chosenTable
+	 * @param keys
+	 * @param smallPrefixComponents
+	 * @param isFirstTable
+	 * @param isJoin
 	 * @return
-	 * @throws BadLocationException 
+	 * @throws BadLocationException
 	 */
-	public JPanel getTablesPanel(String labelText, final JComboBox tablesGui, TapTable chosenTable, List<String> keys,
-			List<JComponent> smallPrefixComponents, boolean isFirstTable) throws BadLocationException {
+	public static JPanel getTablesPanel(TapClient tapClient, final FilterActionClass actionClass, String labelText,
+			final JComboBox tablesGui, TapTable chosenTable, List<String> keys, List<JComponent> smallPrefixComponents,
+			boolean isFirstTable) throws BadLocationException {
     	JPanel tablesPanel = new JPanel();
 		GridBagLayout gridbag = new GridBagLayout();
 		tablesPanel.setLayout(gridbag);
@@ -232,7 +240,7 @@ public abstract class DynamicTapForm extends Server implements FilterActionClass
 		c.anchor = GridBagConstraints.WEST;
 		c.fill = GridBagConstraints.WEST;
 		c.weightx = 0.05;
-		if (smallPrefixComponents!=null && !smallPrefixComponents.isEmpty()) {
+		if (smallPrefixComponents != null && !smallPrefixComponents.isEmpty()) {
 			for (JComponent smallPrefixComponent : smallPrefixComponents) {
 				tablesPanel.add(smallPrefixComponent, c);
 				c.gridx++;
@@ -240,6 +248,7 @@ public abstract class DynamicTapForm extends Server implements FilterActionClass
 		}
 		
 		JLabel label = new JLabel();
+		label.setName(TABLESLABEL);
 		if (labelText == null) {
 			labelText = "Table:";
 		}
@@ -255,26 +264,25 @@ public abstract class DynamicTapForm extends Server implements FilterActionClass
 		}
 		if (tableToolTip != null && !tableToolTip.isEmpty()) {
 			tablesGui.setToolTipText("<html><p width=\"500\">"+tableToolTip+"</p></html>");
-		} else {
-			tablesGui.setToolTipText(null);
 		}
-		if (this.tapClient.mode != TapClientMode.UPLOAD && keys != null &&!keys.isEmpty()) {
+		
+		if (keys != null && !keys.isEmpty()) {
 			tablesGui.setEditable(true);
 			JTextComponent tablesGuiEditor = (JTextComponent) tablesGui.getEditor().getEditorComponent();
-			FilterDocument document = new FilterDocument(this, tablesGui, keys, selectedName);
+			FilterDocument document = new FilterDocument(actionClass, tablesGui, keys, selectedName);
 			tablesGuiEditor.setDocument(document);
 		} else {
 			tablesGui.addActionListener(new ActionListener() {
 				@Override
 				public void actionPerformed(ActionEvent e) {
 					// TODO Auto-generated method stub
-					checkSelectionChanged(tablesGui);
+					actionClass.checkSelectionChanged(tablesGui);
 				}
 			});
 		}
 		
 		tablesGui.setOpaque(false);
-		tablesGui.setName("table");
+		tablesGui.setName(TABLEGUINAME);
 //		tablesGui.setActionCommand(TABLECHANGED);
 		tablesGui.setAlignmentY(SwingConstants.CENTER);
 		c.insets = new Insets(1, 0, 1, 0);
@@ -284,64 +292,68 @@ public abstract class DynamicTapForm extends Server implements FilterActionClass
 		c.weightx = 0.75;
 		tablesPanel.add(tablesGui, c);
 		
-		if (this.tapClient.mode == TapClientMode.UPLOAD) {
-			c.weightx = 0.05;
-			c.gridx++;
-			JButton button = new JButton(DISCARD);
-			button.setActionCommand(DISCARDACTION);
-			button.addActionListener(this.tapClient.tapManager.uploadFrame);
-			button.setToolTipText(DISCARDTIP);
-			tablesPanel.add(button, c);
-		}
+		DynamicTapForm server = null;
+		boolean isJoin = false;
+		if (actionClass instanceof DynamicTapForm) {
+			server = (DynamicTapForm) actionClass;
+		} else if (actionClass instanceof JoinFacade) {
+			server = ((JoinFacade)actionClass).serverTap;
+			isJoin = true;
+		} 
 		
-		if (this instanceof ServerTap) {
+		if (!isJoin && server instanceof ServerTap) {
 			JButton button = new JButton("Set ra, dec");
 			button.setName(RADECBUTTON);
 			button.setActionCommand(OPEN_SET_RADEC);
-			button.addActionListener(this);
+			button.addActionListener(server);
 			button.setToolTipText(SETRADECBUTTONTIP);
 			c.insets = new Insets(1, 3, 1, 3);
 			c.weightx = 0.10;
 			c.gridx++;
 			tablesPanel.add(button,c);
 		
-			if (Aladin.PROTO) {//TODO:: tintinproto
+//			if (Aladin.PROTO) {
 				button = new JButton("Join");
 				c.weightx = 0.05;
 				c.gridx++;
+				button.setActionCommand(JOIN_TABLE);
 				tablesPanel.add(button, c);
-				button.setEnabled(false);
-				button.addActionListener(this);
+//				button.setEnabled(false);
+				button.addActionListener(server);
 				button.setToolTipText(TAPTABLEJOINTIP);
-			}
-		} else if (this instanceof ServerTapExamples) {
+//			}
+		} else if (isJoin || server instanceof ServerTapExamples) {
 			c.insets = new Insets(1, 3, 1, 3);
 			c.weightx = 0.10;
 			c.gridx++;
 			if (isFirstTable) {
-				JToggleButton grab = getGrab();
-				grab.setToolTipText(CHANGETARGETSETTINGSTOOLTIP);
-				tablesPanel.add(grab,c);
+				if (!isJoin) {
+					JToggleButton grab = server.getGrab();
+					grab.setToolTipText(CHANGETARGETSETTINGSTOOLTIP);
+					tablesPanel.add(grab,c);
+				}
 			} else {
-				tablesPanel.add(getUploadButtonIfAvailable("Upload tables.."), c);
+				tablesPanel.add(server.getUploadButtonIfAvailable(null, tablesGui), c);
 			}
-			
 		}
+		
 		return tablesPanel;
 	}
 	
 	public JToggleButton getGrab() {
 		this.target = new JTextField(40);
-
-		JToggleButton b = grab = new JToggleButton(new ImageIcon(Aladin.aladin.getImagette("Grab.png")));
+		Image image = Aladin.aladin.getImagette("Grab.png");
+		if (image == null) {
+			grab = new JToggleButton("Grab");
+		} else {
+			grab = new JToggleButton(new ImageIcon(image));
+		}
+		JToggleButton b = grab;
         Util.toolTip(b, "Grab a position/radius in the view");
         b.setMargin(new Insets(0, 0, 0, 0));
         b.setBorderPainted(false);
         b.setContentAreaFilled(false);
         
-//        Insets m = grab.getMargin();
-//        grab.setMargin(new Insets(m.top,2,m.bottom,2));
-//        grab.setOpaque(false);
         grab.addActionListener(new ActionListener() {
 			public void actionPerformed(ActionEvent e) {
 				aladin.f.toFront();
@@ -433,10 +445,6 @@ public abstract class DynamicTapForm extends Server implements FilterActionClass
         	updateWidgets((GrabItFrame) rootFrame);
 		}
         
-        if (this.aladinLabel.equalsIgnoreCase(Constants.DATALINK_CUTOUT_FORMLABEL)) {
-        	grab.setEnabled(true);//Default true for datalink forms
-		}
-        
         c.gridwidth = 1;
 		c.weightx = 0.05;
 		c.gridx = 3;
@@ -461,7 +469,7 @@ public abstract class DynamicTapForm extends Server implements FilterActionClass
 		gridbag.setConstraints(radius, c);
 		targetPanel.add(radius);
 		
-		this.circleOrSquare = new JComboBox<String>(CIRCLEORSQUARE);
+		this.circleOrSquare = new JComboBox<>(CIRCLEORSQUARE);
 		circleOrSquare.setOpaque(false);
 		circleOrSquare.setName("posConstraintShape");
 //		this.circleOrSquare.setActionCommand(POSCONSTRAINTSHAPECHANGED);
@@ -502,11 +510,11 @@ public abstract class DynamicTapForm extends Server implements FilterActionClass
 	 * Lower buttons panel, just above the tap query text area
 	 * @return
 	 */
-	public JPanel getBottomPanel(boolean addRefreshQuery) {
+	public JPanel getBottomPanel(boolean isForLoaded) {
 		JPanel bottomPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
 		bottomPanel.setBackground(this.tapClient.primaryColor);
 		JButton button = new JButton("Refresh query");
-		if (addRefreshQuery && (this instanceof ServerTap || this instanceof ServerObsTap)) {
+		if (isForLoaded && (this instanceof ServerTap)) {
 			button.setToolTipText(REFRESHQUERYTOOLTIP);
 			button.setActionCommand("WRITEQUERY");
 			button.addActionListener(this);
@@ -519,11 +527,10 @@ public abstract class DynamicTapForm extends Server implements FilterActionClass
 		button.addActionListener(this);
 		bottomPanel.add(button);
 		
-		this.sync_async = new JComboBox<String>(SYNC_ASYNC);
+		this.sync_async = new JComboBox<>(SYNC_ASYNC);
 		this.sync_async.setOpaque(false);
 		if (SYNCASYNCTOOLTIP!=null && !SYNCASYNCTOOLTIP.isEmpty()) {
-			SYNCASYNCTOOLTIP = "<html><p width=\"500\">"+SYNCASYNCTOOLTIP+"</p></html>";
-			this.sync_async.setToolTipText(SYNCASYNCTOOLTIP);
+			this.sync_async.setToolTipText("<html><p width=\"500\">"+SYNCASYNCTOOLTIP+"</p></html>");
 		}
 		bottomPanel.add(this.sync_async);
 		
@@ -533,25 +540,62 @@ public abstract class DynamicTapForm extends Server implements FilterActionClass
 		button.addActionListener(this);
 		bottomPanel.add(button);
 		
-		if (Aladin.BETA) {
-			if (this.tapClient.mode != TapClientMode.UPLOAD) {
-				bottomPanel.add(getUploadButtonIfAvailable("Upload"));
-			}
-		}
 		return bottomPanel;
 	}
 	
-	public JButton getUploadButtonIfAvailable(String label) {
+	public JButton getUploadButtonIfAvailable(String label, JComboBox tablesGui) {
+		String defaultLanel = "Edit table name";
 		String uploadTipText = TAPTABLEUPLOADTIP;
-		JButton button = new JButton(label);
-		button.setActionCommand(UPLOAD);
+		JButton button = null;
+		
+		if (label == null) {
+			Image image = Aladin.aladin.getImagette("settings.png");
+			if (image == null) {
+				button = new JButton(defaultLanel);
+			} else {
+				button = new JButton(new ImageIcon(image));
+				button.setBorderPainted(false);
+				button.setMargin(new Insets(0, 0, 0, 0));
+				button.setContentAreaFilled(true);
+			}
+		} else {
+			button = new JButton(label);
+		}
+		
+		button.setActionCommand(EDITUPLOADTABLENAMEACTION);
+		
+		
 		if (this.tapClient.capabilities != null) {
 			try {
 				VOSICapabilitiesReader meta = this.tapClient.capabilities.get();
 				button.setEnabled(meta.isUploadAllowed());
-				button.addActionListener(this);
+				
+				final JButton editVutton = button;
+				tablesGui.addPropertyChangeListener(new PropertyChangeListener() {
+					
+					@Override
+					public void propertyChange(PropertyChangeEvent evt) {
+						// TODO Auto-generated method stub
+						if (evt.getPropertyName().equals("enabled")) {
+							if (((Boolean) evt.getNewValue())) {
+								editVutton.setEnabled(true);
+							} else {
+								editVutton.setEnabled(false);
+							}
+							((JComboBox)evt.getSource()).setToolTipText(NOSTACKTABLES);
+						}
+					}
+				});
+				
+				button.addActionListener(new ActionListener() {
+					@Override
+					public void actionPerformed(ActionEvent e) {
+						// TODO Auto-generated method stub
+						DynamicTapForm.this.tapClient.tapManager.uploadFacade.uploadSettingsClickAction((JButton) e.getSource());
+					}
+				});
 				if (meta.isUploadAllowed() && meta.getUploadHardLimit() > 0L) {
-					String tip = String.format("Hard limit =%1$s rows", meta.getUploadHardLimit());
+					String tip = String.format(TAPTABLEUPLOADLIMITTOOLTIP, meta.getUploadHardLimit());
 					uploadTipText = uploadTipText.concat(tip);
 				} else if (!meta.isUploadAllowed()) {
 					uploadTipText = TAPTABLENOUPLOADTIP;
@@ -575,6 +619,9 @@ public abstract class DynamicTapForm extends Server implements FilterActionClass
 		if (comboBox.getSelectedItem() != null
 				&& !selectedTableName.equalsIgnoreCase(comboBox.getSelectedItem().toString())) {
 			Aladin.trace(3, "Change table selection from within the document");
+			if (this instanceof ServerTap) {
+				((ServerTap)this).resetJoin();
+			}
 			if (comboBox == this.tablesGui) {
 				selectedTableName = (String) comboBox.getSelectedItem();
 				this.changeTableSelection(selectedTableName);
@@ -608,12 +655,8 @@ public abstract class DynamicTapForm extends Server implements FilterActionClass
 	public Vector<TapTableColumn> getColumnsToLoad(String tableName, Map<String, TapTable> tablesMetaData) {
 		Vector<TapTableColumn> columnNames = tablesMetaData.get(tableName).getColumns();
 		if (columnNames == null) {
-			if (this.tapClient.mode == TapClientMode.UPLOAD) {
-				Aladin.error(this, "Error in uploaded data");
-				return null;
-			}
 			try {
-				List<String> tableNamesToUpdate = new ArrayList<String>();
+				List<String> tableNamesToUpdate = new ArrayList<>();
 				tableNamesToUpdate.add(tableName);
 				this.tapClient.tapManager.updateTableColumnSchemas(this, tableNamesToUpdate);
 				columnNames = tablesMetaData.get(tableName).getColumns();
@@ -655,6 +698,10 @@ public abstract class DynamicTapForm extends Server implements FilterActionClass
 	
 	@Override
 	protected void showStatusReport() {
+		showStatusReport(true);
+	}
+	
+	protected void showStatusReport(boolean showTapInfoText) {
 		if (aladin.frameInfoServer == null || !aladin.frameInfoServer.isOfDynamicTapServerType()
 				|| !aladin.frameInfoServer.isThisInfoPanel(this.tapClient)) {
 			if (aladin.frameInfoServer != null) {
@@ -675,12 +722,16 @@ public abstract class DynamicTapForm extends Server implements FilterActionClass
 					e.printStackTrace();
 			}
 		}
-		aladin.frameInfoServer.show(this);
+		aladin.frameInfoServer.showHidetapInfoText(showTapInfoText);
+		if (!aladin.frameInfoServer.isVisible()) {
+			aladin.frameInfoServer.show(this);
+		}
+		aladin.frameInfoServer.toFront();
 	}
 	
 	@Override
 	public Vector<String> getMatches(String mask, JComboBox<String> comboBox) {
-		Vector<String> matches = new Vector<String>();
+		Vector<String> matches = new Vector<>();
 		if (/*this.tablesGui ==  comboBox && */mask != null && !mask.isEmpty()) {
 			for (String key : getTableNames()) {
 				boolean checkDescription = false;
@@ -700,13 +751,18 @@ public abstract class DynamicTapForm extends Server implements FilterActionClass
 	}
 	
 	@Override
-	public ADQLQuery checkQuery()  throws UnresolvedIdentifiersException {
+	public ADQLQuery checkQuery(Map<String, Object> requestParams)  throws UnresolvedIdentifiersException {
 		ADQLQuery query = null;
 		try {
-			if (Aladin.PROTO) {
-				this.tapClient.updateUploadedTablesToParser(this);
-			}
-			query = super.checkQuery();
+//			if (Aladin.PROTO) {
+				try {
+					this.tapClient.updateUploadedTablesToADQLParser(this, requestParams);
+				} catch (Exception e) {
+					// TODO Auto-generated catch block
+					Aladin.trace(3, "error trying to update adql parser" +e.getMessage());
+				}
+//			}
+			query = super.checkQuery(null);
 //			DefaultDBTable table = new DefaultDBTable(selectedTableName);
 //			System.err.println("getADQLCatalogName "+table.getADQLCatalogName()+"\ngetADQLSchemaName "+table.getADQLSchemaName()+"\ngetADQLName "+table.getADQLName()+"\ntoString "+table.toString());
 		} catch (UnresolvedIdentifiersException uie) {	
@@ -747,7 +803,7 @@ public abstract class DynamicTapForm extends Server implements FilterActionClass
 		List<String> tableNames = null;
 		try {
 			ADQLQuery adqlQuery = syntaxParser.parseQuery(query);
-			tableNames = new ArrayList<String>();
+			tableNames = new ArrayList<>();
 //			DBColumn[] columns = query.getResultingColumns();//match columns with the unresolvedIdentifiers?
 			for (ADQLTable adqlTable : adqlQuery.getFrom().getTables()) {
 				String tableNameKey = getTableMetaCacheKey(adqlTable.getFullTableName());
@@ -787,8 +843,7 @@ public abstract class DynamicTapForm extends Server implements FilterActionClass
 	 * @param tableName
 	 */
 	public void updateQueryChecker(String tableName) {
-		boolean isUpload = this.tapClient.mode == TapClientMode.UPLOAD;
-		updateQueryChecker(isUpload, tableName, this.tapClient.tablesMetaData, this.tapClient.queryCheckerTables);
+		updateQueryChecker(false, tableName, this.tapClient.tablesMetaData, this.tapClient.queryCheckerTables);
 	}
 	
 	/**
@@ -837,8 +892,7 @@ public abstract class DynamicTapForm extends Server implements FilterActionClass
 				}
 			}
 			
-			if (this.tapClient.mode == TapClientMode.UPLOAD
-					|| (queryCheckerTable != null && queryCheckerTables.remove(queryCheckerTable))) {
+			if (queryCheckerTable != null && queryCheckerTables.remove(queryCheckerTable)) {
 				QueryChecker checker = new DBChecker(queryCheckerTables);
 				this.adqlParser.setQueryChecker(checker);
 			}
@@ -859,13 +913,9 @@ public abstract class DynamicTapForm extends Server implements FilterActionClass
 					Aladin.error(this, TapClient.GENERICERROR);
 		            ball.setMode(Ball.NOK);
 				}
-			} else if (action.equals(UPLOAD)) {
-				//disabled based on capability and if user has not created a table
-				this.tapClient.showOnUploadFrame();
-				
 			} else if (action.equals(SHOWAYNCJOBS)) {
 				try {
-					this.tapClient.tapManager.showAsyncPanel();
+					UWSFacade.getInstance(aladin).showAsyncPanel();
 				} catch (Exception e) {
 					Aladin.error(this, TapClient.GENERICERROR);
 		            ball.setMode(Ball.NOK);
@@ -884,7 +934,7 @@ public abstract class DynamicTapForm extends Server implements FilterActionClass
 	
 	//Not bothering for ServerObscore.. yet!
 	public Vector<String> getTableNames() {
-		Vector<String> tables = new Vector<String>();
+		Vector<String> tables = new Vector<>();
 		if (!isFullServer && this.tapClient.nodeTableNames != null) {
 			for (String nodeTableName : this.tapClient.nodeTableNames) {
 				DefaultDBTable ntable = new DefaultDBTable(nodeTableName);
@@ -904,6 +954,17 @@ public abstract class DynamicTapForm extends Server implements FilterActionClass
 			}
 		} else {
 			tables.addAll(this.tapClient.tablesMetaData.keySet());
+			if (TapManager.getInstance(aladin).hideTapSchema) {
+				ListIterator<String> tablesIterator  = tables.listIterator(tables.size());
+				int schemaTables = TAPSCHEMACOUNT;
+				while (tablesIterator.hasPrevious() && schemaTables > 0) {
+					String tableName = tablesIterator.previous();
+					if (TapClient.isSchemaTable(tableName)) {
+						tablesIterator.remove();
+						schemaTables--;
+					}
+				}
+			}
 		}
 		return tables;
 		
@@ -1005,6 +1066,8 @@ public abstract class DynamicTapForm extends Server implements FilterActionClass
 		TAPERRORSTATUSINFO = Aladin.chaine.getString("TAPERRORSTATUSINFO");
 		TAPLOADINGSTATUSINFO = Aladin.chaine.getString("TAPLOADINGSTATUSINFO");
 		MESSAGEUNKNOWNPARAMSINQUERY = Aladin.chaine.getString("MESSAGEUNKNOWNPARAMSINQUERY");
+		TAPTABLEUPLOADLIMITTOOLTIP = Aladin.chaine.getString("TAPTABLEUPLOADLIMITTOOLTIP");
+		NOSTACKTABLES = Aladin.chaine.getString("NOSTACKTABLES");
 	}
 
 }
