@@ -1,10 +1,10 @@
-// Copyright 1999-2018 - Universit de Strasbourg/CNRS
+// Copyright 1999-2020 - Universit de Strasbourg/CNRS
 // The Aladin Desktop program is developped by the Centre de Donnes
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
 //
-//This file is part of Aladin.
+//This file is part of Aladin Desktop.
 //
 //    Aladin Desktop is free software: you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 //    GNU General Public License for more details.
 //
 //    The GNU General Public License is available in COPYING file
-//    along with Aladin.
+//    along with Aladin Desktop.
 //
 
 package cds.aladin;
@@ -470,7 +470,7 @@ public class Fov {
 	  * @param radius
 	  * @return
 	  */
-	public double getPlotRadiusForCircleFromCoord(Projection proj, Coord centre, double radius) {
+	public static double getPlotRadiusForCircleFromCoord(Projection proj, Coord centre, double radius) {
 		Coord c0 = new Coord(centre.al, centre.del);
 		proj.getXY(c0);
 		Coord c1 = new Coord(0, 0);
