@@ -1,10 +1,10 @@
-// Copyright 1999-2018 - Universit de Strasbourg/CNRS
+// Copyright 1999-2020 - Universit de Strasbourg/CNRS
 // The Aladin Desktop program is developped by the Centre de Donnes
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
 //
-//This file is part of Aladin.
+//This file is part of Aladin Desktop.
 //
 //    Aladin Desktop is free software: you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 //    GNU General Public License for more details.
 //
 //    The GNU General Public License is available in COPYING file
-//    along with Aladin.
+//    along with Aladin Desktop.
 //
 
 package cds.aladin;
@@ -183,7 +183,7 @@ public final class FrameColorMap extends JFrame implements MouseListener {
     */
    protected void majCM() { majCM(false); }
    protected void majCM(boolean force) {
-      if( force ) pimg=null;
+       if( force ) pimg=null;
       if( aladin.toolBox.tool[ToolBox.HIST].mode==Tool.DOWN ) {
          memoControl();
 
@@ -399,6 +399,9 @@ public final class FrameColorMap extends JFrame implements MouseListener {
 
       if( aladin.frameAllsky!=null ) aladin.frameAllsky.updateCurrentCM();
       cm.resumePixelTool();
+      
+      // Y a-t-il un PlanBGRgb  l'coute ?
+      updatePlanRgb();
 
       if( canvasPixelRange!=null ) canvasPixelRange.repaint();
       cm.repaint();
@@ -406,6 +409,10 @@ public final class FrameColorMap extends JFrame implements MouseListener {
 
       defaultConf.memo();
    }
+   
+   private void updatePlanRgb() {
+      if( pimg instanceof PlanBG && ((PlanBG)pimg).hasRecutListener() ) pimg.recut();
+   }
 
    private JPanel createPanelTop() {
       JButton b;
@@ -739,7 +746,7 @@ public final class FrameColorMap extends JFrame implements MouseListener {
 
    /** Cration d'un Combo des CM possibles */
    protected static JComboBox<String> createComboCM() {
-      JComboBox<String> c = new JComboBox<String>();
+      JComboBox<String> c = new JComboBox<>();
       for( String s : CanvasColorMap.getCMList() ) c.addItem(s);
       c.addItem(" -- ");
       return c;
@@ -747,7 +754,7 @@ public final class FrameColorMap extends JFrame implements MouseListener {
 
    /** Cration d'un Combo des Fonctions de transferts possibles */
    protected static JComboBox<String> createComboFct() {
-      JComboBox<String> c = new JComboBox<String>( PlanImage.TRANSFERTFCT );
+      JComboBox<String> c = new JComboBox<>( PlanImage.TRANSFERTFCT );
       return c;
    }
 
