@@ -1,10 +1,10 @@
-// Copyright 1999-2018 - Universit de Strasbourg/CNRS
+// Copyright 1999-2020 - Universit de Strasbourg/CNRS
 // The Aladin Desktop program is developped by the Centre de Donnes
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
 //
-//This file is part of Aladin.
+//This file is part of Aladin Desktop.
 //
 //    Aladin Desktop is free software: you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 //    GNU General Public License for more details.
 //
 //    The GNU General Public License is available in COPYING file
-//    along with Aladin.
+//    along with Aladin Desktop.
 //
 
 /*
@@ -376,8 +376,8 @@ public class FrameColumnCalculator extends JFrame implements ActionListener {
                Obj o = it.next();
                if( !(o instanceof Source) ) continue;
                s = (Source)o;
-               for( int k=s.leg.field.length-1;k>=0;k-- ) {
-                  str = s.leg.field[k].name;
+               for( int k=s.getLeg().field.length-1;k>=0;k-- ) {
+                  str = s.getLeg().field[k].name;
                   if( vCol.indexOf(str)<0 ) vCol.addElement(str);
                }
             }
@@ -398,8 +398,8 @@ public class FrameColumnCalculator extends JFrame implements ActionListener {
            Obj o = it.next();
            if( !(o instanceof Source) ) continue;
            s = (Source)o;
-		   for (int k = s.leg.field.length - 1; k >= 0; k--) {
-		      str = s.leg.field[k].name;
+		   for (int k = s.getLeg().field.length - 1; k >= 0; k--) {
+		      str = s.getLeg().field[k].name;
 		      if (v.indexOf(str) < 0) v.addElement(str);
 		   }
 		}
