@@ -1,10 +1,10 @@
-// Copyright 1999-2018 - Universit de Strasbourg/CNRS
+// Copyright 1999-2020 - Universit de Strasbourg/CNRS
 // The Aladin Desktop program is developped by the Centre de Donnes
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
 //
-//This file is part of Aladin.
+//This file is part of Aladin Desktop.
 //
 //    Aladin Desktop is free software: you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -16,22 +16,24 @@
 //    GNU General Public License for more details.
 //
 //    The GNU General Public License is available in COPYING file
-//    along with Aladin.
+//    along with Aladin Desktop.
 //
 
 package cds.aladin;
 
 import java.awt.AWTEvent;
 import java.awt.BorderLayout;
+import java.awt.FlowLayout;
 import java.awt.GridBagConstraints;
 import java.awt.GridBagLayout;
 import java.awt.Insets;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
 import java.awt.event.WindowEvent;
-import java.text.ParseException;
 
 import javax.swing.BorderFactory;
+import javax.swing.JButton;
+import javax.swing.JComboBox;
 import javax.swing.JFrame;
 import javax.swing.JLabel;
 import javax.swing.JPanel;
@@ -71,11 +73,19 @@ public class FrameCooToolbox extends JFrame {
    private double rv=0;              // Vlocit radiale (km/s)
    private double plx=0;             // Parallax (mas)
    private double targetEpoch=2000;  // Epoque pour laquelle on veut calculer la position
+   private int precision=-1;         // Prcision, -1 si non prcise
 
    private JTextField cooField[];   // Les champs des coordonnes dans les diffrents systmes
    private JTextField pmraField,pmdeField,originEpochField,targetEpochField;
    private JTextField distField,plxField,rvField,distALField,distParsecField;
    private SliderEpochTool sliderEpoch;
+   private JComboBox<String> precisionField;
+   
+   static private String PREC_S  [] = { "unknown", "1degree", "0.1degree", "1arcmin", "0.1arcmin", "1arcsec",
+   "0.1arcsec", "10mas", "1mas", "0.1mas", "10\u00b5as",
+   "1\u00b5as", "0.1\u00b5as", "0.01\u00b5as"  };
+
+   static private int    PREC_VAL[] = { -1,   1,2,3,4,5,6,7,8,9,10,11,12 };
    
    protected FrameCooToolbox(Aladin aladin) {
       super();
@@ -87,6 +97,7 @@ public class FrameCooToolbox extends JFrame {
       setLocation( Aladin.computeLocation(this) );
       getContentPane().add( createPanelLeft(), BorderLayout.WEST);
       getContentPane().add( createPanelRight(), BorderLayout.EAST);
+      getContentPane().add( getPanelBottom(), BorderLayout.SOUTH);
       pack();
       setVisible(true);
    }
@@ -135,6 +146,30 @@ public class FrameCooToolbox extends JFrame {
       return p;
    }
    
+   /** Construction du panel des boutons de validation
+    * @return Le panel contenant les boutons Apply/Close
+    */
+   protected JPanel getPanelBottom() {
+      JPanel p = new JPanel();
+      p.setLayout( new FlowLayout(FlowLayout.CENTER));
+      JButton b;
+//      p.add( b=new JButton(aladin.chaine.getString("UPAPPLY"))); 
+//      b.addActionListener( new ActionListener() { public void actionPerformed(ActionEvent e) { modify(); }} );
+//      b.setFont(b.getFont().deriveFont(Font.BOLD));
+      p.add( b=new JButton(aladin.chaine.getString("TAGIT"))); 
+      b.addActionListener( new ActionListener() { public void actionPerformed(ActionEvent e) { tagit(); }} );
+      p.add( b=new JButton(aladin.chaine.getString("CLEAR"))); 
+      b.addActionListener( new ActionListener() { public void actionPerformed(ActionEvent e) { reset(); }} );
+      p.add( b=new JButton(aladin.chaine.getString("UPCLOSE"))); 
+      b.addActionListener( new ActionListener() { 
+         public void actionPerformed(ActionEvent e) { 
+            aladin.frameCooTool=null;
+            dispose(); }
+      });
+      return p;
+   }
+
+   
    // Cration du panel contenant les poques et les composantes du PM
    private JPanel createPanelLeft() {
       GridBagConstraints c = new GridBagConstraints();
@@ -211,6 +246,14 @@ public class FrameCooToolbox extends JFrame {
          }
       });
       
+      JLabel precisionLabel = new JLabel("Precision");
+      precisionField = new JComboBox<>( PREC_S );
+      precisionField.addActionListener( new ActionListener() {
+         public void actionPerformed(ActionEvent e) {
+            modify();
+         }
+      });
+      
       sliderEpoch = new SliderEpochTool(this);
       
       PropPanel.addCouple(p,pmraLabel, pmraField, g,c);
@@ -232,6 +275,7 @@ public class FrameCooToolbox extends JFrame {
       distField = new JTextField(10);
       distField.setEditable(false);
       PropPanel.addCouple(p,distLabel, distField, g,c);
+      PropPanel.addCouple(p,precisionLabel, precisionField, g,c);
 
       return p;
    }
@@ -244,11 +288,12 @@ public class FrameCooToolbox extends JFrame {
     * @param s La source dont on extrait la position
     */
    protected void setSource(Source s)  {
-      Legende leg = s.leg;
+      Legende leg = s.getLeg();
       
       // Rcupration des coordonnes d'origines
       int nra   = leg.getRa();
       int ndec  = leg.getDe();
+      if( nra==-1 && ndec==-1 ) return;
       String sra = s.getValue(nra);
       String sdec= s.getValue(ndec);
       
@@ -261,44 +306,47 @@ public class FrameCooToolbox extends JFrame {
          ra = c.getLon();
          dec = c.getLat();
          
+         pmra=pmdec=0;
+         
          // Rcupration des mouvements propres
          int npmra = leg.getPmRa();
          int npmde = leg.getPmDe();
          if( npmra>0 && npmde>0 ) {
             Unit mu1 = new Unit();
             try {
-               mu1.setUnit(s.getUnit(npmra));
+               mu1.setUnit( Util.adjustFoxUnit( s.getUnit(npmra) ));
                mu1.setValue(s.getValue(npmra));
-            } catch( Exception e1 ) { e1.printStackTrace(); }
+            } catch( Exception e1 ) { }
             Unit mu2 = new Unit();
             try {
-               mu2.setUnit(s.getUnit(npmde));
+               mu2.setUnit( Util.adjustFoxUnit( s.getUnit(npmde) ));
                mu2.setValue(s.getValue(npmde));
-            } catch( Exception e1 ) { e1.printStackTrace();  }
-            if( mu1.getValue()!=0 || mu2.getValue()!=0 ) {
+            } catch( Exception e1 ) { }
+            if( (mu1.getValue()!=0 || mu2.getValue()!=0) 
+                  && !Double.isNaN(mu1.getValue()) && !Double.isNaN(mu2.getValue()) ) {
                try {
                   mu1.convertTo(new Unit("mas/yr"));
                } catch( Exception e) { 
                   // Il faut reinitialiser parce que mu1 a chang d'unit malgr l'chec !
-                  mu1.setUnit(s.getUnit(npmra));
+                  mu1.setUnit( Util.adjustFoxUnit( s.getUnit(npmra) ));
                   mu1.setValue(s.getValue(npmra));
                   mu1.convertTo(new Unit("ms/yr"));
                   double v = 15*mu1.getValue()*Math.cos(c.getLat()*Math.PI/180);
                   mu1 = new Unit(v+"mas/yr");
                };
 
-               pmra = Util.round( mu1.getValue(),3);
+               pmra =mu1.getValue();
                mu2.convertTo(new Unit("mas/yr"));
-               pmdec = Util.round( mu2.getValue(),3);
+               pmdec = mu2.getValue();
             }
-         } else pmra=pmdec=0;
+         }
          
       } catch( Exception e ) {
          e.printStackTrace();
       }
       
       // Rcupration de l'poque
-      originEpoch = 2000;
+      originEpoch = s.plan.getOriginalEpoch().getJyr();
       targetEpoch = s.plan.getEpoch().getJyr();
       
       init=true;
@@ -353,6 +401,7 @@ public class FrameCooToolbox extends JFrame {
          aft.convertTo( Localisation.getAstroframe(Localisation.ICRS) );
          originEpochField.setText(aft.epoch+"");
          targetEpochField.setText(aft.epoch+"");
+         precision=-1;
          ra=aft.getLon();
          dec=aft.getLat();
          init=true;
@@ -378,6 +427,10 @@ public class FrameCooToolbox extends JFrame {
          targetEpoch = (new Astrotime(s)).getJyr();
       } catch( Exception e ) { }
 
+      try {
+         precision = PREC_VAL[ precisionField.getSelectedIndex() ];
+      } catch( Exception e ) { precision=-1; }
+
       try {
          pmra = getField(pmraField);
       } catch( Exception e ) { }
@@ -405,6 +458,20 @@ public class FrameCooToolbox extends JFrame {
       moveRepere();      
    }
    
+   // Cration d'un tag  la position indique, et dplacement des vues  l'emplacement correspondant
+   private void tagit() {
+      try {
+         Plan plan = aladin.calque.selectPlanTool();
+         Coord coo = getCoordInICRS();
+         Tag tag = new Tag(plan,coo,null);
+         aladin.calque.updateToolCatTag(tag,aladin.view.getCurrentView());
+//         aladin.view.setRepere(coo);
+         aladin.view.gotoThere( coo );
+      } catch( Exception e ) {
+         e.printStackTrace();
+      }
+   }
+   
    // Retourne la valeur numrique du champ qui peut tre une expression algbrique
    private double getField(JTextField field) throws Exception {
       String s = field.getText();
@@ -429,10 +496,12 @@ public class FrameCooToolbox extends JFrame {
       
       originEpochField.setText( !init ? "" : originEpoch+"" );
       targetEpochField.setText( !init ? "" : targetEpoch+"" );
-      pmraField.setText(  (pmdec==0 && pmra==0) ? "" : Util.myRound(pmra) );
-      pmdeField.setText(  (pmdec==0 && pmra==0) ? "" :Util.myRound( pmdec) );
-      rvField.setText(    rv==0 ? "" : Util.myRound(rv) );
-      plxField.setText(   getParallaxe() );
+//      int i = Util.indexInArrayOf(precision+"", PREC_S);
+//      precisionField.setSelectedIndex( !init ||i==-1 ? 0 : i );
+      pmraField.setText(  !init ? "" : (pmdec==0 && pmra==0) ? "" : Util.myRound(pmra+"",3) );
+      pmdeField.setText(  !init ? "" : (pmdec==0 && pmra==0) ? "" :Util.myRound( pmdec+"",3) );
+      rvField.setText(    !init ? "" : rv==0 ? "" : Util.myRound(rv+"",3) );
+      plxField.setText(   !init ? "" : getParallaxe() );
       
       distField.setText( getDistance() );
       distALField.setText( getDistAL() );
@@ -443,18 +512,23 @@ public class FrameCooToolbox extends JFrame {
       if( flagLog ) aladin.glu.log("CoordToolbox","");
    }
    
-   
-   /** Dplace le rticule  la position courante */
-   private void moveRepere() {
-      try {
-         Coo c = new Coo(cooField[0].getText() );
-         aladin.view.gotoThere( new Coord(c.getLon(),c.getLat()) );
-      } catch( ParseException e ) {
-         if( Aladin.levelTrace>=3 )  e.printStackTrace();
-      }
+   // Reset complet du formulaire
+   private void reset() {
+      init=false;
+      resume(false);
+      precision=-1;
+      precisionField.setSelectedIndex(0);
    }
    
    
+//   /** Dplace le rticule  la position courante */
+   private void moveRepere() {
+      try {
+         Coord coo = getCoordInICRS();
+         aladin.view.gotoThere( coo );
+      } catch( Exception e ) { }
+   }
+   
    static private double PARSEC2AL = 3.2614945566008;
   
    
@@ -487,6 +561,7 @@ public class FrameCooToolbox extends JFrame {
       Astropos targetCoo = new Astropos(Localisation.AF_ICRS);
       targetCoo.set(ra,dec,originEpoch,null,pmra,pmdec,originEpoch,null,
             new double[] { plx,0}, new double []{ rv,0});
+      if( precision>-1 ) targetCoo.setPrecision(precision);
       targetCoo.toEpoch(targetEpoch);
      
 //      System.out.println("==> "+targetCoo);
@@ -499,6 +574,7 @@ public class FrameCooToolbox extends JFrame {
       Astropos targetCoo = new Astropos(Localisation.AF_ICRS);
       targetCoo.set(ra,dec,originEpoch,null,pmra,pmdec,originEpoch,null,
             new double[] { plx,0}, new double []{ rv,0});
+      if( precision>-1 ) targetCoo.setPrecision(precision);
       targetCoo.toEpoch(targetEpoch);
      
 //      System.out.println("==> "+targetCoo);
@@ -506,6 +582,19 @@ public class FrameCooToolbox extends JFrame {
       return Coord.getUnit( d );
    }
    
+   // Calcule les coordonnes en ICRS  l'poque indique
+   private Coord getCoordInICRS() {
+      Astropos targetCoo = new Astropos(Localisation.AF_ICRS);
+      targetCoo.set(ra,dec,originEpoch,null,pmra,pmdec,originEpoch,null,
+            new double[] { plx,0}, new double []{ rv,0});
+      if( precision>-1 ) targetCoo.setPrecision(precision);
+      targetCoo.convertTo( Localisation.getAstroframe(Localisation.ICRS) );
+      targetCoo.toEpoch(targetEpoch);
+      
+      Coord coo = new Coord( targetCoo.getLon(), targetCoo.getLat() );
+      return coo;
+   }
+   
    // Calcule et dite les coordonnes dans un systme donne (frameTarget) en prenant
    // en compte l'poque d'origine, l'poque cible et le mouvement propre
    private String getCoordIn(int frameTarget) {
@@ -513,6 +602,7 @@ public class FrameCooToolbox extends JFrame {
       Astropos targetCoo = new Astropos(Localisation.AF_ICRS);
       targetCoo.set(ra,dec,originEpoch,null,pmra,pmdec,originEpoch,null,
             new double[] { plx,0}, new double []{ rv,0});
+      if( precision>-1 ) targetCoo.setPrecision(precision);
       targetCoo.convertTo( Localisation.getAstroframe(frameTarget) );
       targetCoo.toEpoch(targetEpoch);
       
