@@ -1,10 +1,10 @@
-// Copyright 1999-2018 - Universit de Strasbourg/CNRS
+// Copyright 1999-2020 - Universit de Strasbourg/CNRS
 // The Aladin Desktop program is developped by the Centre de Donnes
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
 //
-//This file is part of Aladin.
+//This file is part of Aladin Desktop.
 //
 //    Aladin Desktop is free software: you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 //    GNU General Public License for more details.
 //
 //    The GNU General Public License is available in COPYING file
-//    along with Aladin.
+//    along with Aladin Desktop.
 //
 
 package cds.aladin;
@@ -25,6 +25,8 @@ import java.awt.BorderLayout;
 import java.awt.Component;
 import java.awt.Dimension;
 import java.awt.Font;
+import java.awt.event.MouseWheelEvent;
+import java.awt.event.MouseWheelListener;
 import java.util.ArrayList;
 import java.util.Enumeration;
 import java.util.Vector;
@@ -88,7 +90,7 @@ public final class Legende extends AbstractTableModel  {
 
    /** Construction d'une lgende gnrique en fonction d'un tableau de lgendes */
    protected Legende(ArrayList<Legende> leg) {
-      ArrayList<Field> f = new ArrayList<Field>(100);
+      ArrayList<Field> f = new ArrayList<>(100);
       for(int i=0; i<leg.size(); i++ ) {
          Legende lg = leg.get(i);
          for( int j=0; j<lg.field.length; j++) {
@@ -202,6 +204,12 @@ public final class Legende extends AbstractTableModel  {
       for( int i=0; i<field.length; i++ ) if( utype.equals(field[i].utype) ) return i;
       return -1;
    }
+   
+   /** Retourne l'indice du champ TIME (JD,MJD ou ISOTIME), sinon -1 */
+   protected int getTime() {
+      for( int i=0; i<field.length; i++ ) if( field[i].isTime() ) return i;
+      return -1;
+   }
 
    /** Retourne l'indice du champ RA, sinon -1 */
    protected int getRa() {
@@ -438,11 +446,14 @@ public final class Legende extends AbstractTableModel  {
    protected int getPrecision(int i) {
       if( i>=field.length ) return -1;
       Field f = field[i];
-      try { return Integer.parseInt(f.precision); }
-      catch( Exception e ) {}
+      try { 
+         // Il peut y avoir une lettre avant (ex: F8)
+         int offset = Character.isDigit( f.precision.charAt(0) ) ? 0 : 1;
+         return Integer.parseInt(f.precision.substring(offset) ); 
+      } catch( Exception e ) {}
       return -1;
    }
-
+   
    /** Retourne la desc
   /** Retourne le nombre de caracteres associes au champ.
     * @param i numero du champ
@@ -505,6 +516,7 @@ public final class Legende extends AbstractTableModel  {
 
    protected String getID(int i)       { return i>=field.length?null:field[i].ID;     }
    protected String getName(int i)     { return i>=field.length?null:field[i].name;     }
+   protected String getDescription(int i) { return i>=field.length?null:field[i].description;     }
    protected String getHref(int i)     { return i>=field.length?null:field[i].href;     }
    protected String getGref(int i)     { return i>=field.length?null:field[i].gref;     }
    protected String getRefText(int i)  { return i>=field.length?null:field[i].refText;  }
@@ -562,12 +574,48 @@ public final class Legende extends AbstractTableModel  {
       return p;
    }
 
-   protected JComboBox createCombo() {
-      JComboBox combo = new JComboBox();
+   protected JComboBox<String> createCombo() { return createCombo(false); }
+   protected JComboBox<String> createCombo(boolean forPlot) {
+      JComboBox<String> combo = new JComboBox<>();
+      combo.setPrototypeDisplayValue("12345678901234567890123456789");
       combo.setMaximumRowCount(15);
-      for( int i=0; i<field.length; i++ ) combo.addItem(field[i].name);
+      for( int i=0; i<field.length; i++ ) {
+         if( forPlot &&  !(field[i].coo>0 || field[i].isNumDataType()) ) continue;
+         combo.addItem( getNameAndDescription(i) );
+      }
+      combo.addMouseWheelListener( new MouseWheelListener() {
+         public void mouseWheelMoved(MouseWheelEvent e) {
+            JComboBox<String> c = (JComboBox)e.getSource();
+            int sens = e.getWheelRotation();
+            int i = c.getSelectedIndex();
+            i+=sens;
+            if( i<0 ) i=0;
+            if( i>=c.getItemCount() ) i=c.getItemCount()-1;
+            c.setSelectedIndex(i);
+         }
+      });
+
       return combo;
    }
+   
+   protected String getNameAndDescription(int i) {
+      return field[i].name + (field[i].description!=null ? " - "+field[i].description:"");
+   }
+   
+   /** Retourne l'indice du premier champ numrique */
+   protected int getIndexNumericField() { return getIndexNumericField(-1); }
+   
+   /** Retourne l'indice du premier champ numrique aprs l'indique indiqu
+    * @param after
+    * @return
+    */
+   protected int getIndexNumericField(int after) {
+      for( int i=after+1; i<field.length; i++ ) {
+         if( !field[i].isNumDataType() || field[i].coo>0 ) continue;
+         return i;
+      }
+      return -1;
+   }
 
    protected JTable getTable() {
       if( table==null ) createTable();
@@ -641,6 +689,7 @@ public final class Legende extends AbstractTableModel  {
          if( !s.equals( field[i].getCooSignature() ) ) {
             int coo = Util.indexInArrayOf(s, Field.COOSIGN);
             modifyRaDecXYField(i,coo);
+            plan.resetTimeFieldCache();
             frameInfo.epochFieldActivate( Field.isEquatorial(coo) );
          }
          break;
@@ -655,6 +704,7 @@ public final class Legende extends AbstractTableModel  {
          case PRECISION:   field[i].precision = (String)value; break;
       }
       aladin.mesure.redisplay();
+      TapManager.getInstance(aladin).reAddPlan(plan);
    }
 
    class MyRenderer extends DefaultTableCellRenderer {
@@ -786,6 +836,15 @@ public final class Legende extends AbstractTableModel  {
          }
          field[index].coo=coo;
          if( nlon>=0 && nlat>=0 && coo!=0 ) plan.modifyLonLatField(this, nlon,nlat, Localisation.ECLIPTIC);
+       
+         // Pour le temps
+      } else if( Field.isTime(coo) ) {
+         for( int i=0; i<field.length; i++ ) {
+            Field f = field[i];
+            if( f.isTime() ) { f.coo=0; break; }
+         }
+         field[index].coo=coo;
+         plan.modifyTimeField(this, index, coo);
 
          // Pour les coordonnes cartsiennes
       } else {
