@@ -1,10 +1,10 @@
-// Copyright 1999-2018 - Universit de Strasbourg/CNRS
+// Copyright 1999-2020 - Universit de Strasbourg/CNRS
 // The Aladin Desktop program is developped by the Centre de Donnes
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
 //
-//This file is part of Aladin.
+//This file is part of Aladin Desktop.
 //
 //    Aladin Desktop is free software: you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 //    GNU General Public License for more details.
 //
 //    The GNU General Public License is available in COPYING file
-//    along with Aladin.
+//    along with Aladin Desktop.
 //
 
 package cds.aladin;
@@ -29,17 +29,18 @@ import java.awt.Polygon;
 import java.awt.Rectangle;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
+import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Comparator;
 import java.util.Iterator;
 import java.util.Vector;
 
-import cds.aladin.Hist.HistItem;
+import cds.aladin.ZoomHist.HistItem;
 import cds.aladin.prop.Prop;
 import cds.aladin.prop.PropAction;
+import cds.tools.FastMath;
 import cds.tools.Util;
 import cds.tools.pixtools.CDSHealpix;
-import healpix.essentials.FastMath;
 
 /**
  * Objet graphique pour une Ligne.
@@ -637,6 +638,8 @@ public class Ligne extends Position {
    //         e.printStackTrace();
    //      }
    //   }
+   
+   private static final int MAXSTATPIXELS = 1000000;
 
    /**
     * Mthode:
@@ -652,16 +655,20 @@ public class Ligne extends Position {
    protected boolean statCompute(Graphics g,ViewSimple v) {
 
       if( bout!=3 ) return false;
+      
 
-      if( v!=null && !v.isFree() && v.pref.type==Plan.ALLSKYIMG ) {
-         //         ((PlanBG)v.pref).setDebugIn(this);
-         return false;
-      }
+//      if( v!=null && !v.isFree() && v.pref.type==Plan.ALLSKYIMG ) {
+//         //         ((PlanBG)v.pref).setDebugIn(this);
+//         return false;
+//      }
 
+      
+      if( v.flagClicAndDrag) return false;
+      
       if( v==null || v.isFree() || !hasPhot(v.pref) ) return false;
 
       boolean isHiPS = (v.pref.type == Plan.ALLSKYIMG);
-
+      ArrayList<double[]> cooList = isHiPS ? new ArrayList<double[]>() : null;
 
       statInit();
 
@@ -679,16 +686,18 @@ public class Ligne extends Position {
       mina=mind=minx1=miny1=Integer.MAX_VALUE;
       maxa=maxd=maxx1=maxy1=Integer.MIN_VALUE;
 
+      double pixelSurf = 0;
+
 
       tmp = deb=getFirstBout();
       for( nb=0; tmp.finligne!=null; tmp=tmp.finligne) nb++;
       Segment [] seg = new Segment[nb];
 
       for( i=0, tmp=deb; tmp.finligne!=null; tmp=tmp.finligne, i++ ) {
-         int fx=(int)Math.floor(tmp.xv[v.n]-0.5);
-         int tx=(int)Math.ceil(tmp.xv[v.n]-0.5);
-         int fy=(int)Math.floor(tmp.yv[v.n]-0.5);
-         int ty=(int)Math.ceil(tmp.yv[v.n]-0.5);
+         double fx=tmp.xv[v.n];
+         double tx=tmp.xv[v.n];
+         double fy=tmp.yv[v.n];
+         double ty=tmp.yv[v.n];
          if( tx>maxx ) maxx=tx;
          if( fx<minx ) minx=fx;
          if( ty>maxy ) maxy=ty;
@@ -702,8 +711,14 @@ public class Ligne extends Position {
          a=tmp; b=tmp.finligne;
          if( tmp.yv[v.n]> tmp.finligne.yv[v.n] ) { b=tmp ; a=tmp.finligne; }
 
-         if( isHiPS ) seg[i] = new Segment(a.raj,a.dej, b.raj,b.dej);
-         else seg[i] = new Segment(a.xv[v.n]-0.5,a.yv[v.n]-0.5, b.xv[v.n]-0.5,b.yv[v.n]-0.5);
+         if( isHiPS ) {
+            seg[i] = new Segment(a.raj,a.dej, b.raj,b.dej);
+            
+            Coord coo = new Coord(tmp.raj,tmp.dej);
+            coo = Localisation.frameToFrame(coo,Localisation.ICRS,((PlanBG)v.pref).frameOrigin);
+            cooList.add( new double[]{ coo.al, coo.del });
+            
+         } else seg[i] = new Segment(a.xv[v.n]-0.5,a.yv[v.n]-0.5, b.xv[v.n]-0.5,b.yv[v.n]-0.5);
       }
 
       Arrays.sort(seg, seg[0]);
@@ -725,16 +740,53 @@ public class Ligne extends Position {
       if( isHiPS ) {
 
          PlanBG pbg = (PlanBG) v.pref;
-         double d = CDSHealpix.pixRes( CDSHealpix.pow2( pbg.getOrder() + pbg.getTileOrder() ) ) / 3600;
+         double d = CDSHealpix.pixRes( pbg.getOrder() + pbg.getTileOrder() ) / 3600;
          d /= 2;   // Pour tre sur de ne pas sauter une ligne
-         System.out.println("From "+Coord.getUnit(mind)+" to "+Coord.getUnit(maxd)+" Delta = " + Coord.getUnit(d) );
+//         System.out.println("From "+Coord.getUnit(mind)+" to "+Coord.getUnit(maxd)+" Delta = " + Coord.getUnit(d) );
          Coord haut = new Coord( (maxa+mina)/2, mind);
          Coord bas  = new Coord( (maxa+mina)/2, maxd);
          double dist = Coord.getDist(haut, bas);
          int n = (int) (dist/d);
-         System.out.println("Distance entre "+haut+"  et "+bas+" => "+Coord.getUnit(dist)+" soit "+n+" itrations");
+//         System.out.println("Distance entre "+haut+"  et "+bas+" => "+Coord.getUnit(dist)+" soit "+n+" itrations");
+
+//         return false;
+         
+         try {
+            int orderFile = pbg.getOrder();
+//            long nsideFile = CDSHealpix.pow2(orderFile);
+            long nsideLosange = CDSHealpix.pow2(pbg.getTileOrder());
+//            long nside = nsideFile * nsideLosange;
+            int order = orderFile + pbg.getTileOrder();
+            long [] npix=null;
+            
+            // Dgradation de la rsolution si trop de pixel
+            for( long o = order; o>=pbg.getMinOrder(); o--) {
+//               nside = CDSHealpix.pow2(o);
+               pixelSurf = CDSHealpix.pixRes(order)/3600;
+               pixelSurf *= pixelSurf;
+               npix = CDSHealpix.query_polygon(order, cooList,false);
+               if( npix.length<=MAXSTATPIXELS ) break;
+            }; 
+            
+            // Trop de pixels ?
+            if( npix.length>MAXSTATPIXELS ) return false;
+            
+            Coord coo = new Coord();
+            for( i=0; i<npix.length; i++ ) {
+               long npixFile = npix[i]/(nsideLosange*nsideLosange);
+               double pix = pbg.getHealpixPixel(orderFile,npixFile,npix[i],HealpixKey.SYNC);
+               if( Double.isNaN(pix) ) continue;
+               pix = pix*pbg.bScale+pbg.bZero;
+               double polar[] = CDSHealpix.pix2ang_nest(order, npix[i]);
+               polar = CDSHealpix.polarToRadec(polar);
+               coo.al = polar[0]; coo.del = polar[1];
+               coo = Localisation.frameToFrame(coo,pbg.frameOrigin,Localisation.ICRS);
+               statPixel(g,pix,coo.al,coo.del,v,onMouse);
+               if( flagHist ) v.aladin.view.zoomview.addPixelHist(pix);
+            }
+//            System.out.println("==> nombre="+npix.length+" total="+total+" => moyenne="+(total/nombre));
+         } catch( Exception e ) { e.printStackTrace(); }
 
-         return false;
 
       } else {
 
@@ -759,7 +811,10 @@ public class Ligne extends Position {
          }
 
          minx=minx1; maxx=maxx1; miny=miny1; maxy1=maxy;
-      }
+         try {
+            pixelSurf = plan.proj[v.n].getPixResAlpha()*plan.proj[v.n].getPixResDelta();
+         } catch( Exception e ) {}
+     }
 
       if( flagHist ) plan.aladin.view.zoomview.createPixelHist("Pixels");
 
@@ -767,7 +822,6 @@ public class Ligne extends Position {
       // Calculs des statistiques => sera utilis immdiatement par le paint
       // Attention, il s'agit de variables statiques
       try {
-         double pixelSurf = plan.proj[v.n].getPixResAlpha()*plan.proj[v.n].getPixResDelta();
          surface = nombre*pixelSurf;
          moyenne = total/nombre;
          variance = carre/nombre - moyenne*moyenne;
