@@ -1,10 +1,10 @@
-// Copyright 1999-2018 - Universit de Strasbourg/CNRS
+// Copyright 1999-2020 - Universit de Strasbourg/CNRS
 // The Aladin Desktop program is developped by the Centre de Donnes
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
 //
-//This file is part of Aladin.
+//This file is part of Aladin Desktop.
 //
 //    Aladin Desktop is free software: you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 //    GNU General Public License for more details.
 //
 //    The GNU General Public License is available in COPYING file
-//    along with Aladin.
+//    along with Aladin Desktop.
 //
 
 package cds.aladin;
@@ -46,6 +46,7 @@ import cds.astro.FK5;
 import cds.astro.Galactic;
 import cds.astro.ICRS;
 import cds.astro.Supergal;
+import cds.tools.Astrodate;
 import cds.tools.Util;
 
 /**
@@ -492,21 +493,20 @@ public class Localisation extends MyBox  {
 
    
    /** Retourne le frame "gnrique" courant (ICRS | GAL | SGAL | ECLIPTIC | -1) */
-   protected int getFrameGeneric() {
+   public int getFrameGeneric() {
       return frame==GAL ? GAL :
              frame==ECLIPTIC ? ECLIPTIC :
              frame==SGAL ? SGAL :
              (frame!=XY || frame!=XYNAT || frame!=XYLINEAR || frame!=PLANET) ? ICRS : -1;
    }
    
-   
    /** Insre le rsultat d'une rsolution Ssame dans le champ de commande avec le label
     * POSITION histoire que cela se comprenne */
    protected void setSesameResult(String s) {
-      aladin.localisation.setTextSaisie(s);
-      aladin.localisation.readyToClear();
+      setTextSaisie(s);
+      readyToClear();
    }
-
+   
    /** Affiche la position, en fonction du frame defini
     * dans le menu deroulant
     * @param x,y Les coordonnees de la souris dans la View
@@ -519,7 +519,11 @@ public class Localisation extends MyBox  {
 
       // Forcage pour les nuage de point
       ViewSimple view = aladin.view.getMouseView();
-      if( view!=null && view.isPlotView() ) frame=XYLINEAR;
+      boolean isPlotTime = false;
+      if( view!=null && view.isPlot() ) {
+         frame=XYLINEAR;
+         isPlotTime = view.isPlotTime();
+      }
 
       Plan plan = v.pref;
       if( plan==null ) return;
@@ -555,7 +559,11 @@ public class Localisation extends MyBox  {
                s = coo.getDegPlanet();
             } else if( frame==XYLINEAR ) {
                if( !proj.isXYLinear() ) s=NOXYLINEAR;
-               else s=Util.myRound(coo.al+"",4)+" : "+Util.myRound(coo.del+"",4);
+               else {
+                  if( isPlotTime ) {
+                     s=Astrodate.JDToDate(coo.al); //+" , "+Util.myRound(coo.del+"",4);
+                  } else s=Util.myRound(coo.al+"",4)+" , "+Util.myRound(coo.del+"",4);
+               }
             } else {
                if( proj.isXYLinear() ) s=NOPROJECTION;
                else {
@@ -580,7 +588,6 @@ public class Localisation extends MyBox  {
 
       //Affichage du resultat
       if( methode==1 ) {
-         //         Aladin.copyToClipBoard(s);   C'EST VRAIMENT TROP GONFLANT
          setTextSaisie(s);
          setMode(SAISIE);
       } else {
@@ -722,8 +729,8 @@ public class Localisation extends MyBox  {
       
       // Coordonnes planto ? => rien  faire
       if( flagPlaneto ) return coo1;
-
-      // Edition et conversion si ncessaire
+      
+     // Edition et conversion si ncessaire
       try {
          
          Astrocoo aft = new Astrocoo( getAstroframe(frameSource) );
@@ -750,7 +757,7 @@ public class Localisation extends MyBox  {
     * @param al,del : coordonnees (ICRS)
     * @return La chaine decrivant la position
     */
-   protected String J2000ToString(double al,double del) { return J2000ToString(al,del,Astrocoo.ARCSEC+1,false); }
+   protected String J2000ToString(double al,double del) { return J2000ToString(al,del,/* Astrocoo.ARCSEC+1*/Astrocoo.MAS+3 ,false); }
    protected String J2000ToString(double al,double del,int precision,boolean withFox) {
       Coord cTmp = new Coord(al,del);
       cTmp = ICRSToFrame(cTmp);
