@@ -1,10 +1,10 @@
-// Copyright 1999-2018 - Universit de Strasbourg/CNRS
+// Copyright 1999-2020 - Universit de Strasbourg/CNRS
 // The Aladin Desktop program is developped by the Centre de Donnes
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
 //
-//This file is part of Aladin.
+//This file is part of Aladin Desktop.
 //
 //    Aladin Desktop is free software: you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 //    GNU General Public License for more details.
 //
 //    The GNU General Public License is available in COPYING file
-//    along with Aladin.
+//    along with Aladin Desktop.
 //
 
 package cds.aladin;
@@ -42,10 +42,8 @@ import java.awt.event.MouseWheelEvent;
 import java.awt.event.MouseWheelListener;
 import java.util.ArrayList;
 import java.util.Enumeration;
-import java.util.Formatter;
 import java.util.HashSet;
 import java.util.Hashtable;
-import java.util.Locale;
 import java.util.Vector;
 
 import javax.swing.JComponent;
@@ -455,7 +453,7 @@ MouseWheelListener, Widget
          JMenuItem item;
          menuBroadcastSpectrum.removeAll();
          
-         HashSet<String> set = new HashSet<String>();
+         HashSet<String> set = new HashSet<>();
          if( spectrumApps!=null && spectrumApps.size()>0) {
             for (String appName: spectrumApps) {
                if( set.contains(appName) ) continue;
@@ -601,9 +599,11 @@ MouseWheelListener, Widget
             //            int pos = w.precision>0 ? w.precision+1 : w.precision;
             //            if( i+pos<text.length() ) text = text.substring(0,i+pos);
 
+            
             try {
-               double v = Double.parseDouble(w.text);
-               text = (new Formatter(Locale.ENGLISH)).format("%."+w.precision+"f", v).toString();
+//               double v = Double.parseDouble(w.text);
+//               text = (new Formatter(Locale.ENGLISH)).format("%."+w.precision+"f", v).toString();
+               text = Util.myRound(w.text,w.precision, false);
                int i = text.lastIndexOf('.');
                if( i>0 ) {
                   int k;
@@ -616,7 +616,7 @@ MouseWheelListener, Widget
                      text = text.substring(0,k+1)+trail.toString();
                   }
                }
-            } catch( Exception e) { }
+            } catch( Exception e) {}
          }
       }
 
@@ -835,7 +835,7 @@ MouseWheelListener, Widget
 
    /** Redessine l'entte qui si c'est ncessaire (sur un mouseMoved par exemple) */
    private void quickDrawHead(Graphics g,Source o) {
-      if( o!=null && o.leg==oleg  ) return;      // dj fait
+      if( o!=null && o.getLeg()==oleg  ) return;      // dj fait
       drawHead(g,o);
    }
 
@@ -853,10 +853,10 @@ MouseWheelListener, Widget
    protected void drawHead(Graphics g,Source o) { drawHead(g,0,0,o,W); }
    protected void drawHead(Graphics g,int X,int y,Source o,int W) {
       Vector head;
-      if( o==null && oleg==null || o!=null && oleg==o.leg ) head=ligneHead;
+      if( o==null && oleg==null || o!=null && oleg==o.getLeg() ) head=ligneHead;
       else {
          head= o==null ? null : aladin.mesure.getHeadLine(o);
-         oleg= o==null ? null : o.leg;
+         oleg= o==null ? null : o.getLeg();
          ligneHead=head;
       }
 
@@ -961,12 +961,12 @@ MouseWheelListener, Widget
       //      if( nField!=realNField ) System.out.println("nField="+nField+" realdNField="+realNField);
       //      nField = realNField;
       if( nField==-1 ) {
-         o.leg.clearSort();
+         o.getLeg().clearSort();
          triTag = Field.SORT_ASCENDING;
          ascending = true;
       } else {
          triTag = Field.UNSORT;
-         ascending = o.leg.switchSort(nField);
+         ascending = o.getLeg().switchSort(nField);
       }
       ligneHead = aladin.mesure.getHeadLine(o);
       aladin.mesure.tri(o,nField,ascending);
@@ -988,7 +988,7 @@ MouseWheelListener, Widget
    /** Tri des sources de mme type que celle passe en paramtre. */
    protected void tri(Source o,int nField,boolean ascending) {
       //      nField=o.leg.getRealFieldNumber(nField);
-      o.leg.setSort(nField, ascending?Field.SORT_ASCENDING:Field.SORT_DESCENDING);
+      o.getLeg().setSort(nField, ascending?Field.SORT_ASCENDING:Field.SORT_DESCENDING);
       ligneHead = aladin.mesure.getHeadLine(o);
       aladin.mesure.tri(o,nField,ascending);
    }
@@ -1327,9 +1327,9 @@ MouseWheelListener, Widget
    /** Construit la description du champ i de l'objet o */
    private String getDescription(Source o,int i,boolean flagShort) {
       StringBuilder res = new StringBuilder();
-      if( o.leg!=null && o.leg.field!=null && i>=0 && i<o.leg.field.length ) {
-         Field f = o.leg.field[i];
-         A(res,o.leg.name);
+      if( o.getLeg()!=null && o.getLeg().field!=null && i>=0 && i<o.getLeg().field.length ) {
+         Field f = o.getLeg().field[i];
+         A(res,o.getLeg().name);
          if( !flagShort ) {
             A(res," - ",f.name);
             A(res,": ",adjustVizier(f.description));
@@ -1341,13 +1341,14 @@ MouseWheelListener, Widget
 
    /** Construit la description longue (info) du champ i de l'objet o */
    private String getInfo(Source o,int i) {
+      Legende leg = o.getLeg();
       StringBuilder res = new StringBuilder();
-      if( o.leg!=null && o.leg.field!=null && i>=0 && i<o.leg.field.length ) {
-         Field f = o.leg.field[i];
+      if( leg!=null && leg.field!=null && i>=0 && i<leg.field.length ) {
+         Field f = leg.field[i];
          if( f.description==null && f.unit==null && f.ucd==null && f.utype==null ) return "";
          
          // Rcupration du titre de la lgende et du petit nom du obs_collection
-         String s = o.leg.name!=null ? o.leg.name : "";
+         String s = leg.name!=null ? leg.name : "";
          if( o.plan!=null ) {
             MyProperties prop = aladin.directory.getProperties( o.plan.id );
             if( prop!=null ) {
@@ -1365,7 +1366,8 @@ MouseWheelListener, Widget
          if( s.length()>50 ) s=s.substring(0,47)+"...";
          A(res,"\n* Value: ", s );
          A(res,"\n* Unit: ",f.unit);
-         if( (s = convert( v, f.unit))!=null ) A(res,"\n => ",s);
+         if( f.isTime() ) A(res,"\n => "+Astrodate.JDToDate( Astrodate.parseTime(s, f.coo)));
+         else if( (s = convert( v, f.unit))!=null ) A(res,"\n => ",s);
          A(res,"\n* UCD: ",f.ucd);
          A(res,"\n* Utype: ",f.utype);
          A(res,"\n \n",adjustVizier(f.description));
@@ -1379,11 +1381,13 @@ MouseWheelListener, Widget
 
       // Une date en JD ou MJD => ISO
       boolean mjd=false;
-      if( unit.equalsIgnoreCase("jd") || (mjd=unit.equalsIgnoreCase("mjd")) )  {
-         double x = Double.parseDouble(val);
-         if( mjd ) x = Astrodate.MJDToJD( x );
-         return Astrodate.JDToDate( x );
-      }
+      try {
+         if( unit.equalsIgnoreCase("jd") || (mjd=unit.equalsIgnoreCase("mjd")) )  {
+            double x = Double.parseDouble(val);
+            if( mjd ) x = Astrodate.MJDToJD( x );
+            return Astrodate.JDToDate( x );
+         }
+      } catch( NumberFormatException e ) { }
       return null;
    }
    
@@ -1438,7 +1442,7 @@ MouseWheelListener, Widget
          return;
       }
 
-      if( oTimer.leg.isSED() ) {
+      if( oTimer.getLeg().isSED() ) {
          aladin.view.zoomview.setSED(oTimer);
       } else  {
          aladin.calque.zoom.zoomView.setHist(oTimer,onField);
@@ -1453,13 +1457,13 @@ MouseWheelListener, Widget
    // et il y a un dcalage d'une valeur pour tenir compte de la case  cocher en dbut de ligne
    private int getRealIndice(Source s,int indice) {
       indice--;
-      if( s.leg==null ) return indice;
+      if( s.getLeg()==null ) return indice;
       
       // Comme si on affichait  nouveau
       int pos=-1;
-      for( int i=0; i<s.leg.field.length; i++ ) {
-         int nField = s.leg.fieldAt[ i ];
-         if( s.leg.isVisible(nField) ) {
+      for( int i=0; i<s.getLeg().field.length; i++ ) {
+         int nField = s.getLeg().fieldAt[ i ];
+         if( s.getLeg().isVisible(nField) ) {
             pos++;
             if( pos==indice ) return nField;
          }
@@ -1481,7 +1485,6 @@ MouseWheelListener, Widget
       Graphics g = getGraphics();
       if( g==null ) return;
       g.setFont(FONT);
-      String s=null;            // La chaine a afficher dans le status
       Words w=null;        // Mot courant (sous la souris)
       Enumeration e;
       Source o;
@@ -1576,14 +1579,8 @@ MouseWheelListener, Widget
       if( oo!=oshow ) {
          aladin.view.showSource(oshow);
 
-         // On affiche le FoV associ s'il en existe un
-//         PlanField pf = oshow.getFootprint().getFootprint();
-//         if (pf != null) {
-//            pf.setActivated(true);
-//            pf.flagOk = true;
-//         }
-         oshow.setShowFootprintTransient(true,true);
-         if( oo!=null ) oo.setShowFootprintTransient(false,true);
+         if( oshow.isSetFootprint() ) oshow.setShowFootprintTransient(true,true);
+         if( oo!=null && o.isSetFootprint() ) oo.setShowFootprintTransient(false,true);
 
          oo=oshow;
       }
@@ -1636,8 +1633,7 @@ MouseWheelListener, Widget
 //            Util.drawEdge(g,W,H);
 
             // Recuperation de la description du champ et d'un ventuel tooltip
-            if( w.repere ) { /* tip=TIPREP; */ s=w.text; }
-            else if( w.archive )  tip=TIPARCH;
+            if( w.archive )  tip=TIPARCH;
             else if( w.glu )      tip=TIPGLU;
             else if( w.footprint)  tip=TIPFOV;
             try {
@@ -1681,7 +1677,7 @@ MouseWheelListener, Widget
 
 
    protected void showSEDPoint(Source s) {
-      if( s.leg!=null && s.leg.isSED() && aladin.view.zoomview.flagSED )  aladin.view.zoomview.setSED(s);
+      if( s.getLeg()!=null && s.getLeg().isSED() && aladin.view.zoomview.flagSED )  aladin.view.zoomview.setSED(s);
    }
 
    //   public boolean mouseEnter(Event e, int x, int y) {
