@@ -1,10 +1,10 @@
-// Copyright 1999-2018 - Universit de Strasbourg/CNRS
+// Copyright 1999-2020 - Universit de Strasbourg/CNRS
 // The Aladin Desktop program is developped by the Centre de Donnes
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
 //
-//This file is part of Aladin.
+//This file is part of Aladin Desktop.
 //
 //    Aladin Desktop is free software: you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 //    GNU General Public License for more details.
 //
 //    The GNU General Public License is available in COPYING file
-//    along with Aladin.
+//    along with Aladin Desktop.
 //
 
 package cds.aladin;
@@ -25,9 +25,11 @@ import static cds.aladin.Constants.CONTENTTYPE;
 import static cds.aladin.Constants.CONTENT_TYPE_PDF;
 import static cds.aladin.Constants.CONTENT_TYPE_TEXTHTML;
 import static cds.aladin.Constants.CONTENT_TYPE_TEXTPLAIN;
+import static cds.aladin.Constants.CONTENT_TYPE_VOTABLE;
 import static cds.aladin.Constants.SEMANTICS;
 import static cds.aladin.Constants.SEMANTIC_ACCESS;
 import static cds.aladin.Constants.SEMANTIC_CUTOUT;
+import static cds.aladin.Constants.SEMANTIC_PREVIEWPLOT;
 import static cds.aladin.Constants.SEMANTIC_PROC;
 
 import java.awt.BorderLayout;
@@ -170,7 +172,7 @@ public final class Mesure extends JPanel implements Runnable,Iterable<Source>,Wi
        // archive
        // server", o, true);
        if (this.activeDataLinkWord.datalinksInfo == null || this.activeDataLinkWord.datalinksInfo.isEmpty()) {//just instantiating
-    	   this.activeDataLinkWord.datalinksInfo = new ArrayList<SimpleData>();
+    	   this.activeDataLinkWord.datalinksInfo = new ArrayList<>();
     	   datalinkUrl = new URL(url);
        } else if (this.activeDataLinkGlu!=null && this.activeDataLinkWord.datalinksInfo.contains(this.activeDataLinkGlu)) {
 
@@ -179,7 +181,7 @@ public final class Mesure extends JPanel implements Runnable,Iterable<Source>,Wi
 				dataLinkInfoCopy.addAll(datalinksInfo);
 				dataLinkInfoCopy.remove(aladin.mesure.activeDataLinkGlu);*/
 
-    	  this.activeDataLinkWord.datalinksInfo = new ArrayList<SimpleData>();
+    	  this.activeDataLinkWord.datalinksInfo = new ArrayList<>();
           SimpleData activeDatalinkLabel = this.activeDataLinkGlu;
           datalinkUrl = new URL(activeDatalinkLabel.getParams().get(Constants.ACCESSURL));
        }
@@ -224,7 +226,6 @@ public final class Mesure extends JPanel implements Runnable,Iterable<Source>,Wi
 			j.setActionCommand(String.valueOf(i));
 			j.addActionListener(new ActionListener() {
 				public void actionPerformed(ActionEvent clickEvent) {
-					clickEvent.getSource();
 					int menuIndex = Integer.parseInt(clickEvent.getActionCommand());
 					aladin.mesure.getFormInfo(menuIndex);
 					
@@ -246,35 +247,32 @@ public final class Mesure extends JPanel implements Runnable,Iterable<Source>,Wi
 			
 			if (activeDataLinkGlu != null) {
 				Map<String,String> params = activeDataLinkGlu.getParams();
-				if (params!=null) {
+				if (params != null) {
 					String semantics = activeDataLinkGlu.getParams().get(SEMANTICS);
 					String contentType = activeDataLinkGlu.getParams().get(CONTENTTYPE);
 					
 					if (semantics.equalsIgnoreCase(SEMANTIC_CUTOUT) || semantics.equalsIgnoreCase(SEMANTIC_ACCESS)
 							|| semantics.equalsIgnoreCase(SEMANTIC_PROC)) {//TODO:: remove access semantic. added to facilitate testing.
-						if (Aladin.BETA) {
+//						if (Aladin.BETA) {
 							if (aladin.datalinkGlu == null) {
 								aladin.datalinkGlu = new DataLinkGlu(aladin);
 							}
 							aladin.datalinkGlu.createDLGlu(this.activeDataLinkSource, activeDataLinkGlu);
-						} else {
-							Aladin.error(NOCUTOUTCLIENTSUPPORT, 1);
-						}
-					} /*else if (semantics.startsWith(SEMANTIC_PREVIEW) && accessUrl != null
-							&& (contentType.equalsIgnoreCase(CONTENT_TYPE_JPEG)
-							|| contentType.equalsIgnoreCase(CONTENT_TYPE_PNG))) { //for now we just load plot image or votable in Aladin
-						aladin.glu.showDocument("Http", accessUrl, true);
-					}*/ else if (contentType != null && accessUrl != null
+//						} else {
+//							Aladin.error(NOCUTOUTCLIENTSUPPORT, 1);
+//						}
+					} else if (semantics.startsWith(SEMANTIC_PREVIEWPLOT) && accessUrl != null 
+							&& contentType.equalsIgnoreCase(CONTENT_TYPE_VOTABLE)) {
+						int x = this.activeDataLinkWord.x + this.activeDataLinkWord.w/2;
+						//to display at the popup menu click location  y = wordy* menuIndex *Server.HAUT 
+						aladin.mesure.mcanvas.toSamp(accessUrl, x, this.activeDataLinkWord.y);
+					} else if (contentType != null && accessUrl != null
 							&& (contentType.equalsIgnoreCase(CONTENT_TYPE_TEXTHTML)
 									|| contentType.equalsIgnoreCase(CONTENT_TYPE_TEXTPLAIN)
 									|| contentType.equalsIgnoreCase(CONTENT_TYPE_PDF))) {
 						aladin.glu.showDocument("Http", accessUrl, true);
-					} /*else if (contentType!=null && accessUrl!=null && contentType.contains(DATATYPE_DATALINK)) {// we won't be detecting this here. but will occur downstream at ServerFile
-						aladin.mesure.isEnabledDatalinkPopUp = true;
-						aladin.makeCursor(mcanvas, Aladin.WAITCURSOR);
-						this.activeDataLinkWord.callArchive(aladin, activeDataLinkSource, true);
-					}*/ else if (accessUrl!=null && !accessUrl.isEmpty()) {
-						aladin.calque.newPlan(activeDataLinkGlu.getParams().get(ACCESSURL), null, null);//TODO::change to access
+					} else if (accessUrl != null && !accessUrl.isEmpty()) {
+						aladin.calque.newPlan(activeDataLinkGlu.getParams().get(ACCESSURL), null, null, activeDataLinkSource);//TODO::change to access
 					} else {
 						Aladin.error("Error in loading datalink",1);
 					}
@@ -327,7 +325,7 @@ public final class Mesure extends JPanel implements Runnable,Iterable<Source>,Wi
       // Dcompte
       int nb=0;
       for( int i=0; i<nbSrc; i++ ) {
-         if( src[i].leg!=o.leg ) continue;
+         if( src[i].getLeg()!=o.getLeg() ) continue;
          nb++;
       }
 
@@ -335,7 +333,7 @@ public final class Mesure extends JPanel implements Runnable,Iterable<Source>,Wi
 
       // Rcupration des valeurs du champ indiqu
       for( int i=0,j=0; i<nbSrc; i++ ) {
-         if( src[i].leg!=o.leg ) continue;
+         if( src[i].getLeg()!=o.getLeg() ) continue;
          x[j]=Double.NaN;
          String s = src[i].getValue(nField);
          int n = s.length();
@@ -362,7 +360,7 @@ public final class Mesure extends JPanel implements Runnable,Iterable<Source>,Wi
       // Dcompte
       int nb=0;
       for( int i=0; i<nbSrc; i++ ) {
-         if( src[i].leg!=o.leg ) continue;
+         if( src[i].getLeg()!=o.getLeg() ) continue;
          nb++;
       }
 
@@ -370,7 +368,7 @@ public final class Mesure extends JPanel implements Runnable,Iterable<Source>,Wi
 
       // Rcupration des valeurs du champ indiqu
       for( int i=0,j=0; i<nbSrc; i++ ) {
-         if( src[i].leg!=o.leg ) continue;
+         if( src[i].getLeg()!=o.getLeg() ) continue;
          x[j] = src[i].getValue(nField);
          j++;
       }
@@ -532,16 +530,16 @@ public final class Mesure extends JPanel implements Runnable,Iterable<Source>,Wi
             lastOcc=s;
 
             // Y a-t-il un nom de colonne prcise ? dans le cas o je change de lgende
-            if( col.length()>0 && oLeg!=s.leg) {
-               colIndex = s.leg.matchIgnoreCaseColIndex(col.toString());
+            if( col.length()>0 && oLeg!=s.getLeg()) {
+               colIndex = s.getLeg().matchIgnoreCaseColIndex(col.toString());
                if( colIndex==-1 ) break;  // Pas dans ce plan
-               numeric = s.leg.isNumField(colIndex);
+               numeric = s.getLeg().isNumField(colIndex);
                if( numeric ) {
                   try { numS = Double.parseDouble(masq); }
                   catch(Exception e) {}
                }
                //System.out.println(s.plan.label+" mode="+mode+" col="+col+" val="+v+" colIndex="+colIndex+" dataType="+dataType+" numeric="+numeric+" numS="+numS);
-               oLeg=s.leg;
+               oLeg=s.getLeg();
             }
 
             String val[] = s.getValues();
@@ -627,7 +625,7 @@ public final class Mesure extends JPanel implements Runnable,Iterable<Source>,Wi
       aladin.calque.zoom.zoomView.stopHist();
       aladin.calque.zoom.zoomView.resumeSED();
       aladin.console.setEnabledDumpButton(true);
-      if( s.leg!=null && s.leg.isSorted() ) { s.leg.clearSort(); mcanvas.reloadHead(); }
+      if( s.getLeg()!=null && s.getLeg().isSorted() ) { s.getLeg().clearSort(); mcanvas.reloadHead(); }
       if( mcanvas.triTag!=Field.UNSORT ) mcanvas.triTag=Field.UNSORT;
    }
 
@@ -766,7 +764,7 @@ public final class Mesure extends JPanel implements Runnable,Iterable<Source>,Wi
    /** Gnration de la HeadLine associe  la source passe en paramtre */
    protected Vector getHeadLine(Source o) {
       Vector wordLine;
-      Legende leg = o.leg;
+      Legende leg = o.getLeg();
       wordLine = new Vector(leg.field.length+2);
 
       wordLine.addElement(o);           // L'objet lui-meme est tjrs en premiere place
@@ -775,8 +773,8 @@ public final class Mesure extends JPanel implements Runnable,Iterable<Source>,Wi
       for( int i=0; i<leg.field.length; i++ )  {
          int nField = leg.fieldAt[i];
          if( !leg.isVisible(nField) ) continue;
-         Words w = new Words(leg.field[nField].name,null,o.leg.getWidth(nField),o.leg.getPrecision(nField),
-               Words.CENTER,o.leg.computed.length==0?false:o.leg.computed[nField],
+         Words w = new Words(leg.field[nField].name,null,o.getLeg().getWidth(nField),o.getLeg().getPrecision(nField),
+               Words.CENTER,o.getLeg().computed.length==0?false:o.getLeg().computed[nField],
                      leg.field[nField].sort,-1);
          w.pin = i==0;
          wordLine.addElement(w);
@@ -786,7 +784,7 @@ public final class Mesure extends JPanel implements Runnable,Iterable<Source>,Wi
 
    // Liste des liens qui ont dj t cliqu (on mmorise le hashcode de l'obj et l'index
    // de la colonne concerne
-   private HashSet<String> haspushedSet = new HashSet<String>();
+   private HashSet<String> haspushedSet = new HashSet<>();
    protected void setHaspushed(Obj o, int numField) {
       String key = o.hashCode()+"/"+numField;
       haspushedSet.add( key );
@@ -824,7 +822,7 @@ public final class Mesure extends JPanel implements Runnable,Iterable<Source>,Wi
          } else {
             int nField=0;
             try {
-               nField = o.leg.fieldAt[i-1];
+               nField = o.getLeg().fieldAt[i-1];
             } catch( Exception e ) {
                // Y a un problme
                System.out.println("Y a un prob. =>"+s);
@@ -832,22 +830,22 @@ public final class Mesure extends JPanel implements Runnable,Iterable<Source>,Wi
             }
             tag = tags[nField];
 
-            if( !o.leg.isVisible(nField) ) continue;
+            if( !o.getLeg().isVisible(nField) ) continue;
 
             // Determination de l'alignement en fonction du type de donnees
-            int align= o.leg.isNumField(nField) ? Words.RIGHT : Words.LEFT;
+            int align= o.getLeg().isNumField(nField) ? Words.RIGHT : Words.LEFT;
 
             // Creation d'un mot dans le cas d'un footprint associ (Thomas, VOTech)
             if( indexFootPrint==nField ) {
-               w = new Words("  FoV",o.leg.getWidth(i-1),o.leg.getPrecision(nField),Words.LEFT,
+               w = new Words("  FoV",o.getLeg().getWidth(i-1),o.getLeg().getPrecision(nField),Words.LEFT,
                      false,true,num);
             }
             // Creation du nouveau mot
             else {
-               if( o.leg.isNullValue(tag, i-1) ) tag="";
-               w = new Words(tag,o.leg.getRefText(nField),o.leg.getWidth(nField),
-                     o.leg.getPrecision(nField),align,
-                     o.leg.computed.length==0?false:o.leg.computed[nField],Field.UNSORT,num);
+               if( o.getLeg().isNullValue(tag, i-1) ) tag="";
+               w = new Words(tag,o.getLeg().getRefText(nField),o.getLeg().getWidth(nField),
+                     o.getLeg().getPrecision(nField),align,
+                     o.getLeg().computed.length==0?false:o.getLeg().computed[nField],Field.UNSORT,num);
             }
          }
          w.show= (o==mcanvas.objSelect || o==mcanvas.objShow );
@@ -950,7 +948,7 @@ public final class Mesure extends JPanel implements Runnable,Iterable<Source>,Wi
    
    private Plan oldPlanHighlighted=null;
    
-   /** Affichage d'une info au dessous de la pile */
+   /** Affichage d'une info au dessus de la pile */
    protected void showInfo(String s, Plan p) {
       if( s==null || s.length()==0 ) aladin.calque.select.hideMessage();
       else aladin.calque.select.setMessageInfo( s );
@@ -1045,7 +1043,7 @@ public final class Mesure extends JPanel implements Runnable,Iterable<Source>,Wi
       StringBuilder sb = new StringBuilder();
       Source s = mcanvas.objSelect;
       
-      Legende leg = s.leg;
+      Legende leg = s.getLeg();
       String [] values = s.getValues();
       boolean first=true;
       
@@ -1128,7 +1126,7 @@ public final class Mesure extends JPanel implements Runnable,Iterable<Source>,Wi
       mcanvas.currentselect=-2;
 
       Source s = aladin.mesure.getFirstSrc();
-      if( s==null && aladin.view.zoomview.flagSED || s!=null && s.leg!=null && s.leg.isSED() ) {
+      if( s==null && aladin.view.zoomview.flagSED || s!=null && s.getLeg()!=null && s.getLeg().isSED() ) {
          aladin.view.zoomview.setSED(s);
       }
       mcanvas.repaint();
