@@ -1,10 +1,10 @@
-// Copyright 1999-2018 - Universit de Strasbourg/CNRS
+// Copyright 1999-2020 - Universit de Strasbourg/CNRS
 // The Aladin Desktop program is developped by the Centre de Donnes
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
 //
-//This file is part of Aladin.
+//This file is part of Aladin Desktop.
 //
 //    Aladin Desktop is free software: you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -16,12 +16,10 @@
 //    GNU General Public License for more details.
 //
 //    The GNU General Public License is available in COPYING file
-//    along with Aladin.
+//    along with Aladin Desktop.
 //
 
 package cds.aladin;
-import static cds.aladin.Constants.REGEX_NUMBER;
-
 import java.awt.Color;
 import java.awt.Composite;
 import java.awt.Graphics;
@@ -41,8 +39,6 @@ import java.util.Iterator;
 import java.util.Map;
 import java.util.StringTokenizer;
 import java.util.Vector;
-import java.util.regex.Matcher;
-import java.util.regex.Pattern;
 
 import javax.swing.JFrame;
 import javax.swing.JScrollPane;
@@ -183,9 +179,10 @@ public final class Pcat implements TableParserConsumer/* , VOTableConsumer */ {
 
       if( v.isFree() ) return;
 
-
       Projection proj = v.getProj();
-      if( plan.proj[v.n]==proj && Projection.isOk(proj)
+      if( !Projection.isOk( proj ) ) return;
+      
+      if( plan.proj[v.n]==proj /* && Projection.isOk(proj) */
             && (!(plan instanceof PlanBGCat)
                ||  plan instanceof PlanBGCat && !(plan instanceof PlanMoc) && projpcat[v.n]==proj )   // Dans le cas d'un planBGCat
             ) {
@@ -196,8 +193,8 @@ public final class Pcat implements TableParserConsumer/* , VOTableConsumer */ {
       }
 
       // Test sur le recouvrement des champs
-      if( Projection.isOk(proj) && plan.type!=Plan.TOOL && plan.type!=Plan.APERTURE
-            && !(plan.isSimpleCatalog() && (plan.hasXYorig || v.isPlotView()) )
+      if( /* Projection.isOk(proj) && */plan.type!=Plan.TOOL && plan.type!=Plan.APERTURE
+            && !(plan.isSimpleCatalog() && (plan.hasXYorig || v.isPlot()) )
             && !proj.agree(plan.projd,v) ) return;
 
       // Memorisation de la projection courante appliquee
@@ -256,6 +253,8 @@ public final class Pcat implements TableParserConsumer/* , VOTableConsumer */ {
    boolean flagTarget=false;
    boolean flagEndResource;
    boolean flagFirstRecord=true;		// True si on n'a pas encore traite le premier enr.
+   long timeStartTable;                 // Date du dbut du parsing de la table
+   int lastNb_o;                        // juste pour connaitre le nombre d'objets dans chaque table
    double minRa,maxRa,minDec,maxDec;
 
    /** L'interface AstroRes */
@@ -312,6 +311,7 @@ public final class Pcat implements TableParserConsumer/* , VOTableConsumer */ {
       if( flagXY ) {
 
          plan.hasXYorig=true;
+         flagXY=false;   // on resete ce flag sinon a posera souci en cas de positionnement ultrieur d'une projection via RA,DEC manuel
 
          //         aladin.info(aladin.chaine.getString("INFOXY"));
          //
@@ -333,10 +333,11 @@ public final class Pcat implements TableParserConsumer/* , VOTableConsumer */ {
             0.0,false,
             typeProj,Calib.FK5));
 
-      // Positionnement du centre
-      plan.co=new Coord(rajc,dejc);
+      // Positionnement du centre que si ce n'est pas la valeur par dfaut 0,0
+      if( rajc!=0 && dejc!=0 ) plan.co=new Coord(rajc,dejc);
    }
-
+   
+   
    /** L'interface AstroRes */
    public void startTable(String name) {
       Aladin.trace(3,"startTable "+name);
@@ -352,6 +353,8 @@ public final class Pcat implements TableParserConsumer/* , VOTableConsumer */ {
       nId=-1;
       nIdVraisemblance=0;
       group=null;
+      timeStartTable=Util.getTime();
+      lastNb_o=nb_o;
    }
 
    private Vector<String> group=null;  // Liste des GROUP servant aux dfinitions des systmes de coordonnes ou des Flux
@@ -359,7 +362,7 @@ public final class Pcat implements TableParserConsumer/* , VOTableConsumer */ {
    // Ajoute un GROUP  la liste des GROUPs  associer  la prochaine lgende qui sera cre
    private void addGroup(String s) {
       if( s==null ) { group=null; return; } // reset forc
-      if( group==null ) group = new Vector<String>();
+      if( group==null ) group = new Vector<>();
       group.addElement(s);
    }
 
@@ -386,7 +389,13 @@ public final class Pcat implements TableParserConsumer/* , VOTableConsumer */ {
    }
 
    /** L'interface AstroRes */
-   public void endTable() { }
+   public void endTable() { 
+      long t = Util.getTime() - timeStartTable;
+      int nbObj = nb_o-lastNb_o;
+      tableParserInfo("   -Table loaded & parsed in "+Util.getTemps(t)
+            + " for "+nbObj+" object"+(nbObj>1?"s":"")
+            + (nbObj<1000?"":" ("+Util.myRound(""+1000.*nbObj/t)+" objects per sec)"));
+   }
 
    /** L'interface AstroRes */
    public void setField(Field f) {
@@ -510,19 +519,19 @@ public final class Pcat implements TableParserConsumer/* , VOTableConsumer */ {
    private int indexOID=-1;          // Position de la colonne OID eventuelle
    private TableParser res;          // Parser utilis pour crer les objets
    private StringBuilder line = new StringBuilder(500);
-   private Map<Integer, Field> standardisedColumns = new HashMap<Integer, Field>();
+   private Map<Integer, Field> standardisedColumns = new HashMap<>();
 
 
-   // Lgende gnrique qui vient remplacer toutes les lgendes propres
-   private Legende genericLeg=null;
-
-   /** Positionnement d'une lgende gnrique pour tous les objets (concerne PlanBGCat) */
-   protected void setGenericLegende(Legende leg) {
-      genericLeg=leg;
-   }
+//   // Lgende gnrique qui vient remplacer toutes les lgendes propres
+//   private Legende genericLeg=null;
+//
+//   /** Positionnement d'une lgende gnrique pour tous les objets (concerne PlanBGCat) */
+//   protected void setGenericLegende(Legende leg) {
+//      genericLeg=leg;
+//   }
 
    /** L'interface TableParserConsumer */
-   public void setRecord(double ra, double dec, String[] value) {
+   public void setRecord(double ra, double dec, double jdTime, String[] value) {
       int n;
       String oid = null; // OID trouve s'il y a lieu
 
@@ -531,6 +540,7 @@ public final class Pcat implements TableParserConsumer/* , VOTableConsumer */ {
          // uniquement fait a la premiere source (test sur flagFirstRecord)
 
          if( flagFirstRecord ) {
+            leg=null;   // Reinitialisation si plusieurs tables dans une mme resource
             firstTrace = true; // Pour afficher la premiere source en cas de trace
             indexAccessFormat=indexAccessUrl=indexDataProductType=-1;
             n = vField.size();
@@ -547,8 +557,10 @@ public final class Pcat implements TableParserConsumer/* , VOTableConsumer */ {
             String siaStandardColumns = null;
             String siaHideColumns = null;
             if (flagSIAV2) {
-            	siaStandardColumns = ConfigurationReader.getInstance().getPropertyValue("SIAV2StandardColumns"); //column to standardize in SIAV2 results table;
-            	siaHideColumns = ConfigurationReader.getInstance().getPropertyValue("SIAV2HideColumns"); //column to hide in SIAV2 results table;
+                //column to standardize in SIAV2 results table;
+            	siaStandardColumns = ConfigurationReader.getInstance().getPropertyValue("SIAV2StandardColumns"); 
+            	//column to hide in SIAV2 results table;
+            	siaHideColumns = ConfigurationReader.getInstance().getPropertyValue("SIAV2HideColumns"); 
 			}
             
             for( int i = 0; e.hasMoreElements(); i++ ) {
@@ -567,10 +579,11 @@ public final class Pcat implements TableParserConsumer/* , VOTableConsumer */ {
                      underDE = i;
                      fDE = f;
                   }
-                  else if( f.name.equals("RAJ2000") ) RA = i;
-                  else if( f.name.equals("DEJ2000") ) DE = i;
-                  else if( f.name.equals("RA(ICRS)") ) RA = i;
-                  else if( f.name.equals("DE(ICRS)") ) DE = i;
+                  else if( f.name.equals("RAJ2000") )   RA = i;
+                  else if( f.name.equals("DEJ2000") )   DE = i;
+                  else if( f.name.equals("RA(ICRS)") )  RA = i;
+                  else if( f.name.equals("DE(ICRS)") )  DE = i;
+                  else if( f.name.equals("s_region") )  indexSTC=i;
                }
 
                if( f.type != null
@@ -580,17 +593,21 @@ public final class Pcat implements TableParserConsumer/* , VOTableConsumer */ {
                }
 
                v.addElement(f);
-               if(flagSIAV2 && f.name!=null){
-            	   if (siaHideColumns!=null && siaHideColumns.contains(f.name)) {
+               if(flagSIAV2 && f.name!=null ){
+            	   if (siaHideColumns!=null && siaHideColumns.indexOf(f.name)>-1) {
             		   hiddenField[i] = true;
                        f.visible=false;
             	   }
             	   
-            	   if (siaStandardColumns != null && siaStandardColumns.contains(f.name)) {
+            	   if (siaStandardColumns != null && siaStandardColumns.indexOf(f.name)>-1) {
             		   Field displayField = new Field(f);
-            		   displayField.name = Aladin.getChaine().getString(f.name);
+            		   
+            		   // COde CHaitra => bien trop lent
+                       displayField.name = Aladin.getChaine().getString(f.name);
+//                       displayField.name = f.name;
+                       
             		   //below logic to check if standardized columns are to be hidden
-            		   if (siaHideColumns != null && siaHideColumns.contains(displayField.name)) {
+            		   if (siaHideColumns != null && siaHideColumns.indexOf(displayField.name)>-1) {
             			   displayField.visible = false;
                 	   } else {
                 		   displayField.visible = true;
@@ -609,15 +626,16 @@ public final class Pcat implements TableParserConsumer/* , VOTableConsumer */ {
                fRA.visible = fDE.visible = false;
             }
 
-            if( genericLeg!=null ) leg=genericLeg;
-            else {
+            // En cas de plan HiPS catalog, la lgende est globale et externe
+            if( plan instanceof PlanBGCat ) leg=((PlanBGCat)plan).getFirstLegende();
+            if( leg==null ) {
                leg = new Legende(v);
                leg.name=table;
             }
 
             // Ajout de GROUPs ventuels
             if( group!=null ) leg.setGroup(group);
-
+            
             nbTable++;
             flagFirstRecord = false;
          }
@@ -633,8 +651,8 @@ public final class Pcat implements TableParserConsumer/* , VOTableConsumer */ {
          }
 
          // Generation de la ligne d'info
-         //         line = new StringBuffer(500);
-         Util.resetString(line);
+          line = new StringBuilder(1000);
+//         Util.resetString(line);
          //         line.append(table);
          if( catalog!=null && catalog.equals("Simbad") ) line.append("<&_SIMBAD |Simbad>");
          else if( catalog!=null && (catalog.equals("NED") || catalog.equals("Ned")) ) line.append("<&_NED |NED>");
@@ -645,13 +663,7 @@ public final class Pcat implements TableParserConsumer/* , VOTableConsumer */ {
          int j = -1; // Veritable index de la mesure (en fonction des champs
          // caches)
          for( int i = 0; i < n; i++ ) {
-            // "<" et ">" n'taient pas dcods
-            // ajout thomas pour dmo AVO
-            //        value[i] = cds.savot.samples.WriteDocument.Decode(value[i]);
-            // Rectification Pierre pour viter de charger WriteDocument dans le
-            // .jar
             if( value[i]==null ) value[i]="";	// En cas de VOTable <TD/>
-            //            value[i] = cds.xml.XMLParser.XMLDecode(value[i]);
 
             // Memorisation d'un eventuel OID
             if( indexOID >= 0 && i == indexOID ) oid = value[i];
@@ -662,7 +674,8 @@ public final class Pcat implements TableParserConsumer/* , VOTableConsumer */ {
             // pas d'info sur la mesure ou mesure vide ou nulle
             String a=value[i].trim();
             if( leg == null || !leg.hasInfo(j) || a.length() == 0
-                  || a.equals("0") || a.equals("-") /* || a.equalsIgnoreCase("null") */ ) {
+                  || a.equals("0") || a.equals("-") /* || a.equalsIgnoreCase("null") */ 
+                  ) {
             	String displayString = "\t" + ((a.length() == 0) ? " " : value[i]);
             	line.append(displayString);
             	if (flagSIAV2 && standardisedColumns.containsKey(i)) {
@@ -674,16 +687,16 @@ public final class Pcat implements TableParserConsumer/* , VOTableConsumer */ {
             // Construction des ancres
             String href = leg.getHref(j);
             String gref = leg.getGref(j);
-            String refText = leg.getRefText(j);
+//            String refText = leg.getRefText(j);
             String flagArchive = leg.getRefValue(j);
             String utype = leg.getUtype(j);
             String name = leg.getName(j);
             
-            try {
-               if( indexSTC==-1 && /* flagEPNTAP && */ leg.getID(j).equals("s_region") ) {
-                  indexSTC=j;
-               }
-            } catch( Exception e ) { }
+//            try {
+//               if( indexSTC==-1 && /* flagEPNTAP && */ leg.getID(j).equals("s_region") ) {
+//                  indexSTC=j;
+//               }
+//            } catch( Exception e ) { }
             
             if( indexSTC==-1 && Util.indexOfIgnoreCase( value[i], "Polygon ")==0 ) {
                indexSTC=j;
@@ -709,7 +722,7 @@ public final class Pcat implements TableParserConsumer/* , VOTableConsumer */ {
                      indexAccessUrl=-2;  // Pour viter de le traiter par la suite
                   }
                }
-               // SSA - preview
+               // SSA + preview
                if( flagArchive==null ) {
                   String ucd = leg.getUCD(j);
                   if( ucd!=null && Util.indexOfIgnoreCase(ucd,"meta.ref.url;datalink.preview")>=0 ) {
@@ -744,7 +757,7 @@ public final class Pcat implements TableParserConsumer/* , VOTableConsumer */ {
             // Les TABs ne peuvent tre prsents dans les valeurs individuelles (au risque de ne plus pouvoir relire les donnes
             // correctement). Je les remplace par un espace
             if( text.indexOf('\t')>=0 ) text=text.replace("\t"," ");
-
+            
             line.append('\t');
             if( tag != null ) {
                line.append("<&" + dollarSub(tag, value, (href != null) ? 1 : 0));
@@ -776,7 +789,7 @@ public final class Pcat implements TableParserConsumer/* , VOTableConsumer */ {
             Aladin.trace(3, "setRecord "
                   + (oid != null ? "(oid=" + oid + ")" : "") + " \"" + lab
                   + "\" " + (flagXY ? "XY" : "pos") + "=(" + ra + "," + dec
-                  + ") [" + line + "]");
+                  + ")"+(!Double.isNaN(jdTime)?" time="+Astrodate.JDToDate(jdTime):"")+" [" + line + "]");
             firstTrace = false;
 
             // Dans le cas d'un rsultat ObsTAP, on devra post-traiter le tag sur le champ "access_url" en fonction
@@ -791,6 +804,7 @@ public final class Pcat implements TableParserConsumer/* , VOTableConsumer */ {
          Source source;
          if( flagXY ) source = new Source(plan, ra, dec, 0, 0, Obj.XY, lab, line.toString(), leg);
          else source = new Source(plan, ra, dec, lab, line.toString(), leg);
+         source.jdtime = jdTime;
          if( oid != null ) source.setOID(oid);
          o[nb_o++] = source;
 
@@ -814,7 +828,9 @@ public final class Pcat implements TableParserConsumer/* , VOTableConsumer */ {
             }
             
          // Peut tre un FoV peut tre tout de mme gnr par SIA1 ou SSA ?
-         } else if( flagSIA && !flagSIAV2 ) {
+         } else 
+         
+         if( flagSIA && !flagSIAV2 ) {
             String fov = source.createSIAFoV();
             if( fov!=null ) {
                source.setFootprint(fov);
@@ -836,7 +852,7 @@ public final class Pcat implements TableParserConsumer/* , VOTableConsumer */ {
                   else if( fmt.startsWith("spectr") && fmt.indexOf('/')>0 ) tag="";
                   val = "<&"+tag+val.substring(2);
                   source.setValue(indexAccessUrl, val);
-                  source.leg.field[indexAccessUrl].flagArchive=true;
+                  source.getLeg().field[indexAccessUrl].flagArchive=true;
                }
             } catch( Exception e ) {
                e.printStackTrace();
@@ -844,7 +860,7 @@ public final class Pcat implements TableParserConsumer/* , VOTableConsumer */ {
          }
 
       } catch( Exception e ) {
-         System.out.println("setRecord (3p) " + e);
+         if( aladin.levelTrace>=3 ) System.err.println("Pcat setRecord (3p) exception " + e);
          e.printStackTrace();
       }
    }
@@ -859,33 +875,54 @@ public final class Pcat implements TableParserConsumer/* , VOTableConsumer */ {
     * @param text
     * @return formated string
     */
+   
+   // J'INVALIDE CES CONVERSIONS QUI RALENTISSENT BIEN TROP LE CHARGEMENT - PF 7/5/2019
 	public String processValuesToStandardRepresentation(Field field, String text) {
-		if (!flagSIAV2) {//for now only siav2 are known to get time in MJD and spectral data in m.
+		if( !flagSIAV2) {//for now only siav2 are known to get time in MJD and spectral data in m.
 			return text;
 		}
-
+		
 		String standardRepresentation = text;
 		try {
 			if (text != null && !text.trim().isEmpty() && field != null && field.ucd != null) {
-				if (field.ucd.split("\\.")[0].equalsIgnoreCase("time") && "D".equalsIgnoreCase(field.datatype)
-						&& "d".equalsIgnoreCase(field.unit)) {
-					Pattern regex = Pattern.compile(REGEX_NUMBER);
-					Matcher matcher = regex.matcher(text);
-					if (matcher.find()) {
-						standardRepresentation = this.convertMJDToISO(text);
-					}
-				} else if (field.ucd.split("\\.")[0].equalsIgnoreCase("em") && "D".equalsIgnoreCase(field.datatype)
-						&& "m".equalsIgnoreCase(field.unit)) {
-					standardRepresentation = this.setStandardSpectralRepresentation(text);
-				}
+			   
+			      int i;
+                  String s = ((i=field.ucd.indexOf('.'))<0 ? field.ucd : field.ucd.substring(0,i)).toLowerCase();
+			      
+	              if( s.equals("time") && "D".equalsIgnoreCase(field.datatype) && "d".equalsIgnoreCase(field.unit)) {
+	                 if( isNumber(text) ) standardRepresentation = this.convertMJDToISO(text);
+	              } else if( s.equals("em") && "D".equalsIgnoreCase(field.datatype) && "m".equalsIgnoreCase(field.unit)) {
+                     standardRepresentation = this.setStandardSpectralRepresentation(text);
+                }
+
+// Code Chaitra => bien trop lent [le split est  fuir]
+//				if (field.ucd.split("\\.")[0].equalsIgnoreCase("time") && "D".equalsIgnoreCase(field.datatype)
+//						&& "d".equalsIgnoreCase(field.unit)) {
+//					Pattern regex = Pattern.compile(REGEX_NUMBER);
+//					Matcher matcher = regex.matcher(text);
+//					if (matcher.find()) {
+//						standardRepresentation = this.convertMJDToISO(text);
+//					}
+//				} else if (field.ucd.split("\\.")[0].equalsIgnoreCase("em") && "D".equalsIgnoreCase(field.datatype)
+//						&& "m".equalsIgnoreCase(field.unit)) {
+//					standardRepresentation = this.setStandardSpectralRepresentation(text);
+//				}
 			}
 		} catch (Exception e) {
-			if (Aladin.levelTrace >= 3)
-				e.printStackTrace();
+			if (Aladin.levelTrace >= 3) e.printStackTrace();
 			standardRepresentation = text;
 		}
 		return standardRepresentation;
 	}
+	
+	private boolean isNumber( String s) {
+	   try { Double.parseDouble(s); return true; } catch( Exception e ) { }
+	   return false;
+	   
+//       Pattern regex = Pattern.compile(REGEX_NUMBER);
+//       Matcher matcher = regex.matcher(s);
+//       return matcher.find();
+	}
 
    public String convertMJDToISO(String timeWord) {
 		double valueInProcess= Astrodate.MJDToJD(Double.valueOf(timeWord)); 
@@ -908,8 +945,7 @@ public final class Pcat implements TableParserConsumer/* , VOTableConsumer */ {
 		String defaultWavelengthUnit = "m";
 		Double spectralVal = Double.parseDouble(text);
 		Unit unitToProcess = null;
-		Double wavelengthRange1 = Double
-				.parseDouble(ConfigurationReader.getInstance().getPropertyValue("WAVELENGTHRANGE1"));
+		Double wavelengthRange1 = Double.parseDouble(ConfigurationReader.getInstance().getPropertyValue("WAVELENGTHRANGE1"));
 		Double wavelengthRange2 = Double
 				.parseDouble(ConfigurationReader.getInstance().getPropertyValue("WAVELENGTHRANGE2"));
 		try {
@@ -953,11 +989,14 @@ public final class Pcat implements TableParserConsumer/* , VOTableConsumer */ {
       }
       return false;
    }
+   
+   /** Retourne true si on dispose d'informations techniques sur le parsing */
+   protected boolean hasParsingInfo() { return parsingInfo!=null || description!=null; }
 
    /** Affichage de la fentre contenant les informations sur le parsing
        et les donnes */
    protected void seeCatalogInfo() {
-      if( parsingInfo==null && description==null ) return;
+      if( !hasParsingInfo() ) return;
       JFrame f = new JFrame("Catalog information");
       Util.setCloseShortcut(f, false, aladin);
       //      f.setBackground(Aladin.BKGD);
@@ -1124,15 +1163,14 @@ public final class Pcat implements TableParserConsumer/* , VOTableConsumer */ {
 
       // Cas particulier pour un plan hirarchique
       if( ok && plan instanceof PlanBGCat ) {
-         if( nb_o==0 ) setRecord(0, 0, null);  // pour initialiser tout de mme la lgende
+         if( nb_o==0 ) setRecord(0, 0, Double.NaN, null);  // pour initialiser tout de mme la lgende
          return nb_o;
       }
 
       if( ok ) {
          if( !flagEndResource ) endResource();
          long duree=System.currentTimeMillis()-d;
-         String s = "Catalog parsed in "+Util.myRound(""+duree/1000.,3)+"s"+(nb_o<1000?"":" ("
-               +Util.myRound(""+1000.*nb_o/duree)+" objects per sec)");
+         String s = "Catalog queried, loaded and parsed in "+Util.getTemps(duree);
          tableParserInfo("\n"+s);
          Aladin.trace(3,s);
          if( !flagXY && rm==0.0 ) plan.error = aladin.error = "no RA or DE columns";
@@ -1170,9 +1208,11 @@ public final class Pcat implements TableParserConsumer/* , VOTableConsumer */ {
             rajc = (minRa+alpha)/2 - alpha;
          } else rajc = (minRa+maxRa)/2;
          dejc = (minDec+maxDec)/2;
-         double r = Math.max(Math.abs(minRa-rajc)*Math.cos( (dejc*Math.PI)/180.0 ),
-               Math.abs(minDec-dejc));
-         rm = (nb_o==1 || r==0.)?7:r*60.0*1.4142;
+//         double r = Math.max(Math.abs(minRa-rajc)*Math.cos( (dejc*Math.PI)/180.0 ), Math.abs(minDec-dejc));
+//         rm = (nb_o==1 || r==0.)?7:r*60.0*1.4142;
+         double r = Coord.getDist( new Coord(minRa, minDec), new Coord(maxRa, maxDec) )/2;
+         rm = (nb_o==1 || r==0.)?7:r*60.0;
+
       }
       Aladin.trace(3,"computeTarget ra=["+minRa+".."+maxRa+"]=>"+rajc+" de=["+minDec+".."+maxDec+"]=>"+dejc+" rm=["+rm+"]");
    }
@@ -1256,7 +1296,7 @@ public final class Pcat implements TableParserConsumer/* , VOTableConsumer */ {
          long type=dis.getType();
          flagVOTable=(type & MyInputStream.VOTABLE)!=0;
          flagFootprint = (type & MyInputStream.FOV)!=0;
-         flagSIAV2 = (type & MyInputStream.SIAV2)!= 0;
+//         flagSIAV2 = (type & MyInputStream.SIAV2)!= 0;
          flagSIA = (type & MyInputStream.SIA)!= 0;
          flagEPNTAP = (type & MyInputStream.EPNTAP)!= 0;
          if( flagFootprint ) {
@@ -1407,7 +1447,7 @@ public final class Pcat implements TableParserConsumer/* , VOTableConsumer */ {
          if( !(o[i] instanceof Source) ) continue;
          
          // On a trouv ?
-         if( ((Source)o[i]).leg==src.leg ) {
+         if( ((Source)o[i]).getLeg()==src.getLeg() ) {
             int n = nextIndex();
             for( int j=n; j>i+1; j--) o[j] = o[j-1];  // dcalage
             o[i+1] = src;
@@ -1433,7 +1473,7 @@ public final class Pcat implements TableParserConsumer/* , VOTableConsumer */ {
       for( int i=0; i<nb_o; i++ ) {
          if( !(o[i] instanceof Source) ) continue;
          Source s = (Source)o[i];
-         if( s.leg==leg ) { s.fixInfo(); j++; }
+         if( s.getLeg()==leg ) { s.fixInfo(); j++; }
       }
       //System.out.println("FIX J'ai fix "+j+" sources du plan "+plan);
    }
