@@ -1,10 +1,10 @@
-// Copyright 1999-2018 - Universit de Strasbourg/CNRS
+// Copyright 1999-2020 - Universit de Strasbourg/CNRS
 // The Aladin Desktop program is developped by the Centre de Donnes
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
 //
-//This file is part of Aladin.
+//This file is part of Aladin Desktop.
 //
 //    Aladin Desktop is free software: you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 //    GNU General Public License for more details.
 //
 //    The GNU General Public License is available in COPYING file
-//    along with Aladin.
+//    along with Aladin Desktop.
 //
 
 package cds.aladin;
@@ -40,11 +40,15 @@ import cds.tools.Util;
  */
 public class PlanCatalog extends Plan {
    URL url = null;
+   boolean autoSelect=false;    // si true slectionne automatiquement tous les objets s'ils n'ont pas de coordonnes
 
   /** Creation d'un plan de type CATALOG (via une fichier)
    * @param file  Le nom du fichier
    */
    protected PlanCatalog(Aladin aladin, String file, MyInputStream in,boolean skip,boolean doClose) {
+      this(aladin,file,in,skip,doClose,true);
+   }
+   protected PlanCatalog(Aladin aladin, String file, MyInputStream in,boolean skip,boolean doClose,boolean autoSelect) {
       this.doClose=doClose;
       flagSkip = skip;
       String label = "Cat";
@@ -68,7 +72,7 @@ public class PlanCatalog extends Plan {
       flagLocal=true;
       flagWaitTarget=true;  // Voir Command.waitingPlanInProgress
 
-      Suite(aladin,label,"","",null,null, null);
+      Suite(aladin,label,"","",null,null, null, null);
    }
 
   /** Creation d'un plan de type CATALOG (via un InputStream)
@@ -76,15 +80,15 @@ public class PlanCatalog extends Plan {
    * @param label le label du plan (propos)
    * @param origin origine du catalogue
    */
-   protected PlanCatalog(Aladin aladin, MyInputStream in,String label,String origin) {
+   protected PlanCatalog(Aladin aladin, MyInputStream in,String label,String origin, Server server) {
       this.dis = in;
       if( label==null) label="VOApp";
       flagWaitTarget=true;
-      Suite(aladin,label,"","",origin,null, null);
+      Suite(aladin,label,"","",origin,server, null, null);
    }
 
    protected PlanCatalog(Aladin aladin, MyInputStream in,String label) {
-   	this(aladin,in,label,null);
+   	this(aladin,in,label,null, null);
    }
 
    private HttpURLConnection httpConn;
@@ -93,15 +97,16 @@ public class PlanCatalog extends Plan {
       this.httpConn = httpConn;
       if( label==null) label="HttpConn";
       flagWaitTarget=true;
-      Suite(aladin,label,"","",null,null, null);
+      Suite(aladin,label,"","",null,null, null, null);
      }
 	
-	protected PlanCatalog(Aladin aladin, MyInputStream in, String label, Server server, String query) {
+	protected PlanCatalog(Aladin aladin, MyInputStream in, String label,String origin, Server server, URL url, String query, int requestId) {
 		this.dis = in;
 		if (label == null)
 			label = "VOApp";
 		flagWaitTarget = true;
-		Suite(aladin, label, "", "", null, server, query);
+		this.requestId = requestId;
+		Suite(aladin, label, "", "", origin, server, url, query);
 	}
 
   /** Creation d'un plan de type CATALOG (sans info)
@@ -130,10 +135,11 @@ public class PlanCatalog extends Plan {
       this.u     = u;
       flagLocal  = false;
 
-      Suite(aladin,label,objet,param,from,server, null);
+//      Suite(aladin,label,objet,param,from,server, null, null);  <= Chaitra bug
+      Suite(aladin,label,objet,param,from,server, u, null);
 
    }
-
+   
   /** Creation d'un plan de type CATALOG
    * @param aladin reference
    * @param label  le nom du plan (dans la pile des plans)
@@ -143,7 +149,7 @@ public class PlanCatalog extends Plan {
    * @param server le serveur d'origine
  * @param query 
    */
-   protected void Suite(Aladin aladin, String label,String objet,String param,String from,Server server, String query ) {
+   protected void Suite(Aladin aladin, String label,String objet,String param,String from,Server server, URL url, String query ) {
       setLogMode(true);
       this.aladin= aladin;
       type       = CATALOG;
@@ -156,6 +162,7 @@ public class PlanCatalog extends Plan {
       headerFits=null;
       this.server=server;
       this.query = query;
+      this.u = url;
       if( server!=null ) {
          filters=server.filters;
          filterIndex=aladin.configuration.getFilter()==0? server.getFilterChoiceIndex() : -1;
@@ -171,7 +178,7 @@ public class PlanCatalog extends Plan {
    
    /** retourne le nom de la table associe  une source */
    protected String getTableName(Source o) {
-      String s = o.leg==null ? null : o.leg.name;
+      String s = o.getLeg()==null ? null : o.getLeg().name;
       if( s==null ) {
          if( o.info==null ) return "Table";
          int i = o.info.indexOf('|');
@@ -212,9 +219,9 @@ public class PlanCatalog extends Plan {
       if( getCounts()<=0 || isSED() ) return true;
       
       aladin.view.deSelect(this);
-      if (Aladin.PROTO) {//TODO:: tintinproto
+//      if (Aladin.PROTO) {
     	  TapManager.getInstance(aladin).updateDeleteUploadPlans(this);
-      }
+//      }
       super.Free();
       aladin.view.free(this);
       headerFits=null;
@@ -225,8 +232,9 @@ public class PlanCatalog extends Plan {
    }
    
    protected boolean isCatalog() { return true; }
-
-
+   
+   protected boolean isTime() { return isCatalogTime(); }
+   
    protected String getDescription() {
       if( pcat.description==null ) return null;
       return pcat.description.toString();
@@ -266,6 +274,9 @@ public class PlanCatalog extends Plan {
    
    protected boolean setActivated() {
       if( !hasSources() ) return false;
+      
+      if( autoSelect && !aladin.view.hasSelectedSource() ) aladin.view.selectAllInPlan(this);
+      
       return super.setActivated();
    }
 
@@ -323,9 +334,9 @@ public class PlanCatalog extends Plan {
          setFilter(filterIndex);
          
          //to add loaded plan into upload options
-         if (Aladin.PROTO) {//TODO:: tintinproto
+//         if (Aladin.PROTO) {
         	 TapManager.getInstance(aladin).updateAddUploadPlans(this);
-		}
+//		}
       }
 
       if( getNbTable()>1 ) aladin.calque.splitCatalog(this);
@@ -363,14 +374,14 @@ public class PlanCatalog extends Plan {
 
    /** Retourne la liste des lgendes des tables qui composent le catalogue */
    protected Vector<Legende> getLegende() {
-      Vector<Legende> leg = new Vector<Legende>(10);
+      Vector<Legende> leg = new Vector<>(10);
       Iterator<Obj> it = iterator();
       while( it.hasNext() ) {
          Obj o = it.next();
          if( !(o instanceof Source) ) continue;
          Source s = (Source)o;
-         if( s.leg==null ) continue;
-         if( !leg.contains(s.leg) ) leg.addElement(s.leg);
+         if( s.getLeg()==null ) continue;
+         if( !leg.contains(s.getLeg()) ) leg.addElement(s.getLeg());
       }
       return leg;
    }
@@ -382,7 +393,7 @@ public class PlanCatalog extends Plan {
          Obj o = it.next();
          if( !(o instanceof Source) ) continue;
          Source s = (Source)o;
-         if( s.leg!=null ) return s.leg;
+         if( s.getLeg()!=null ) return s.getLeg();
       }
       return null;
    }
@@ -396,6 +407,8 @@ public class PlanCatalog extends Plan {
 
 	 /** retourne true si le plan a des sources */
 	 protected boolean hasSources() { return pcat!=null && pcat.hasObj(); }
+	 
+	 protected boolean hasCatalogInfo() { return  pcat!=null && pcat.hasCatalogInfo(); }
 
 	 /******************************************************** QUELQUES TESTS UNITAIRES *******************************************************/
 
