@@ -1,10 +1,10 @@
-// Copyright 1999-2018 - Universit de Strasbourg/CNRS
+// Copyright 1999-2020 - Universit de Strasbourg/CNRS
 // The Aladin Desktop program is developped by the Centre de Donnes
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
 //
-//This file is part of Aladin.
+//This file is part of Aladin Desktop.
 //
 //    Aladin Desktop is free software: you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 //    GNU General Public License for more details.
 //
 //    The GNU General Public License is available in COPYING file
-//    along with Aladin.
+//    along with Aladin Desktop.
 //
 
 package cds.aladin;
@@ -42,6 +42,7 @@ import cds.allsky.Constante;
 import cds.allsky.Context;
 import cds.fits.Fits;
 import cds.fits.HeaderFits;
+import cds.moc.Moc;
 import cds.tools.Util;
 import cds.tools.pixtools.CDSHealpix;
 
@@ -85,7 +86,7 @@ public class PlanHealpix extends PlanBG {
    private boolean isGZ = false;
    private boolean isPartial = false; // true if partial Healpix mode
    private boolean isARGB=false;      // True if ARGB Healpix map (32 int bits, COLORMOD ARGB)
-   private double[] partialHpxPixIdx; // indexes of healpix pixels
+   private long[] partialHpxPixIdx; // indexes of healpix pixels
    private int nbRecordsPartial;
 
    private String originalPath;
@@ -99,7 +100,9 @@ public class PlanHealpix extends PlanBG {
    //    private int myAllskyMode = FIRST; // mode de creation des images de niveau superieur et de l'image all sky : FIRST, MOYENNE, ...
 
    private boolean fromProperties; // true si la cration du plan a t demande depuis la fenetre des properties. Dans ce cas, on ne touche pas  idxTFormToRead, mme pour les fichiers partiels
-
+   
+    // pour afficher une map plantaire dans le bon sens et avec la bonne projection
+   boolean isAPlanet = false;
 
    /** @param mode : DRAWPIXEL : les pixels, DRAWPOLARISATION : les segments de polarisation, DRAWANGLE : les angles sous forme d'une image */
    public PlanHealpix(Aladin aladin, String file, MyInputStream in, String label, int mode, int idxTFormToRead, boolean fromProperties, Coord c,double radius) {
@@ -364,6 +367,7 @@ public class PlanHealpix extends PlanBG {
       prop.setProperty(Constante.OLD_LENHPX, Util.join(lenHpx, ','));
       prop.setProperty(Constante.OLD_TYPEHPX, Util.join(typeHpx, ','));
       prop.setProperty(Constante.OLD_ALADINVERSION, aladin.VERSION);
+
       
       OutputStreamWriter out = null;
       try {
@@ -404,7 +408,7 @@ public class PlanHealpix extends PlanBG {
 
       active=selected=true;
 
-      pixList = new Hashtable<String, HealpixKey>(1000);
+      pixList = new Hashtable<>(1000);
       //       allsky=null;
       loader = new HealpixLoader();
       
@@ -418,8 +422,15 @@ public class PlanHealpix extends PlanBG {
       int defaultProjType = aladin.configuration.getProjAllsky();
       Plan base = aladin.calque.getPlanBase();
       if( base instanceof PlanBG ) defaultProjType = base.projd.t;
+      
+      boolean longAsc = isAPlanet;
+      if( isAPlanet ) {
+         defaultProjType = Calib.SIN;
+         setSpecificProj(true);
+      }
 
-      Projection p = new Projection("allsky",Projection.WCS,co.al,co.del,60*4,60*4,250,250,500,500,0,false, defaultProjType,Calib.FK5);
+      Projection p =  new Projection("allsky",Projection.WCS,co.al,co.del,60*4,60*4,250,250,500,500,0,
+            longAsc, defaultProjType,Calib.FK5);
 
       p.frame = getCurrentFrameDrawing();
 //      if( Aladin.OUTREACH ) p.frame = Localisation.GAL;
@@ -493,9 +504,19 @@ public class PlanHealpix extends PlanBG {
 
       int nsideImage=0;
       int minLevel = 3; // Norder minimum dsir
-      if( nside==0 ) nside = headerFits.getIntFromHeader("NSIDE");
+      if( nside==0 ) {  
+         try { nside = headerFits.getIntFromHeader("NSIDE"); } catch( Exception e ) {} 
+         
+         // Ce doit tre un fichier HEALPIX MOC  la Giuseppe
+         if( nside==0 ) {
+            try { 
+               int o = headerFits.getIntFromHeader("MOCORDER"); 
+               nside = (int)CDSHealpix.pow2(o);
+            } catch( Exception e ) {} 
+         }
+      }
       int maxSizeGeneratedImage = 512;
-      if( nside<maxSizeGeneratedImage ) maxSizeGeneratedImage=nside;      // PF : Pour pouvoir charger des "petits cieux"
+      if( nside!=0 && nside<maxSizeGeneratedImage ) maxSizeGeneratedImage=nside;      // PF : Pour pouvoir charger des "petits cieux"
       Aladin.trace(3, "maxSizeGeneratedImage: "+maxSizeGeneratedImage);
       nbPixGeneratedImage = 2*maxSizeGeneratedImage;
       nSideFile = nside;
@@ -643,8 +664,7 @@ public class PlanHealpix extends PlanBG {
          fillRingValues(0, 12*newNSideFile*newNSideFile, idxTForm);
       }
 
-
-      if (tfieldNames.length>0 && tfieldNames[0].equals("PIXEL") ) {
+      if ( ordering.equals("NUNIQ") || tfieldNames.length>0 && tfieldNames[0].equals("PIXEL") ) {
          isPartial = true;
 
          // on skippe le premier champ qui donne les numros Healpix des pixels
@@ -663,13 +683,17 @@ public class PlanHealpix extends PlanBG {
 
          Aladin.trace(3, "nbRecordsPartial: "+nbRecordsPartial);
 
-
-         if (ordering.equals("NESTED") ) {
-            // on passe 0 comme indice, car le champ PIXEL est en position 0
-            partialHpxPixIdx = getValuesNested(0, nbRecordsPartial, rafHpx, initialOffsetHpx, 0);
-         }
-         else {
-            partialHpxPixIdx = getValuesRing(0, nbRecordsPartial, newNSideFile);
+         // on passe 0 comme indice, car le champ PIXEL est en position 0
+         if (ordering.equals("NESTED") || ordering.equals("NUNIQ") ) {
+            partialHpxPixIdx = getIndex(0, nbRecordsPartial, rafHpx, initialOffsetHpx, 0);
+//            partialHpxPixIdx = getValuesNested(0, nbRecordsPartial, rafHpx, initialOffsetHpx, 0);
+            
+         } else {
+            
+            double [] x =  getValuesRing(0, nbRecordsPartial, newNSideFile);
+            partialHpxPixIdx = new long[ x.length ];
+            for( int i=0; i<x.length; i++ ) partialHpxPixIdx[i]= (long) x[i];
+//            partialHpxPixIdx = getValuesRing(0, nbRecordsPartial, newNSideFile);
          }
          fillPartialValues(idxTForm);
       }
@@ -686,7 +710,7 @@ public class PlanHealpix extends PlanBG {
          Fits out = new Fits(nbPixGeneratedImage, nbPixGeneratedImage, curTFormBitpix);
          buildDoubleHealpix(newNSideImage, i, newNSideFile, out, idxTForm, dir);
       }
-
+      
       // on rend la mmoire
       ringValues = null;
 
@@ -823,6 +847,9 @@ public class PlanHealpix extends PlanBG {
          String s = prop.getProperty(Constante.KEY_HIPS_FRAME);
          if( s==null ) s = prop.getProperty(Constante.OLD_HIPS_FRAME);
          if( s==null ) s = "galactic";
+         
+         isAPlanet = isPlanet(s);
+         
          coordsys = coordsys(s);
          frameOrigin = coordsysToFrame(coordsys);
 
@@ -957,16 +984,17 @@ public class PlanHealpix extends PlanBG {
 
    private double[] getValues(long lowHealpixIdx, long highHealpixIdx,
          RandomAccessFile raf, long initialOffset, long idxTForm) {
-      if (isPartial && ordering.equals("NESTED")) {
+      
+      if (isPartial && (ordering.equals("NESTED") || ordering.equals("NUNIQ")) ) {
          return getValuesPartialNested(lowHealpixIdx, highHealpixIdx);
-      }
-      else if (isPartial && ordering.equals("RING")) {
+         
+      } else if (isPartial && ordering.equals("RING")) {
          return getValuesPartialRing(lowHealpixIdx, highHealpixIdx);
-      }
-      else if (ordering.equals("RING")) {
+         
+      } else if (ordering.equals("RING")) {
          return getValuesRing(lowHealpixIdx, highHealpixIdx, nSideFile);
-      }
-      else {
+         
+      } else {
          return getValuesNested(lowHealpixIdx, highHealpixIdx, raf, initialOffset, idxTForm);
       }
    }
@@ -1059,6 +1087,86 @@ public class PlanHealpix extends PlanBG {
       return result;
    }
 
+   private long[] getIndex(long lowHealpixIdx, long highHealpixIdx,
+         RandomAccessFile raf, long initialOffset, long idxTForm) {
+
+
+      // TODO : ATTENTION : si lowHealpixIdx n'est pas un multiple du format (1024E par exemple), a ne marchera pas
+      long val = 0; // Valeur du champ courant
+
+      int nbValues = (int) (highHealpixIdx - lowHealpixIdx);
+      long[] result = new long[nbValues];
+
+
+      // On prend un buffer assez grand pour tout recuperer en une fois
+      // (raisonnable quand on genere des images 512*512)
+      int nbRowsToRead = nbValues / lenHpx[(int)idxTForm];
+      // dans ce cas, on ne rcupre qu'une partie d'une ligne de donnes (bug fix pour fichier Caroline toto.fits)
+      if (nbRowsToRead==0) {
+         nbRowsToRead = 1;
+      }
+      //        System.out.println("nbValues: "+nbValues);
+      //        System.out.println("NB ROWS TO READ: "+nbRowsToRead);
+      //        System.out.println(sizeRecord);
+      int sizeBuf = nbRowsToRead * sizeRecord;
+      byte[] buf = new byte[sizeBuf];
+
+      // (bug fix pour fichier Caroline toto.fits)
+      int inrowSkip = 0;
+      if (nbValues<lenHpx[(int)idxTForm]) {
+         inrowSkip = ((int)lowHealpixIdx % lenHpx[(int)idxTForm]) * Util.binSizeOf(typeHpx[(int)idxTForm], 1);
+      }
+
+      int nbRowsToSkip = (int) lowHealpixIdx / lenHpx[(int)idxTForm];
+      // on se place  l'endroit qui nous intresse et on lit
+      try {
+
+         // System.out.println("NB ROWS TO SKIP: "+ nbRowsToSkip);
+         raf.seek(initialOffset + nbRowsToSkip * sizeRecord + inrowSkip);
+         try {
+            raf.readFully(buf);
+         } catch (EOFException e) {
+         } catch (IOException e) {
+            e.printStackTrace();
+         }
+         // System.out.println(buf.length);
+      } catch (IOException e1) {
+         // TODO Auto-generated catch block
+         e1.printStackTrace();
+      }
+
+      int resultIdx = 0;
+      int offset = 0;
+      int offsetField;
+      for( int idxRow=0; idxRow<nbRowsToRead; idxRow++, offset+=sizeRecord ) {
+         offsetField = 0;
+         for( int idxField=0; idxField<nField; idxField++ ) {
+            if( idxField!=idxTForm ) {
+               offsetField += Util.binSizeOf(typeHpx[idxField], lenHpx[idxField]);
+               continue;
+            }
+
+            int totalOffset = offset+offsetField;
+            for( int k=0; k<lenHpx[idxField] && resultIdx<result.length; k++, resultIdx++ ) {
+               int offsetc = Util.binSizeOf(typeHpx[idxField],k);
+               if (k==0) {
+               }
+               switch(typeHpx[idxField]) {
+                  case 'B': val = getByte(buf,totalOffset+offsetc); break;
+                  case 'I': val = getShort(buf,totalOffset+offsetc); break;
+                  case 'J': val = getInt(buf,totalOffset+offsetc); break;
+                  case 'K': val = (((long)getInt(buf,totalOffset+offsetc))<<32)
+                        | ((getInt(buf,totalOffset+offsetc+4))& 0xFFFFFFFFL); break;
+                  default: val=-1;
+               }
+               result[resultIdx] = val;
+            }
+         }
+      }
+
+      return result;
+   }
+
    public static long getHealpixMin(int n1, long n, int n2, boolean nside) {
       if (nside)
          return n*(long)(Math.pow(4,(log2(n2) - log2(n1))/log2(2)));
@@ -1080,34 +1188,27 @@ public class PlanHealpix extends PlanBG {
       //        partialValues = new double[12*newNSideFile*newNSideFile];
       partialValues = new HashMap();
       double[] values;
-      if( ordering.equals("NESTED")) {
+      
+      if( ordering.equals("NUNIQ") || ordering.equals("NESTED")) {
          values = getValuesNested(0, nbRecordsPartial, rafHpx, initialOffsetHpx, idxTForm);
       }
       else {
          values = getValuesRing(0, nbRecordsPartial, newNSideFile);
       }
-      // on initialize les valeurs  NaN
-      //        for (int i = 0; i < partialValues.length; i++) {
-      //            partialValues[i] = Double.NaN;
-      //        }
-      //        // on boucle sur les indices partiels
-      //        for (int i = 0; i < partialHpxPixIdx.length; i++) {
-      //            partialValues[(int)partialHpxPixIdx[i]] = values[i];
-      //        }
-      // on boucle sur les indices partiels
-      for (int i = 0; i < partialHpxPixIdx.length; i++) {
-         partialValues.put((long)partialHpxPixIdx[i],values[i]);
-      }
 
+      for (int i = 0; i < partialHpxPixIdx.length; i++) {
+         partialValues.put(partialHpxPixIdx[i],values[i]);
+      }
    }
 
    private double[] getValuesPartialRing(long low, long high) {
       int nbVal = (int)(high-low);
       double[] ret = new double[nbVal];
       try {
+         int order = (int)CDSHealpix.log2(nSideFile);
          for (int i=0; i<nbVal; i++) {
             //             ret[i] = partialValues[(int)CDSHealpix.nest2ring(nSideFile, low + i)];
-            Double a = (Double)partialValues.get( CDSHealpix.nest2ring(nSideFile, low + i));
+            Double a = (Double)partialValues.get( CDSHealpix.nest2ring(order, low + i));
             ret[i] = a==null ? Double.NaN : a.doubleValue();
          }
       } catch( Exception e ) { e.printStackTrace(); }
@@ -1118,14 +1219,54 @@ public class PlanHealpix extends PlanBG {
       int nbVal = (int)(high-low);
       double[] ret = new double[nbVal];
       for (int i=0; i<nbVal; i++) {
-         //            ret[i] = partialValues[(int)low+i];
-         Double a = (Double)partialValues.get( low+i );
-         ret[i] = a==null ? Double.NaN : a.doubleValue();
+//         Double a = (Double)partialValues.get( low+i );
+//         ret[i] = a==null ? Double.NaN : a.doubleValue();
+         ret[i] = getPartialValue(low+i);
       }
 
       return ret;
    }
 
+   
+   private int lastOrder=0;
+//   boolean X=true;
+//   private int histo[] = new int[30];
+
+   private double getPartialValue(long npix) {
+      if( !ordering.equals("NUNIQ") ) {
+         Double a = (Double)partialValues.get( npix );
+         return a==null ? Double.NaN : a.doubleValue();
+      }
+      
+//      if( X ) {
+//         X=false;
+//         for( Object k : partialValues.keySet() ) {
+//            long nuniq = (long)k;
+//            long [] hpix = Moc.uniq2hpix(nuniq);
+//            histo[ (int)hpix[0] ] ++;
+//            if( nuniq==29389659 ) {
+//               System.out.println("J'y suis");
+//            }
+//         }
+//         
+//         int somme=0;
+//         for( int i=0; i<histo.length; i++ )  { System.out.println("histo["+i+"] = "+histo[i]); somme+=histo[i]; };
+//         System.out.println("Somme ="+somme);
+//      }
+      
+      // On va chercher dans tous les NUNIQ possibles pour chaque ordre.
+      // On repart toujours du dernier ordre trait car il y a une forte probabilit
+      // que deux pixels conscutifs soient de mme ordre
+      int maxOrder = (int) log2( newNSideFile );
+      for( int i=0; i<=maxOrder; i++,lastOrder++ ) {
+         if( lastOrder>maxOrder ) lastOrder=0;
+         int gap = 2*(maxOrder-lastOrder);
+         long nuniq = Moc.hpix2uniq(lastOrder, npix>>>gap);
+         Double a = (Double)partialValues.get( nuniq );
+         if( a!=null ) return a; 
+      }
+      return Double.NaN;
+   }
 
    private double[] ringValues; // tableau des valeurs des pixels RING, indexes en RING
    private void fillRingValues(int minIdx, int maxIdx, long idxTForm) {
@@ -1149,8 +1290,9 @@ public class PlanHealpix extends PlanBG {
       int length = maxNested - minNested;
       double[] val = new double[length];
       try {
+         int order = (int)CDSHealpix.log2(nside);
          for (int i=0; i<length; i++) {
-            val[i] = ringValues[(int)CDSHealpix.nest2ring(nside, minNested + i)];
+            val[i] = ringValues[(int)CDSHealpix.nest2ring(order, minNested + i)];
          }
       } catch( Exception e ) {
          e.printStackTrace();
@@ -1163,30 +1305,24 @@ public class PlanHealpix extends PlanBG {
          Fits out, long idxTForm, String dir) {
       return buildDoubleHealpix(nside_file, npix_file, nside, out, true, idxTForm, dir);
    }
-
+   
    boolean buildDoubleHealpix(int nside_file, long npix_file, int nside,
          Fits out, boolean write, long idxTForm, String dir) {
       boolean empty = true;
       long min, max;
       long index;
-
+      
       // cherche les numros de pixels Healpix dans ce losange
       min = getHealpixMin(nside_file, npix_file, nside, true);
       max = min+out.width*out.height;
 
 
-      double[] values = getValues(min, max, rafHpx, initialOffsetHpx, idxTForm);
+      double[] values = getValues(min, max, rafHpx, initialOffsetHpx, idxTForm);  // PF ICI
       double value;
       // cherche la valeur  affecter dans chacun des pixels healpix
       for( int y=0; y<out.height; y++ ) {
          for( int x=0; x<out.width; x++ ) {
-
-
             index = min + xy2hpx(y*out.width+x);
-            //                for( int i=0; i<overSamplingFactor; i++ ) {
-            //                    indexFather = getFather(indexFather);
-            //                }
-
             value = values[(int) (index-min)];
             if( empty && !Double.isNaN(value) ) empty=false;  // PF - juin 2013
             out.setPixelDouble(x, y, value);
@@ -1776,7 +1912,7 @@ public class PlanHealpix extends PlanBG {
     * @param survey nom du survey
     * @param order order Healpix
     * @param outLosangeWidth largeur des losanges pour le Allsky (typiquement 64 ou 128 pixels)
-    * @param mode FIRST, MAX, MEDIANE, MOYENNE, SIGMA
+    * @param acceleration FIRST, MAX, MEDIANE, MOYENNE, SIGMA
     */
    public void createAllSky(String path,String survey,int order,int outLosangeWidth) throws Exception {
       long t=System.currentTimeMillis();
@@ -1965,7 +2101,7 @@ public class PlanHealpix extends PlanBG {
     * @param order Ordre Healpix du losange
     * @param npix Numro Healpix du losange
     * @param fils les 4 fils du losange
-    * @param mode mthode de cumul des pixels (FIRST, MOYENNE, MEDIANE...)
+    * @param acceleration mthode de cumul des pixels (FIRST, MOYENNE, MEDIANE...)
     */
    //    Fits createNodeHpx(String file,int order,long npix,Fits fils[], int mode) throws Exception {
    Fits createNodeHpx(String file,int order,long npix,Fits fils[], int bitpix, double blank) throws Exception {
