@@ -1,10 +1,10 @@
-// Copyright 1999-2018 - Universit de Strasbourg/CNRS
+// Copyright 1999-2020 - Universit de Strasbourg/CNRS
 // The Aladin Desktop program is developped by the Centre de Donnes
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
 //
-//This file is part of Aladin.
+//This file is part of Aladin Desktop.
 //
 //    Aladin Desktop is free software: you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -16,15 +16,15 @@
 //    GNU General Public License for more details.
 //
 //    The GNU General Public License is available in COPYING file
-//    along with Aladin.
+//    along with Aladin Desktop.
 //
 
 package cds.aladin;
 
-import cds.tools.*;
+import java.io.RandomAccessFile;
+import java.util.Date;
 
-import java.io.*;
-import java.util.*;
+import cds.tools.Util;
 
 /**
  * Plan dedie a une image tres grande (IMAGEHUGE)
@@ -171,7 +171,7 @@ public class PlanImageHuge extends PlanImage implements Runnable {
             nextPixelsSub=0;
             for( int i=0; i<vPixelsSub.length; i++ ) {
                PixelsSub px = vPixelsSub[i];
-               for( int j=0; j<aladin.view.getModeView(); j++ ) {
+               for( int j=0; j<aladin.view.getNbView(); j++ ) {
                   ViewSimple v = aladin.view.viewSimple[j];
                   if( v.isFree() || v.pref!=this || !v.flagHuge ) continue;
                   if( px.agree(v.xHuge,v.yHuge,v.wHuge,v.hHuge) ) continue;
