@@ -1,10 +1,10 @@
-// Copyright 1999-2018 - Universit de Strasbourg/CNRS
+// Copyright 1999-2020 - Universit de Strasbourg/CNRS
 // The Aladin Desktop program is developped by the Centre de Donnes
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
 //
-//This file is part of Aladin.
+//This file is part of Aladin Desktop.
 //
 //    Aladin Desktop is free software: you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 //    GNU General Public License for more details.
 //
 //    The GNU General Public License is available in COPYING file
-//    along with Aladin.
+//    along with Aladin Desktop.
 //
 
 package cds.aladin;
@@ -28,15 +28,12 @@ import java.util.ArrayList;
 import java.util.Iterator;
 
 import cds.moc.Array;
-import cds.moc.Healpix;
-import cds.moc.HealpixMoc;
+import cds.moc.Moc;
 import cds.moc.MocCell;
+import cds.moc.SMoc;
 import cds.tools.Util;
 import cds.tools.pixtools.CDSHealpix;
 import cds.tools.pixtools.Hpix;
-import healpix.essentials.HealpixBase;
-import healpix.essentials.Pointing;
-import healpix.essentials.Vec3;
 
 /**
  * Gnration d'un plan MOC  partir d'un flux
@@ -54,17 +51,23 @@ public class PlanMoc extends PlanBGCat {
    static final public int DRAW_FILLIN   = 0x4;      // Remplissage avec aplat de demi-opacit
    static final public int DRAW_PERIMETER     = 0x8; // Trac du primtres
 
-   protected HealpixMoc moc = null;                 // Le MOC
-   private int wireFrame=DRAW_BORDER | DRAW_FILLIN; // Mode de tracage par dfaut
+   protected Moc moc = null;                 // Le MOC
+   protected int wireFrame=DRAW_BORDER | DRAW_FILLIN; // Mode de tracage par dfaut
+   
+   static int MAXGAPORDER=3;
+   protected int gapOrder=0;
 
-   private HealpixMoc [] arrayMoc =null;          // Le MOC  tous les ordres */
-   private ArrayList<Hpix> arrayHpix = null;      // Liste des cellules correspondant aux cellules tracs (order courant)
+   protected Moc [] arrayMoc =null;        // Le MOC  tous les ordres */
+   protected ArrayList<Hpix> arrayHpix = null;    // Liste des cellules correspondant aux cellules tracs (order courant)
    private ArrayList<Hpix> arrayPeri = null;      // Liste des cellules correspondant au primtre trac (ordre courant)
    
-   public PlanMoc(Aladin a) { super(a); }
+   public PlanMoc(Aladin a) {
+      super(a);
+      type = ALLSKYMOC;
+   }
 
    /** Cration d'un Plan MOC  partir d'un MOC pr-xistant */
-   protected PlanMoc(Aladin aladin, HealpixMoc moc, String label, Coord c, double radius) {
+   protected PlanMoc(Aladin aladin, SMoc moc, String label, Coord c, double radius) {
       this(aladin,null,moc,label,c,radius);
    }
 
@@ -72,16 +75,15 @@ public class PlanMoc extends PlanBGCat {
    protected PlanMoc(Aladin aladin, MyInputStream in, String label, Coord c, double radius) {
       this(aladin,in,null,label,c,radius);
    }
-
-   protected PlanMoc(Aladin aladin, MyInputStream in, HealpixMoc moc, String label, Coord c, double radius) {
+   
+   protected PlanMoc(Aladin aladin, MyInputStream in, SMoc moc, String label, Coord c, double radius) {
       super(aladin);
-      arrayMoc = new HealpixMoc[CDSHealpix.MAXORDER+1];
       this.dis   = in;
       this.moc   = moc;
       useCache = false;
       frameOrigin = Localisation.ICRS;
       if( moc!=null ) {
-         String f = moc.getCoordSys();
+         String f = moc.getSys();
          frameOrigin = f.equals("E")?Localisation.ECLIPTIC :
             f.equals("G")?Localisation.GAL:Localisation.ICRS;
          
@@ -95,7 +97,7 @@ public class PlanMoc extends PlanBGCat {
       setLabel(label);
       co=c;
       coRadius=radius;
-      aladin.trace(3,"AllSky creation: "+Plan.Tp[type]+(c!=null ? " around "+c:""));
+      aladin.trace(3,"MOC creation: "+Plan.Tp[type]+(c!=null ? " around "+c:""));
       suite();
    }
 
@@ -104,71 +106,71 @@ public class PlanMoc extends PlanBGCat {
       super.copy(p1);
       PlanMoc pm = (PlanMoc)p1;
       pm.frameOrigin=frameOrigin;
-      pm.moc = (HealpixMoc)moc.clone();
+      pm.moc = moc!=null ? moc.clone() : null;
       pm.wireFrame=wireFrame;
       pm.gapOrder=gapOrder;
       pm.arrayHpix = arrayPeri = null;
-      pm.arrayMoc = new HealpixMoc[CDSHealpix.MAXORDER+1];
+//      pm.arrayMoc = new Moc[CDSHealpix.MAXORDER+1];
    }
    
    /** Ajoute des infos sur le plan */
    protected void addMessageInfo( StringBuilder buf, MyProperties prop ) {
-      double cov = moc.getCoverage();
+      double cov = ((SMoc)moc).getCoverage();
       double degrad = Math.toDegrees(1.0);
       double skyArea = 4.*Math.PI*degrad*degrad;
       ADD( buf, "\n* Space: ",Coord.getUnit(skyArea*cov, false, true)+"^2, "+Util.round(cov*100, 3)+"% of sky");
-      ADD( buf, "\n* Best ang.res: ",Coord.getUnit(moc.getAngularRes()));
+      ADD( buf, "\n* Best ang.res: ",Coord.getUnit(((SMoc)moc).getAngularRes()));
       
-      int order = getRealMaxOrder(moc);
+      int order = moc.getMocOrder();
       int drawOrder = getDrawOrder();
-      ADD( buf,"\n","* MOC order: "+ (order==drawOrder ? order+"" : drawOrder+"/"+order));
+      ADD( buf,"\n","* SMOC order: "+ (order==drawOrder ? order+"" : "draw:"+drawOrder+"/"+order));
    }
 
    /** Changement de rfrentiel si ncessaire */
-   public HealpixMoc toReferenceFrame(String coordSys) throws Exception {
-      HealpixMoc moc1 = convertTo(moc,coordSys);
+   public SMoc toReferenceFrame(String coordSys) throws Exception {
+      SMoc moc1 = SMoc.convertTo((SMoc)moc,coordSys);
       if( moc!=moc1 ) {
-         aladin.trace(2,"Moc reference frame conversion: "+moc.getCoordSys()+" => "+moc1.getCoordSys());
+         aladin.trace(2,"Moc reference frame conversion: "+moc.getSys()+" => "+moc1.getSys());
       }
       return moc1;
    }
 
-   /** Changement de rfrentiel si ncessaire */
-   static public HealpixMoc convertTo(HealpixMoc moc, String coordSys) throws Exception {
-      if( coordSys.equals( moc.getCoordSys()) ) return moc;
-
-      char a = moc.getCoordSys().charAt(0);
-      char b = coordSys.charAt(0);
-      int frameSrc = a=='G' ? Localisation.GAL : a=='E' ? Localisation.ECLIPTIC : Localisation.ICRS;
-      int frameDst = b=='G' ? Localisation.GAL : b=='E' ? Localisation.ECLIPTIC : Localisation.ICRS;
-
-      Healpix hpx = new Healpix();
-      int order = moc.getMaxOrder();
-      HealpixMoc moc1 = new HealpixMoc(coordSys,moc.getMinLimitOrder(),moc.getMocOrder());
-      moc1.setCheckConsistencyFlag(false);
-      long onpix1=-1;
-      Iterator<Long> it = moc.pixelIterator();
-      while( it.hasNext() ) {
-         long npix = it.next();
-         for( int i=0; i<4; i++ ) {
-            double [] coo = hpx.pix2ang(order+1, npix*4+i);
-            Coord c = new Coord(coo[0],coo[1]);
-            c = Localisation.frameToFrame(c, frameSrc, frameDst);
-            long npix1 = hpx.ang2pix(order+1, c.al, c.del);
-            if( npix1==onpix1 ) continue;
-            onpix1=npix1;
-            moc1.add(order,npix1/4);
-         }
-
-      }
-      moc1.setCheckConsistencyFlag(true);
-      return moc1;
-   }
+//   /** Changement de rfrentiel si ncessaire */
+//   static public SMoc convertTo(SMoc moc, String coordSys) throws Exception {
+//      if( coordSys.equals( moc.getCoordSys()) ) return moc;
+//
+//      char a = moc.getCoordSys().charAt(0);
+//      char b = coordSys.charAt(0);
+//      int frameSrc = a=='G' ? Localisation.GAL : a=='E' ? Localisation.ECLIPTIC : Localisation.ICRS;
+//      int frameDst = b=='G' ? Localisation.GAL : b=='E' ? Localisation.ECLIPTIC : Localisation.ICRS;
+//
+//      Healpix hpx = new Healpix();
+//      int order = moc.getMaxOrder();
+//      SMoc moc1 = new SMoc(coordSys,moc.getMinLimitOrder(),moc.getMocOrder());
+//      moc1.setCheckConsistencyFlag(false);
+//      long onpix1=-1;
+//      Iterator<Long> it = moc.pixelIterator();
+//      while( it.hasNext() ) {
+//         long npix = it.next();
+//         for( int i=0; i<4; i++ ) {
+//            double [] coo = hpx.pix2ang(order+1, npix*4+i);
+//            Coord c = new Coord(coo[0],coo[1]);
+//            c = Localisation.frameToFrame(c, frameSrc, frameDst);
+//            long npix1 = hpx.ang2pix(order+1, c.al, c.del);
+//            if( npix1==onpix1 ) continue;
+//            onpix1=npix1;
+//            moc1.add(order,npix1/4);
+//         }
+//
+//      }
+//      moc1.setCheckConsistencyFlag(true);
+//      return moc1;
+//   }
    
    /** Retourne le Moc.maxOrder rel, mme pour les vieux MOCs dont le Norder est gnralement
     * faux */
-   static private int getRealMaxOrder(HealpixMoc m) {
-      int nOrder = m.getMaxOrder();
+   static protected int getRealMaxOrder(SMoc m) {
+      int nOrder = m.getMaxUsedOrder();
       if( nOrder<=0 ) return nOrder;
       Array a;
       while( ( (a=m.getArray(nOrder))==null || a.getSize()==0) && nOrder>0 ) nOrder--;
@@ -177,7 +179,7 @@ public class PlanMoc extends PlanBGCat {
    
    // POUR LE MOMENT ON N'UTILISE PAS ENCORE CETTE FONCTION
    // Cration du Pcat qui contient les lignes du primtre du MOC
-//   private Pcat createPerimetre(ViewSimple v,HealpixMoc moc) throws Exception {
+//   private Pcat createPerimetre(ViewSimple v,SMoc moc) throws Exception {
 //      Pcat pcat = new Pcat(this);
 //      ArrayList<double[]> a = getPerimeter(moc);
 //      Ligne oo=null;
@@ -204,7 +206,7 @@ public class PlanMoc extends PlanBGCat {
     * @return
     * @throws Exception
     */
-   static public String createPerimeterString(HealpixMoc moc) throws Exception {
+   static public String createPerimeterString(SMoc moc) throws Exception {
       StringBuilder res = null;
       ArrayList<double[]> a = getPerimeter(moc);
 
@@ -224,25 +226,25 @@ public class PlanMoc extends PlanBGCat {
     * @return
     * @throws Exception
     */
-   static public ArrayList<double[]> getPerimeter(HealpixMoc moc) throws Exception {
+   static public ArrayList<double[]> getPerimeter(SMoc moc) throws Exception {
       if( moc==null ) return null;
       int maxOrder = getRealMaxOrder(moc);
-      ArrayList<double[]> a = new ArrayList<double[]>();
-      if( maxOrder==-1 || moc.isAllSky() ) return a;
-      HealpixMoc done = new HealpixMoc( moc.getCoordSys(), moc.getMinLimitOrder(),maxOrder );
-      HealpixBase hpx= CDSHealpix.getHealpixBase(maxOrder);
+      ArrayList<double[]> a = new ArrayList<>();
+      if( maxOrder==-1 || moc.isFull() ) return a;
+      SMoc done = new SMoc( moc.getSys(), moc.getMinOrder(),maxOrder );
+//      long nside = CDSHealpix.pow2( maxOrder );
 
       Iterator<Long> it = moc.pixelIterator();
       while( it.hasNext() ) {
          long pix = it.next();
-         parcoursBord(hpx,moc,done,a,maxOrder,pix,0,0);
+         parcoursBord(maxOrder,moc,done,a,maxOrder,pix,0,0);
          if( a.size()>0 && a.get(a.size()-1)!=null ) a.add(null);
       }
 
       return a;
    }
    
-   static private void parcoursBord(HealpixBase hpx, HealpixMoc moc, HealpixMoc done, ArrayList<double[]> a, 
+   static private void parcoursBord(int order, SMoc moc, SMoc done, ArrayList<double[]> a, 
          int maxOrder, long pix, int sens, int rec ) throws Exception {
       if( done.isIntersecting(maxOrder,pix) ) return;
       
@@ -250,13 +252,13 @@ public class PlanMoc extends PlanBGCat {
       
       done.add(maxOrder,pix);
       
-      long [] voisins = getVoisins(hpx,moc,maxOrder,pix);
+      long [] voisins = getVoisins(order,moc,maxOrder,pix);
       double [][] corners = null;
       for( int j=0; j<4; j++ ) {
          int i = (sens+j)%4;
          long voisin = voisins[i];
          if( voisin!=-1 ) continue;
-         if( corners==null ) corners = getCorners(hpx,pix);
+         if( corners==null ) corners = getCorners(order,pix);
          
          boolean flagAdd=true;
          if( flagAdd && a.size()>0 ) {
@@ -270,12 +272,12 @@ public class PlanMoc extends PlanBGCat {
          long nextVoisin = voisins[ i<3 ? i+1 : 0 ];
          if( nextVoisin!=-1 ) {
             
-            long [] vVoisins = getVoisins(hpx,moc,maxOrder,nextVoisin);
+            long [] vVoisins = getVoisins(order,moc,maxOrder,nextVoisin);
             long nNextVoisin = vVoisins[i];
             if( nNextVoisin!=-1 ) {
-               parcoursBord(hpx,moc,done,a,maxOrder,nNextVoisin,i==0?3:i-1,rec+1);
+               parcoursBord(order,moc,done,a,maxOrder,nNextVoisin,i==0?3:i-1,rec+1);
             }
-            else parcoursBord(hpx,moc,done,a,maxOrder,nextVoisin,i,rec+1);
+            else parcoursBord(order,moc,done,a,maxOrder,nextVoisin,i,rec+1);
             if( a.size()>0 && a.get(a.size()-1)!=null ) a.add(null);
          }
       }
@@ -285,27 +287,37 @@ public class PlanMoc extends PlanBGCat {
 
    // Retourne les coordonnes des 4 coins du pixel HEALPix indiqu
    // Ordre des coins => S, W, N, E
-   static private double [][] getCorners(HealpixBase hpx,long pix) throws Exception {
-      Vec3[] tvec = hpx.boundaries(pix,1);   // N W S E
-      double [][] corners = new double[tvec.length][2];
-      for (int i=0; i<tvec.length; ++i) {
-         Pointing pt = new Pointing(tvec[i]);
+   static private double [][] getCorners(int order,long pix) throws Exception {
+      double [][] radec =  CDSHealpix.borders(order,pix,1);
+      double [][] corners = new double[radec.length][2];
+      for (int i=0; i<radec.length; ++i) {
          int j=A[i];
-         corners[j][0] = ra(pt);
-         corners[j][1] = dec(pt);
+         corners[j] = CDSHealpix.polarToRadec( radec[i] );
       }
       return corners;
    }
-   
-   public static final double cPr = Math.PI / 180;
-   static private double dec(Pointing ptg) { return (Math.PI*0.5 - ptg.theta) / cPr; }
-   static private double ra(Pointing ptg) { return ptg.phi / cPr; }
+
+//   static private double [][] getCorners(long nside,long pix) throws Exception {
+//      Vec3[] tvec = hpx.boundaries(pix,1);   // N W S E
+//      double [][] corners = new double[tvec.length][2];
+//      for (int i=0; i<tvec.length; ++i) {
+//         Pointing pt = new Pointing(tvec[i]);
+//         int j=A[i];
+//         corners[j][0] = ra(pt);
+//         corners[j][1] = dec(pt);
+//      }
+//      return corners;
+//   }
+//   
+//   public static final double cPr = Math.PI / 180;
+//   static private double dec(Pointing ptg) { return (Math.PI*0.5 - ptg.theta) / cPr; }
+//   static private double ra(Pointing ptg) { return ptg.phi / cPr; }
 
    // Retourne la liste des numros HEALPix des 4 voisins directs ou -1 s'ils sont en dehors du MOC   
    // Ordre des voisins => W, N, E, S
-   static private long [] getVoisins(HealpixBase hpx, HealpixMoc moc, int maxOrder,long npix) throws Exception {
+   static private long [] getVoisins(int order, SMoc moc, int maxOrder,long npix) throws Exception {
       long [] voisins = new long[4];
-      long [] neib = hpx.neighbours(npix);
+      long [] neib = CDSHealpix.neighbours(order,npix);
       for( int i=0,j=0; i<voisins.length; i++, j+=2 ) {
          voisins[i] = moc.isIntersecting(maxOrder, neib[j]) ? neib[j] : -1;
       }
@@ -314,9 +326,9 @@ public class PlanMoc extends PlanBGCat {
    
    // Retourne la liste des numros HEALPix des 4 voisins directs ou -1 s'il n'y en a pas du mme ordre   
    // Ordre des voisins => W, N, E, S
-   private long [] getVoisinsSameOrder(HealpixBase hpx, HealpixMoc moc, int maxOrder,long npix) throws Exception {
+   private long [] getVoisinsSameOrder(int order, SMoc moc, int maxOrder,long npix) throws Exception {
       long [] voisins = new long[4];
-      long [] neib = hpx.neighbours(npix);
+      long [] neib = CDSHealpix.neighbours(order,npix);
       for( int i=0,j=0; i<voisins.length; i++, j+=2 ) {
          voisins[i] = moc.isIn(maxOrder, neib[j]) ? neib[j] : -1;
       }
@@ -326,10 +338,10 @@ public class PlanMoc extends PlanBGCat {
    protected int getMocOrder() { return moc.getMocOrder(); }
 
    /** Retourne le Moc */
-   protected HealpixMoc getMoc() { return moc; }
+   protected Moc getMoc() { return moc; }
 
    protected void suiteSpecific() { isOldPlan=false; }
-   protected boolean isSync() { return isReady(); }
+   protected boolean isSync() { return isReady() && !isLoading(); }
    protected void reallocObjetCache() { }
 
    public void setWireFrame(int wireFrame) { this.wireFrame=wireFrame; }
@@ -374,25 +386,30 @@ public class PlanMoc extends PlanBGCat {
    }
    
    /** Retourne une chaine dcrivant le pourcentage de couverture */
-   public String getPropCoverage() { return Util.myRound(moc.getCoverage()); }
+   public String getPropCoverage() { return Util.myRound(((SMoc)moc).getCoverage()); }
 
    /** Retourne une chaine dcrivant le MOC order */
-   public String getPropMocOrder() { return getRealMaxOrder(moc)+""; }
+   public String getPropMocOrder() { return getRealMaxOrder((SMoc)moc)+""; }
 
    protected boolean isCatalog() { return false; }
+   protected boolean isTime() { return false; }
    protected boolean hasSources() { return false; }
+   protected boolean hasCatalogInfo() { return false; }
    protected int getCounts() { return 0; }
+   
+   // Pas de MOc  charger pour un plan Moc
+   protected void planReadyMoc() {}
 
    protected boolean waitForPlan() {
       if( dis!=null ) {
          super.waitForPlan();
          try {
             if( moc==null && dis!=null ) {
-               moc = new HealpixMoc();
+               moc = new SMoc();
                if(  (dis.getType() & MyInputStream.FITS)!=0 ) moc.readFits(dis);
                else moc.readASCII(dis);
             }
-            String c = moc.getCoordSys();
+            String c = moc.getSys();
             frameOrigin = ( c==null || c.charAt(0)=='G' ) ? Localisation.GAL : Localisation.ICRS;
             if( moc.getSize()==0 ) error="Empty MOC";
          }
@@ -401,44 +418,47 @@ public class PlanMoc extends PlanBGCat {
             return false;
          }
       }
+      
+      // Dtermination ou correction du target
+      if( !(/* flagNoTarget || */ moc==null || moc.isEmpty()) ) {
+         
+         // est-ce que le target tombe bien dans le moc ? sinon on va forcer le recalcul sur la premire cellule du MOC
+         if( co!=null ) {
+            int order = moc.getMocOrder();
+            Coord c = Localisation.frameToFrame(co, Localisation.ICRS, frameOrigin);
+            double[] radec = CDSHealpix.radecToPolar(new double[] { c.al, c.del });
+            try {
+               long npix = CDSHealpix.ang2pix_nest(order, radec[0], radec[1]);
+               if( !((SMoc)moc).isIntersecting(order, npix) ) co=null;
+            } catch( Exception e ) {}
+         }
 
-      // Mmoristion de la position de la premire cellule
-      if( (co==null || flagNoTarget ) && moc.getSize()>0 && frameOrigin==Localisation.ICRS ) {
-         try {
-            MocCell cell = moc.iterator().next();
-            double res[] = CDSHealpix.pix2ang_nest(CDSHealpix.pow2(cell.getOrder()), cell.getNpix());
-            double[] radec = CDSHealpix.polarToRadec(new double[] { res[0], res[1] });
-            co = new Coord(radec[0],radec[1]);
-         } catch( Exception e ) {
-            if( aladin.levelTrace>=3 ) e.printStackTrace();
+         // Mmorisation en tant que target de la position de la premire cellule
+         if( co==null || co.al==0 && co.del==0  ) {
+            try {
+               MocCell cell = moc.iterator().next();
+//               long nside = CDSHealpix.pow2(cell.getOrder());
+               double res[] = CDSHealpix.pix2ang_nest(cell.getOrder(), cell.getNpix());
+               double[] radec = CDSHealpix.polarToRadec(new double[] { res[0], res[1] });
+               co = Localisation.frameToFrame( new Coord(radec[0],radec[1]) , frameOrigin, Localisation.ICRS);
+               aladin.trace(3,"MOC target (re)computed from the first MOC HEALPix cell => "+co);
+            } catch( Exception e ) {
+               if( aladin.levelTrace>=3 ) e.printStackTrace();
+            }
          }
       }
 
-      // Je force le MOC minOrder  3 pour que l'affichage soit propre
-      if( moc!=null && moc.getMinLimitOrder()<3 ) {
+      // Je force le MOC minOrder  3 pour que l'affichage soit toujours propre
+      if( moc!=null && ((SMoc)moc).getMinOrder()<3 ) {
          try {
             if( moc.getMocOrder()<3 ) moc.setMocOrder(3);   
-            moc.setMinLimitOrder(3);
+            ((SMoc)moc).setMinOrder(3);
          } catch( Exception e ) {
             if( aladin.levelTrace>=3 ) e.printStackTrace();
             error="MOC error";
             return false;
          }
       }
-      
-      // Je rectifie une ventuelle erreur de dclaration du maxOrder
-      int maxOrder = getRealMaxOrder(moc);
-      int o = moc.getMaxOrder();
-      if( maxOrder!=o ) {
-         try {
-            moc.setMocOrder(maxOrder);
-            aladin.console.printError("Moc order probably wrong ("+o+"), assuming "+maxOrder);
-         } catch( Exception e ) {
-            if( aladin.levelTrace>=3 ) e.printStackTrace();
-            return false;
-         }
-         
-      }
 
       return true;
    }
@@ -449,23 +469,27 @@ public class PlanMoc extends PlanBGCat {
    protected boolean isDrawn() { return true; }
 
    // Fournit le MOC qui couvre le champ de vue courant
-   private HealpixMoc getViewMoc(ViewSimple v,int order) throws Exception {
-      Coord center = getCooCentre(v);
-      long [] pix = getPixList(v,center,order);
-
-      HealpixMoc m = new HealpixMoc();
-      m.setCheckConsistencyFlag(false);
-      m.setCoordSys(moc.getCoordSys());
-      for( int i=0; i<pix.length; i++ ) m.add(order,pix[i]);
-      m.setCheckConsistencyFlag(true);
-      return m;
+   protected SMoc getViewMoc(ViewSimple v,int order) throws Exception {
+      return (SMoc) v.getMoc(order);
    }
+//   protected SMoc getViewMoc(ViewSimple v,int order) throws Exception {
+//      Coord center = getCooCentre(v);
+//      long [] pix = getPixList(v,center,order);
+//      if( pix==null ) {
+//         if( Aladin.levelTrace>=3 ) System.err.println("Bizarre PlanMoc.getViewMoc=null center="+center+" order="+order);
+//         return null;
+//      }
+//
+//      SMoc m = new SMoc();
+//      m.setCheckConsistencyFlag(false);
+//      m.setCoordSys(moc.getCoordSys());
+//      for( int i=0; i<pix.length; i++ ) m.add(order,pix[i]);
+//      m.setCheckConsistencyFlag(true);
+//      return m;
+//   }
 
-   static int MAXGAPORDER=3;
-   private int gapOrder=0;
-   
    protected void setMaxGapOrder() { setGapOrder(MAXGAPORDER); }
-
+   
    protected int getGapOrder() { return gapOrder; }
    protected void setGapOrder(int gapOrder) {
       if( Math.abs(gapOrder)>MAXGAPORDER ) return;
@@ -501,38 +525,74 @@ public class PlanMoc extends PlanBGCat {
       }
    }
    
-   // retourne/construit la liste du MOC
-   //  l'ordre courant (mode progressif)
-   private HealpixMoc getHealpixMocLow(int order,int gapOrder) {
-      
-      // On fournit le meilleur MOC dans le cas de la gnration d'une image
-      if( aladin.NOGUI ) return moc;
-
-      int mo = moc.getMaxOrder();
+   protected SMoc getSpaceMoc() { return (SMoc)moc; }
+   
+   protected  boolean isTimeModified() { return false; }
+   
+   
+   protected Moc getSpaceMocLow(ViewSimple v,int order,int gapOrder) {
+      SMoc m = (SMoc) getSpaceMocLow1(v,order,gapOrder);
+      try { m.setMinOrder(3); } catch( Exception e ) { }
+      return m;
+   }
+   
+   /** Retourne l'order  utiliser pour l'affichage courant en fonction de la position
+    * du slider "densit" */
+   protected int getLowOrder(int order, int gapOrder ) {
+      int mo = moc.getSpaceOrder();
+      if( mo==-1 ) {
+         try { mo = getRealMaxOrder( (SMoc) moc); } catch( Exception e ) {  e.printStackTrace(); }
+      }
       if( mo<3 ) mo=3;
       order += 5;
       if( order<7 ) order=7;
       order += gapOrder;
       if( order<5 ) order=5;
       if( order>mo ) order=mo;
-      //      System.out.println("getHpixListProg("+o+") => "+order);
-      if( arrayMoc[order]==null ) {
-         arrayMoc[order] = new HealpixMoc();   // pour viter de lancer plusieurs threads sur le meme calcul
-         final int myOrder = order;
-         final int myMo=mo;
-         (new Thread("PlanMoc building order="+order){
+      return order;
+   }
+   
+   // retourne/construit la liste du MOC  l'ordre courant (mode progressif)
+   protected Moc getSpaceMocLow1(ViewSimple v,int order,int gapOrder) {
+      if( aladin.NOGUI ) return moc;
 
-            public void run() {
-               Aladin.trace(4,"PlanMoc.getHealpixMocLow("+myOrder+") running...");
-               HealpixMoc mocLow = myOrder==myMo ? moc : (HealpixMoc)moc.clone();
-               try { mocLow.setMocOrder(myOrder); }
-               catch( Exception e ) { e.printStackTrace(); }
-               arrayMoc[myOrder]=mocLow;
-               Aladin.trace(4,"PlanMoc.getHealpixMocLow("+myOrder+") done !");
-               askForRepaint();
-            }
-
-         }).start();
+      int mo = moc.getMaxUsedOrder();
+      if( mo<3 ) mo=3;
+      order = getLowOrder(order,gapOrder);
+      
+      // Pour les petits champs, on travaille sans cache, en dcoupant le MOC  la bonne
+      // rsolution et  la bonne taille.
+      if( v.getTaille()<20 ) {
+         try {
+            // On amliore un peu la rsolution
+            if( order<moc.getSpaceOrder() ) order++;
+            
+            // Peut tre y a-t-il dans le cache des MocLows la Moc complet  la bonne rsolution ?
+            // Sinon on prend le MOC d'origine
+            Moc mocP = arrayMoc!=null && arrayMoc[order]!=null 
+                  && arrayMoc[order].getSize()>0 ? arrayMoc[order] : moc;
+            
+            // On dcoupe la zone concerne
+            Moc moclow = mocP.intersection( v.getMoc() );
+            
+            // On ajuste l'ordre spatial max
+            moclow.setSpaceOrder(order);
+            return moclow;
+            
+         } catch( Exception e ) {
+            if( aladin.levelTrace>=3 ) e.printStackTrace();
+         }
+      }
+      
+      // Pour les grands champs on va prcalculer les MOC  chaque rsolution pour pouvoir
+      /// fournir rapidement celui  la rsolution la plus adquate.
+      if( arrayMoc==null || arrayMoc[order]==null ) {
+         initArrayMoc(order);
+         BuildLow t = new BuildLow(moc,order,mo);
+         
+         // Si petit, je ne threade pas
+         if( moc.getSize()<100000 || (lastBuildingTime>=0 && lastBuildingTime<30) ) t.run();
+         else t.start();
 
       }
       // peut tre y a-t-il dj un MOC de plus basse rsolution dj prt
@@ -546,15 +606,61 @@ public class PlanMoc extends PlanBGCat {
       lastOrderDrawn = order;
       return arrayMoc[order];
    }
+   
+   protected void initArrayMoc(int order) {
+      if( arrayMoc==null ) arrayMoc = new Moc[Moc.MAXORDER+1];
+      arrayMoc[order] = new SMoc();   // pour viter de lancer plusieurs threads sur le meme calcul
+   }
+   
+   private long lastBuildingTime=-1;
+   
+   class BuildLow extends Thread {
+      int myOrder,myMo;
+      Moc moc;
 
-   private boolean isLoading=false;
+      BuildLow(Moc moc,int myOrder,int myMo) {
+         super("BuidLow order="+myOrder);
+         setPriority( getPriority()-1 );
+         this.moc = moc;
+         this.myOrder= myOrder;
+         this.myMo = myMo;
+      }
+      
+      public void run() {
+         long t0 = System.currentTimeMillis();
+         Aladin.trace(4,"PlanMoc.getHealpixMocLow("+myOrder+") running...");
+         Moc mocLow=null;
+         if( myOrder==myMo ) mocLow = moc;
+         else {
+            // Si dj calcul, on va dgrader le MOC  partir d'un autre  une meilleure rsolution la plus proche
+            for( int o=myOrder; o<myMo; o++ ) {
+               if( arrayMoc[o]!=null && arrayMoc[o].getSize()!=0) {
+                  mocLow = arrayMoc[o].clone();
+                  break;
+               }
+            }
+            if( mocLow==null ) mocLow = moc.clone();
+         }
+//         Moc mocLow = myOrder==myMo ? moc : moc.clone();
+         try {
+            mocLow.setSpaceOrder(myOrder);
+         } catch( Exception e ) { e.printStackTrace(); }
+         arrayMoc[myOrder]=mocLow;
+         lastBuildingTime = System.currentTimeMillis() - t0;
+         Aladin.trace(4,"PlanMoc.getHealpixMocLow("+myOrder+") done in "+lastBuildingTime+"ms");
+         askForRepaint();
+      }
+
+   }
+
+   protected boolean isLoading=false;
    protected boolean isLoading() { return isLoading; }
 
-   private int lastOrderDrawn=-1;   // Dernier order trac
+   protected int lastOrderDrawn=-1;   // Dernier order trac
 
    /** Retourne true si tout a t dessine, sinon false */
    protected boolean hasMoreDetails() {
-      return moc!=null && lastOrderDrawn < moc.getMaxOrder();
+      return moc!=null && lastOrderDrawn < moc.getMaxUsedOrder();
    }
    
    /** Retourne l'ordre du dernier MOC dessin effectivement */
@@ -578,75 +684,85 @@ public class PlanMoc extends PlanBGCat {
          if( (++i)>=order ) { bord++; i=1; }
          return res;
       }
-	@Override
-	public void remove() {
-		// TODO Auto-generated method stub
-	}
+    @Override
+    public void remove() {
+        // TODO Auto-generated method stub
+    }
    }
    
-   private long oiz=-1;
+   
+   protected void memoNewTime() { }
+   
+   protected long oiz=-1;
    private boolean oFlagPeri;
    private int oGapOrder;
    
    // Trac du MOC visible dans la vue
    protected void draw(Graphics g,ViewSimple v) {
+      drawInSpaceView(g,v);
+   }
+   
+   private SMoc lastDrawMoc=null;
+   protected SMoc getLastDrawMoc() { return lastDrawMoc; }
+
+      
+   protected void drawInSpaceView(Graphics g, ViewSimple v) {
            
       long t1 = Util.getTime();
       g.setColor(c);
       int max = Math.min(maxOrder(v),maxOrder)+1;
       
       try {
-         HealpixMoc m = v.isAllSky() ? null : getViewMoc(v,max);
          
          long t=0;
          int myOrder = max+ (v.isAllSky()?0:1);
          
-         t = System.currentTimeMillis();
+//         t = System.currentTimeMillis();
          
          int drawingOrder = 0;
-         HealpixBase hpx=null;
-         HealpixMoc lowMoc = null;
+         Moc lowMoc = null;
          boolean flagPeri = isDrawingPerimeter();
          boolean flagBorder = isDrawingBorder();
          boolean flagFill = isDrawingFillIn();
 
          int gapOrder = this.gapOrder;
-         if( mustDrawFast() ) {
-            gapOrder--;
-         }
+         
+         
+         // En cas de ralentissement de l'affichage, on dgrade d'un cran la rsolution
+         if( mustDrawFast() ) gapOrder--;
 
          // Gnration des Hpix concernes par le champ de vue
          if( oiz!=v.getIZ() || flagPeri!=oFlagPeri || gapOrder!=oGapOrder ) {
-            lowMoc = getHealpixMocLow(myOrder,gapOrder);
-            drawingOrder = getRealMaxOrder(lowMoc);
+            lowMoc = getSpaceMocLow(v,myOrder,gapOrder);
+            drawingOrder = getRealMaxOrder((SMoc)lowMoc);
             if( drawingOrder==-1 ) return;
             //            System.out.println("Rcupration Hpix order "+drawingOrder);
-            hpx= CDSHealpix.getHealpixBase(drawingOrder);
-            ArrayList<Hpix> a1 = new ArrayList<Hpix>(10000);
+//            long drawingNside = CDSHealpix.pow2(drawingOrder);
+            
+            lastDrawMoc = getViewMoc(v,max);
+            String coordsys = moc.getSys();
+            boolean notEquatorial = moc!=null && coordsys!=null && !coordsys.equals("C");
+            SMoc viewMoc = v.isAllSky() || notEquatorial ? null : lastDrawMoc ;
+            ArrayList<Hpix> a1 = new ArrayList<>(10000);
             ArrayList<Hpix> a2 = !flagPeri ? null : new ArrayList<Hpix>(10000);
             Iterator<MocCell> it = lowMoc.iterator();
             while( it.hasNext() ) {
                MocCell c = it.next();
-               if( m!=null && !m.isIntersecting(c.order, c.npix)) continue;
+               if( viewMoc!=null && !viewMoc.isIntersecting(c.order, c.npix)) continue;
                Hpix p = new Hpix(c.order, c.npix, frameOrigin);
                if( p.isOutView(v) ) continue;
                a1.add(p);
 
                if( flagPeri )  {
-                  long [] vo = getVoisinsSameOrder(CDSHealpix.getHealpixBase(p.order), lowMoc, p.order, p.npix);
+                  long [] vo = getVoisinsSameOrder(p.order, (SMoc)lowMoc, p.order, p.npix);
                   if( vo[0]!=-1 && vo[1]!=-1 && vo[2]!=-1 && vo[3]!=-1 ) continue;
-
-                  //                  long deb = p.npix << 2*(drawingOrder-p.order);
-                  //                  long fin = (p.npix+1) << 2*(drawingOrder-p.order);
-                  //                  for( long pix=deb; pix<fin; pix++ ) {
-
                   long base = p.npix << 2*(drawingOrder-p.order);
                   Bord bord = new Bord((int)CDSHealpix.pow2(drawingOrder-p.order));
                   while( bord.hasNext() ) {
                      long b = bord.next();
                      long pix = base | b;
 
-                     vo = getVoisins(hpx, lowMoc, drawingOrder, pix);
+                     vo = getVoisins(drawingOrder, (SMoc)lowMoc, drawingOrder, pix);
                      int mask = (vo[0]==-1?0x1:0) | (vo[1]==-1?0x2:0) | (vo[2]==-1?0x4:0) | (vo[3]==-1?0x8:0);
                      if( mask!=0 ) {
                         Hpix p1 = new Hpix(drawingOrder, pix, frameOrigin);
@@ -671,7 +787,7 @@ public class PlanMoc extends PlanBGCat {
             try {
                g2d.setComposite( Util.getImageComposite(getOpacityLevel()*getFactorOpacity()) );
                for( Hpix p : arrayHpix ) {
-                  boolean small = isDrawingBorder() && p.getDiag2(v)<25;
+                  boolean small = flagBorder && p.getDiag2(v)<25;
                   if( !small ) p.fill(g, v);
                }
             } finally {
@@ -703,5 +819,6 @@ public class PlanMoc extends PlanBGCat {
    
    protected float factorOpacity = 0.5f;
    protected float getFactorOpacity() { return factorOpacity; }
+   
 }
 
