@@ -1,10 +1,10 @@
-// Copyright 1999-2018 - Universit de Strasbourg/CNRS
+// Copyright 1999-2020 - Universit de Strasbourg/CNRS
 // The Aladin Desktop program is developped by the Centre de Donnes
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
 //
-//This file is part of Aladin.
+//This file is part of Aladin Desktop.
 //
 //    Aladin Desktop is free software: you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 //    GNU General Public License for more details.
 //
 //    The GNU General Public License is available in COPYING file
-//    along with Aladin.
+//    along with Aladin Desktop.
 //
 
 package cds.aladin;
@@ -25,14 +25,16 @@ import java.util.ArrayList;
 import java.util.Collections;
 import java.util.Iterator;
 import java.util.List;
+import java.util.TreeSet;
 
 import cds.aladin.stc.STCObj;
 import cds.moc.Healpix;
-import cds.moc.HealpixMoc;
+import cds.moc.SMoc;
 import cds.tools.pixtools.CDSHealpix;
 
 /** Generation d'un plan MOC  partir d'une liste de plans (Image, Catalogue ou map HEALPix) 
  * @author P.Fernique [CDS]
+ * @version 1.2 - mar 2019 - correction HipstoMoc + correction
  * @version 1.1 - mar 2016 - ajout probability sky map
  * @version 1.0 - nov 2012
  */
@@ -67,24 +69,40 @@ public class PlanMocGen extends PlanMoc {
       log();
    }
    
-   protected void suite1() {}
+   protected void launchLoading() {}
    
-   private void addMocFromCatFov(Plan p1,int order) {
+   protected boolean Free() {
+      stop=true;
+      return super.Free();
+   }
+   
+   private boolean stop;
+   
+   private void addMocFromCatFov(Plan p1,int order) throws Exception {
       Iterator<Obj> it = p1.iterator();
       int m= p1.getCounts();
-      int n=0;
       double incrPourcent = gapPourcent/m;
+      int n=0;
+      stop=false;
+      
       while( it.hasNext() ) {
+         n++;
          Obj o = it.next();
          if( !(o instanceof Source) ) continue;
-         if( m<100 || n%100==0 ) pourcent+=incrPourcent;
+         
+         if( m<100 || n%100==0 ) {
+            pourcent+=incrPourcent;
+            if( stop ) throw new Exception("Abort");
+         }
+         
+         pourcent+=incrPourcent;
          Source s = (Source)o;
          SourceFootprint sf = s.getFootprint();
          if( sf==null ) continue;
          List<STCObj> listStcs = sf.getStcObjects();
-         if( listStcs==null ) continue;
+         if( listStcs==null || listStcs.size()==0 ) continue;
          try {
-            HealpixMoc m1 = aladin.createMocRegion(listStcs,order);
+            SMoc m1 = aladin.createMocRegion(listStcs,order);
             if( m1!=null ) moc.add(m1);
          } catch( Exception e ) {
             if( aladin.levelTrace>=3 ) e.printStackTrace();
@@ -93,18 +111,23 @@ public class PlanMocGen extends PlanMoc {
    }
       
    // Ajout d'un plan catalogue au moc en cours de construction
-   private void addMocFromCatalog(Plan p1,double radius,int order) {
+   private void addMocFromCatalog(Plan p1,double radius,int order) throws Exception {
       Iterator<Obj> it = p1.iterator();
       Healpix hpx = new Healpix();
       int o1 = order;
       Coord coo = new Coord();
       int n=0;
+      stop=false;
       int m= p1.getCounts();
       double incrPourcent = gapPourcent/m;
       while( it.hasNext() ) {
          Obj o = it.next();
          if( !(o instanceof Position) ) continue;
-         if( m<100 || n%100==0 ) pourcent+=incrPourcent;
+         
+         if( m<100 || n%100==0 ) {
+            pourcent+=incrPourcent;
+            if( stop ) throw new Exception("Abort");
+         }
          try {
             coo.al = ((Position)o).raj;
             coo.del = ((Position)o).dej;
@@ -112,9 +135,9 @@ public class PlanMocGen extends PlanMoc {
             if( radius==0 ) npix = new long[] { hpx.ang2pix(o1, coo.al, coo.del) };
             else npix = hpx.queryDisc(o1, coo.al, coo.del, radius);
             
-            for( long pix : npix ) moc.add(o1,pix);
+            for( long pix : npix ) ((SMoc)moc).add(o1,pix);
             n+=npix.length;
-            if( n>10000 ) { moc.checkAndFix(); n=0; }
+            if( n>100000 ) { ((SMoc)moc).checkAndFix(); n=0; }
          } catch( Exception e ) {
             if( aladin.levelTrace>=3 ) e.printStackTrace();
          }
@@ -132,19 +155,21 @@ public class PlanMocGen extends PlanMoc {
       double gapA=0,gapD=0;
       try { 
          gapA = Math.min(p1.projd.getPixResAlpha(),p1.projd.getPixResDelta());
-         for( o1=order; CDSHealpix.pixRes( CDSHealpix.pow2(o1) )/3600. <= gapA*2; o1--);
+         for( o1=order; CDSHealpix.pixRes( o1 )/3600. <= gapA*2; o1--);
       } catch( Exception e1 ) {
          e1.printStackTrace();
       }
 //      if( gap<1 || Double.isNaN(gap) ) gap=1;
 //      
-      gapD = CDSHealpix.pixRes( CDSHealpix.pow2(o1) )/3600.;
+      gapD = CDSHealpix.pixRes( o1 )/3600.;
 //      System.out.println("res="+res+" order="+order+" gapA ="+Coord.getUnit(gapA)+" gapD ="+Coord.getUnit(gapD)+" gap="+gap);
       
       // Pour garder en mmoire les pixels 
       pimg.setLockCacheFree(true);
       pimg.pixelsOriginFromCache();
 
+      int n=0;
+
       double incrPourcent = gapPourcent/pimg.naxis2;
       long oNpix=-1;  
       for( double y=0; y<pimg.naxis2; y+=gap ) {
@@ -153,124 +178,160 @@ public class PlanMocGen extends PlanMoc {
             try {
                coo.x = x;
                coo.y = (pimg.naxis2-y-1);
-               
+
                // dans du vide - on test d'abord le buffers 8bits, et on vrifie si on tombe sur 0
                if( pimg.getPixel8Byte((int)x,(int)coo.y)==0 && Double.isNaN(pimg.getPixel((int)x,(int)y)) ) continue;
-               
+
                // Hors de la plage de pixels
                if( flagRange ) {
                   double pix = pimg.getPixel((int)x,(int)y);
                   if( !Double.isNaN(pixMin) && pix<pixMin ) continue;
                   if( !Double.isNaN(pixMax) && pix>pixMax ) continue;
                }
-               
+
                pimg.projd.getCoord(coo);
                long npix=0;
                npix = hpx.ang2pix(o1, coo.al, coo.del);
 
                // Juste pour viter d'insrer 2x de suite le mme npix
                if( npix==oNpix ) continue;
-               
-               moc.add(o1,npix);
+
+               ((SMoc)moc).add(o1,npix);
+               if( n>100000 ) { ((SMoc)moc).checkAndFix(); n=0; }
                oNpix=npix;
             } catch( Exception e ) {
                if( aladin.levelTrace>=3 ) e.printStackTrace();
             }
          }
       }
-      
+
       // Les pixels peuvent dsormais tre librs
       pimg.setLockCacheFree(false);
    }
    
    // Ajout d'un plan map Healpix au MOC en cours de construction
-   private void addMocFromPlanBG(Plan p1,int order, double pixMin,double pixMax) {
+   private void addMocFromPlanBG(Plan p1,int order, double pixMin,double pixMax) throws Exception { addMocFromPlanBG(p1,order,-1,pixMin,pixMax); }
+   
+   private void addMocFromPlanBG(Plan p1,int order, int infoTileWidth, double pixMin,double pixMax) throws Exception {
       boolean flagRange = !Double.isNaN(pixMin) || !Double.isNaN(pixMax);
       PlanBG p = (PlanBG)p1;
       
-      
       // Passage en mode FITS
       if( !p.hasOriginalPixels() ) p.switchFormat();
       
-      // Dtermination de l'ordre pixel (order) et tuiles (fileOrder)
-      int tileOrder = p.getTileOrder();
+      // Recherche de la taille d'une tuile si non explicitement mentionne
+      int tileOrder = infoTileWidth==-1 ? p.getTileOrder() : (int) CDSHealpix.log2(infoTileWidth);
+      int tileWidth = (int)CDSHealpix.pow2(tileOrder);
       int z = (int)p.getZ();
       
-      int divOrder=0;
-      int fileOrder = order - tileOrder;
-      
-      // L'ordre des tuiles ne peut tre infrieur  3
-      if( fileOrder<3 ) {
-         divOrder=(3-fileOrder)*2;
-         fileOrder=3;
+      if( order> tileOrder+ p.maxOrder ) {
+         order = tileOrder+p.maxOrder;
+         aladin.warning("MOC order greater than HiPS resolution. Assuming MOC order "+order);
       }
+
       
-      /// L'ordre des tuiles ne peut entrainer le dpassement de la rsolution
-      // de la map HEALPix
-      if( fileOrder>p.getMaxFileOrder() ) {
-         fileOrder=p.getMaxFileOrder();
-         order = fileOrder+tileOrder;
+      // Quel est l'ordre des tuiles requis
+      int requiredHipsOrder = order-tileOrder;
+      
+      // Par dfaut on suppose que cet order Hips est disponible
+      int hipsOrder = requiredHipsOrder;
+      
+      // Par dfaut on considre qu'un pixel de tuile correspond  un pixel du MOC
+      int cellOrder = 0;
+      
+      // Cet ordre HiPS est-il disponible ? et sinon quel est le niveau sup le plus proche
+      // Cela impactera le nombre de pixels  moyenner dans la tuile de dpart pour un pixel du MOC : 2^cellOrder = taille de ce bloc
+      int minHipsOrder = p.getMinOrder();
+      if( minHipsOrder>requiredHipsOrder ) {
+         cellOrder = minHipsOrder - requiredHipsOrder;
+         hipsOrder=minHipsOrder;
       }
       
       // On gnre d'abord un MOC dans le systme de rfrence de la map HEALPix
       // on fera la conversion en ICRS  la fin du processus
-      moc.setCoordSys( p.frameOrigin==Localisation.GAL ? "G" : 
-                       p.frameOrigin==Localisation.ECLIPTIC ? "E" : "C");
+      moc.setMocOrder(order);
+      ((SMoc)moc).setSys( p.frameOrigin==Localisation.GAL ? "G" : 
+                               p.frameOrigin==Localisation.ECLIPTIC ? "E" : "C");
       frameOrigin = p.frameOrigin;
       try { moc.setCheckConsistencyFlag(false); } catch(Exception e) {}
       
       // Nombre de losanges  traiter
-      int n = (int)CDSHealpix.pow2(fileOrder); 
+      int n = (int)CDSHealpix.pow2(hipsOrder); 
       n=12*n*n;
       
 //      System.out.println("Nombre de losanges  traiter : "+n);
       
-      // Sans doute inutile car dj fait
-      try { p.createHealpixOrder(tileOrder); } catch( Exception e1 ) { e1.printStackTrace(); }
-      long nsize = CDSHealpix.pow2(tileOrder);
+      try { createHealpixOrder(tileOrder-cellOrder); }
+      catch( Exception e1 ) { }
       
       double incrPourcent = gapPourcent/n;
+      boolean first=true;
+      
+      int cellWidth = (int)CDSHealpix.pow2(cellOrder);   // taille d'un bloc (cot)
+      int nbCell = tileWidth/cellWidth;                  // Nombre de blocs  considrer dans une ligne de la tuile
+      int virtualTileWidth = tileWidth/cellWidth;
       
       for( int npixFile=0; npixFile<n; npixFile++ ) {
 //         System.out.println("Traitement de "+npixFile);
          pourcent += incrPourcent;
-         HealpixKey h = p.getHealpixLowLevel(fileOrder,npixFile,z,HealpixKey.SYNC);
+         if( p.moc!=null && p.isOutMoc(hipsOrder, npixFile) ) continue;
+         
+         HealpixKey h = p.getHealpixLowLevel(hipsOrder,npixFile,z,HealpixKey.SYNC);
          if( h==null ) continue;
          
-         long min = nsize * nsize * npixFile;
+         // Dans le cas o la taille de la tuile annonce ou suppose est fausse
+         // on recommence en indiquant explicitement cette taille
+         if( first && infoTileWidth==-1 && h.width!=CDSHealpix.pow2(tileOrder) ) {
+            if( aladin.levelTrace>=3) System.err.println("Warning, the HiPS \""+p.label+"\" has missing or erroneous tileWidth propertie value => assuming "+h.width);
+            addMocFromPlanBG(p1,order,h.width,pixMin,pixMax);
+            return;
+         }
+         first=false;
+         
+         long min = virtualTileWidth *virtualTileWidth * npixFile;
          try {
             int nb=0;
-            long oNpix=-1;  
-            for( int y=0; y<h.height; y++ ) {
-               for( int x=0; x<h.width; x++ ) {
-                  try {
-                     int idx = y * h.width + x;
-                     double pixel = h.getPixel(idx,HealpixKey.NOW);
-                     
-                     // Pixel vide
-                     if( Double.isNaN( pixel ) || pixel==blank ) continue;
-                     
-                     // En dehors de la plage
-                     if( flagRange ) {
-                        if( !Double.isNaN(pixMin) && pixel<pixMin ) continue;
-                        if( !Double.isNaN(pixMax) && pixel>pixMax ) continue;
+            
+            // Parcours des blocs
+            for( int yCell=0; yCell<nbCell; yCell++ ) {
+               for( int xCell=0; xCell<nbCell; xCell++ ) {
+                  
+                  // Parcours des pixels du bloc courant et dtermination de la moyenne de ses pixels
+                  double average = 0;
+                  int nbPixels=0;
+                  for( int y=0; y<cellWidth; y++ ) {
+                     for( int x=0; x<cellWidth; x++ ) {
+                        int idx = ((yCell*cellWidth)+y) * h.width + (xCell*cellWidth)+x;
+                        double pixel = h.getPixel(idx,HealpixKey.NOW);
+                        
+                        // Pixel vide
+                        if( Double.isNaN( pixel ) || isBlank && pixel==blank ) continue;
+                        
+                        pixel = pixel  * p.bScale+ p.bZero;
+                        
+                        average += pixel;
+                        nbPixels++;
                      }
-                     
-                     long npix = min + p.xy2hpx(idx);
-
-                     // Juste pour viter d'insrer 2x de suite le mme npix
-                     if( npix==oNpix ) continue;
-                     
-                     moc.add(order,npix>>>divOrder);
-                     oNpix=npixFile;
-                     nb++;
-                  } catch( Exception e ) { e.printStackTrace(); }
-               }
+                  }
+                  if( nbPixels==0 ) continue;
+                  average /= nbPixels;
+                  
+                  // En dehors de la plage
+                  if( flagRange ) {
+                     if( !Double.isNaN(pixMin) && average<pixMin ) continue;
+                     if( !Double.isNaN(pixMax) && average>pixMax ) continue;
+                  }
+                  
+                  
+                  long npixMoc = min + xy2hpx(yCell*virtualTileWidth + xCell);
+                  ((SMoc)moc).add(order,npixMoc);
+                  nb++;
+              }
             }
             
             // On remet immdiatement au propre le MOC uniquement si on a insr
-            // au-moins 1000 cellules (histoire de ne pas exploser la mmoire)
-            if( nb>10000 ) moc.checkAndFix();
+            // au-moins 100000 cellules (histoire de ne pas exploser la mmoire)
+            if( nb>100000 ) { ((SMoc)moc).checkAndFix(); nb=0; }
             
          } catch( Exception e ) {
             e.printStackTrace();
@@ -286,120 +347,213 @@ public class PlanMocGen extends PlanMoc {
          } catch( Exception e ) { e.printStackTrace(); }
       }
    }
+
    
    /** Creation d'un plan Moc  partir d'un HiPS en prenant toutes les pixels qui reprsentent
     * threshold (sommation) de la totalit des pixels. On commence par les valeurs les plus grandes.
     * Permet par exemple de crer un MOC  10% (threshold=0.1) pour des maps de probabilit
     * issues de l'tude des ondes gravitationnelles.
     */
-   private void addMocFromPlanBG(Plan p1, double threshold) throws Exception {
+   // Ajout d'un plan map Healpix au MOC en cours de construction
+   private void addMocFromPlanBG(Plan p1, int order, double threshold) throws Exception { addMocFromPlanBG(p1,order, -1,threshold); }
+   
+   private void addMocFromPlanBG(Plan p1, int order, int infoTileWidth, double threshold) throws Exception {
       PlanBG p = (PlanBG)p1;
+      if( order==-1 ) {
+         order= infoTileWidth==-1 ? p.getMaxHealpixOrder() : p.getMaxFileOrder() + (int)CDSHealpix.log2(infoTileWidth);
+         if( order>12 ) order=12;
+      }
       
-      order=p.getMaxHealpixOrder();
+      // Passage en mode FITS
+      if( !p.hasOriginalPixels() ) p.switchFormat();
       
-      // Dtermination de l'ordre pixel (o1) et tuiles (fileOrder)
-      int o1 = p.getTileOrder();
+      // Recherche de la taille d'une tuile si non explicitement mentionne
+      int tileOrder = infoTileWidth==-1 ? p.getTileOrder() : (int) CDSHealpix.log2(infoTileWidth);
+      int tileWidth = (int)CDSHealpix.pow2(tileOrder);
       int z = (int)p.getZ();
       
-      int divOrder=0;
-      int fileOrder = order - o1;
-      
-      // L'ordre des tuiles ne peut tre infrieur  3
-      if( fileOrder<3 ) {
-         divOrder=(3-fileOrder)*2;
-         fileOrder=3;
+      if( order> tileOrder+ p.maxOrder ) {
+         order = tileOrder+p.maxOrder;
+         aladin.warning("MOC order greater than HiPS resolution. Assuming MOC order "+order);
       }
       
-      /// L'ordre des tuiles ne peut entrainer le dpassement de la rsolution
-      // de la map HEALPix
-      if( fileOrder>p.getMaxFileOrder() ) {
-         fileOrder=p.getMaxFileOrder();
-         order = fileOrder+o1;
-      }
+      // Quel est l'ordre des tuiles requis
+      int requiredHipsOrder = order-tileOrder;
       
-      moc.setMaxLimitOrder(order);
+      // Par dfaut on suppose que cet order Hips est disponible
+      int hipsOrder = requiredHipsOrder;
+      
+      // Par dfaut on considre qu'un pixel de tuile correspond  un pixel du MOC
+      int cellOrder = 0;
+      
+      // Cet ordre HiPS est-il disponible ? et sinon quel est le niveau sup le plus proche
+      // Cela impactera le nombre de pixels  moyenner dans la tuile de dpart pour un pixel du MOC : 2^cellOrder = taille de ce bloc
+      int minHipsOrder = p.getMinOrder();
+      if( minHipsOrder>requiredHipsOrder ) {
+         cellOrder = minHipsOrder - requiredHipsOrder;
+         hipsOrder=minHipsOrder;
+      }
       
       // On gnre d'abord un MOC dans le systme de rfrence de la map HEALPix
       // on fera la conversion en ICRS  la fin du processus
-      moc.setCoordSys( p.frameOrigin==Localisation.GAL ? "G" : 
-                       p.frameOrigin==Localisation.ECLIPTIC ? "E" : "C");
+      ((SMoc)moc).setSys( p.frameOrigin==Localisation.GAL ? "G" : 
+                               p.frameOrigin==Localisation.ECLIPTIC ? "E" : "C");
       frameOrigin = p.frameOrigin;
+      try { moc.setCheckConsistencyFlag(false); } catch(Exception e) {}
       
       // Nombre de losanges  traiter
-      int n = (int)CDSHealpix.pow2(fileOrder); 
+      int n = (int)CDSHealpix.pow2(hipsOrder); 
       n=12*n*n;
       
-//      System.out.println("Nombre de losanges  traiter : "+n);
+      System.out.println("Nombre de losanges  traiter (order="+hipsOrder+") : "+n);
       
-      // Sans doute inutile car dj fait
-      try { p.createHealpixOrder(o1); } catch( Exception e1 ) { e1.printStackTrace(); }
-      long nsize = CDSHealpix.pow2(o1);
+      try { createHealpixOrder(tileOrder-cellOrder); }
+      catch( Exception e1 ) { }
       
       double incrPourcent = gapPourcent/n;
+      boolean first=true;
       
-      // Principe de l'algo: on parcours la map, pixel aprs pixel qu'on ajoute
-      //  la liste cumul en la triant immdiatement en ordre dcroissant, 
-      // tout en calculant la somme.
-      //Ds qu'on dpasse le threshold, l'insertion sera conditionne au fait qu'il faut
-      // que le nouveau pixel soit plus petit que la dernire valeur de la liste 
-      // Et si oui, on va l'insrer, mais virer autant des pixels les plus petits que ncessaire
-      ArrayList<PixCum> list;
-      try {
-         list = new ArrayList<PixCum>((int)(n*nsize*nsize));
-      } catch( Exception e1 ) {
-         throw new Exception("Sorry! too large probability sky map !");
-      }
+      int cellWidth = (int)CDSHealpix.pow2(cellOrder);   // taille d'un bloc (cot)
+      int nbCell = tileWidth/cellWidth;                  // Nombre de blocs  considrer dans une ligne de la tuile
+      int virtualTileWidth = tileWidth/cellWidth;
+      
+      // Liste des numros de tuiles  traiter. Dans la premire tape on se contentera
+      // de calculer la moyenne de l'ensemble des pixels concernes dans chaque tuile.
+      // Puis avant la deuxime tape, on triera ces numros de tuiles en commenant par celles
+      // qui ont la moyenne la plus leve (le but est de commencer par les valeurs
+      // les plus grandes pour viter de saturer "queue" (voir ci-dessous)
+      ArrayList<PixCum> tileList = new ArrayList<>(n);
+      
+      // La queue va servir dans la deuxime tape.
+      // On ne va garder que les pixels qui ont les plus grandes valeurs, et  chaque fois qu'on devra en insrer un nouveau, on virera
+      // autant de pixels de plus petites valeurs dont la somme est gale  la nouvelle valeur insr
+      Queue<PixCum> queue = new Queue<>( threshold );
+
+      // Somme totale dtermine  la premire tape, et qui permettra de normaliser les valeurs
+      // entre 0 et 1  la deuxime tape.
       double somme=0;
-      
-      for( int npixFile=0; npixFile<n; npixFile++ ) {
-         pourcent += incrPourcent;
-         HealpixKey h = p.getHealpixLowLevel(fileOrder,npixFile,z,HealpixKey.SYNC);
-         if( h==null ) continue;
+
+      for( int step=0; step<2; step++ ) {
          
-         long min = nsize * nsize * npixFile;
-         try {
-            for( int y=0; y<h.height; y++ ) {
-               for( int x=0; x<h.width; x++ ) {
-                  try {
-                     int idx = y * h.width + x;
-                     double pixel = h.getPixel(idx,HealpixKey.NOW);
-                     
-                     // Pixel vide
-                     if( Double.isNaN( pixel ) || pixel==blank ) continue;
-                     
-                     long npix = min + p.xy2hpx(idx);
-                     list.add( new PixCum(npix,pixel) );
-                     somme += pixel;
-                     
-                  } catch( Exception e ) {
-                     e.printStackTrace();
-                  }
+         // 1re tape, Initilisation de la liste des losanges =>  suit l'ordre naturel
+         if( step==0 ) {
+            for( int i=0; i<n; i++ ) tileList.add( new PixCum(i,0) );
+            
+         // 2me tape: Tri de la liste des losanges => suit l'odre dcroissant des moyennes
+         } else  Collections.sort(tileList);
+         
+         // Parcours des tuiles
+         for( PixCum tile : tileList ) {
+            if( tile.npix==-1 ) continue;   // c'est une tuile carte  l'tape 1
+            int tileNpix = (int) tile.npix;
+            tile.npix = -1;   // par dfaut, la tuile sera suppose rejete;
+            
+            aladin.trace(4,"Processing tile "+tileNpix+" (step "+(step+1)+
+                  (step==1 ? " mean="+tile.val+" max="+tile.max+" queue.size="+queue.size() : "")+")");
+            
+            double tileTotal = 0;  // Cumul des pixels de la tuile courante
+            double tileMax = 0;    // Valeur max sur les pixels de la tuile courante
+            int tileNbPixel = 0;   // Nomre de pixels de la tuile courante
+         
+            pourcent += incrPourcent/2;
+            if( p.moc!=null && p.isOutMoc(hipsOrder, tileNpix) ) continue;
+
+            HealpixKey h = p.getHealpixLowLevel(hipsOrder,tileNpix,z,HealpixKey.SYNC);
+            if( h==null ) continue;
+
+            // Dans le cas o la taille de la tuile annonce ou suppose est fausse
+            // on recommence en indiquant explicitement cette taille
+            if( first && infoTileWidth==-1 && h.width!=CDSHealpix.pow2(tileOrder) ) {
+               if( aladin.levelTrace>=3) System.err.println("Warning, the HiPS \""+p.label+"\" has missing or erroneous tileWidth propertie value => assuming "+h.width);
+               addMocFromPlanBG(p1,order, h.width, threshold );
+               return;
+            }
+            first=false;
+            
+            // Dans la deuxime tape, si le max de la tuile est infrieur  la plus petite
+            // valeur de queue, on peut tout de suite carter cette tuile
+            if( step==1 && queue.size()>0 ) {
+               PixCum last = queue.last();
+//               if( last.val>(tile.max/somme) ) {
+               if( last.val*somme>tile.max ) {
+//                  System.out.println(" tilemax="+(tile.max/somme)+" less that the min of queue="+last.val+" => rejected");
+                  continue;
                }
             }
-         } catch( Exception e ) {
-            e.printStackTrace();
+
+            long min = virtualTileWidth *virtualTileWidth * tileNpix;
+            try {
+               
+               // Parcours des blocs
+               for( int yCell=0; yCell<nbCell; yCell++ ) {
+                  for( int xCell=0; xCell<nbCell; xCell++ ) {
+
+                     // Parcours des pixels du bloc courant et dtermination de la moyenne de ses pixels
+                     double pixel = 0;
+                     int nbPixels=0;
+                     for( int y=0; y<cellWidth; y++ ) {
+                        for( int x=0; x<cellWidth; x++ ) {
+                           int idx = ((yCell*cellWidth)+y) * h.width + (xCell*cellWidth)+x;
+                           double localPixel = h.getPixel(idx,HealpixKey.NOW);
+
+                           // Pixel vide
+                           if( Double.isNaN( localPixel ) || isBlank && localPixel==blank ) continue;
+
+                           localPixel = localPixel  * p.bScale+ p.bZero;
+
+                           pixel += localPixel;
+                           nbPixels++;
+                        }
+                     }
+                     if( nbPixels==0 ) continue; // aucune valeur dans ce bloc
+//                     pixel /= nbPixels;        // moyenne de la valeur des pixels du bloc courant
+                     
+                     // Etape 1: mesures statistiques par tuile et globales
+                     if( step==0 ) {
+                        somme += pixel;
+                        tileTotal += pixel;
+                        if( tileNbPixel==0 || pixel>tileMax ) tileMax = pixel;
+                        tileNbPixel ++;
+                        
+                     // Etape 2: valeur ramene entre 0 et 1, et mmorisation ventuelle
+                     // dans la file des candidats
+                     } else  {
+                        pixel /= somme;
+                        long npixMoc = min + xy2hpx(yCell*virtualTileWidth + xCell);
+                        queue.add( new PixCum(npixMoc,pixel) );
+                     }
+                  }
+               }
+
+            } catch( Exception e ) {
+               e.printStackTrace();
+            }
+            
+            // Etape 1: la tuile sera-t-elle traite  l'tape 2 ? si oui, mmorisation de la moyenne
+            // de ses valeurs
+            if( step==0 && tileNbPixel>0 ) {
+               tile.npix = tileNpix;
+//               tile.val  = tileTotal / tileNbPixel;
+               tile.val  = tileTotal;
+               tile.max  = tileMax;
+            }
          }
       }
-      
-      Collections.sort(list);
-      
-      // Normalisation ventuelle
-      if( Math.abs(1-somme)>1e-8 ) {
-         for( PixCum pc : list ) pc.val/=somme;
-      }
-      
+
       somme=0;
       try {
          // Remplissage du Moc
          moc.setCheckConsistencyFlag(false);
          int nb=0;
-         for( PixCum pc : list ) {
+         Iterator<PixCum> it = queue.iterator();
+         while( it.hasNext() ) {
+            PixCum pc = it.next();
             long npix = pc.npix;
             somme += pc.val;
             if( somme>threshold ) break;
-            moc.add(order,npix>>>divOrder);
+            ((SMoc)moc).add(order,npix);
             nb++;
-            if( nb>10000 ) { moc.checkAndFix(); nb=0; }
+            if( nb>100000 ) { ((SMoc)moc).checkAndFix(); nb=0; }
          }
 
          // Conversion en ICRS si ncessaire
@@ -412,10 +566,52 @@ public class PlanMocGen extends PlanMoc {
       }
    }
    
+   public class Queue<E> extends TreeSet<PixCum> {
+
+      private double threshold;
+      private double somme;
+
+      public Queue(double threshold) {
+          super();
+          this.threshold = threshold;
+      }
+
+      public boolean add(PixCum e) {
+         if( somme<=0 || somme+e.val<threshold ) {
+            somme += e.val;
+            super.add(e);
+            return true;
+
+         } else {
+            double tot=0L;
+            while( true ) {
+               PixCum last = last();
+               if( last==null ) break;
+               tot += last.val;
+               somme -= last.val;
+               if( tot<e.val ) pollLast();
+               else break;
+            }
+            if( tot!=0 ) {
+               somme += e.val;
+               super.add(e);
+               return true;
+            }
+         }
+         return false;
+      }
+      
+      public String toString() {
+         PixCum e = first(), f = last();
+         return size()+"/"+threshold+"/"+somme+"["+e+".."+f+"]";
+      }
+   }
+
    // Pour grer un pixel Healpix
    class PixCum implements Comparable {
       long npix;   // indice dans la map
       double val;  // valeur du pixel
+      double max;
       
       PixCum(long npix,double val) {
          this.npix=npix;
@@ -429,8 +625,131 @@ public class PlanMocGen extends PlanMoc {
          }
          return val < ((PixCum)o).val ? 1 : -1;
       }
+      
+      public String toString() { return npix+"/"+val; }
    }
    
+
+   // ANCIENNE METHODE -> A JETER
+//   private void addMocFromPlanBG(Plan p1, double threshold) throws Exception {
+//      PlanBG p = (PlanBG)p1;
+//      
+//      order=p.getMaxHealpixOrder();
+//      
+//      // Dtermination de l'ordre pixel (o1) et tuiles (fileOrder)
+//      int o1 = p.getTileOrder();
+//      int z = (int)p.getZ();
+//      
+//      int divOrder=0;
+//      int fileOrder = order - o1;
+//      
+//      // L'ordre des tuiles ne peut tre infrieur  3
+//      if( fileOrder<3 ) {
+//         divOrder=(3-fileOrder)*2;
+//         fileOrder=3;
+//      }
+//      
+//      /// L'ordre des tuiles ne peut entrainer le dpassement de la rsolution
+//      // de la map HEALPix
+//      if( fileOrder>p.getMaxFileOrder() ) {
+//         fileOrder=p.getMaxFileOrder();
+//         order = fileOrder+o1;
+//      }
+//      
+//      moc.setMocOrder(order);
+//      
+//      // On gnre d'abord un MOC dans le systme de rfrence de la map HEALPix
+//      // on fera la conversion en ICRS  la fin du processus
+//      moc.setCoordSys( p.frameOrigin==Localisation.GAL ? "G" : 
+//                       p.frameOrigin==Localisation.ECLIPTIC ? "E" : "C");
+//      frameOrigin = p.frameOrigin;
+//      
+//      // Nombre de losanges  traiter
+//      int n = (int)CDSHealpix.pow2(fileOrder); 
+//      n=12*n*n;
+//      
+////      System.out.println("Nombre de losanges  traiter : "+n);
+//      
+//      // Sans doute inutile car dj fait
+//      try { p.createHealpixOrder(o1); } catch( Exception e1 ) { e1.printStackTrace(); }
+//      long nsize = CDSHealpix.pow2(o1);
+//      
+//      double incrPourcent = gapPourcent/n;
+//      
+//      // Principe de l'algo: on parcours la map, pixel aprs pixel qu'on ajoute
+//      //  la liste cumul en la triant immdiatement en ordre dcroissant, 
+//      // tout en calculant la somme.
+//      //Ds qu'on dpasse le threshold, l'insertion sera conditionne au fait qu'il faut
+//      // que le nouveau pixel soit plus petit que la dernire valeur de la liste 
+//      // Et si oui, on va l'insrer, mais virer autant des pixels les plus petits que ncessaire
+//      ArrayList<PixCum> list;
+//      try {
+//         list = new ArrayList<>((int)(n*nsize*nsize));
+//      } catch( Exception e1 ) {
+//         throw new Exception("Sorry! too large probability sky map !");
+//      }
+//      double somme=0;
+//      
+//      for( int npixFile=0; npixFile<n; npixFile++ ) {
+//         pourcent += incrPourcent;
+//         HealpixKey h = p.getHealpixLowLevel(fileOrder,npixFile,z,HealpixKey.SYNC);
+//         if( h==null ) continue;
+//         
+//         long min = nsize * nsize * npixFile;
+//         try {
+//            for( int y=0; y<h.height; y++ ) {
+//               for( int x=0; x<h.width; x++ ) {
+//                  try {
+//                     int idx = y * h.width + x;
+//                     double pixel = h.getPixel(idx,HealpixKey.NOW);
+//                     
+//                     // Pixel vide
+//                     if( Double.isNaN( pixel ) || pixel==blank ) continue;
+//                     
+//                     long npix = min + p.xy2hpx(idx);
+//                     list.add( new PixCum(npix,pixel) );
+//                     somme += pixel;
+//                     
+//                  } catch( Exception e ) {
+//                     e.printStackTrace();
+//                  }
+//               }
+//            }
+//         } catch( Exception e ) {
+//            e.printStackTrace();
+//         }
+//      }
+//      
+//      Collections.sort(list);
+//      
+//      // Normalisation ventuelle
+//      if( Math.abs(1-somme)>1e-8 ) {
+//         for( PixCum pc : list ) pc.val/=somme;
+//      }
+//      
+//      somme=0;
+//      try {
+//         // Remplissage du Moc
+//         moc.setCheckConsistencyFlag(false);
+//         int nb=0;
+//         for( PixCum pc : list ) {
+//            long npix = pc.npix;
+//            somme += pc.val;
+//            if( somme>threshold ) break;
+//            moc.add(order,npix>>>divOrder);
+//            nb++;
+//            if( nb>100000 ) { moc.checkAndFix(); nb=0; }
+//         }
+//
+//         // Conversion en ICRS si ncessaire
+//         moc.setCheckConsistencyFlag(true);
+//         moc=toReferenceFrame("C");
+//         frameOrigin=Localisation.ICRS;
+//         
+//      } catch( Exception e ) {
+//         e.printStackTrace();
+//      }
+//   }
    
 //   /** Creation d'un plan Moc  partir d'un HiPS en prenant tous les pixels dont les
 //    * valeurs sont comprises dans un interval (bornes incluses)
@@ -529,10 +848,10 @@ public class PlanMocGen extends PlanMoc {
    
    protected boolean waitForPlan() {
       try {
-         moc = new HealpixMoc();
-         moc.setMinLimitOrder(3);
-         if( order!=-1) moc.setMaxLimitOrder(order);
-         moc.setCoordSys("C");
+         moc = new SMoc();
+         ((SMoc)moc).setMinOrder(3);
+         if( order!=-1) moc.setMocOrder(order);
+         ((SMoc)moc).setSys("C");
          frameOrigin=Localisation.ICRS;
          moc.setCheckConsistencyFlag(false);
          for( Plan p1 : p ) {
@@ -541,7 +860,7 @@ public class PlanMocGen extends PlanMoc {
                else addMocFromCatalog(p1,radius,order);
             }
             else if( p1.isImage() ) addMocFromImage(p1,pixMin,pixMax);
-            else if( p1 instanceof PlanBG && !Double.isNaN(threshold) ) addMocFromPlanBG(p1,threshold);
+            else if( p1 instanceof PlanBG && !Double.isNaN(threshold) ) addMocFromPlanBG(p1,order,threshold);
             else if( p1 instanceof PlanBG ) addMocFromPlanBG(p1,order,pixMin,pixMax);
          }
          moc.setCheckConsistencyFlag(true);
