@@ -1,10 +1,10 @@
-// Copyright 1999-2018 - Universit de Strasbourg/CNRS
+// Copyright 1999-2020 - Universit de Strasbourg/CNRS
 // The Aladin Desktop program is developped by the Centre de Donnes
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
 //
-//This file is part of Aladin.
+//This file is part of Aladin Desktop.
 //
 //    Aladin Desktop is free software: you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 //    GNU General Public License for more details.
 //
 //    The GNU General Public License is available in COPYING file
-//    along with Aladin.
+//    along with Aladin Desktop.
 //
 
 package cds.aladin;
@@ -29,9 +29,9 @@ import java.awt.Polygon;
 import java.awt.Rectangle;
 import java.io.OutputStream;
 
-import cds.aladin.Hist.HistItem;
+import cds.aladin.ZoomHist.HistItem;
+import cds.tools.FastMath;
 import cds.tools.Util;
-import healpix.essentials.FastMath;
 
 /**
  * Manipulation d'un objet graphique affichable dans la vue
@@ -175,6 +175,13 @@ public class Position extends Obj {
       xv[vt.n]=xv[vs.n];
       yv[vt.n]=yv[vs.n];
    }
+   
+   /** Vrai si le temps associ  la position se trouve dans l'intervalle temps d'affichage
+    * de la vue */
+   protected boolean inTime(ViewSimple v) {
+      if( !plan.isTime() ) return true;
+      return v.inTime( jdtime );
+   }
 
    /** Positionne le flag VISIBLE */
    protected final void setVisible(boolean visible) {
@@ -184,7 +191,7 @@ public class Position extends Obj {
 
    /** Retourne true si la source a le flag VISIBLE positionn */
    final protected boolean isVisible() { return (flags & VISIBLE) !=0; /* == VISIBLE;*/ }
-
+   
    /** Positionne le flag LABEL */
    protected void setWithLabel(boolean withLabel) {
       if( withLabel ) flags |= WITHLABEL;
@@ -303,6 +310,7 @@ public class Position extends Obj {
 
       // Sinon on calcule les xy en fonction des coord. ra,de
       } else {
+         
          // Calcul initial d'un objet n'ayant des des positions xy natives
          if( raj==Double.NaN && xv[v.n]!=0 ) setCoord(v,proj);
 
@@ -330,6 +338,7 @@ public class Position extends Obj {
 
       xv[v.n] = x;
       yv[v.n] = y;
+      
       setCoord(v);
    }
 
@@ -498,6 +507,8 @@ public class Position extends Obj {
    */
    protected boolean draw(Graphics g,ViewSimple v,int dx, int dy) {
       if( !isVisible() ) return false;
+      if( !inTime(v) ) return false;
+      
       Point p = getViewCoord(v,0,0);
       if( p==null ) return false;
 
