@@ -1,10 +1,10 @@
-// Copyright 1999-2018 - Universit de Strasbourg/CNRS
+// Copyright 1999-2020 - Universit de Strasbourg/CNRS
 // The Aladin Desktop program is developped by the Centre de Donnes
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
 //
-//This file is part of Aladin.
+//This file is part of Aladin Desktop.
 //
 //    Aladin Desktop is free software: you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 //    GNU General Public License for more details.
 //
 //    The GNU General Public License is available in COPYING file
-//    along with Aladin.
+//    along with Aladin Desktop.
 //
 
 package cds.aladin;
@@ -63,6 +63,7 @@ import javax.swing.JScrollPane;
 import javax.swing.JSlider;
 import javax.swing.JTextArea;
 import javax.swing.JTextField;
+import javax.swing.SwingUtilities;
 import javax.swing.border.Border;
 import javax.swing.border.TitledBorder;
 import javax.swing.event.ChangeEvent;
@@ -72,6 +73,9 @@ import cds.aladin.bookmark.FrameBookmarks;
 import cds.aladin.prop.PropPanel;
 import cds.allsky.Constante;
 import cds.astro.Astrotime;
+import cds.moc.SMoc;
+import cds.moc.STMoc;
+import cds.moc.TMoc;
 import cds.tools.Astrodate;
 import cds.tools.Util;
 import cds.tools.pixtools.CDSHealpix;
@@ -418,95 +422,96 @@ public class Properties extends JFrame implements ActionListener, ChangeListener
 
       return p;
    }
+   
+   
+   private TimeField timeField=null;  // Voir createTimeField()
+   
+   /** Cration des deux champs temporaires sde saisie de dates, notamment pour les MOC */
+   protected void createTimeField() {
+      double [] t = aladin.calque.getDefaultTimeRange();
+      timeField = new TimeField(t[0],t[1]);
+      showProp(true);
+   }
+   
+   // Transforme le plan SMOC en STMOC avec un intervalle de temps par dfaut
+   private boolean moc2STmoc() {
+      try {
+         plan=aladin.calque.moc2STMoc( (PlanMoc)plan ,24, timeField.getJdmin(), timeField.getJdmax());
+         aladin.view.repaintAll();
+         SwingUtilities.invokeLater( new Runnable() {
+            public void run() {
+               showProp(true);
+               aladin.view.createView4TMOC(plan);
+            }
+         });
+      } catch( Exception e ) {
+         if( aladin.levelTrace>=3 ) e.printStackTrace();
+         return false;
+      }
+      return true;
+   }
+   
+   // Gre les deux textfields pour l'ajustement des dates d'une STMOC  un seul intervalle temporel
+   class TimeField extends JPanel implements KeyListener {
+      PlanSTMoc p;
+      JTextField start,end;
+      
+      // Objet temporaire qui sera utilis pour un MOC qu'on est entrain de transformer en STMOC
+      TimeField( double jdmin, double jdmax) {
+         setLayout( new GridLayout(2, 1) );
+         try {
+            start = new JTextField( Astrodate.JDToDate( jdmin ) );
+            add( start);
+            end= new JTextField( Astrodate.JDToDate( jdmax) );
+            add(end);
+         } catch( Exception e ) {
+            e.printStackTrace();
+         }
+      }
+      
+      // Objet de saisie d'un range de date pour un STMOC  un seul intervalle temporel
+      TimeField( PlanSTMoc p) {
+         this.p=p;
+         setLayout( new GridLayout(2, 1) );
+         try {
+            TMoc timeMoc =  ((STMoc) ((PlanMoc)p).moc).getTimeMoc();
+            start = new JTextField( Astrodate.JDToDate(timeMoc.getTimeMin()) );
+            start.addKeyListener( this );
+            add( start);
+            end= new JTextField( Astrodate.JDToDate(timeMoc.getTimeMax()) );
+            end.addKeyListener( this );
+            add(end);
+         } catch( Exception e ) {
+            e.printStackTrace();
+         }
+      }
 
-   //   private void showInfo() {
-   //      aladin.info(this,plan.verboseDescr);
-   //   }
-
-
-//   /** Genre un Label, ventuellement sur plusieurs lignes, qui peut avoir  la fin un lien (more...) pour de l'info
-//    * additionnel, ou une url complte associe. Les deux simultanment ne sont pas possibles.
-//    */
-//   class MyAnchor extends JLabel {
-//      String url,more;
-//      int width;
-//
-//      /**
-//       * @param text Texte du baratin (ou null si dbut du texte supplmentaire  afficher)
-//       * @param width nombre de caractres avant repli (-1 si pas de repli), ou csure si text==null
-//       * @param more texte supplmentaire accessible par (more...), null sinon
-//       * @param url url associe, null sinon
-//       */
-//      MyAnchor(String text,int width, String more,final String url) {
-//         super();
-//
-//         if( text==null && more==null && url!=null ) text=url;
-//
-//         if( text==null && more!=null ) {
-//            if( more.length()>width ) {
-//               int n = more.lastIndexOf(' ',width);
-//               if( n<=0 ) n=width;
-//               text=more.substring(0,n)+"...";
-//            }
-//            else { text=more; more=null; }
-//         }
-//         if( text==null ) text="";
-//         this.more = more;
-//         this.url=url;
-//         if( width>0 ) {
-//            if( (text.startsWith("http://") || text.startsWith("ftp://")) && text.length()>width ) text=text.substring(0,width)+"...";
-//            else {
-//               if( url!=null ) text = Util.fold(text,width,true);
-//               text = Util.fold(text,width);
-//            }
-//         }
-//         if( url!=null ) {
-//            text = "<html><A HREF=\"\">"+text+"</A></html>";
-//            setToolTipText(url);
-//         }
-//         if( more!=null ) text = "<html>"+text+" <A HREF=\"\">(more...)</A></html>";
-//         setText(text);
-//         setFont(getFont().deriveFont(Font.ITALIC));
-//         final String more1 = more;
-//         if( url!=null || more!=null ) {
-//            final Component c = this;
-//            addMouseMotionListener(new MouseMotionListener() {
-//               public void mouseMoved(MouseEvent e) { Aladin.makeCursor(c,Aladin.HANDCURSOR); }
-//               public void mouseDragged(MouseEvent e) { }
-//            });
-//            addMouseListener(new MouseListener() {
-//               public void mouseReleased(MouseEvent e) {
-//                  if( (e.getModifiers() & java.awt.event.InputEvent.BUTTON3_MASK) !=0 ) return;
-//                  if( url!=null ) aladin.glu.showDocument(url);
-//                  else aladin.info(c,more1.replace("\\n","\n"));
-//               }
-//               public void mousePressed(MouseEvent e)  { 
-//                  if( (e.getModifiers() & java.awt.event.InputEvent.BUTTON3_MASK) !=0 ) {
-//                     showPopMenu(e.getX(),e.getY());
-//                  }
-//               }
-//               public void mouseExited(MouseEvent e)   { Aladin.makeCursor(c,Aladin.DEFAULTCURSOR); }
-//               public void mouseEntered(MouseEvent e)  { }
-//               public void mouseClicked(MouseEvent e) { }
-//            });
-//         }
-//      }
-//      
-//      // Affiche le popup
-//      private void showPopMenu(int x,int y) {
-//         JPopupMenu popMenu = new JPopupMenu();
-//         popMenu.setLightWeightPopupEnabled(false);
-//         JMenuItem j=new JMenuItem(aladin.chaine.getString("MFCOPYURL"));
-//         popMenu.add(j);
-//         j.addActionListener( new ActionListener() {
-//            public void actionPerformed(ActionEvent e) {
-//               aladin.copyToClipBoard(url);
-//            }
-//         });
-//         popMenu.show(this,x,y);
-//      }
-//   }
-//
+      public void keyTyped(KeyEvent e) { }
+      public void keyPressed(KeyEvent e) { }
+      public void keyReleased(KeyEvent e) {
+         if( e.getKeyCode()==KeyEvent.VK_ENTER ) submit();
+      }
+      
+      double getJdmin() { return Astrodate.dateToJD( start.getText() ); }
+      double getJdmax() { return Astrodate.dateToJD( end.getText() ); }
+      
+      // Modification de l'intervalle associ au STMOC
+      void submit() {
+         boolean error=false;
+         double jdmin=Double.NaN;
+         double jdmax=Double.NaN;
+         try { jdmin = getJdmin(); } catch( Exception e) { start.setForeground( Color.red ); error=true;};
+         try { jdmax = getJdmax(); } catch( Exception e) { end.setForeground( Color.red ); error=true;};
+         if( error ) return;
+         try { p.changeTimeRange( jdmin, jdmax ); } catch( Exception e ) { e.printStackTrace(); }
+         start.setForeground( Color.black );
+         end.setForeground( Color.black );
+         aladin.calque.resetTimeRange();
+         aladin.view.repaintAll();
+     }
+               
+   }
+   
    /** Construction du panel des proprietes du plan courant.
     * @return Le panel des proprietes du plan courant
     */
@@ -620,6 +625,15 @@ public class Properties extends JFrame implements ActionListener, ChangeListener
             pscope.add( scopeLocal );
             pscope.add( b=new JButton(" ? ")); b.addActionListener(this);
             PropPanel.addCouple(p,SCOPE, pscope, g,c);
+            
+            if( hasCatalogInfo() ) {
+               b = new JButton(aladin.chaine.getString("PARSING"));
+               b.addActionListener(new ActionListener() {
+                  public void actionPerformed(ActionEvent e) { seeCatalogInfo(); }
+               });
+               PropPanel.addCouple(p,"", b, g,c);
+            }
+
          } else scope=null;
 
          // cas d'un PlanContour
@@ -745,14 +759,17 @@ public class Properties extends JFrame implements ActionListener, ChangeListener
 
          // Survey de l'image
          String survey=pimg.survey();
-         if( survey!=null && survey.length()>0 )
-            PropPanel.addCouple(p,INF, new JLabel(pimg.survey()), g,c);
+         if( survey!=null && survey.length()>0 && !survey.equals(pimg.getUrl()) ) {
+            PropPanel.addCouple(p,INF, new MyAnchor(aladin,survey,50,null,survey), g,c);
+         }
 
          // Format d'image
          JLabel fmtl = new JLabel(
                plan instanceof PlanHealpix ? "HEALPix map" :
-                  plan instanceof PlanMoc ? "Multi-Order Coverage map (MOC)" :
-                     plan instanceof PlanBG ? "HiPS" :
+               plan instanceof PlanSTMoc ?   "Space Time Multi-Order Coverage map (STMOC)" :
+               plan instanceof PlanTMoc ?    "Temporal Multi-Order Coverage map (TMOC)" :
+               plan instanceof PlanMoc ?     "Spatial Multi-Order Coverage map (MOC)" :
+               plan instanceof PlanBG ?      "Hierarchical Progressive Survey (HiPS)" :
                         PlanImage.describeFmtRes(pimg.dis,pimg.res));
 
          // Bouton de visualisation du header FITS
@@ -894,19 +911,29 @@ public class Properties extends JFrame implements ActionListener, ChangeListener
 
       if( plan.isCatalog() ) {
          Vector<Legende> legs = plan.getLegende();
+         
+         JPanel pt = new JPanel();
          b=new JButton(SEEPARSING);
          b.addActionListener(this);
+         pt.add(b);
+         if( hasCatalogInfo() ) {
+            b = new JButton(aladin.chaine.getString("PARSING"));
+            b.addActionListener(new ActionListener() {
+               public void actionPerformed(ActionEvent e) { seeCatalogInfo(); }
+            });
+            pt.add(b);
+         }
          int n = legs.size();
-         if( n==1 ) PropPanel.addCouple(p,TABLEINFO, b, g,c);
+         if( n==1 ) PropPanel.addCouple(p,TABLEINFO, pt, g,c);
          else {
             StringBuffer s = new StringBuffer("<html>");
             for( int i=0; i<legs.size(); i++ ) s.append((i>0?"<br>":"")+(i+1)+": "+legs.elementAt(i).name);
             s.append("</html>");
             PropPanel.addCouple(p,TABLEINFO, new JLabel(s.toString()), g,c);
-            PropPanel.addCouple(p,"", b, g,c);
+            PropPanel.addCouple(p,"", pt, g,c);
          }
       }
-
+      
       // Gestion des filtres prdfinis
       if( plan.filters!=null ) {
          PropPanel.addFilet(p,g,c);
@@ -1020,17 +1047,70 @@ public class Properties extends JFrame implements ActionListener, ChangeListener
          PropPanel.addCouple(p, "HEALPix NSide:",  new JLabel(CDSHealpix.pow2(res)+" (2^"+res+")"), g, c);
       }
 
-      if( plan.type==Plan.ALLSKYMOC ) {
+      if( plan.isMoc() ) {
          final PlanMoc pmoc = (PlanMoc)plan;
 //         final Frame frameProp = this;
-         double cov = pmoc.getMoc().getCoverage();
-         double degrad = Math.toDegrees(1.0);
-         double skyArea = 4.*Math.PI*degrad*degrad;
-//         final long mocSize = pmoc.getMoc().getSize();
-         PropPanel.addCouple(p,"Coverage: ",new JLabel(Util.round(cov*100, 3)+"% of sky => "+Coord.getUnit(skyArea*cov, false, true)+"^2"),g,c);
-         PropPanel.addCouple(p,"Best MOC ang.res: ",new JLabel(Coord.getUnit(pmoc.getMoc().getAngularRes())
-               +" (moc order="+pmoc.getMoc().getMocOrder()+")"),g,c);
-//         PropPanel.addCouple(p,"Size: ",new JLabel(mocSize+" cells - about "+Util.getUnitDisk(pmoc.getMoc().getMem())),g,c);
+         
+         if( plan.type==Plan.ALLSKYMOC ) {
+            double cov = ((SMoc)pmoc.getMoc()).getCoverage();
+            double degrad = Math.toDegrees(1.0);
+            double skyArea = 4.*Math.PI*degrad*degrad;
+            PropPanel.addCouple(p,"Coverage: ",new JLabel(Util.round(cov*100, 3)+"% of sky => "+Coord.getUnit(skyArea*cov, false, true)+"^2"),g,c);
+            PropPanel.addCouple(p,"Best ang.res: ",new JLabel(Coord.getUnit(((SMoc)pmoc.getMoc()).getAngularRes())
+                  +" (order="+pmoc.getMoc().getMocOrder()+")"),g,c);
+            
+            if( Aladin.BETA ) {
+               if( timeField!=null ) {
+                  PropPanel.addCouple(p,"Time : ", timeField, g,c);
+               } else {
+                  JButton t = new JButton("Range?");
+                  t.addActionListener(new ActionListener() {
+                     public void actionPerformed(ActionEvent e) { createTimeField();  }
+                  });
+                  PropPanel.addCouple(p,"Time",t,g,c);
+               }
+            }
+         }
+
+         else if( plan.type==Plan.ALLSKYTMOC ) {
+            TMoc moc = (TMoc) ((PlanTMoc)plan).moc;
+            long nbSec = moc.getNbCells();
+            int order = moc.getMocOrder();
+            PropPanel.addCouple(p,"Start", new JLabel( Astrodate.JDToDate( moc.getTimeMin()) ), g,c);
+            PropPanel.addCouple(p,"End", new JLabel( Astrodate.JDToDate( moc.getTimeMax()) ), g,c);
+            PropPanel.addCouple(p,"Accuracy", new JLabel(
+                  TMoc.getTemps(  TMoc.getDuration(moc.getMocOrder()))), g,c);
+//            PropPanel.addCouple(p,"Sum", new JLabel( Util.getTemps(nbSec*1000000, true) ), g,c);
+         }
+
+         else if( plan.type==Plan.ALLSKYSTMOC ) {
+            try {
+               SMoc spaceMoc =  ((STMoc) ((PlanMoc)plan).moc).getSpaceMoc();
+               double cov = spaceMoc.getCoverage();
+               double degrad = Math.toDegrees(1.0);
+               double skyArea = 4.*Math.PI*degrad*degrad;
+               PropPanel.addCouple(p,"Coverage: ",new JLabel(Util.round(cov*100, 3)+"% of sky => "+Coord.getUnit(skyArea*cov, false, true)+"^2"),g,c);
+               PropPanel.addCouple(p,"Best ang.res: ",new JLabel(Coord.getUnit(spaceMoc.getAngularRes())
+                     +" (order="+spaceMoc.getMocOrder()+")"),g,c);
+
+               TMoc timeMoc =  ((STMoc) ((PlanMoc)plan).moc).getTimeMoc();
+               long nbSec = timeMoc.getNbCells();
+               int order = timeMoc.getMocOrder();
+               if( ((PlanSTMoc)plan).isOneTimeRange() ) {
+                  PropPanel.addCouple(p,"Time : ", new TimeField((PlanSTMoc)plan), g,c);
+
+               } else {
+                  PropPanel.addCouple(p,"Start", new JLabel( Astrodate.JDToDate( timeMoc.getTimeMin()) ), g,c);
+                  PropPanel.addCouple(p,"End", new JLabel( Astrodate.JDToDate( timeMoc.getTimeMax()) ), g,c);
+                  PropPanel.addCouple(p,"Time Accuracy", new JLabel( 
+                        TMoc.getTemps(  TMoc.getDuration(timeMoc.getMocOrder()))), g,c);
+               }
+               
+            } catch( Exception e1 ) {
+               if( aladin.levelTrace>=3 ) e1.printStackTrace();
+            }
+         }
+
 
          final JCheckBox b1 = new JCheckBox("cell borders");
          final JCheckBox b3 = new JCheckBox("perimeter");
@@ -1094,7 +1174,7 @@ public class Properties extends JFrame implements ActionListener, ChangeListener
       //         PropPanel.addCouple(p,cat+" coverage map",p1,g,c);
       //      }
 
-      if( plan instanceof PlanBG ) {
+      if( plan instanceof PlanBG && !(plan instanceof PlanTMoc) ) {
          final PlanBG pbg = (PlanBG) plan;
          PropPanel.addCouple(p, "Coord.sys.:", new JLabel(pbg.getHipsFrame()), g, c);
 
@@ -1258,15 +1338,32 @@ public class Properties extends JFrame implements ActionListener, ChangeListener
                plong.setBorder( BorderFactory.createEmptyBorder());
                longitude = new ButtonGroup();
                longitudeDescending = new JRadioButton("descending");
-               longitudeDescending.addActionListener(this);
+               longitudeDescending.addActionListener( new ActionListener() {
+                  public void actionPerformed(ActionEvent e) {
+                     ((PlanBG)plan).setSpecificProj(true);
+                     plan.projd.setProjSym( !plan.projd.sym );
+                     aladin.view.newView();
+                     actionSpecificProj();
+                     aladin.calque.repaintAll();
+                  }
+               });
                longitude.add(longitudeDescending);
                longitudeAscending = new JRadioButton("ascending");
-               longitudeAscending.addActionListener(this);
+               longitudeAscending.addActionListener( new ActionListener() {
+                  public void actionPerformed(ActionEvent e) {
+                     ((PlanBG)plan).setSpecificProj(true);
+                     plan.projd.setProjSym( !plan.projd.sym );
+                     aladin.view.newView();
+                     actionSpecificProj();
+                     aladin.calque.repaintAll();
+                  }
+               });
+
                longitude.add(longitudeAscending); 
+               if( plan.projd.sym ) longitudeAscending.setSelected(true);
+               else  longitudeDescending.setSelected(true);
                plong.add( longitudeAscending );
                plong.add( longitudeDescending );
-               longitudeAscending.setSelected(plan.projd.sym);
-               longitudeDescending.setSelected(!plan.projd.sym);
                if( Aladin.BETA ) PropPanel.addCouple(p,".longitude", plong, g,c);
 
             }
@@ -1502,6 +1599,7 @@ public class Properties extends JFrame implements ActionListener, ChangeListener
                String s = ((JMenuItem)e.getSource()).getActionCommand();
                if( aladin.glu.setIndirectionByUrl(plan.gluTag, s) ) {
                   plan.url = s;
+                  plan.resetStats();
                   showProp(true);
                }
             }
@@ -1735,11 +1833,11 @@ public class Properties extends JFrame implements ActionListener, ChangeListener
    }
 
    // Changement de motif pour les sources
-   private void actionSourceType() {
-      if( sourceType==null ) return;
+   private boolean actionSourceType() {
+      if( sourceType==null ) return false;
       plan.setSourceType(sourceType.getSelectedIndex());
       aladin.calque.repaintAll();
-
+      return true;
    }
 
    // Changement de projection pour le calcul des alpha,delta
@@ -1814,6 +1912,20 @@ public class Properties extends JFrame implements ActionListener, ChangeListener
       String param1 = param.length()>0 ? param.toString() : null;
       fb.createNewBookmark(name,param1,"Load "+plan.label+(param1!=null?"":" on the view"), code);
    }
+   
+   private boolean hasCatalogInfo() {
+      try {
+         if( plan instanceof PlanBGCat ) return ((PlanBGCat)plan).getGenericPcat().hasCatalogInfo();
+         return plan.pcat.hasCatalogInfo();
+      } catch( Exception e ) {
+         return false;
+      }
+   }
+   
+   private void seeCatalogInfo() { 
+      if( plan instanceof PlanBGCat ) ((PlanBGCat)plan).getGenericPcat().seeCatalogInfo();
+      else plan.pcat.seeCatalogInfo();
+   }
 
    private void apply() {
       setFlagFullMaj(true);// Pour que toutes les fenetres soient maj par la suite
@@ -1825,16 +1937,6 @@ public class Properties extends JFrame implements ActionListener, ChangeListener
 
       if( blankField!=null ) actionBlank();
       
-      // Modification du sens de la longitude
-      // ATTENTION, DOIT NECESSAIREMENT ETRE AVANT actionDefCatProj() !!!
-      if( longitude!=null && plan.projd!=null ) {
-         boolean flagAsc = longitudeAscending.isSelected();
-         if( plan.projd.sym!=flagAsc ) {
-            plan.projd.setProjSym(flagAsc);
-            aladin.view.newView();
-         }
-      }
-      
       // Changement de projection catalogue par dfaut ?
       actionSpecificProj();
 
@@ -1940,8 +2042,7 @@ public class Properties extends JFrame implements ActionListener, ChangeListener
       }
 
 
-      actionSourceType();
-      aladin.calque.repaintAll();
+      if( ! actionSourceType() ) aladin.calque.repaintAll();
 
    }
 
@@ -1950,9 +2051,10 @@ public class Properties extends JFrame implements ActionListener, ChangeListener
       Object src = e.getSource();
       String what = src instanceof JButton ? ((JButton)src).getActionCommand() : "";
 
-      if( (src==longitudeDescending || src==longitudeAscending) && plan instanceof PlanBG ) {
-         apply();
-         return;
+      // Prise en compte d'un intervalle temporaire pour un MocSpatial
+      // => transformation en STMOC
+      if( timeField!=null ) {
+         if( moc2STmoc() ) timeField=null;
       }
       
       // Bouton d'dition d'un filtre ddi
@@ -2071,6 +2173,7 @@ public class Properties extends JFrame implements ActionListener, ChangeListener
 
    // Supprime la fenetre des Properties
    public void dispose() {
+      timeField=null;   // Si on tait en train de fournir un intervalle de temps, sans l'avoir valider
       frameProp.removeElement(this);
       super.dispose();
    }
