@@ -1,10 +1,10 @@
-// Copyright 1999-2018 - Universit de Strasbourg/CNRS
+// Copyright 1999-2020 - Universit de Strasbourg/CNRS
 // The Aladin Desktop program is developped by the Centre de Donnes
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
 //
-//This file is part of Aladin.
+//This file is part of Aladin Desktop.
 //
 //    Aladin Desktop is free software: you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 //    GNU General Public License for more details.
 //
 //    The GNU General Public License is available in COPYING file
-//    along with Aladin.
+//    along with Aladin Desktop.
 //
 
 package cds.aladin;
@@ -35,12 +35,12 @@ import java.util.Vector;
 
 import javax.swing.JTextField;
 
-import cds.aladin.Hist.HistItem;
+import cds.aladin.ZoomHist.HistItem;
 import cds.aladin.prop.Prop;
 import cds.aladin.prop.PropAction;
+import cds.tools.FastMath;
 import cds.tools.Util;
 import cds.tools.pixtools.CDSHealpix;
-import healpix.essentials.FastMath;
 
 
 /**
@@ -59,7 +59,6 @@ public class Repere extends Position {
    static final int ARROW  =3;  // Graphisme pour l'objet demand (flche)
    static final int CENTER =4;  // Graphisme pour le centre d'un FoV (croix)
    static final int ROTCENTER =5;  // Graphisme pour le centre de rotationd'un FoV (rond)
-   static final int CARTOUCHE  =6;  // Graphisme pour un cartouche
    protected int L = 5;         // demi-taille du repere
    private double radius;      // Rayon en degrs image du cercle englobant
    protected int dw,dh;         // mesure du label
@@ -83,6 +82,11 @@ public class Repere extends Position {
       super(plan,null,0,0,c.al,c.del,RADE,null);
    }
 
+   /** Creation d'un repere speciale de positionnement dans l'ecran */
+   protected Repere(Plan plan, ViewSimple v, Coord c) {
+      super(plan,v,0,0,c.al,c.del,RADE,null);
+   }
+
    /** Creation d'un repere graphique en connaissant RA/DE.
     * Cela signifie que ce repere est issu d'une capture de sources
     * @param plan plan d'appartenance de la ligne
@@ -195,24 +199,10 @@ public class Repere extends Position {
    }
 
    /** Determine le decalage pour ecrire l'id */
-   void setD() {
+   protected void setD() {
       FontMetrics m = Toolkit.getDefaultToolkit().getFontMetrics(DF);
       dw = m.stringWidth(id)+25;
       dh=  HF;
-      if( type==CARTOUCHE ) { 
-         try {
-            dw = m.stringWidth( getId(id) )+25;
-            int w = m.stringWidth( "Type : "+getType(id) )+25;
-            if( w>dw ) dw = w;
-            w = m.stringWidth( "Mag : "+getMag(id) )+25;
-            if( w>dw ) dw = w;
-            dh*=5.4; 
-         } catch( Exception e ) { 
-            int w = m.stringWidth("unknown by Simbad")+20;
-            if( w>dw ) dw = w;
-            dh*=3; 
-         }
-      }
    }
 
    /** Set specifical color (dedicated for catalog sources) */
@@ -272,8 +262,6 @@ public class Repere extends Position {
          double xc,yc;
          xc = xv[v.n];
          yc = yv[v.n];
-
-         if( type==CARTOUCHE ) return(xc<=x+l+dw/2 && xc>=x-l-dw/2 && yc<=y+l+dh/2 && yc>=y-l-dh/2);
          return(xc<=x+l && xc>=x-l && yc<=y+l && yc>=y-l);
       }
    }
@@ -345,8 +333,7 @@ public class Repere extends Position {
       if( type==TARGET || type==TARGETL ) couleur=Color.magenta.darker();
       if( couleur!=null ) return couleur;
 
-      if( type==CARTOUCHE ) couleur = Color.blue;
-      else if( type==ARROW ) couleur = Color.red;
+      if( type==ARROW ) couleur = Color.red;
       else if( plan!=null ) {
          if( plan.type==Plan.APERTURE ) couleur = ((PlanField)plan).getColor(this);
          else return plan.c;
@@ -412,25 +399,26 @@ public class Repere extends Position {
          try {
             PlanBG pbg = (PlanBG)v.pref;
             int orderFile = pbg.getOrder();
-            long nsideFile = CDSHealpix.pow2(orderFile);
+//            long nsideFile = CDSHealpix.pow2(orderFile);
             long nsideLosange = CDSHealpix.pow2(pbg.getTileOrder());
-            long nside = nsideFile * nsideLosange;
-            pixelSurf = CDSHealpix.pixRes(nside)/3600;
+//            long nside = nsideFile * nsideLosange;
+            int orderPix = orderFile + pbg.getTileOrder();
+            pixelSurf = CDSHealpix.pixRes(orderPix)/3600;
             pixelSurf *= pixelSurf;
             //            System.out.println("order="+CDSHealpix.log2(nside)+" => surf="
             //                  +Coord.getUnit(pixelSurf, false, true));
             Coord coo = new Coord(raj,dej);
             coo = Localisation.frameToFrame(coo,Localisation.ICRS,pbg.frameOrigin);
             double radiusRadian = Math.toRadians(getRadius());
-            long [] npix = CDSHealpix.query_disc(nside, coo.al, coo.del, radiusRadian, false);
+            long [] npix = CDSHealpix.query_discFXCenters(orderPix, coo.al, coo.del, radiusRadian);
             //            System.out.println("npix="+npix.length+" coo="+coo+" nside="+nside+" radius="+getRadius()+" nsideFile="+nsideFile+" nsideLosange="+nsideLosange);
             for( int i=0; i<npix.length; i++ ) {
                long npixFile = npix[i]/(nsideLosange*nsideLosange);
-               double pix = pbg.getHealpixPixel(orderFile,npixFile,npix[i],HealpixKey.NOW);
+               double pix = pbg.getHealpixPixel(orderFile,npixFile,npix[i],HealpixKey.SYNC);
                //               double pix = pbg.getHealpixPixel(orderFile,npixFile,npix[i],HealpixKey.ONLYIFDISKAVAIL);
                if( Double.isNaN(pix) ) continue;
                pix = pix*pbg.bScale+pbg.bZero;
-               double polar[] = CDSHealpix.pix2ang_nest(nside, npix[i]);
+               double polar[] = CDSHealpix.pix2ang_nest(orderPix, npix[i]);
                polar = CDSHealpix.polarToRadec(polar);
                coo.al = polar[0]; coo.del = polar[1];
                coo = Localisation.frameToFrame(coo,pbg.frameOrigin,Localisation.ICRS);
@@ -512,19 +500,20 @@ public class Repere extends Position {
          PlanBG pbg = (PlanBG)p;
          int orderFile = pbg.getOrder();
          //         if( pbg.maxOrder!=pbg.getOrder() ) return false;
-         long nsideFile = CDSHealpix.pow2(orderFile);
+//         long nsideFile = CDSHealpix.pow2(orderFile);
          long nsideLosange = CDSHealpix.pow2(pbg.getTileOrder());
-         long nside = nsideFile * nsideLosange;
-         pixelSurf = CDSHealpix.pixRes(nside)/3600;
+//         long nside = nsideFile * nsideLosange;
+         int orderPix = orderFile + pbg.getTileOrder();
+         pixelSurf = CDSHealpix.pixRes(orderPix)/3600;
          pixelSurf *= pixelSurf;
          Coord coo = new Coord(raj,dej);
          coo = Localisation.frameToFrame(coo,Localisation.ICRS,pbg.frameOrigin);
          double radiusRadian = Math.toRadians(getRadius());
-         long [] npix = CDSHealpix.query_disc(nside, coo.al, coo.del, radiusRadian, false);
+         long [] npix = CDSHealpix.query_disc(orderPix, coo.al, coo.del, radiusRadian, false);
          for( int i=0; i<npix.length; i++ ) {
             long npixFile = npix[i]/(nsideLosange*nsideLosange);
             //            double pix = pbg.getHealpixPixel(orderFile,npixFile,npix[i],HealpixKey.ONLYIFDISKAVAIL);
-            double pix = pbg.getHealpixPixel(orderFile,npixFile,npix[i],HealpixKey.NOW);
+            double pix = pbg.getHealpixPixel(orderFile,npixFile,npix[i],HealpixKey.SYNC);
             if( Double.isNaN(pix) ) continue;
             pix = pix*pbg.bScale+pbg.bZero;
             nombre++;
@@ -715,76 +704,11 @@ public class Repere extends Position {
             g.drawLine(p.x,   p.y-3,   p.x-3,   p.y-6);
             g.drawLine(p.x,   p.y-3,   p.x+3,   p.y-6);
             break;
-         case CARTOUCHE:
-            g.setColor(JAUNEPALE);
-            g.drawLine(p.x,   p.y-L, p.x,   p.y-3);
-            g.setColor(Color.black);
-            g.drawLine(p.x+1,   p.y-L, p.x+1,   p.y-3);
-            g.setColor(JAUNEPALE);
-            Util.drawCircle5(g, p.x, p.y);
-            g.setColor(Color.black);
-            Util.drawCircle7(g, p.x, p.y);
-            break;
-            //         case TARGET:
-            //            g.drawLine(p.x-L, p.y,   p.x-3, p.y);
-            //            g.drawLine(p.x+3, p.y,   p.x+L, p.y);
-            //            g.drawLine(p.x,   p.y-L, p.x,   p.y-3);
-            //            g.drawLine(p.x,   p.y+3, p.x,   p.y+L);
-            //            break;
       }
 
       if( isWithLabel() && !hasRayon() ) {
          if( id==null ) setId();
-         if( type==CARTOUCHE ) {
-//            Util.drawCartouche(g,p.x-dw/2,p.y-L-dh-1, dw-2, dh+3, 1f, Color.black,JAUNEPALE);
-//            g.setColor( getColor() );
-//            g.setFont(Aladin.SPLAIN);
-//            g.drawString(id,p.x-dw/2,p.y-L-1);
-            
-            g.setColor( CARTOUCHE_BACKGROUND );
-            g.fillRoundRect(p.x-dw/2,p.y-L-dh-1, dw-2, dh+3, 10, 10);
-//            g.setColor( CARTOUCHE_BACKGROUND.brighter().brighter() );
-//            g.drawRoundRect(p.x-dw/2,p.y-L-dh-1, dw-2, dh+3, 10, 10);
-            
-            try {
-               g.setColor( Aladin.COLOR_BLUE );
-               g.setFont(Aladin.BOLD);
-               int x=p.x-dw/2+5;
-               int y=p.y-4*L-6;
-               String s = getId(id);
-               g.drawString( s,x,y);
-               if( flagIn ) {
-                  int w = g.getFontMetrics().stringWidth(s);
-                  g.drawLine(x-1,y+3,x+w+2,y+3);
-                  g.drawLine(x-1,y+4,x+w+2,y+4);
-               }
-               
-               g.setFont(Aladin.ITALIC);
-               g.setColor( CARTOUCHE_FOREGROUND );
-               g.drawString( "Type: "+getType(id),p.x-dw/2+15,p.y-3*L-4);
-               g.drawString( "Mag : "+getMag(id),p.x-dw/2+15,p.y-2*L-4);
-               
-               g.setColor( CARTOUCHE_FOREGROUND.darker() );
-               s = "by Simbad";
-               g.setFont( g.getFont().deriveFont( g.getFont().getSize2D()-2));
-               x = p.x+dw/2-5 - g.getFontMetrics().stringWidth(s);
-               y = p.y-L-2;
-               g.drawString( s,x,y);
-               
-            } catch( Exception e ) {
-               g.setColor( CARTOUCHE_FOREGROUND );
-               g.drawString( id,p.x-dw/2+5,p.y-2*L-5);
-               g.setColor( CARTOUCHE_FOREGROUND.darker() );
-               g.setFont(Aladin.ITALIC);
-               g.setFont( g.getFont().deriveFont( g.getFont().getSize2D()-2));
-               String s = "unknown by Simbad";
-               int x = p.x+dw/2-5 - g.getFontMetrics().stringWidth(s);
-               int y = p.y-L-2;
-               g.drawString( s,x,y );
-            }
-            
-
-         } else g.drawString(id,p.x-dw/2,p.y-L-1);
+         g.drawString(id,p.x-dw/2,p.y-L-1);
       }
 
       if( isSelected()  ) {
