@@ -1,10 +1,10 @@
-// Copyright 1999-2018 - Universit de Strasbourg/CNRS
+// Copyright 1999-2020 - Universit de Strasbourg/CNRS
 // The Aladin Desktop program is developped by the Centre de Donnes
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
 //
-//This file is part of Aladin.
+//This file is part of Aladin Desktop.
 //
 //    Aladin Desktop is free software: you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 //    GNU General Public License for more details.
 //
 //    The GNU General Public License is available in COPYING file
-//    along with Aladin.
+//    along with Aladin Desktop.
 //
 
 package cds.aladin;
@@ -229,7 +229,7 @@ public class SED extends JPanel {
             Util.pause(10);
             MyInputStream inParam =  null;
             try {
-               inParam = Util.openAnyStream(url);
+               inParam = Util.openAnyStream(url,20000);
                plan.pcat.tableParsing(inParam, "TABLE");
                parseAndDraw();
             } catch( Exception e ) {
@@ -268,7 +268,7 @@ public class SED extends JPanel {
    
    // Procdure interne d'accs aux valeurs numriques du SED
    private double getSEDValue(Source s,int sed) {
-      Legende leg = s.leg;
+      Legende leg = s.getLeg();
       for( int i = 0; i<leg.field.length; i++ ) {
          if( leg.field[i].sed == sed ) {
             String val = s.getValue(i);
@@ -284,7 +284,7 @@ public class SED extends JPanel {
    
    /** Retourne l'identificateur du filtre associ  la source */
    protected String getSEDId(Source s) {
-      Legende leg = s.leg;
+      Legende leg = s.getLeg();
       String sId=null;
       for( int i = 0; i<leg.field.length; i++ ) {
          if( leg.field[i].sed == Field.SEDID ) sId = s.getValue(i);
@@ -294,7 +294,7 @@ public class SED extends JPanel {
    
    // Gnre la liste des points SED "prmachs" sous la forme d'un ArrayList de SEDItem
    private void createSEDlist(Iterator<?> it) {
-       if( sedList==null ) sedList = new ArrayList<SEDItem>();
+       if( sedList==null ) sedList = new ArrayList<>();
        while( it.hasNext() ) {
           Source s = (Source) it.next();
           double freq = getFreq(s);
@@ -734,7 +734,8 @@ public class SED extends JPanel {
 //      String target = URLEncoder.encode(source);
 //      target = target.replaceAll("[+]","%20");
 //      aladin.glu.showDocument("Widget.VizPhot", target+" "+radius);
-      aladin.glu.showDocument("Widget.VizPhot", Glu.quote(source)+" "+radius);
+//      aladin.glu.showDocument("Widget.VizPhot", Glu.quote(source)+" "+radius);
+      aladin.glu.showDocument("VizieR.sed.home", Glu.quote(source)+" "+radius);
    }
    
    // Affiche un message explicatif sur le SED
