@@ -1,10 +1,10 @@
-// Copyright 1999-2018 - Universit de Strasbourg/CNRS
+// Copyright 1999-2020 - Universit de Strasbourg/CNRS
 // The Aladin Desktop program is developped by the Centre de Donnes
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
 //
-//This file is part of Aladin.
+//This file is part of Aladin Desktop.
 //
 //    Aladin Desktop is free software: you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 //    GNU General Public License for more details.
 //
 //    The GNU General Public License is available in COPYING file
-//    along with Aladin.
+//    along with Aladin Desktop.
 //
 
 package cds.aladin;
@@ -489,13 +489,6 @@ Runnable, SwingWidgetFinder, Widget {
       popMenu.show(this,x-15,y);
    }
 
-   //  /** Retourne vrai si on est dans l'oeil.
-   //   * @param x,y Position de la souris
-   //   * @return <I>true</I> si on est dans l'oeil <I>false</I> sinon
-   //   */
-   //   protected boolean inEye(int x, int y) { return(x-gapL<eyeWidth && y<eyeHeight); }
-
-
    /** Generation si necessaire du message "Image in progress...".
     * @param p le plan a attendre
     * @return <I>true</I> si c'est pret <I>false</I> sinon
@@ -504,10 +497,6 @@ Runnable, SwingWidgetFinder, Widget {
       String s;
       if( p.type!=Plan.NO && p.type!=Plan.FILTER && !p.flagOk ) {
          if( p.error==null ) {
-            //            if( p.isImage() ) s=WAITMIN+"\n \n"+((PlanImage)p).getStatus();
-            //            else s=WAITMIN;
-            //            Aladin.warning(s);
-
             a.status.setText("*** "+WAITMIN+" ***");
          }
          return false;
@@ -516,6 +505,7 @@ Runnable, SwingWidgetFinder, Widget {
    }
 
    boolean memoBoutonDroit = false;
+   
    /** Gestion de la souris */
    public void mousePressed(MouseEvent e) {
       if( a.inHelp ) return;
@@ -539,27 +529,17 @@ Runnable, SwingWidgetFinder, Widget {
       // Fin du message d'accueil
       a.endMsg();
 
-
-      //      // Gestion de l'oeil (selection de tous les plans simultanement, ou avec
-      //      if( inEye(x,y) ) {
-      //         a.calque.clinDoeil();
-      //         a.view.repaintAll();
-      //         clinDoeil();
-      //      }
-
       // Recherche du plan clique
       if( (currentPlan = getPlan(y))==null ) return;
       
+      //set current plan in all tap guis by default of what is chosen on plan stack.
+      TapManager.getInstance(a).setCurrentUploadPlane(currentPlan.label);
+      
       // Par defaut, memorisation de la position
       this.x=x; this.y=y;
       oldy=y; oldx=x;
    }
 
-   //   /** Pour faire un clin d'oeil */
-   //   protected void clinDoeil() {
-   //      clinDoeil=true;
-   //      repaint();
-   //   }
 
    /** Permet de dterminer si le plan peut devenir de rfrence
     * comme si l'utilisateur avait cliqu sur le petit triangle */
@@ -587,7 +567,7 @@ Runnable, SwingWidgetFinder, Widget {
       
       // Un plan sans position ne peut tre pris comme rfrence
       if( currentPlan.hasNoPos ) return false;
-
+      
       // les plans FILTER et CONTOUR ne peuvent PAS etre pris comme rfrence
       if( currentPlan.type==Plan.FILTER
             || currentPlan instanceof PlanContour
@@ -937,7 +917,7 @@ Runnable, SwingWidgetFinder, Widget {
 
       boolean itsDone=false;
 
-      if( !canBeNewRef(e,x,p) ||  (!a.view.isMultiView() && p.ref)  ) newRef=null;
+      if( !canBeNewRef(e,x,p) || (!a.view.isMultiView() && p.ref)  ) newRef=null;
       else newRef = p;
 
       if( !itsDone && p.type!=Plan.NO ) {
@@ -1042,7 +1022,7 @@ Runnable, SwingWidgetFinder, Widget {
          // Slection de tous les objets du plan par double-clic
          if( x>gapL && !boutonDroit && e.getClickCount()==2 && (p.isCatalog() ||
                p instanceof PlanTool && !(p instanceof PlanContour) ) && p.active ) {
-            a.view.calque.selectAllObjectInPlans();
+            a.select();
 
             // On repasse en mode SELECT si ncessaire
             int i=a.toolBox.getTool();
@@ -1437,6 +1417,9 @@ Runnable, SwingWidgetFinder, Widget {
             : Aladin.COLOR_LABEL;
    }
    
+   /** Retourne le message courant */
+   protected String getMessage() { return message; }
+   
    /** Positionnement d'un message d'annonce CDS
     * FORMAT [ttt] [^en:]This is an <&http://aladin.fr|link>\\n...[^fr:Ceci est ....]
     * @param s
@@ -1486,6 +1469,7 @@ Runnable, SwingWidgetFinder, Widget {
     */
    protected void setMessageError(String s) {
       if( s!=null ) s=WARN+"\n \n"+s;
+      if( s.equals(message) ) return;
       message=s;
       messageType=MESSAGE_ERROR;
       messageKey=null;
@@ -1500,6 +1484,12 @@ Runnable, SwingWidgetFinder, Widget {
       repaint();
    }
    
+   /** arrte l'affichage du message courant s'il s'agit d'une erreur uniquement */
+   protected void hideMessageError() {
+      if( message==null || messageType!=MESSAGE_ERROR ) return;
+      hideMessage();
+   }
+   
    // Mise en forme du message CDS
    // Syntaxe d'entre [timeUnix] Message avec des \\n etc
    // Le message peut tre prcd d'un code de langue ^xx: (ex: ^en:), et dans ce cas il peut y avoir
@@ -1655,7 +1645,7 @@ Runnable, SwingWidgetFinder, Widget {
    
    private void resetUrl() { listUrl=null; }
    private void addUrl(String url, Rectangle r) {
-      if( listUrl==null ) listUrl = new ArrayList<ActionUrl>();
+      if( listUrl==null ) listUrl = new ArrayList<>();
       listUrl.add( new ActionUrl(url, r ) );
    }
    
@@ -1664,7 +1654,16 @@ Runnable, SwingWidgetFinder, Widget {
       if( listUrl==null ) return null;
       for( ActionUrl au : listUrl ) {
          if( au.rect.contains(x, y) ) {
-            if( au.url!=null && au.url.length()>0 ) a.glu.showDocument(au.url);
+            if( au.url!=null && au.url.length()>0 ) {
+               
+               // Une URL effectivement ?
+               if( au.url.startsWith("http") ) a.glu.showDocument(au.url);
+               
+               // En fait une commande script
+               else {
+                  a.execAsyncCommand( au.url);
+               }
+            }
             return au.url;
          }
       }
@@ -1674,15 +1673,19 @@ Runnable, SwingWidgetFinder, Widget {
    private boolean onUrl(int x, int y) {
       if( listUrl==null ) return false;
       for( ActionUrl au : listUrl ) {
-         if( au.rect.contains(x, y) ) return true;
+         if( au.rect.contains(x, y) ) {
+            a.urlStatus.setText(au.url);
+            return true;
+         }
       }
+      a.urlStatus.setText("");
       return false;
    }
    
    private int drawUrl(Graphics g,String s, String url,int x, int y) {
       if( s.length()==0 ) return x;
       
-      g.setColor( Aladin.COLOR_BLUE );
+      g.setColor( url.startsWith("http") ? Aladin.COLOR_BLUE : Aladin.COLOR_LABEL );
       g.drawString(s,x,y);
       int w = g.getFontMetrics().stringWidth(s);
       g.drawLine(x, y+2, x+w, y+2);
@@ -1941,7 +1944,10 @@ Runnable, SwingWidgetFinder, Widget {
       a.resumeVariousThinks();
 
       // Reaffichage du status du plan sous la souris
-      if( planIn!=null ) setInfo(planIn);
+      if( planIn!=null && planIn.type!=Plan.NO ) {
+         setInfo(planIn);
+         setMessageInfo( planIn );
+      }
 
       //Clignotement des voyants si besoin
       if( slideBlink ) startBlink();
