@@ -1,10 +1,10 @@
-// Copyright 1999-2018 - Universit de Strasbourg/CNRS
+// Copyright 1999-2020 - Universit de Strasbourg/CNRS
 // The Aladin Desktop program is developped by the Centre de Donnes
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
 //
-//This file is part of Aladin.
+//This file is part of Aladin Desktop.
 //
 //    Aladin Desktop is free software: you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 //    GNU General Public License for more details.
 //
 //    The GNU General Public License is available in COPYING file
-//    along with Aladin.
+//    along with Aladin Desktop.
 //
 
 package cds.aladin;
@@ -52,6 +52,7 @@ import java.util.Arrays;
 import java.util.Comparator;
 import java.util.Date;
 import java.util.Enumeration;
+import java.util.HashMap;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
@@ -81,7 +82,7 @@ import adql.parser.ADQLParser;
 import adql.parser.TokenMgrError;
 import adql.query.ADQLQuery;
 import adql.query.TextPosition;
-import cds.moc.HealpixMoc;
+import cds.moc.SMoc;
 import cds.tools.Astrodate;
 import cds.tools.ScientificUnitsUtil;
 import cds.tools.Util;
@@ -122,7 +123,7 @@ public class Server extends JPanel
 	protected String TARGET, RAD, /* GRABIT = "", */ DEFAULT_METHODE, TARGET_EX, RADIUS_EX, WNEEDOBJ, WNEEDRAD, WNEEDDATE,
 			WNEEDCAT, WERROR, WTOOLARGE, WERRORDATE, WDEJA, HASFILTER1, HASFILTER2, NOINPUTITEM, WNEEDCHECK, UNKNOWNOBJ,
 			NOTTOOMANY, DATEFORMATINCORRECT, BANDFORMATINCORRECT, TARGETOUTOFBOUNDSMESSAGE, CHECKQUERY_SUCCESS,
-			CHECKQUERY_ISBLANK;
+			CHECKQUERY_ISBLANK, INVALIDNUMBERINPUT;
 
    // Pour le positionnement des widgets en absolu
    static final int XTAB1=10;		// Abscisse des labels des champs  saisir
@@ -197,7 +198,7 @@ public class Server extends JPanel
    static final int IMGs   = 2;		// Input : URLs d'images (FITS)
    static final int CAT    = 4;		// Input : une URL de catalogue (VOTABLE)
    static final int CATs   = 8;		// Input : URLs de catalogues (VOTABLE)
-   static final int ALLIMG = 16;     // Input : une URL d'image qq soit son type (FITS)
+   static final int ALLIMG = 16;    // Input : une URL d'image qq soit son type (FITS)
 
    static int ORDRE=0;
 
@@ -289,7 +290,8 @@ public class Server extends JPanel
       TARGETOUTOFBOUNDSMESSAGE = aladin.chaine.getString("TARGETOUTOFBOUNDSMESSAGE");
       CHECKQUERY_ISBLANK = Aladin.chaine.getString("CHECKQUERY_ISBLANK");
       CHECKQUERY_SUCCESS = Aladin.chaine.getString("CHECKQUERY_SUCCESS");
-
+      INVALIDNUMBERINPUT = Aladin.chaine.getString("INVALIDNUMBERINPUT");
+      
       statusAllVO=new JLabel(" "); // Le status pour le mode ALLVO
 
    }
@@ -645,9 +647,6 @@ public class Server extends JPanel
 
             
             grab.setEnabled(false);
-            if (this.aladinLabel.equalsIgnoreCase(Constants.DATALINK_CUTOUT_FORMLABEL)) {//TODO:change this logic?
-            	grab.setEnabled(true);//deefault true for datalink forms
-			}
             p.add(grab);
          }
 
@@ -774,9 +773,9 @@ public void layout() {
      * @throws NumberFormatException
      * @throws Exception
      */
-    protected String isWithinBounds(HealpixMoc posBounds, List<Coord> rectVertices) throws NumberFormatException, Exception {
+    protected String isWithinBounds(SMoc posBounds, List<Coord> rectVertices) throws NumberFormatException, Exception {
     	String result = null;
-    	HealpixMoc userSpecified = null;
+    	SMoc userSpecified = null;
     	
 		if (posBounds!=null) {
     		String radiusInput  = radius.getText().trim();
@@ -792,10 +791,14 @@ public void layout() {
 			}
     		
     		if (userSpecified!=null) {
-    			userSpecified.toRangeSet();
-            	if (!posBounds.rangeSet.containsAll(userSpecified.rangeSet)) {
-    				result = TARGETOUTOFBOUNDSMESSAGE;
-    			}
+               if( aladin.levelTrace>=5) {
+                  aladin.execCommand("bounds= draw MOC "+posBounds.toASCII());
+                  aladin.execCommand("users= draw MOC "+userSpecified.toASCII());
+               }
+    		   
+    		   if( posBounds.intersection( userSpecified ).isEmpty() ) {
+                  result = TARGETOUTOFBOUNDSMESSAGE;
+    		   }
 			} else {
 				result = TARGETOUTOFBOUNDSMESSAGE;
 			}
@@ -996,20 +999,12 @@ public void layout() {
       aladin.dialog.setDefaultParameters(aladin.dialog.getCurrent(),5);
    }
 
-  /** Pre-remplissage du(des) champ target + activation eventuelle
-   * du bouton GrabIt
+  /** Pre-remplissage du(des) champ target
    * @param s La chaine a mettre dans le champ target
    */
    protected void setTarget(String s) {
       if( target==null || s.equals(aladin.GETOBJ) ) return;
       target.setText(s);
-
-      // Activation ou non du bouton GrabIt
-//      if( aladin.dialog!=null && !aladin.dialog.isGrabIt() && grab!=null ) {
-//         Plan pref = aladin.calque.getPlanRef();
-//         boolean grabEnable = pref!=null && Projection.isOk(pref.projd);
-//         grab.setEnabled(grabEnable);
-//      }
    }
 
    protected boolean updateWidgets() {
@@ -1106,10 +1101,11 @@ public void layout() {
     public StringBuffer setDateInMJDFormat(boolean replaceUserField, String input, String[] range, String outputDelimiter) throws Exception {
 		StringBuffer error = null;
 		StringBuffer processedText = null;
-		if(input != null && !input.isEmpty()) {
+		if(input != null && !input.trim().isEmpty()) {
 			processedText = new StringBuffer();
 			
 			String delimiterRegex = REGEX_TIME_RANGEINPUT;
+			input = input.replaceAll("\\s+"," ").trim();
 			String delimiter = getDelimiter(delimiterRegex, input);
 			String split = delimiter;
 			if (split.equals("..")) {
@@ -1119,10 +1115,10 @@ public void layout() {
 			}
 			String[] time = input.split(split);
 			Pattern p = Pattern.compile(REGEX_NUMBERNOEXP);
+			Pattern regexOpValue = Pattern.compile(REGEX_OPANYVAL);// find no special chars
 			for (int i = 0; i < time.length; i++) {
 				time[i] = time[i].trim();
 				String op = null;
-				Pattern regexOpValue = Pattern.compile(REGEX_OPANYVAL);// find no special chars
 				Matcher matcherOpValue = regexOpValue.matcher(time[i]);
 				if (matcherOpValue.find()) {
 					if (matcherOpValue.group("operator") != null) {
@@ -1198,9 +1194,10 @@ public void layout() {
 		StringBuffer result = null;
 		String op = null;
 		String valueToProcess = null;
-		if (input != null && !input.isEmpty()) {
+		if (input != null && !input.trim().isEmpty()) {
 			result = new StringBuffer();
 			String delimiterRegex = REGEX_BAND_RANGEINPUT;
+			input = input.trim().replaceAll("\\s+"," ");
 			String delimiter = getDelimiter(delimiterRegex, input);
 			String split = delimiter;
 			if (split != null && !split.isEmpty()) {
@@ -1211,21 +1208,22 @@ public void layout() {
 				}
 			}
 			String[] spectralBand = input.split(split);
+			String errorMessage = this.getNumberIncorrectMessage(input);
+			Pattern regexOpValue = Pattern.compile(REGEX_OPANYVAL);// find no special chars
 			for (int i = 0; i < spectralBand.length; i++) {
 				valueToProcess = spectralBand[i].trim();
-				Pattern regexOpValue = Pattern.compile(REGEX_OPANYVAL);// find no special chars
 				Matcher matcherOpValue = regexOpValue.matcher(valueToProcess);
 				if (matcherOpValue.find()) {
 					if (matcherOpValue.group("operator") != null) {
 						op = matcherOpValue.group("operator");
 						if (spectralBand.length > 1) {
-							throw new Exception(input+" is incorrect! Please rectify. Valid examples are: >3, 10..12, <=-788 etc..");
+							throw new Exception(errorMessage);
 						}
 					}
 					if (matcherOpValue.group("value") != null) {
 						valueToProcess = matcherOpValue.group("value");
 					} else {
-						throw new Exception(input+" is incorrect! Please rectify. Valid examples are: >3, 10..12, <=-788 etc..");
+						throw new Exception(errorMessage);
 					}
 				}
 //				op = TapClient.getValidOperatorNumber(spectralBand[i]);
@@ -1256,13 +1254,26 @@ public void layout() {
 		return result;
 	}
 	
+	public String getNumberIncorrectMessage(String input) {
+		String errorMessage = null;
+		try {
+			errorMessage = String.format(INVALIDNUMBERINPUT, input);
+		} catch (Exception e) {
+			// TODO: handle exception
+		}
+		if (errorMessage == null) {
+			errorMessage = input + " is incorrect! Please rectify. Valid examples are: >3, 10..12, <=-0.0788 etc...With no space/blank between the number and the operator.";
+		}
+		return errorMessage;
+	}
+	
 	/**
 	 * Extract delimiter or return default -1;
 	 * \\s*(?<delimiter>,|\\.\\.|\\s+\\band\\b\\s+|\\s+\\bAND\\b\\s+)\\s*
 	 * @param matcher
 	 * @return
 	 */
-	public String getDelimiter(String delimiterRegex, String input) {
+	public static String getDelimiter(String delimiterRegex, String input) {
 		String delimiter = null;
 		Pattern regex = Pattern.compile(delimiterRegex);
 		Matcher matcher = regex.matcher(input);
@@ -1320,8 +1331,8 @@ public void layout() {
 	}
 	
 	public static void appendRangeInvalidMessage(StringBuffer output, String[] range) {
-		String lowerLimitString = (isValidNumberRange(range[0])) ? EMPTYSTRING: range[0];
-		String upperLimitString = (isValidNumberRange(range[0])) ? EMPTYSTRING: range[1];
+		String lowerLimitString = (isValidNumberRange(range[0])) ? range[0]: EMPTYSTRING;
+		String upperLimitString = (isValidNumberRange(range[1])) ? range[1]: EMPTYSTRING;
 		if (!(lowerLimitString.isEmpty() && upperLimitString.isEmpty())) {
 			output.append(" within limits. Valid range: ").append(lowerLimitString);
 			output.append(RANGE_DELIMITER).append(upperLimitString);
@@ -1523,6 +1534,15 @@ public void layout() {
      protected boolean isBaseUrl(JComponent c) {
         return baseUrl==c;
      }
+     
+     
+     /** Retourne true si le Component pass en paramtre ne sert qu' reprer un champ qui
+      * recevra le suffixe d'une URL HiPS */
+     protected boolean isFieldHiPS(JComponent c ) {
+        return c instanceof JHiPS;
+     }
+     
+     class JHiPS extends JComponent { }
 
      /** Retourne true si le Component pass en paramtre est input */
      protected boolean isFieldInput(JComponent c) {
@@ -2061,7 +2081,7 @@ public void layout() {
     */
 	public void checkQueryFlagMessage() {
 		try {
-			if (this.checkQuery() != null) {
+			if (this.checkQuery(null) != null) {
 				Aladin.info(this, CHECKQUERY_SUCCESS);
 			}
 		} catch (UnresolvedIdentifiersException uie) {
@@ -2081,7 +2101,7 @@ public void layout() {
 	 * @return the adql query
 	 * @throws UnresolvedIdentifiersException 
 	 */
-	public ADQLQuery checkQuery() throws UnresolvedIdentifiersException {
+	public ADQLQuery checkQuery(Map<String, Object> requestParams) throws UnresolvedIdentifiersException {
 		if (tap.getText().isEmpty()) {
 			Aladin.error(this, CHECKQUERY_ISBLANK);
 			return null;
@@ -2111,10 +2131,11 @@ public void layout() {
 	 * @param name
 	 * @param url
 	 */
-	public void submitTapServerRequest(boolean sync, Map<String, Object> requestParams, String name, String url, String queryString) {
+	public void submitTapServerRequest(boolean sync, String name, String url, String queryString) {
 		ADQLQuery query = null;
+		Map<String, Object> requestParams = new HashMap<>();
 		try {
-			query = checkQuery();
+			query = checkQuery(requestParams);
 		} catch (UnresolvedIdentifiersException e) {
 			//error is handled in the respective checkQuery() methods
 			ADQLParser syntaxParser = new ADQLParser();
@@ -2133,9 +2154,9 @@ public void layout() {
 						/*+ "\n ADQLQuery: " + query.toADQL()*/ + "\n requestParams: " + requestParams);
 				if (sync) {
 					//Spec: Synchronous requests may issue a redirect to the result using HTTP code 303: See Other.
-					tapManager.fireSync(this, url, queryString, query, requestParams);
+					tapManager.fireSync(this, url, queryString, requestParams);
 				} else {
-					tapManager.fireASync(this, url, queryString, query, requestParams);
+					tapManager.fireASync(this, url, queryString, requestParams);
 				}
 				aladin.glu.log(TAP, "Submit query at "+this.getClass().getSimpleName());//log
 			} catch (Exception e) {
@@ -2146,6 +2167,11 @@ public void layout() {
 		}
 	}
 	
+	public void displayWarning(Server server, int requestNumber, String message) {
+		Aladin.error(aladin.dialog, message);
+		server.setStatusForCurrentRequest(requestNumber, Ball.NOK);
+	}
+	
 	public void highlightQueryError(Highlighter highlighter, adql.parser.ParseException pe) {
 		TextPosition errPosition = pe.getPosition();
 		if (errPosition != null) {
