@@ -1,10 +1,10 @@
-// Copyright 1999-2018 - Universit de Strasbourg/CNRS
+// Copyright 1999-2020 - Universit de Strasbourg/CNRS
 // The Aladin Desktop program is developped by the Centre de Donnes
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
 //
-//This file is part of Aladin.
+//This file is part of Aladin Desktop.
 //
 //    Aladin Desktop is free software: you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 //    GNU General Public License for more details.
 //
 //    The GNU General Public License is available in COPYING file
-//    along with Aladin.
+//    along with Aladin Desktop.
 //
 
 package cds.aladin;
@@ -329,7 +329,7 @@ DropTargetListener, DragSourceListener, DragGestureListener, GrabItFrame {
       Aladin.setIcon(this);
 
       int i;
-      Vector<Server> sv = new Vector<Server>(100); // Temporaire pour la creation de serveur[]
+      Vector<Server> sv = new Vector<>(100); // Temporaire pour la creation de serveur[]
       JPanel actions = new JPanel();
       createChaine();
       setTitle(TITLE);
@@ -355,7 +355,10 @@ DropTargetListener, DragSourceListener, DragGestureListener, GrabItFrame {
       status.setForeground( Color.blue );
 
       // Les serveurs Images par programme
-      if( Aladin.NETWORK ) sv.addElement(aladinServer = new ServerAladin(aladin));
+      if( Aladin.NETWORK ) { 
+         aladinServer = new ServerAladin(aladin);
+         if( Aladin.OLD ) sv.addElement(aladinServer);
+      }
 
       // Les serveurs Images via GLU
       if( Aladin.NETWORK ) addGluServer(sv, Glu.vGluServer, Server.IMAGE);
@@ -367,13 +370,16 @@ DropTargetListener, DragSourceListener, DragGestureListener, GrabItFrame {
          vizierServer = svizier = new ServerVizieR(aladin, this, actions);
 //         if( !Aladin.OUTREACH ) {
             sv.addElement(svizier);
+            if( !Aladin.OLD ) svizier.HIDDEN=true; 
+            
+            
 //               sv.addElement(vizierSurveys = new ServerVizieRSurvey(aladin,
 //                     ((ServerVizieR) svizier).vSurveys));
 //               sv.addElement(vizierArchives = new ServerVizieRMission(aladin,
 //                     ((ServerVizieR) svizier).vArchives));
             
             if( aladin.glu.get("Simbad")==null ) sv.addElement(new ServerSimbad(aladin));
-            if( aladin.glu.get("Ned")==null ) sv.addElement(new ServerNED(aladin));
+//            if( aladin.glu.get("Ned")==null ) sv.addElement(new ServerNED(aladin));
             
 //               sv.addElement(new ServerSWarp(aladin));
 //               sv.addElement(new ServerMocQuery(aladin));
@@ -858,17 +864,17 @@ DropTargetListener, DragSourceListener, DragGestureListener, GrabItFrame {
     * fonction du x,y de la souris
     * @param x,y Position dans la vue
     */
-   public void setGrabItCoord(double x, double y) {
-      ViewSimple v = aladin.view.getCurrentView();
-      Plan pr = v.pref;
-      if( pr == null ) return;
-      Projection proj = pr.projd;
-      if( proj == null ) return;
-      PointD p = v.getPosition(x, y);
-      Coord c = new Coord();
-      c.x = p.x;
-      c.y = p.y;
-      proj.getCoord(c);
+   public void setGrabItCoord(Coord c) { //double x, double y) {
+//      ViewSimple v = aladin.view.getCurrentView();
+//      Plan pr = v.pref;
+//      if( pr == null ) return;
+//      Projection proj = pr.projd;
+//      if( proj == null ) return;
+//      PointD p = v.getPosition(x, y);
+//      Coord c = new Coord();
+//      c.x = p.x;
+//      c.y = p.y;
+//      proj.getCoord(c);
       if(  Double.isNaN(c.al) ) return;
       server[current].setTarget(aladin.localisation.getFrameCoord(c.getSexa()));
    }
