@@ -1,10 +1,10 @@
-// Copyright 1999-2018 - Universit de Strasbourg/CNRS
+// Copyright 1999-2020 - Universit de Strasbourg/CNRS
 // The Aladin Desktop program is developped by the Centre de Donnes
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
 //
-//This file is part of Aladin.
+//This file is part of Aladin Desktop.
 //
 //    Aladin Desktop is free software: you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 //    GNU General Public License for more details.
 //
 //    The GNU General Public License is available in COPYING file
-//    along with Aladin.
+//    along with Aladin Desktop.
 //
 
 package cds.aladin;
@@ -43,6 +43,9 @@ import javax.swing.JScrollPane;
 import javax.swing.JTextField;
 
 import cds.allsky.Constante;
+import cds.moc.SMoc;
+import cds.moc.STMoc;
+import cds.moc.TMoc;
 import cds.tools.Util;
 import cds.xml.Field;
 import cds.xml.XMLConsumer;
@@ -236,8 +239,12 @@ public class ServerFile extends Server implements XMLConsumer {
     * @param f path du fichier
     * @param resNode noeud dcrivant le fichier  charger, peut tre <i>null</i>
     */
-   protected int creatLocalPlane(String f,String label,String origin, Obj o,
-         ResourceNode resNode,InputStream is,Server server,String target,String radius) {
+   protected int creatLocalPlane(String f, String label, String origin, Obj o, ResourceNode resNode, InputStream is,
+			Server server, String target, String radius) {
+	   return creatLocalPlane(f, label, origin, o, resNode, is, server, target, radius, null, null, -1);
+   }
+	protected int creatLocalPlane(String f, String label, String origin, Obj o, ResourceNode resNode, InputStream is,
+			Server server, String target, String radius, URL url, String query, int requestId) {
       String serverTaskId = aladin.synchroServer.start("ServerFile.creatLocalPlane/"+label);
       try {
          //         setSync(false);
@@ -334,7 +341,8 @@ public class ServerFile extends Server implements XMLConsumer {
             //Obtention du stream (donnee locale ou distante)
             if( is==null && (f.startsWith("http:")||f.startsWith("https:")) ) {
                u = aladin.glu.getURL("Http",getNameWithoutBrackets(f),true,true);
-               try { in = Util.openStream(u); } catch( Exception e ) { }
+               
+               try { in = Util.openStream(u); } catch( Exception e ) { if( aladin.levelTrace>=3 ) e.printStackTrace(); }
                mode="http";
             }
 
@@ -392,11 +400,11 @@ public class ServerFile extends Server implements XMLConsumer {
             else if( (type & MyInputStream.AJ)!=0) n=loadAJ(in)?1:0;
             else if( (type & MyInputStream.AJTOOL)!=0 ) n=loadTool(in,label)?1:0;
             else if( (type & MyInputStream.IDHA)!=0) n=updateMetaData(in,server,"",null)?1:0;
-//            else if( (type & MyInputStream.SIA_SSA)!=0)  n=updateMetaData(in,server,"",null)?1:0;
+ 
+            else if( (type & MyInputStream.SMOC)!=0 ) n=aladin.calque.newPlanMOC(in,label);
+            else if( (type & MyInputStream.TMOC)!=0 ) n=aladin.calque.newPlanTMOC(in,label);
+            else if( (type & MyInputStream.STMOC)!=0) n=aladin.calque.newPlanSTMOC(in,label);
 
-            else if( (type & MyInputStream.HPXMOC)!=0 ) {
-               n=aladin.calque.newPlanMOC(in,label);
-            }
             else if( (type & MyInputStream.FITS)!=0 && (type & MyInputStream.RGB)!=0 ) {
                if( u!=null ) {
                   n=aladin.calque.newPlanImageRGB(u,in,PlanImage.OTHER,
@@ -438,7 +446,7 @@ public class ServerFile extends Server implements XMLConsumer {
                   MyInputStream.CSV|MyInputStream.BSV|MyInputStream.IPAC))!=0 ) {
                if( u!=null ) n=aladin.calque.newPlanCatalog(u,in,label,"",f,null,server);
                else if( f!=null) n=aladin.calque.newPlanCatalog(f,in);
-               else n=aladin.calque.newPlanCatalog(in,label,origin);
+               else n=aladin.calque.newPlanCatalog(in,label,origin, server, url, query, requestId);
 
                // C'est peut tre une image native ?
             } else if( (type & MyInputStream.NativeImage())!=0 ) {
@@ -475,9 +483,10 @@ public class ServerFile extends Server implements XMLConsumer {
 
                // Cubes ?
                TreeObjDir gSky=null;
-               try { gSky = new TreeObjDir(aladin, f); }
-               catch( Exception e ) {
-                  aladin.trace(4, "ServerFile.creatLocalPlane(...) HiPS properties file not found => autodiscovery");
+               try {
+                  gSky = new TreeObjDir(aladin, f);
+               } catch( Exception e ) {
+                  aladin.trace(3, "ServerFile.creatLocalPlane(...) HiPS properties file not found => autodiscovery");
                }
                if( gSky!=null ) n=aladin.calque.newPlanBG(gSky,label,null,null);
 
@@ -720,6 +729,8 @@ public class ServerFile extends Server implements XMLConsumer {
       ViewMemoItem vmi = new ViewMemoItem();
       String s;
       if( (s=(String)atts.get("zoom"))!=null )       vmi.zoom=Double.valueOf(s).doubleValue();
+      if( (s=(String)atts.get("jdMin"))!=null )      vmi.jdmin=Double.valueOf(s).doubleValue();
+      if( (s=(String)atts.get("jdMax"))!=null )      vmi.jdmax=Double.valueOf(s).doubleValue();
       if( (s=(String)atts.get("xzoomView"))!=null )  vmi.xzoomView=Double.valueOf(s).doubleValue();
       if( (s=(String)atts.get("yzoomView"))!=null )  vmi.yzoomView=Double.valueOf(s).doubleValue();
       if( (s=(String)atts.get("rzoomWidth"))!=null ) vmi.rzoomWidth=Double.valueOf(s).doubleValue();
@@ -781,6 +792,12 @@ public class ServerFile extends Server implements XMLConsumer {
                ((PlanTool)plan).hasXYorig = (new Boolean(s)).booleanValue();
             }
             break;
+         case Plan.ALLSKYMOC:
+         case Plan.ALLSKYTMOC:
+         case Plan.ALLSKYSTMOC:
+         case Plan.ALLSKYIMG:
+         case Plan.ALLSKYCAT:
+         case Plan.ALLSKYCUBE:
          case Plan.IMAGEMOSAIC:
          case Plan.IMAGERSP:
          case Plan.IMAGEALGO:
@@ -788,6 +805,12 @@ public class ServerFile extends Server implements XMLConsumer {
          case Plan.IMAGE:
             boolean hasOrigPixel = atts.get("cacheID")!=null;
             if( typePlan==Plan.IMAGE ) plan = ( new PlanImage(aladin));
+            else if( typePlan==Plan.ALLSKYIMG )   plan = ( new PlanBG(aladin));
+            else if( typePlan==Plan.ALLSKYCAT )   plan = ( new PlanBGCat(aladin));
+            else if( typePlan==Plan.ALLSKYCUBE )  plan = ( new PlanBGCube(aladin));
+            else if( typePlan==Plan.ALLSKYMOC )   plan = ( new PlanMoc(aladin));
+            else if( typePlan==Plan.ALLSKYTMOC )  plan = ( new PlanTMoc(aladin));
+            else if( typePlan==Plan.ALLSKYSTMOC ) plan = ( new PlanSTMoc(aladin));
             else if( typePlan==Plan.IMAGEMOSAIC ) plan = ( new PlanImageMosaic(aladin));
             else if( typePlan==Plan.IMAGERSP )    plan = ( new PlanImageResamp(aladin));
             else if( typePlan==Plan.IMAGEALGO )   plan = ( new PlanImageAlgo(aladin));
@@ -795,6 +818,39 @@ public class ServerFile extends Server implements XMLConsumer {
                plan = ( new PlanImageRGB(aladin));
                ((PlanImageRGB)plan).RGBControl = new int[9];
             }
+            
+            if( plan instanceof PlanBG ) {
+               PlanBG pbg = (PlanBG)plan;
+               if( (s=(String)atts.get("hipsgluTag"))!=null )     pbg.gluTag=s;
+               if( (s=(String)atts.get("hipssurvey"))!=null )     pbg.survey=s;
+               if( (s=(String)atts.get("hipsurl"))!=null )        pbg.url=s;
+               if( (s=(String)atts.get("hipsminOrder"))!=null )   pbg.minOrder = Integer.parseInt(s);
+               if( (s=(String)atts.get("hipsmaxOrder"))!=null )   pbg.maxOrder = Integer.parseInt(s);
+               if( (s=(String)atts.get("hipscube"))!=null )       pbg.cube = (new Boolean(s)).booleanValue();
+               if( (s=(String)atts.get("hipscolor"))!=null )      pbg.color = (new Boolean(s)).booleanValue();
+               if( (s=(String)atts.get("hipscolorPNG"))!=null )   pbg.colorPNG = (new Boolean(s)).booleanValue();
+               if( (s=(String)atts.get("hipscolorUnknown"))!=null )  pbg.colorUnknown = (new Boolean(s)).booleanValue();
+               if( (s=(String)atts.get("hipsfitsGzipped"))!=null )   pbg.fitsGzipped = (new Boolean(s)).booleanValue();
+               if( (s=(String)atts.get("hipstruePixels"))!=null )    pbg.truePixels = (new Boolean(s)).booleanValue();
+               if( (s=(String)atts.get("hipsinFits"))!=null )    pbg.inFits = (new Boolean(s)).booleanValue();
+               if( (s=(String)atts.get("hipsinJPEG"))!=null )    pbg.inJPEG = (new Boolean(s)).booleanValue();
+               if( (s=(String)atts.get("hipsinPNG"))!=null )     pbg.inPNG = (new Boolean(s)).booleanValue();
+               if( (s=(String)atts.get("hipshasMoc"))!=null )    pbg.hasMoc = (new Boolean(s)).booleanValue();
+               if( (s=(String)atts.get("hipshasHpxFinder"))!=null )   pbg.hasHpxFinder = (new Boolean(s)).booleanValue();
+               if( (s=(String)atts.get("hipsbody"))!=null )      pbg.body = s;
+               if( (s=(String)atts.get("hipsframeOrigin"))!=null )   pbg.frameOrigin = Integer.parseInt(s);
+               if( (s=(String)atts.get("hipsframeDrawing"))!=null )  pbg.frameDrawing = Integer.parseInt(s);
+               if( (s=(String)atts.get("hipslive"))!=null )      pbg.live = (new Boolean(s)).booleanValue();
+               if( (s=(String)atts.get("hipspixelCut"))!=null )  pbg.pixelCut = s;
+               if( (s=(String)atts.get("hipstransferFct4Fits"))!=null )     pbg.transferFct4Fits = Integer.parseInt(s);
+               if( (s=(String)atts.get("hipstransferFct4Preview"))!=null )  pbg.transferFct4Preview = Integer.parseInt(s);
+               if( (s=(String)atts.get("hipstrahipstileOrdernsferFct4Preview"))!=null )  pbg.tileOrder = Integer.parseInt(s);
+
+               if( plan instanceof PlanBGCube ) {
+                  if( (s=(String)atts.get("hipsdepth"))!=null )  ((PlanBGCube)pbg).depth = Integer.parseInt(s);
+               }
+            }
+            
             if( (s=(String)atts.get("object"))!=null )     plan.objet = s;
             PlanImage pi = (PlanImage)plan;
             if( (s=(String)atts.get("param"))!=null )      plan.param = s+".";
@@ -809,6 +865,7 @@ public class ServerFile extends Server implements XMLConsumer {
             if( (s=(String)atts.get("RA"))!=null )     ra=Double.valueOf(s).doubleValue();
             if( (s=(String)atts.get("DE"))!=null )     de=Double.valueOf(s).doubleValue();
             if( (s=(String)atts.get("radius"))!=null ) rm=Double.valueOf(s).doubleValue();
+            if( (s=(String)atts.get("color"))!=null )  plan.c=Action.getColor(s);
             pi.transfertFct=PlanImage.LINEAR;
             pi.bScale=1.0;
             pi.bitpix=8;
@@ -1043,7 +1100,7 @@ public class ServerFile extends Server implements XMLConsumer {
                pi.cm=CanvasColorMap.getCM(pi.cmControl[0],pi.cmControl[1],pi.cmControl[2],
                      pi.video==PlanImage.VIDEO_INVERSE,
                      pi.typeCM,
-                     pi.transfertFct);
+                     pi.getTransfertFct());
                pi.calculPixelsZoom();
                pi.changeImgID();
             }
@@ -1060,7 +1117,15 @@ public class ServerFile extends Server implements XMLConsumer {
          aladin.view.scrollOn(firstView,0,1);
       } else if( name.equals("SCRIPT") )  { inFilterScript =false;
       } else if( name.equals("TABLE") )   { leg=null; vField=null; if( flagCatalogSource ) typePlan=CATALOGTOOL;
-      } else if( name.equals("VALUE") )   { inValue=false;
+      } else if( name.equals("VALUE") )   {
+         inValue=false;
+         if( plan instanceof PlanMoc ) {
+            try {
+               ((PlanMoc)plan).moc.read( inMoc.getInputStream() );
+            } catch( Exception e ) {
+               e.printStackTrace();
+            }
+         }
       } else if( name.equals("ALADINJAVA") ) {
 
          // mise  jour des diffrents filtres mmoriss
@@ -1128,7 +1193,7 @@ public class ServerFile extends Server implements XMLConsumer {
 
       return ch[cur]=='\t'?cur+1:cur;
    }
-
+   
    /** Cherche les infos de la source courante et ajoute l'objet correspondant
     * au plan courant
     * utilise la variable de classe rec
@@ -1223,8 +1288,6 @@ public class ServerFile extends Server implements XMLConsumer {
       if( o!=null ) {
          o.raj=ra; o.dej=de;
          o.x=x;    o.y=y;
-         o.setWithLabel(withlabel);
-         o.setSpecificAJInfo(id);
 
          // Suivi de ligne
          try {
@@ -1235,10 +1298,11 @@ public class ServerFile extends Server implements XMLConsumer {
          } catch( Exception e ) { }
 
          // Fin d'un polygone
-         try {
-            if( flagSpecial ) ((Ligne)o).bout=3;
-         } catch( Exception e ) { }
+         try { if( flagSpecial ) ((Ligne)o).bout=3; } catch( Exception e ) { }
 
+         try { o.setWithLabel(withlabel); } catch( Exception e ) {  }
+         o.setSpecificAJInfo(id);
+         
          // Enregistrement de l'objet dans le plan
          plan.pcat.setObjetFast(o);
 
@@ -1249,6 +1313,28 @@ public class ServerFile extends Server implements XMLConsumer {
 
       return cur;
    }
+   
+   
+   private MyByteArrayStream inMoc = null;  // Contenu du MOC courant
+   
+   /** Lecture du MOC ASCII, et l'associe au plan courant. C'est procedure peut tre appelee plusieurs
+    * fois en sequence.
+    * @param ch
+    * @param cur
+    * @param length
+    * @throws Exception
+    */
+   private void getMoc(char [] ch, int cur, int length) {
+      if( ((PlanMoc)plan).moc==null ) {
+         ((PlanMoc)plan).moc = plan instanceof PlanSTMoc ? new STMoc()
+            : plan instanceof PlanTMoc  ? new TMoc()
+            : new SMoc();
+         inMoc =  new MyByteArrayStream( length );
+      }
+      inMoc.write( new String( ch, cur,length) );
+   }
+
+
 
    /** Cherche les pixels RGB de l'image et l'associe au plan courant.
     * Cette procedure peut etre appelee plusieurs fois en sequence.
@@ -1352,6 +1438,11 @@ public class ServerFile extends Server implements XMLConsumer {
                }
                prevFlagSuite=false; // pour ne pas accoler deux objets
                break;
+            case Plan.ALLSKYMOC:
+            case Plan.ALLSKYTMOC:
+            case Plan.ALLSKYSTMOC:
+               getMoc(ch,start,length);
+               break;
             case Plan.IMAGE:
             case Plan.IMAGEHUGE:
             case Plan.IMAGEMOSAIC:
