@@ -1,10 +1,10 @@
-// Copyright 1999-2018 - Universit de Strasbourg/CNRS
+// Copyright 1999-2020 - Universit de Strasbourg/CNRS
 // The Aladin Desktop program is developped by the Centre de Donnes
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
 //
-//This file is part of Aladin.
+//This file is part of Aladin Desktop.
 //
 //    Aladin Desktop is free software: you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 //    GNU General Public License for more details.
 //
 //    The GNU General Public License is available in COPYING file
-//    along with Aladin.
+//    along with Aladin Desktop.
 //
 
 package cds.aladin;
@@ -24,6 +24,9 @@ package cds.aladin;
 import static cds.aladin.Constants.CHANGESERVER;
 import static cds.aladin.Constants.CHECKQUERY;
 import static cds.aladin.Constants.COMMA_CHAR;
+import static cds.aladin.Constants.DATALINK_FORM;
+import static cds.aladin.Constants.DATALINK_FORM_ASYNC;
+import static cds.aladin.Constants.DATALINK_FORM_SYNC;
 import static cds.aladin.Constants.EMPTYSTRING;
 import static cds.aladin.Constants.GETRESULTPARAMS;
 import static cds.aladin.Constants.LASTPANEL;
@@ -43,6 +46,8 @@ import static cds.aladin.Constants.UTF8;
 import java.awt.Component;
 import java.awt.Dimension;
 import java.awt.event.ActionEvent;
+import java.awt.event.ItemEvent;
+import java.awt.event.ItemListener;
 import java.io.File;
 import java.io.InputStream;
 import java.io.UnsupportedEncodingException;
@@ -61,6 +66,7 @@ import java.util.Map;
 import java.util.NoSuchElementException;
 import java.util.StringTokenizer;
 import java.util.Vector;
+import java.util.concurrent.RejectedExecutionException;
 
 import javax.swing.JButton;
 import javax.swing.JComboBox;
@@ -81,7 +87,7 @@ import adql.parser.ADQLParser;
 import adql.query.ADQLQuery;
 import cds.aladin.stc.STCObj;
 import cds.aladin.stc.STCStringParser;
-import cds.moc.HealpixMoc;
+import cds.moc.SMoc;
 import cds.tools.Util;
 
 /**
@@ -97,9 +103,10 @@ public class ServerGlu extends Server implements Runnable {
    private boolean flagSIAIDHA=false;
    private boolean flagTAP=false;
    private boolean flagTAPV2=false;
+   private boolean flagHiPS=false;
    int fmt;		// Format de retour (PlanImage.fmt)
-   String info1, /* info2, */filter, PARSEMJDFIELDHINT, GENERICERROR, CHECKQUERYTOOLTIP, SYNCASYNCTOOLTIP,
-			SHOWASYNCTOOLTIP;
+   String info1, /* info2, */filter, PARSEMJDFIELDHINT, GENERICERROR, CHECKQUERYTOOLTIP, SYNCASYNCTOOLTIP, SYNCASYNCSELECTIONCHANGE,
+			SHOWASYNCTOOLTIP, ASYNCTOOLTIP;
    String system;       // appel systme dans le cas d'un enregistrement concernant une application locale, null sinon
    String dir;          // Rpertoire d'excution du system, null sinon
    StringBuffer record;
@@ -115,12 +122,12 @@ public class ServerGlu extends Server implements Runnable {
 
    protected int lastY;
    
-   HashMap<Integer, String[]> rangeValues = new HashMap<Integer, String[]>();
+   HashMap<Integer, String[]> rangeValues = new HashMap<>();
    
    private Source dataLinkSource = null;
-   private HealpixMoc posBounds = null;
+   private SMoc posBounds = null;
    private String boundaryAreaStcs = null;
-   private Map<String, Vector> tapTableMapping = new HashMap<String, Vector>();
+   private Map<String, Vector> tapTableMapping = new HashMap<>();
    private Map<String, GluAdqlTemplate> gluAdqlQueryTemplates;
    private String currentSelectedTapTable;
    private Hashtable<String, String> adqlFunc = null;
@@ -128,6 +135,10 @@ public class ServerGlu extends Server implements Runnable {
    JComboBox sync_async;
    String LISTDELIMITER = SPACESTRING;
    private boolean cleanupFov;
+	// -1 - do not show this gui, 0- show disabled drop-down with async, 
+	// 1- show drop-down with sync selected,
+	// 2-show drop-down with async selected
+   int showSyncAsync = -1;
 
    protected void createChaine() {
       super.createChaine();
@@ -141,6 +152,8 @@ public class ServerGlu extends Server implements Runnable {
       CHECKQUERYTOOLTIP = Aladin.chaine.getString("CHECKQUERYTOOLTIP");
       SYNCASYNCTOOLTIP = Aladin.chaine.getString("SYNCASYNCTOOLTIP");
       SHOWASYNCTOOLTIP = Aladin.chaine.getString("SHOWASYNCTOOLTIP");
+      SYNCASYNCSELECTIONCHANGE = Aladin.chaine.getString("SYNCASYNCSELECTIONCHANGE");
+      ASYNCTOOLTIP = Aladin.chaine.getString("ASYNCTOOLTIP");
    }
 
    /**
@@ -156,10 +169,10 @@ public class ServerGlu extends Server implements Runnable {
     */
 	protected ServerGlu(Aladin aladin, String actionName, String description, String verboseDescr, String aladinMenu,
 			String aladinMenuNumber, String aladinLabel, String planeLabel, String docUser, String[] paramDescription,
-			String[] paramDataType, String[] paramValue, String[][] paramRange, String resultDataType, String institute,
-			String[] aladinFilter, String aladinLogo, String dir, String system, StringBuffer record,
-			String aladinProtocol, String[] tapTables, GluAdqlTemplate gluAdqlTemplate, TapClient tapClient,
-			boolean flagTapUpload) {
+			String[] paramDataType, String[] paramValue, String[][] paramRange, String resultDataType,
+			int showSyncAsync, String institute, String[] aladinFilter, String aladinLogo, String dir,
+			String system, StringBuffer record, String aladinProtocol, String[] tapTables,
+			GluAdqlTemplate gluAdqlTemplate, TapClient tapClient, boolean flagTapUpload) {
 
       this.aladin = aladin;
       createChaine();
@@ -178,6 +191,7 @@ public class ServerGlu extends Server implements Runnable {
       this.ordre       = aladinMenuNumber;
       this.system      = system;
       this.dir         = dir;
+      
       JScrollPane sc=null;
 //      try { this.aladinMenuNumber=Double.parseDouble(aladinMenuNumber); } catch( Exception e) { }
 
@@ -207,7 +221,8 @@ public class ServerGlu extends Server implements Runnable {
       DISCOVERY=flagSIAIDHA || type==SPECTRUM || type==CATALOG;
 
       fmt = getFmt(resultDataType);
-
+      this.showSyncAsync = showSyncAsync;
+      
       setLayout(null);
       setFont(Aladin.PLAIN);
 
@@ -220,7 +235,7 @@ public class ServerGlu extends Server implements Runnable {
 			LISTDELIMITER = COMMA_CHAR;
 			this.tapClient = tapClient;
 			this.tapClient.serverGlu = this;
-			this.gluAdqlQueryTemplates = new HashMap<String, GluAdqlTemplate>();
+			this.gluAdqlQueryTemplates = new HashMap<>();
 			for (int i = 0; i < tapTables.length; i++) {
 				this.gluAdqlQueryTemplates.put(tapTables[i], new GluAdqlTemplate());
 			}
@@ -324,7 +339,7 @@ public class ServerGlu extends Server implements Runnable {
          StringTokenizer pv = (paramValue[i]!=null)?new StringTokenizer(paramValue[i],"\t"):null;
 
          if (checkIfMultiSelect(paramDataType[i])) {
-        	 list = new JList<String>(paramValue[i].split("\t"));
+        	 list = new JList<>(paramValue[i].split("\t"));
              list.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
              list.setVisibleRowCount(5);
              if (flagTAPV2) {
@@ -470,7 +485,7 @@ public class ServerGlu extends Server implements Runnable {
 			button.addActionListener(this);
 			linePanel.add(button);
 			
-			this.sync_async = new JComboBox<String>(SYNC_ASYNC);
+			this.sync_async = new JComboBox<>(SYNC_ASYNC);
 			this.sync_async.setToolTipText(SYNCASYNCTOOLTIP);
 			this.sync_async.setOpaque(false);
 			linePanel.add(sync_async);
@@ -506,7 +521,15 @@ public class ServerGlu extends Server implements Runnable {
          y+=15;
          sc.setBounds(XTAB1,y,XWIDTH-2*XTAB1,180); y+=180;
          add(sc);
-      }
+	} else if (showSyncAsync > -1) {
+		setSyncAsyncOption();
+		if (this.sync_async != null) {
+			this.sync_async.setBounds(XTAB2, y, XWIDTH - 2 * XTAB2, HAUT);
+//			co.setBounds(XTAB2,y,XWIDTH-XTAB2,HAUT); y+=HAUT+MARGE;
+			y += HAUT ;
+			add(this.sync_async);
+		}
+	}
 
       // Gestion des filtres prdfinis s'il y a lieu
       if( aladinFilter!=null ) {
@@ -539,7 +562,42 @@ public class ServerGlu extends Server implements Runnable {
 
    }
    
-  /**
+	private void setSyncAsyncOption() {
+		// TODO Auto-generated method stub
+		this.sync_async = new JComboBox<>(SYNC_ASYNC);
+		this.sync_async.setOpaque(false);
+		
+		if (showSyncAsync > 0) {
+			if (SYNCASYNCTOOLTIP != null && !SYNCASYNCTOOLTIP.isEmpty()) {
+				this.sync_async.setToolTipText("<html><p width=\"500\">"+SYNCASYNCTOOLTIP+"</p></html>");
+			}
+			if (showSyncAsync == 2) {
+				this.sync_async.setSelectedIndex(1);
+			}
+		} else {
+			this.sync_async.setSelectedIndex(1);
+			this.sync_async.setEnabled(false);
+			this.sync_async.setToolTipText(ASYNCTOOLTIP);
+		}
+		this.sync_async.addItemListener(new ItemListener() {
+			@Override
+			public void itemStateChanged(ItemEvent e) {
+				// TODO Auto-generated method stub
+				Object url = null;
+				if (ServerGlu.this.sync_async.getSelectedItem().equals("SYNC")) {
+					url = aladin.glu.aladinDic.get(DATALINK_FORM_SYNC);
+				} else {
+					url = aladin.glu.aladinDic.get(DATALINK_FORM_ASYNC);
+				}
+				if (url != null) {
+					aladin.glu.aladinDic.put(DATALINK_FORM, url);
+				}
+				Aladin.info(ServerGlu.this, SYNCASYNCSELECTIONCHANGE);
+			}
+		});
+	}
+
+/**
    * Method for TAP : change table: to reset fields and columns as per the first table
    * @param tableSelected
    */
@@ -567,7 +625,7 @@ public class ServerGlu extends Server implements Runnable {
 			y += 20;
 		}
 		y += 20;
-		Vector<Component> components = new Vector<Component>();
+		Vector<Component> components = new Vector<>();
 //		components.addAll(tapTableMapping.get("GENERAL"));
 		components.addAll(tapTableMapping.get(this.currentSelectedTapTable));
 		for (Component co : components) {
@@ -809,7 +867,8 @@ public class ServerGlu extends Server implements Runnable {
 
        for( i=0; i<a.length && a[i]!='('; i++);
        String prefixe = new String(a,0,i);
-       return !( prefixe.equalsIgnoreCase("Target") || prefixe.equalsIgnoreCase("Field") );
+       return !( prefixe.equalsIgnoreCase("Target") || prefixe.equalsIgnoreCase("Field") 
+             || prefixe.equalsIgnoreCase("HiPS"));
     }
     
     //Change this if isShownField(String PK) changes it logic from identifying target panel components
@@ -909,7 +968,7 @@ public class ServerGlu extends Server implements Runnable {
         	   classifyAsPerTapTables(prefixe, s, gluIndex+1, sourceGluAdqlTemplate);
            } else if( flagTAPV2 && s.equalsIgnoreCase("OP") && f instanceof JTextField) {
         	   if( adqlOpInputs==null ) {
-    			   adqlOpInputs = new Vector<JTextField>();
+    			   adqlOpInputs = new Vector<>();
                 }
     		   adqlOpInputs.addElement((JTextField) f);
         	   
@@ -949,7 +1008,7 @@ public class ServerGlu extends Server implements Runnable {
                classifyAsPerTapTables(prefixe, s, gluIndex+1, sourceGluAdqlTemplate);
             } else if( flagTAPV2 && s.equalsIgnoreCase("OP") && f instanceof JTextField) {
                if( adqlOpInputs==null ) {
-                  adqlOpInputs = new Vector<JTextField>();
+                  adqlOpInputs = new Vector<>();
                }
                adqlOpInputs.addElement((JTextField) f);
 
@@ -989,7 +1048,7 @@ public class ServerGlu extends Server implements Runnable {
          	   classifyAsPerTapTables(prefixe, s, gluIndex+1, sourceGluAdqlTemplate);
             } else if( flagTAPV2 && s.equalsIgnoreCase("OP") && f instanceof JTextField) {
           	   if( adqlOpInputs==null ) {
-      			   adqlOpInputs = new Vector<JTextField>();
+      			   adqlOpInputs = new Vector<>();
                   }
       		   adqlOpInputs.addElement((JTextField) f);
           	   
@@ -1021,7 +1080,7 @@ public class ServerGlu extends Server implements Runnable {
            	   classifyAsPerTapTables(prefixe, s, gluIndex+1, sourceGluAdqlTemplate);
               } else if( flagTAPV2 && s.equalsIgnoreCase("OP") && f instanceof JTextField) {
            	   if( adqlOpInputs==null ) {
-       			   adqlOpInputs = new Vector<JTextField>();
+       			   adqlOpInputs = new Vector<>();
                    }
        		   adqlOpInputs.addElement((JTextField) f);
            	   
@@ -1035,6 +1094,11 @@ public class ServerGlu extends Server implements Runnable {
            band = (JTextField)f;
         } else if( flagTAPV2) {
         	setGluAdqlForAllTapTables(prefixe, i, a, gluIndex+1, sourceGluAdqlTemplate, f);
+        	
+        // Cas spcial pour reprer le champ qui devra recevoir le suffixe de l'URL HiPS
+        } else if( prefixe.equalsIgnoreCase("HiPS") ) {
+           f = new JHiPS();
+           flagHiPS=true;
         }
 
       return f;
@@ -1049,7 +1113,7 @@ public class ServerGlu extends Server implements Runnable {
            String s = mode.toString();
            if(s.equalsIgnoreCase("OP") && f instanceof JTextField) {
         	   if( adqlOpInputs==null ) {
-    			   adqlOpInputs = new Vector<JTextField>();
+    			   adqlOpInputs = new Vector<>();
                 }
     		   adqlOpInputs.addElement((JTextField) f);
         	   
@@ -1379,13 +1443,17 @@ public class ServerGlu extends Server implements Runnable {
                if( !verif(Plan.IMAGE,objet,param) ) {
                   return -1;
                }
-               if( fmt==PlanImage.NATIVE ) {
-                  n=aladin.calque.newPlanImageColor(u,null,PlanImage.OTHER,label,objet,param, "provided by "+institute,
-                        fmt,PlanImage.UNDEF,null,null);
-               } else {
-                  n = aladin.calque.newPlanImage(u,PlanImage.OTHER,
-                        label,objet,param, "provided by "+institute, fmt,PlanImage.UNDEF, null);
-               }
+               
+               // AVEC L'ARRIVEE DE hips2fits IL VAUT MIEUX DETECTER LE TYPE D'IMAGE A POSTERIORI
+               n = ((ServerFile)aladin.dialog.localServer).creatLocalPlane(u+"",label,origin,null,null,null,this,null,null);
+
+//               if( fmt==PlanImage.NATIVE ) {
+//                  n=aladin.calque.newPlanImageColor(u,null,PlanImage.OTHER,label,objet,param, "provided by "+institute,
+//                        fmt,PlanImage.UNDEF,null,null);
+//               } else {
+//                  n = aladin.calque.newPlanImage(u,PlanImage.OTHER,
+//                        label,objet,param, "provided by "+institute, fmt,PlanImage.UNDEF, null);
+//               }
 
                // Ou d'un serveur de MOC (on prend on compte les mirroirs)
             } else if( type==MOC ) {
@@ -1455,7 +1523,7 @@ public class ServerGlu extends Server implements Runnable {
       Enumeration e;
       String code=null;
       boolean flagScriptEquiv=true;	// Par dfaut, il existe tjs une commande script quivalent
-      List<Coord> rectVertices = new ArrayList<Coord>();//code for poly addition --in progress 1 line
+      List<Coord> rectVertices = new ArrayList<>();//code for poly addition --in progress 1 line
       
       // Resolution par Simbad necessaire ?
       if(target!=null) {
@@ -1503,7 +1571,12 @@ public class ServerGlu extends Server implements Runnable {
       while( e.hasMoreElements() ) {
          JComponent c = (JComponent)e.nextElement();
          String crit=null;
-         if( isFieldInput(c) ) {
+         if( isFieldHiPS(c) ) {
+            v.addElement("_HIPS_");
+            vbis.addElement("HiPS");
+            flagScriptEquiv=false;
+            
+         } else  if( isFieldInput(c) ) {
             v.addElement( getInputUrl(c) );
             vbis.addElement( getInputPlaneName(c) );
             flagScriptEquiv=false;
@@ -1706,26 +1779,35 @@ public class ServerGlu extends Server implements Runnable {
           } catch( Exception e1) {
  		    String message = e1.getMessage();
 		    if( message==null ) message="unknown error";
-            Aladin.error(this,ERR+"\n["+message+"]");
-			if (Aladin.levelTrace>=3) e1.printStackTrace();
-			defaultCursor();
-            ball.setMode(Ball.HS);
+		    Aladin.error(this,ERR+"\n["+message+"]");
+		    if (Aladin.levelTrace>=3) e1.printStackTrace();
+		    defaultCursor();
+		    ball.setMode(Ball.HS);
           }
           return;
 
       } else defaultCursor();
 
       if (flagTAPV2) {
-    	  String url = (String) aladin.glu.aladinDic.get(gluTag);
-    	  boolean sync = this.sync_async.getSelectedItem().equals("SYNC");
-    	  this.submitTapServerRequest(sync, null, label, url, tap.getText());
-	  } else {
-		  lastPlan = aladin.calque.createPlan(u+"",label,"provided by "+institute,this);
-	  }
-      
+         String url = (String) aladin.glu.aladinDic.get(gluTag);
+         boolean isSync = this.sync_async.getSelectedItem().equals("SYNC");
+         this.submitTapServerRequest(isSync, label, url, tap.getText());
+
+
+      } else if( flagHiPS ) {
+         aladin.calque.newPlanBG(u,label,null,null);
+
+      } else {
+         if (showSyncAsync > -1 && this.sync_async != null && this.sync_async.getSelectedItem().equals("ASYNC")) {
+            this.fireASync(label, u);
+         } else {
+            lastPlan = aladin.calque.createPlan(u + "", label, "provided by " + institute, this);
+         }
+      }
+
       if( code!=null && lastPlan!=null ) lastPlan.setBookmarkCode(code+" $TARGET $RADIUS");
-    }
-   
+   }
+
    /**
     * Method shows fov from the stc string
     */
@@ -1741,7 +1823,7 @@ public class ServerGlu extends Server implements Runnable {
 	}
 	
 	protected void cleanUpFOV() {
-		if (dataLinkSource!=null && dataLinkSource.isShowingFootprint()) {
+		if (dataLinkSource != null && dataLinkSource.isShowingFootprint()) {
 			dataLinkSource.setShowFootprint(false, true);
 		}
 		if (dataLinkSource!=null && cleanupFov) dataLinkSource.resetFootprint();
@@ -1797,11 +1879,11 @@ public class ServerGlu extends Server implements Runnable {
  	      String adqlQueryEncoded = URLEncoder.encode(adqlQueryUnEncoded, UTF8);
  	     
  		  url = url.replace(adqlQueryUnEncoded, adqlQueryEncoded);
- 		  if(Aladin.levelTrace >= 3){
- 			  System.out.println("adqlQueryUnEncoded: "+adqlQueryUnEncoded);//TODO:: tintin sysouts
- 			  System.out.println("adqlQueryEncoded: "+adqlQueryEncoded);
- 			  System.out.println("Tryin to decode again: "+URLDecoder.decode(adqlQueryEncoded, UTF8));
- 		  }
+// 		  if(Aladin.levelTrace >= 3){
+// 			  System.out.println("adqlQueryUnEncoded: "+adqlQueryUnEncoded);
+// 			  System.out.println("adqlQueryEncoded: "+adqlQueryEncoded);
+// 			  System.out.println("Tryin to decode again: "+URLDecoder.decode(adqlQueryEncoded, UTF8));
+// 		  }
  		  aladin.glu.aladinDic.put(gluTag+"v1", url);
  		  u = new URL(url);
 		} catch (UnsupportedEncodingException e1) {
@@ -1903,7 +1985,7 @@ public class ServerGlu extends Server implements Runnable {
 //      setRadius(radius);
 
       // Mise en place des paramtres
-      Vector<String> v = new Vector<String>(10);
+      Vector<String> v = new Vector<>(10);
       e = vc.elements();
       while( e.hasMoreElements() ) {
          Component c = (Component)e.nextElement();
@@ -1987,6 +2069,55 @@ public class ServerGlu extends Server implements Runnable {
       super.clear();
    }
    
+   /**
+	 * Submits async requests
+	 * @param name
+	 * @param url
+	 */
+	public void fireASync(String name, final URL url) {
+		final int requestNumber = -1;//newRequestCreation(); 
+		try {
+			final Map<String, Object> requestParams = new HashMap<>();
+			String query = url.getQuery();
+			String[] pairs = query.split("&");
+			for (String pair : pairs) {
+				int pairIndex = pair.indexOf("=");
+				requestParams.put(URLDecoder.decode(pair.substring(0, pairIndex), UTF8),
+						URLDecoder.decode(pair.substring(pairIndex + 1), UTF8));
+			}
+			Aladin.trace(3, "Firing async for " + name + " url: " + url + "\n requestParams: " + requestParams);
+			aladin.initThreadPool();
+			aladin.executor.execute(new Runnable() {
+				@Override
+				public void run() {
+					final Thread currentT = Thread.currentThread();
+					final String oldTName = currentT.getName();
+					currentT.setName("TsubmitAsync: " + url.toString());
+					try {
+						int index = url.toString().indexOf("?");
+						String urlString = url.toString().substring(0, index);
+						UWSFacade.fireAsync(aladin, ServerGlu.this, ServerGlu.this.planeLabel, urlString, requestParams,
+								requestNumber);
+						currentT.setName(oldTName);
+					} catch (Exception e) {
+						// TODO Auto-generated catch block
+						if (Aladin.levelTrace >= 3)
+							e.printStackTrace();
+						displayWarning(ServerGlu.this, requestNumber, Aladin.getChaine().getString("GENERICERROR"));
+					}
+				}
+			});
+		} catch (RejectedExecutionException ex) {
+			displayWarning(ServerGlu.this, requestNumber,
+					"Unable to submit\n Request overload! Please wait and try again.");
+		} catch (Exception e) {
+			// TODO Auto-generated catch block
+			if (Aladin.levelTrace >= 3)
+				e.printStackTrace();
+			Aladin.error(aladin.dialog, "Server error!");
+		}
+	}
+	
    protected boolean updateWidgets() {
       if( flagTAP ) {
 //         System.out.println("Je dois rditer la chaine ADQL");
@@ -1996,11 +2127,11 @@ public class ServerGlu extends Server implements Runnable {
    }
    
 	@Override
-	public ADQLQuery checkQuery() throws UnresolvedIdentifiersException {
+	public ADQLQuery checkQuery(Map<String, Object> requestParams) throws UnresolvedIdentifiersException {
 		// TODO Auto-generated method stub
 		ADQLQuery result = null;
 		try {
-			result = super.checkQuery();
+			result = super.checkQuery(requestParams);
 		} catch (UnresolvedIdentifiersException uie) {
 			Aladin.trace(3, "Number of errors in the query:"+uie.getNbErrors());
 			Iterator<adql.parser.ParseException> it = uie.getErrors();
@@ -2045,17 +2176,11 @@ public class ServerGlu extends Server implements Runnable {
 				}
 			} else if (action.equals(SHOWAYNCJOBS)) {
 				try {
-					tapManager.showAsyncPanel();
+					UWSFacade.getInstance(aladin).showAsyncPanel();
 				} catch (Exception e1) {
 					Aladin.error(this, GENERICERROR);
 		            ball.setMode(Ball.NOK);
 				}
-			} else if (action.equals(UPLOAD)) {
-				if (tapManager.uploadFrame == null) {
-					tapManager.uploadFrame = new FrameUploadServer(aladin, this.tapClient.tapBaseUrl);
-				}
-				tapManager.uploadFrame.show(this.tapClient);
-				
 			}
 
   		} else if (o instanceof JComboBox) {
@@ -2071,11 +2196,11 @@ public class ServerGlu extends Server implements Runnable {
       super.actionPerformed(e);
    }
 
-   public HealpixMoc getPosBounds() {
+   public SMoc getPosBounds() {
       return posBounds;
    }
 
-   public void setPosBounds(HealpixMoc posBounds) {
+   public void setPosBounds(SMoc posBounds) {
       this.posBounds = posBounds;
    }
 
