@@ -1,10 +1,10 @@
-// Copyright 1999-2018 - Universit de Strasbourg/CNRS
+// Copyright 1999-2020 - Universit de Strasbourg/CNRS
 // The Aladin Desktop program is developped by the Centre de Donnes
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
 //
-//This file is part of Aladin.
+//This file is part of Aladin Desktop.
 //
 //    Aladin Desktop is free software: you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 //    GNU General Public License for more details.
 //
 //    The GNU General Public License is available in COPYING file
-//    along with Aladin.
+//    along with Aladin Desktop.
 //
 
 /**
@@ -63,6 +63,7 @@ import java.awt.event.ActionEvent;
 import java.awt.event.ItemEvent;
 import java.awt.event.ItemListener;
 import java.util.ArrayList;
+import java.util.Collections;
 import java.util.HashMap;
 import java.util.Iterator;
 import java.util.List;
@@ -260,9 +261,9 @@ public class ServerObsTap extends DynamicTapForm implements ItemListener {
 			params.put(ServerObsTap.SPATIALRESOLUTION, obsCoreColumns.get(ServerObsTap.SPATIALRESOLUTION));
 			params.put(RA, obsCoreColumns.get(RA));
 			params.put(DEC, obsCoreColumns.get(DEC));
+			params.values().removeAll(Collections.singleton(null));
 			this.spatialFieldValueOptions.put(tableName, params);
 			
-			
 			if (this.timeFieldValueOptions == null) {
 				this.timeFieldValueOptions = new HashMap<String, Map<String, String>>();
 			}
@@ -271,6 +272,7 @@ public class ServerObsTap extends DynamicTapForm implements ItemListener {
 			params.put(T_MAX, obsCoreColumns.get(T_MAX));
 			params.put(ServerObsTap.EXPOSURETIME, obsCoreColumns.get(ServerObsTap.EXPOSURETIME));
 			params.put(ServerObsTap.TIMERESOLUTION, obsCoreColumns.get(ServerObsTap.TIMERESOLUTION));
+			params.values().removeAll(Collections.singleton(null));
 			this.timeFieldValueOptions.put(tableName, params);
 			
 			if (this.spectralFieldValueOptions == null) {
@@ -281,6 +283,7 @@ public class ServerObsTap extends DynamicTapForm implements ItemListener {
 			params.put(EM_MAX, obsCoreColumns.get(EM_MAX));
 			params.put(ServerObsTap.SPECTRALRESOLUTION, obsCoreColumns.get(ServerObsTap.SPECTRALRESOLUTION));
 			params.put(ServerObsTap.SPECTRALRESOLUTIONPOWER, obsCoreColumns.get(ServerObsTap.SPECTRALRESOLUTIONPOWER));
+			params.values().removeAll(Collections.singleton(null));
 			this.spectralFieldValueOptions.put(tableName, params);
 			this.addOtherParams(tableName);
 		}
@@ -308,8 +311,8 @@ public class ServerObsTap extends DynamicTapForm implements ItemListener {
 		Map<String, TapTable> tablesMetaData = this.tapClient.tablesMetaData;
 		getColumnsToLoad(selectedTableName, tablesMetaData);
 		
-		this.raColumnName = tablesMetaData.get(selectedTableName).getRaColumnName();
-		this.decColumnName = tablesMetaData.get(selectedTableName).getDecColumnName();
+		this.raColumnName = tablesMetaData.get(selectedTableName).getRaColumnName(false);
+		this.decColumnName = tablesMetaData.get(selectedTableName).getDecColumnName(false);
 		
 		tablesGui = new JComboBox(tables);
 		tablesGui.setToolTipText(this.tapClient.tablesMetaData.get(selectedTableName).getDescription());
@@ -460,6 +463,7 @@ public class ServerObsTap extends DynamicTapForm implements ItemListener {
 		//Spectral constraints
 		if (!spectralFieldValueOptions.get(selectedTableName).isEmpty()) {
 			c.gridy++;
+			c.gridwidth = 1;
 			c.gridx = 0;
 			spectral_fields = new JComboBox(spectralFieldValueOptions.get(selectedTableName).keySet().toArray());
 			spectral_fields.setFont(BOLD);
@@ -495,7 +499,7 @@ public class ServerObsTap extends DynamicTapForm implements ItemListener {
 		if (!timeFieldValueOptions.get(selectedTableName).isEmpty()) {
 			c.gridy++;
 			c.gridx = 0;
-			
+			c.gridwidth = 1;
 			time_fields = new JComboBox(timeFieldValueOptions.get(selectedTableName).keySet().toArray());
 			time_fields.setFont(BOLD);
 			c.weightx = 0.05;
@@ -529,6 +533,7 @@ public class ServerObsTap extends DynamicTapForm implements ItemListener {
 		if (columns != null && !columns.isEmpty()) {
 			c.gridy++;
 			c.gridx = 0;
+			c.gridwidth = 1;
 			Vector<TapTableColumn> model = new Vector<TapTableColumn>();
 			model.addAll(columns);
 			free_fields = new JComboBox(model);
@@ -628,8 +633,8 @@ public class ServerObsTap extends DynamicTapForm implements ItemListener {
 		if (columnNames == null) {
 			return;
 		}
-		this.raColumnName = tablesMetaData.get(selectedTableName).getRaColumnName();
-		this.decColumnName = tablesMetaData.get(selectedTableName).getDecColumnName();
+		this.raColumnName = tablesMetaData.get(selectedTableName).getRaColumnName(false);
+		this.decColumnName = tablesMetaData.get(selectedTableName).getDecColumnName(false);
 		if(Aladin.levelTrace >= 3) System.out.println("ra and dec: "+(this.raColumnName!=null && this.decColumnName!=null));
 		if(Aladin.levelTrace >= 3) System.out.println("and target panel: "+target);
 		if  (this.raColumnName != null && this.decColumnName != null) {
@@ -900,7 +905,7 @@ public class ServerObsTap extends DynamicTapForm implements ItemListener {
 				StringBuffer whereClause = new StringBuffer();
 				String raColumNameForQuery = TapTable.getQueryPart(this.raColumnName, false);
 				String decColumNameForQuery = TapTable.getQueryPart(this.decColumnName, false);
-				whereClause.append(String.format(POSQuery, raColumNameForQuery, decColumNameForQuery,
+				whereClause.append(String.format(POSQuery, EMPTYSTRING, raColumNameForQuery, decColumNameForQuery,
 						Util.myRound(coo[0].getText(), 5), Util.myRound(coo[1].getText(), 5),
 						Util.myRound(rad[0].getText(), 5))).append(SPACESTRING);
 				
@@ -922,7 +927,10 @@ public class ServerObsTap extends DynamicTapForm implements ItemListener {
 			String value, String defaultValue) {
 		ADQLQuery query = null;
 		try {
-			query = this.checkQuery();
+			if (tap.getText().isEmpty()) {
+				writeQuery();
+			}
+			query = this.checkQuery(null);
 		} catch (UnresolvedIdentifiersException uie) {
 			// TODO Auto-generated catch block
 			Iterator<adql.parser.ParseException> it = uie.getErrors();
@@ -958,12 +966,13 @@ public class ServerObsTap extends DynamicTapForm implements ItemListener {
 			}
 			
 			String processedInput = null;
-			if (value == null || value.isEmpty()) {// tintin below if condn
-				if (inRange) {
+			if (value == null || value.isEmpty()) {
+				processedInput = defaultValue;
+				/*if (inRange) {
 					processedInput = defaultValue;
 				} else {
 					processedInput = defaultValue;//constraint+SPACESTRING+defaultValue;
-				}
+				}*/
 			} else {
 				if (processAsNumber) {
 					if (inRange) {
@@ -1003,24 +1012,28 @@ public class ServerObsTap extends DynamicTapForm implements ItemListener {
 
 	}
 	
+//	@Override
+//	public void submit() {
+//		submit(null);//no request params
+//	}
+	
 	/**
 	 * Method fires sync or async submissions based on what is selected by user
 	 * Incase of upload as file is posted, diff sync methods are followed
 	 * @param requestParams
 	 */
-	public void submit(Map<String, Object> requestParams) {
+	@Override
+	public void submit() {
 	      //check again
 		if (this.sync_async != null &&  this.tap != null) {
 			boolean sync = this.sync_async.getSelectedItem().equals("SYNC");
-	  	  	this.submitTapServerRequest(sync, requestParams, this.tapClient.tapLabel, this.tapClient.tapBaseUrl, this.tap.getText());
+	  	  	this.submitTapServerRequest(sync, this.tapClient.tapLabel, this.tapClient.tapBaseUrl, this.tap.getText());
+
+	  	  	// Echo of the equivalent script command
+	  	  	aladin.console.printCommand("get TAP("+Tok.quote(this.tapClient.tapLabel)+","+Tok.quote(this.tap.getText())+")");
 		}
 	}
 	
-	@Override
-	public void submit() {
-		submit(null);//no request params
-	}
-	
 	/**
 	 * Method parses adql query from user using Grgory
 	 * Mantelet's (ARI/ZAH) adql parser lib
