@@ -1,10 +1,10 @@
-// Copyright 1999-2018 - Universit de Strasbourg/CNRS
+// Copyright 1999-2020 - Universit de Strasbourg/CNRS
 // The Aladin Desktop program is developped by the Centre de Donnes
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
 //
-//This file is part of Aladin.
+//This file is part of Aladin Desktop.
 //
 //    Aladin Desktop is free software: you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 //    GNU General Public License for more details.
 //
 //    The GNU General Public License is available in COPYING file
-//    along with Aladin.
+//    along with Aladin Desktop.
 //
 
 /**
@@ -53,23 +53,30 @@ import static cds.aladin.Constants.T_MAX;
 import static cds.aladin.Constants.T_MIN;
 import static cds.tools.CDSConstants.BOLD;
 
+import java.awt.Dimension;
 import java.awt.GridBagConstraints;
 import java.awt.GridBagLayout;
 import java.awt.GridLayout;
+import java.awt.Image;
 import java.awt.Insets;
 import java.awt.event.ActionEvent;
+import java.awt.event.ActionListener;
+import java.awt.event.ComponentEvent;
+import java.awt.event.ComponentListener;
 import java.awt.event.ItemEvent;
 import java.awt.event.ItemListener;
+import java.beans.PropertyChangeEvent;
+import java.beans.PropertyChangeListener;
 import java.util.ArrayList;
-import java.util.HashMap;
 import java.util.LinkedHashMap;
 import java.util.List;
 import java.util.Locale;
 import java.util.Map;
 import java.util.Vector;
 
-import javax.swing.BoxLayout;
+import javax.swing.BorderFactory;
 import javax.swing.ComboBoxModel;
+import javax.swing.ImageIcon;
 import javax.swing.JButton;
 import javax.swing.JCheckBox;
 import javax.swing.JComboBox;
@@ -77,9 +84,12 @@ import javax.swing.JComponent;
 import javax.swing.JList;
 import javax.swing.JPanel;
 import javax.swing.JScrollPane;
+import javax.swing.JSplitPane;
 import javax.swing.JTextArea;
 import javax.swing.ListSelectionModel;
 import javax.swing.MutableComboBoxModel;
+import javax.swing.border.Border;
+import javax.swing.border.TitledBorder;
 import javax.swing.event.ListSelectionEvent;
 import javax.swing.event.ListSelectionListener;
 import javax.swing.text.BadLocationException;
@@ -102,11 +112,14 @@ public class ServerTapExamples extends DynamicTapForm {
 	 */
 	private static final long serialVersionUID = -9113338791629047699L;
 	
-	public static String TAPSERVICEEXAMPLESTOOLTIP, SETTARGETTOOLTIP, TAPEXDEFAULTMAXROWS, CHANGESETTINGSTOOLTIP, NODUPLICATESELECTION;
+	public static String TAPSERVICEEXAMPLESTOOLTIP, SETTARGETTOOLTIP, TAPEXDEFAULTMAXROWS, CHANGESETTINGSTOOLTIP,
+			NODUPLICATESELECTION, JOINTABLETOOLTIP, QUERIESREFRESHED, TAPEXGENERATEDLABEL, TAPEXGENERATEDTOOLTIP,
+			TAPEXSERVICEPROVIDEDLABEL, TAPEXSERVICEPROVIDEDTOOLTIP, SHOWSERVICEEXTIP, HIDESERVICEEXAMPLES;
+	
 	public static final String TAPEXDEFAULTMAXROWS_INT = "2000";
 	Map serviceExamples = null;
 	
-	Map<String, CustomListCell> basicExamples = new LinkedHashMap<String, CustomListCell>();
+	Map<String, CustomListCell> basicExamples = new LinkedHashMap<>();
 	Map<String, String> serviceExamples2;
 
 	String secondaryTable; 
@@ -125,6 +138,10 @@ public class ServerTapExamples extends DynamicTapForm {
 
 	JComboBox secondaryTablesGui;
 
+	private MySplitPane centerPanel;
+
+	private JButton reduce;
+
 	public ServerTapExamples(Aladin aladin) {
 		// TODO Auto-generated constructor stub
 		super(aladin);
@@ -187,25 +204,27 @@ public class ServerTapExamples extends DynamicTapForm {
 		selectedTableName = chosenTable.getTable_name();
 		getColumnsToLoad(selectedTableName, this.tapClient.tablesMetaData);
 		
-		Map<String, TapTable> uploadMeta = this.tapClient.tapManager.getUploadedTables();
+		TapManager tapManager = TapManager.getInstance(aladin);
 		
 		TapTable chosenTable2 = null;
-		if (this.tapClient.isUploadAllowed() && uploadMeta != null && !uploadMeta.isEmpty()) {
-			if (secTableChoice == null || !uploadMeta.containsKey(secTableChoice)) {
-				if (uploadMeta.size() > 1) {
-					secondaryTable = uploadMeta.keySet().iterator().next();
+		if (this.tapClient.isUploadAllowed()) {
+			Map<String, TapTable> uploadMeta = tapManager.initUploadFrameAndGetUploadedTables();
+			if (uploadMeta != null && !uploadMeta.isEmpty()) {
+				if (secTableChoice == null || !uploadMeta.containsKey(secTableChoice)) {
+					if (uploadMeta.size() > 1) {
+						secondaryTable = uploadMeta.keySet().iterator().next();
+					} else {
+						secondaryTable = null;
+					}
 				} else {
-					secondaryTable = null;
+					secondaryTable = secTableChoice;
+				}
+				if (secondaryTable != null) {
+					chosenTable2 = uploadMeta.get(secondaryTable);
 				}
-			} else {
-				secondaryTable = secTableChoice;
 			}
 		}
 		
-		if (secondaryTable != null) {
-			chosenTable2 = uploadMeta.get(secondaryTable);
-		}
-		
 		setBasics();
 		max = null;
 		
@@ -217,9 +236,10 @@ public class ServerTapExamples extends DynamicTapForm {
 		JPanel tablesPanel = null;
 		try {
 			tablesGui = new JComboBox(tables);
-			tablesPanel = getTablesPanel(null, tablesGui, chosenTable, tables, null, true);
+			tablesPanel = getTablesPanel(this.tapClient, this, null, tablesGui, chosenTable, tables, null, true);
 			tablesPanel.setBackground(this.tapClient.primaryColor);
 			tablesPanel.setFont(BOLD);
+			
 			c.weighty = 0.02;
 	        c.weightx = 0.10;
 	        c.anchor = GridBagConstraints.NORTHWEST;
@@ -230,72 +250,22 @@ public class ServerTapExamples extends DynamicTapForm {
 		    c.weighty = 0.57;
 		    
 		    if (this.tapClient.isUploadAllowed()) {
-			    JPanel secondaryTablePanel = new JPanel();
-				secondaryTablePanel.setLayout(new BoxLayout(secondaryTablePanel, BoxLayout.PAGE_AXIS));
-				MutableComboBoxModel uploadModel = (MutableComboBoxModel) this.tapClient.tapManager.getUploadClientModel();
-				
+				MutableComboBoxModel uploadModel = (MutableComboBoxModel) tapManager.getUploadClientModel();
 				addSecondTable = new JCheckBox();
-				addSecondTable.setToolTipText("Join with an uploaded table");
+				addSecondTable.setToolTipText(JOINTABLETOOLTIP);
 				addSecondTable.setEnabled((uploadModel.getSize() > 0));
-				addSecondTable.addItemListener(new ItemListener() {
-					@Override
-					public void itemStateChanged(ItemEvent e) {
-						// TODO Auto-generated method stub
-						ComboBoxModel uploadModel = tapClient.tapManager.getUploadClientModel();
-						if (e.getStateChange() == ItemEvent.SELECTED && uploadModel.getSize() > 0) {
-							secondaryTablesGui.setEnabled(true);
-							secondaryTable = (String) secondaryTablesGui.getSelectedItem();
-						} else {
-							secondaryTable = null;
-							secondaryTablesGui.setEnabled(false);
-						}
-//						resetExamplesGui();
-						regenerateBasicExamplesRepaint();
-					}
-				});	
-//				Vector uploadTableNames = new Vector();
-//				if (uploadMeta != null) {
-//					uploadTableNames.addAll(uploadMeta.keySet());
-//				}
-				secondaryTablesGui = new JComboBox();
-				secondaryTablesGui.setModel(uploadModel);
-//				model.addListDataListener(new ListDataListener() {
-//					
-//					@Override
-//					public void intervalRemoved(ListDataEvent e) {
-//						// TODO Auto-generated method stub
-//						if (((MutableComboBoxModel)e.getSource()).getSize() > 0) {
-//							secondaryTablesGui.setEnabled(true);
-//							secondaryTable = (String) secondaryTablesGui.getSelectedItem();
-//						} else {
-//							secondaryTable = null;
-//							secondaryTablesGui.setEnabled(false);
-//						}
-//						changeTableSelection(selectedTableName);
-//					}
-//					
-//					@Override
-//					public void intervalAdded(ListDataEvent e) {
-//						// TODO Auto-generated method stub
-//						if (((MutableComboBoxModel)e.getSource()).getSize() > 0) {
-//							secondaryTablesGui.setEnabled(true);
-//							secondaryTable = (String) secondaryTablesGui.getSelectedItem();
-//						} else {
-//							secondaryTable = null;
-//							secondaryTablesGui.setEnabled(false);
-//						}
-//						changeTableSelection(selectedTableName);
-//					}
-//					
-//					@Override
-//					public void contentsChanged(ListDataEvent e) {
-//						// TODO Auto-generated method stub
-//						
-//					}
-//				});
-				ArrayList<JComponent> compToPrefix = new ArrayList<JComponent>();
+				
+				secondaryTablesGui = new JComboBox(uploadModel);
+				ArrayList<JComponent> compToPrefix = new ArrayList<>();
 				compToPrefix.add(addSecondTable);
-				tablesPanel = getTablesPanel("Join:", secondaryTablesGui, chosenTable2, null, compToPrefix, false);
+				tablesPanel = getTablesPanel(this.tapClient, this, "Join:", secondaryTablesGui, chosenTable2, null, compToPrefix, false);
+				UploadTablesRenderer uploadTableRenderer = UploadTablesRenderer.getInstance(this.aladin);
+				secondaryTablesGui.setRenderer(uploadTableRenderer);
+				if (secondaryTablesGui.getSelectedItem() != null) {
+					String uploadTable = (String) secondaryTablesGui.getSelectedItem();
+					String planeName = uploadTableRenderer.getUploadPlaneName((String) secondaryTablesGui.getSelectedItem());
+					secondaryTablesGui.setToolTipText(UploadTablesRenderer.getToolTip(planeName, uploadTable));
+				}
 				tablesPanel.setBackground(this.tapClient.primaryColor);
 				tablesPanel.setFont(BOLD);
 				if (secondaryTable != null) {
@@ -305,7 +275,31 @@ public class ServerTapExamples extends DynamicTapForm {
 					addSecondTable.setSelected(false);
 					secondaryTablesGui.setEnabled(false);
 				}
-				secondaryTablePanel.add(tablesPanel);
+				
+				addSecondTable.addItemListener(new ItemListener() {
+					@Override
+					public void itemStateChanged(ItemEvent e) {
+						// TODO Auto-generated method stub
+						ComboBoxModel uploadModel = tapClient.tapManager.getUploadClientModel();
+						boolean doScroll = false;
+						if (e.getStateChange() == ItemEvent.SELECTED && uploadModel.getSize() > 0) {
+							doScroll = true;
+							secondaryTablesGui.setEnabled(true);
+							secondaryTable = (String) secondaryTablesGui.getSelectedItem();
+						} else {
+							secondaryTable = null;
+							secondaryTablesGui.setEnabled(false);
+						}
+//						resetExamplesGui();
+						regenerateBasicExamplesRepaint();
+						if (doScroll) {
+							int index = examplesGui.getModel().getSize() - 1;
+							if (index >= 0) {
+								examplesGui.ensureIndexIsVisible(index);
+							}
+						}
+					}
+				});	
 				
 				c.weightx = 0.12;
 				c.weighty = 0.02;
@@ -313,7 +307,7 @@ public class ServerTapExamples extends DynamicTapForm {
 			    c.fill = GridBagConstraints.NONE;
 			    c.insets = new Insets(0, 4, 0, 0);
 			    c.gridy++;
-			    containerPanel.add(secondaryTablePanel, c);	
+			    containerPanel.add(tablesPanel, c);	
 			    c.weighty = 0.55;
 			}
 		} catch (BadLocationException e) {
@@ -333,7 +327,7 @@ public class ServerTapExamples extends DynamicTapForm {
 				// TODO Auto-generated method stub
 				String queryLabel = (String) examplesGui.getSelectedValue();
 				if (queryLabel != null && basicExamples.containsKey(queryLabel) ) {
-					tap.setText(((CustomListCell)basicExamples.get(queryLabel)).label);
+					tap.setText(basicExamples.get(queryLabel).label);
 					if (serviceExamplesGui != null && !examplesGui.isSelectionEmpty()) {
 						serviceExamplesGui.clearSelection();
 					}
@@ -344,10 +338,9 @@ public class ServerTapExamples extends DynamicTapForm {
 		});
 		
 		queryDisplays = new JPanel();
-		JScrollPane scrolley = new JScrollPane(this.examplesGui);
-		
+		queryDisplays.setBackground(this.tapClient.primaryColor);
 		queryDisplays.setLayout(new GridLayout(1, 1));
-		queryDisplays.add(scrolley);
+		queryDisplays.add(getScrollPane1());
 		c.fill = GridBagConstraints.BOTH;
 		c.gridy++;
 		c.insets = new Insets(0, 4, 0, 0);
@@ -367,7 +360,7 @@ public class ServerTapExamples extends DynamicTapForm {
 		tap.setWrapStyleWord(true);
 		tap.setLineWrap(true);
 		tap.setEditable(true);
-		scrolley = new JScrollPane(tap);
+		JScrollPane scrolley = new JScrollPane(tap);
 //		c.weightx = 0.35;
 		c.weighty = 0.35;
 		c.weightx = 1;
@@ -380,7 +373,20 @@ public class ServerTapExamples extends DynamicTapForm {
 	    add(containerPanel);
 	    
 	    formLoadStatus = TAPFORM_STATUS_LOADED;
-	    this.tapClient.tapManager.initTapExamples(this);
+	    tapManager.initTapExamples(this);
+	}
+	
+	public Border getFancyTitleBorder(String title) {
+//		Border compound, raisedbevel, loweredbevel ;
+//		raisedbevel = BorderFactory.createBevelBorder(BevelBorder.RAISED, Aladin.COLOR_VERTDEAU, Aladin.COLOR_VERTDEAU);
+//		loweredbevel = BorderFactory.createBevelBorder(BevelBorder.LOWERED, Aladin.COLOR_VERTDEAU, Aladin.COLOR_BLUE);
+		Border colorLine = BorderFactory.createLineBorder(Aladin.COLOR_MEASUREMENT_BACKGROUND_SELECTED_LINE);
+//		compound = BorderFactory.createCompoundBorder(raisedbevel, loweredbevel);
+		TitledBorder titleb = BorderFactory.createTitledBorder(title);
+		titleb.setBorder(colorLine);
+//		compound = BorderFactory.createCompoundBorder(colorLine, compound);
+//		compound = BorderFactory.createTitledBorder(compound, title,TitledBorder.LEFT,TitledBorder.ABOVE_TOP);
+		return titleb;
 	}
 	
 	public void loadTapExamples() {
@@ -392,6 +398,7 @@ public class ServerTapExamples extends DynamicTapForm {
 		} 
 		if (serviceExamples != null && !serviceExamples.isEmpty()) {
 			this.serviceExamplesGui = new JList(serviceExamples.keySet().toArray());
+//			this.serviceExamplesGui.setBackground(this.tapClient.secondColor);
 			this.serviceExamplesGui.setVisibleRowCount(4);
 			this.serviceExamplesGui.setToolTipText(TAPSERVICEEXAMPLESTOOLTIP);
 			this.serviceExamplesGui.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
@@ -399,7 +406,6 @@ public class ServerTapExamples extends DynamicTapForm {
 				@Override
 				public void valueChanged(ListSelectionEvent e) {
 					// TODO Auto-generated method stub
-					//tintin todo
 					String queryLabel = (String) serviceExamplesGui.getSelectedValue();
 					if (queryLabel != null) {
 						tap.setText(serviceExamples.get(queryLabel).toString());
@@ -411,12 +417,7 @@ public class ServerTapExamples extends DynamicTapForm {
 			});
 			waitCursor();
 			synchronized (queryDisplays) {
-				JScrollPane scrolley = new JScrollPane(this.examplesGui);
-				queryDisplays.removeAll();
-				queryDisplays.setLayout(new GridLayout(1, 2));
-				queryDisplays.add(scrolley);
-				scrolley = new JScrollPane(this.serviceExamplesGui);
-				queryDisplays.add(scrolley);
+				setOnlyClientGenExamples();
 				queryDisplays.revalidate();
 				queryDisplays.repaint();
 			}
@@ -426,6 +427,155 @@ public class ServerTapExamples extends DynamicTapForm {
 		info1.setText(CLIENTINSTR);
 	}
 	
+	/**
+	 * Show only generated examples. with an flag that you might have server provided examples.
+	 */
+	public void setOnlyClientGenExamples() {
+		queryDisplays.removeAll();
+		queryDisplays.setLayout(new GridBagLayout());
+		queryDisplays.setBackground(this.tapClient.primaryColor);
+		GridBagConstraints c = new GridBagConstraints();
+		c.gridx = 1;
+		c.insets = new Insets(-10, 0, 0, 0);
+		c.weightx = 0.01;
+		c.anchor = GridBagConstraints.NORTHWEST;
+		c.fill = GridBagConstraints.NONE;
+		JButton button = null;
+		Image image = Aladin.aladin.getImagette("Load.gif");
+		if (image == null) {
+			button = new JButton("+");
+		} else {
+			button = new JButton(new ImageIcon(image));
+			button.setBorderPainted(false);
+			button.setMargin(new Insets(0, 0, 0, 0));
+			button.setContentAreaFilled(true);
+		}
+		button.addActionListener(new ActionListener() {
+			@Override
+			public void actionPerformed(ActionEvent e) {
+				// TODO Auto-generated method stub
+				showBothExamples();
+			}
+		});
+		button.setToolTipText(SHOWSERVICEEXTIP);
+		queryDisplays.add(button, c);
+		
+		c.gridx = 0;
+		c.gridy = 0;
+		c.insets = new Insets(0, 0, 0, -15);
+		c.fill = GridBagConstraints.BOTH;
+		c.anchor = GridBagConstraints.NORTH;
+		c.weightx = 0.99;
+		c.weighty = 1;
+		queryDisplays.add(getScrollPane1(), c);
+		
+		queryDisplays.revalidate();
+		queryDisplays.repaint();
+	}
+	
+	/**
+	 * Show both static(server provided) and dynamic(client generated) on a split pane
+	 */
+	public void showBothExamples() {
+		queryDisplays.removeAll();
+		queryDisplays.setLayout(new GridBagLayout());
+		queryDisplays.setBackground(this.tapClient.primaryColor);
+		
+		GridBagConstraints c = new GridBagConstraints();
+		c.gridx = 1;
+		c.insets = new Insets(-12, -12, 0, 0);
+		c.weightx = 0.01;
+		c.anchor = GridBagConstraints.NORTHWEST;
+		c.fill = GridBagConstraints.NONE;
+		Image image = Aladin.aladin.getImagette("Reduire.gif");
+		if (image == null) {
+			reduce = new JButton("-");
+		} else {
+			reduce = new JButton(new ImageIcon(image));
+			reduce.setBorderPainted(false);
+			reduce.setMargin(new Insets(0, 0, 0, 0));
+			reduce.setContentAreaFilled(false);
+		}
+		reduce.addActionListener(new ActionListener() {
+			@Override
+			public void actionPerformed(ActionEvent arg0) {
+				// TODO Auto-generated method stub
+				setOnlyClientGenExamples();
+			}
+		});
+		reduce.setToolTipText(HIDESERVICEEXAMPLES);
+		queryDisplays.add(reduce,c);
+		
+		c.gridx = 0;
+		c.gridy = 0;
+		c.insets = new Insets(0, 0, 0, 0);
+		c.fill = GridBagConstraints.BOTH;
+		c.anchor = GridBagConstraints.NORTH;
+		c.weightx = 0.99;
+		c.weighty = 1;
+		
+		JScrollPane scrolley1 = getScrollPane1();
+		scrolley1.setMinimumSize(new Dimension(250, 100));
+		
+		JScrollPane scrolley2 = new JScrollPane(this.serviceExamplesGui);
+		scrolley2.setBackground(this.tapClient.primaryColor);
+		scrolley2.setToolTipText(TAPEXSERVICEPROVIDEDTOOLTIP);
+		scrolley2.setBorder(getFancyTitleBorder(TAPEXSERVICEPROVIDEDLABEL));
+		centerPanel = new MySplitPane(aladin, JSplitPane.HORIZONTAL_SPLIT, scrolley1, scrolley2 , 1);
+		centerPanel.setBackground(this.tapClient.primaryColor);
+		centerPanel.addPropertyChangeListener(new PropertyChangeListener() {
+			@Override
+			public void propertyChange(PropertyChangeEvent evt) {
+				// TODO Auto-generated method stub
+//				System.err.println("centerPanel propertyChange : "+evt.getPropertyName()+" "+evt.getNewValue()+" "+centerPanel.getRightComponent().getWidth()+"  "+centerPanel.getWidth());
+				reduce.revalidate();
+				reduce.repaint();
+			}
+		});
+		scrolley2.addComponentListener(new ComponentListener() {
+			@Override
+			public void componentShown(ComponentEvent e) {
+				// TODO Auto-generated method stub
+//				System.err.println("scrolley2 componentShown :"+serviceExamplesGui.getParent().getSize()+e.paramString());
+			}
+			
+			@Override
+			public void componentResized(ComponentEvent e) {
+				// TODO Auto-generated method stub
+//				System.err.println("scrolley2 componentResized :"+serviceExamplesGui.getParent().getSize()+e.paramString());
+				reduce.revalidate();
+				reduce.repaint();
+			}
+			
+			@Override
+			public void componentMoved(ComponentEvent e) {
+				// TODO Auto-generated method stub
+//				System.err.println("scrolley2 componentMoved :"+serviceExamplesGui.getParent().getSize()+e.paramString());
+				reduce.revalidate();
+				reduce.repaint();
+			}
+			
+			@Override
+			public void componentHidden(ComponentEvent e) {
+				// TODO Auto-generated method stub
+//				System.err.println("scrolley2 componentHidden :"+serviceExamplesGui.getParent().getSize()+e.paramString());
+			}
+		});
+		centerPanel.setDividerLocation(0.7);
+		queryDisplays.add(centerPanel, c);
+
+		queryDisplays.revalidate();
+		queryDisplays.repaint();
+	}
+	
+	private JScrollPane getScrollPane1() {
+		JScrollPane scrolley = new JScrollPane(this.examplesGui);
+		scrolley.setToolTipText(TAPEXGENERATEDTOOLTIP);
+		scrolley.setBorder(BorderFactory.createTitledBorder(TAPEXGENERATEDLABEL));
+		scrolley.setBackground(tapClient.primaryColor);
+		return scrolley;
+    }
+  
 	@Override
 	public void submit() {
 		if (this.sync_async != null &&  this.tap != null) {
@@ -434,7 +584,7 @@ public class ServerTapExamples extends DynamicTapForm {
 				Aladin.error(this, CHECKQUERY_ISBLANK);
 				return;
 			}
-			Map<String, Object> requestParams = null;
+			/*Map<String, Object> requestParams = null; //added off in  updateUploadedTablesToParser. keeping for ref
 			if (secondaryTable != null) {
 				TapManager tapManager = TapManager.getInstance(aladin);
 				if (!this.tapClient.tablesMetaData.containsKey(secondaryTable)) {
@@ -452,7 +602,7 @@ public class ServerTapExamples extends DynamicTapForm {
 						}
 					}
 				}
-			}
+			}*/
 			
 			String fullQuery = tapQuery.toUpperCase();
 			if (!fullQuery.startsWith("SELECT ")) {
@@ -460,7 +610,7 @@ public class ServerTapExamples extends DynamicTapForm {
 				Aladin.trace(3,tapQuery);
 			}
 			boolean sync = this.sync_async.getSelectedItem().equals("SYNC");
-	  	  	this.submitTapServerRequest(sync, requestParams, this.tapClient.tapLabel, this.tapClient.tapBaseUrl, tapQuery);
+	  	  	this.submitTapServerRequest(sync, this.tapClient.tapLabel, this.tapClient.tapBaseUrl, tapQuery);
 		}
 		
 	}
@@ -472,19 +622,7 @@ public class ServerTapExamples extends DynamicTapForm {
 		Object source = arg0.getSource();
 		if (source instanceof JButton) {
 			String action = ((JButton) source).getActionCommand();
-			/*if (action.equals(SETTINGS)) {//tintin remove comments
-				try {
-					TapManager tapManager = TapManager.getInstance(aladin);
-					if (tapManager.settingsFrame == null) {
-						tapManager.settingsFrame = new FrameTapSettings(aladin);
-					}
-					tapManager.settingsFrame.show(this, "Settings for " + this.tapClient.tapLabel);
-				} catch (Exception e) {
-					if (Aladin.levelTrace >= 3) e.printStackTrace();
-					Aladin.warning(this, TapClient.GENERICERROR);
-		            ball.setMode(Ball.NOK);
-				}
-			} else*/ if (action.equals(CHECKQUERY)) {
+			if (action.equals(CHECKQUERY)) {
 				checkQueryFlagMessage();
 			} 
 		}
@@ -520,6 +658,12 @@ public class ServerTapExamples extends DynamicTapForm {
 						secondaryTable = chosen;
 					}
 					addSecondTable.setEnabled(true);
+					if (secondaryTablesGui.getSelectedItem() != null) {
+						String uploadTable = (String) secondaryTablesGui.getSelectedItem();
+						UploadTablesRenderer uploadTableRenderer = (UploadTablesRenderer) secondaryTablesGui.getRenderer();
+						String planeName = uploadTableRenderer.getUploadPlaneName((String) secondaryTablesGui.getSelectedItem());
+						secondaryTablesGui.setToolTipText(UploadTablesRenderer.getToolTip(planeName, uploadTable));
+					}
 				} else {
 					secondaryTable = null;
 					addSecondTable.setEnabled(false);
@@ -545,8 +689,7 @@ public class ServerTapExamples extends DynamicTapForm {
 		String priTableNameForQuery = TapTable.getQueryPart(selectedTableName, true);
 		this.basicExamples.put("Select all", new CustomListCell("Select * from " + priTableNameForQuery, EMPTYSTRING));
 		this.basicExamples.put("Select top 1000", new CustomListCell("Select TOP 1000 * from " + priTableNameForQuery, EMPTYSTRING));
-		// this.basicExamples.put("Select few columns", "Select oidref, filter,
-		// flux, bibcode from "+primaryTable);
+		// this.basicExamples.put("Select few columns", "Select oidref, filter, flux, bibcode from "+primaryTable);
 		this.basicExamples.put("Get the number of rows", new CustomListCell("Select count(*) from " + priTableNameForQuery, EMPTYSTRING));
 
 		String areaConstraint = "0.09";
@@ -566,8 +709,8 @@ public class ServerTapExamples extends DynamicTapForm {
 		String conesearchtemplate = " where 1=CONTAINS(POINT('ICRS', %s, %s), CIRCLE('ICRS', %s, %s, %s))";
 		String primaryTableSelectAllQuery = String.format(tableSelectAllQuery, max, priTableNameForQuery);
 
-		String priRaColumnName = TapTable.getQueryPart(priTableMetaData.getRaColumnName(), false);
-		String priDecColumnName = TapTable.getQueryPart(priTableMetaData.getDecColumnName(), false);
+		String priRaColumnName = TapTable.getQueryPart(priTableMetaData.getRaColumnName(false), false);
+		String priDecColumnName = TapTable.getQueryPart(priTableMetaData.getDecColumnName(false), false);
 
 		String targetQuery = primaryTableSelectAllQuery + conesearchtemplate;
 		String coneSearchPart = null;
@@ -595,8 +738,8 @@ public class ServerTapExamples extends DynamicTapForm {
 		StringBuffer hints = null;
 		CustomListCell setCell = null;
 		String queryName = null;
-		List<String> mandateParamsToAdd = new ArrayList<String>();
-		List<String> optionalParamsToAdd = new ArrayList<String>();
+		List<String> mandateParamsToAdd = new ArrayList<>();
+		List<String> optionalParamsToAdd = new ArrayList<>();
 
 		// some nice select statements..also set param names
 		TapTableColumn parallax = priTableMetaData.getFlaggedColumn(PARALLAX), radialVelocity = priTableMetaData.getFlaggedColumn(RADIALVELOCITY);
@@ -620,7 +763,7 @@ public class ServerTapExamples extends DynamicTapForm {
 		}
 		 
 		if (!mandateParamsToAdd.isEmpty()) {
-			addSelectsToBasicExamples("Get bibliographic data", mandateParamsToAdd, optionalParamsToAdd, null, null, null);
+			addSelectsToBasicExamples("Get bibliographic data", mandateParamsToAdd, optionalParamsToAdd, null, null, null, null);
 		}
 
 		// radial velocity and parallax
@@ -646,7 +789,7 @@ public class ServerTapExamples extends DynamicTapForm {
 			setCell = new CustomListCell();
 			setCell.tooltip = hints.toString();
 			addSelectsToBasicExamples(queryName, mandateParamsToAdd, optionalParamsToAdd, setCell, null,
-					coneSearchPart);
+					coneSearchPart, SETTARGETTOOLTIP);
 		}
 
 		// position and proper motion
@@ -671,7 +814,7 @@ public class ServerTapExamples extends DynamicTapForm {
 			}
 			setCell.tooltip = hints.toString();
 			addSelectsToBasicExamples("Position and proper motion", mandateParamsToAdd, optionalParamsToAdd, setCell, null,
-					coneSearchPart);
+					coneSearchPart, SETTARGETTOOLTIP);
 
 			String properMotionQuery = "SQRT(POWER(" + pmra.getColumnNameForQuery() + ",2)+POWER(" + pmdec.getColumnNameForQuery() + ",2))";
 			mandateParamsToAdd.add(properMotionQuery + " as pm");
@@ -679,7 +822,7 @@ public class ServerTapExamples extends DynamicTapForm {
 			setCell = new CustomListCell();
 			setCell.tooltip = hints.toString();
 			addSelectsToBasicExamples("Proper motion", mandateParamsToAdd, optionalParamsToAdd, setCell, 
-					"Proper motion limits ", properMotionQuery);
+					"Proper motion limits ", properMotionQuery, "");
 		}
 
 		// red shift
@@ -691,7 +834,7 @@ public class ServerTapExamples extends DynamicTapForm {
 			hints = new StringBuffer();
 			setHint(hints, redshift);
 			setCell.tooltip = hints.toString();
-			addSelectsToBasicExamples(null, mandateParamsToAdd, optionalParamsToAdd, setCell, null, coneSearchPart);
+			addSelectsToBasicExamples(null, mandateParamsToAdd, optionalParamsToAdd, setCell, null, coneSearchPart, SETTARGETTOOLTIP);
 		}
 
 		// operations with those params
@@ -713,8 +856,8 @@ public class ServerTapExamples extends DynamicTapForm {
 			if (uploadedTables != null) {
 				secTableMetaData = uploadedTables.get(secondaryTable);
 				if (secTableMetaData != null) {
-					secRaColumnName = TapTable.getQueryPart(secTableMetaData.getRaColumnName(), false); 
-					secDecColumnName = TapTable.getQueryPart(secTableMetaData.getDecColumnName(), false);
+					secRaColumnName = TapTable.getQueryPart(secTableMetaData.getRaColumnName(false), false); 
+					secDecColumnName = TapTable.getQueryPart(secTableMetaData.getDecColumnName(false), false);
 					secTableNameForQuery = TapTable.getQueryPart(secondaryTable, true);
 				}
 			}
@@ -771,7 +914,7 @@ public class ServerTapExamples extends DynamicTapForm {
 				optionalParamsToAdd.add(0, obs_id);
 			}
 			if (!mandateParamsToAdd.isEmpty()) {
-				addSelectsToBasicExamples("Get access_url, access_formats...", mandateParamsToAdd, optionalParamsToAdd, null, null, null);
+				addSelectsToBasicExamples("Get access_url, access_formats...", mandateParamsToAdd, optionalParamsToAdd, null, null, null, null);
 			}
 			
 			/**
@@ -885,8 +1028,6 @@ public class ServerTapExamples extends DynamicTapForm {
 			}
 		}
 		
-		 
-		
 		/**
 		 * Join queries for target in table2 contained in region of table1
 		 */
@@ -918,8 +1059,7 @@ public class ServerTapExamples extends DynamicTapForm {
 				
 				
 				spQuery.append(" AND 1=CONTAINS(POINT('ICRS',")
-						.append(priTableNameForQuery).append(DOT_CHAR).append(priRaColumnName).append(", ")
-						.append(priTableNameForQuery).append(DOT_CHAR).append(priDecColumnName)
+						.append(priRaColumnName).append(", ").append(priDecColumnName)
 						.append("), CIRCLE('ICRS', ").append(grabItX1).append(COMMA_CHAR).append(grabItY1)
 						.append(COMMA_CHAR).append(grabItR1).append("))");
 						
@@ -961,6 +1101,7 @@ public class ServerTapExamples extends DynamicTapForm {
 		this.examplesGui.setListData(this.basicExamples.keySet().toArray());
 		this.examplesGui.revalidate();
 		this.examplesGui.repaint();
+		tapClient.tapManager.eraseNotification(info1, QUERIESREFRESHED, CLIENTINSTR);
 	}
 	
 	public void changeTargetSettings() {
@@ -1046,10 +1187,10 @@ public class ServerTapExamples extends DynamicTapForm {
 	 * @param wherePart
 	 */
 	public void addSelectsToBasicExamples(String queryName, List<String> mandatoryParams,
-			List<String> optionalParamsToAdd, CustomListCell setCell, String whereLabel, String wherePart) {
+			List<String> optionalParamsToAdd, CustomListCell setCell, String whereLabel, String wherePart, String wherePartToolTip) {
 		String tableSelectQuery = "Select %s %s from %s";
 		StringBuffer spQuery = new StringBuffer();
-		List<String> selectParamsToAdd = new ArrayList<String>();
+		List<String> selectParamsToAdd = new ArrayList<>();
 		selectParamsToAdd.addAll(optionalParamsToAdd);
 		selectParamsToAdd.addAll(mandatoryParams);
 		appendSelectString(selectParamsToAdd, spQuery);
@@ -1072,7 +1213,7 @@ public class ServerTapExamples extends DynamicTapForm {
 			if (whereLabel == null) {
 				whereLabel = queryName + " plus conesearch";
 			}
-			this.basicExamples.put(whereLabel, new CustomListCell(queryToDisplay + wherePart, SETTARGETTOOLTIP));
+			this.basicExamples.put(whereLabel, new CustomListCell(queryToDisplay + wherePart, wherePartToolTip));
 		}
 	}
 	
@@ -1119,6 +1260,14 @@ public class ServerTapExamples extends DynamicTapForm {
 		TAPEXDEFAULTMAXROWS = ConfigurationReader.getInstance().getPropertyValue("TAPEXDEFAULTMAXROWS");
 		CHANGESETTINGSTOOLTIP = Aladin.chaine.getString("CHANGESETTINGSTOOLTIP");
 		NODUPLICATESELECTION = Aladin.chaine.getString("NODUPLICATESELECTION");
+		JOINTABLETOOLTIP = Aladin.chaine.getString("JOINTABLETOOLTIP");
+		QUERIESREFRESHED = Aladin.chaine.getString("QUERIESREFRESHED");
+		TAPEXGENERATEDLABEL = Aladin.chaine.getString("TAPEXGENERATEDLABEL");
+		TAPEXGENERATEDTOOLTIP = Aladin.chaine.getString("TAPEXGENERATEDTOOLTIP");
+		TAPEXSERVICEPROVIDEDLABEL = Aladin.chaine.getString("TAPEXSERVICEPROVIDEDLABEL");
+		TAPEXSERVICEPROVIDEDTOOLTIP = Aladin.chaine.getString("TAPEXSERVICEPROVIDEDTOOLTIP");
+		SHOWSERVICEEXTIP = Aladin.chaine.getString("SHOWSERVICEEXTIP");
+		HIDESERVICEEXAMPLES = Aladin.chaine.getString("HIDESERVICEEXAMPLES");
 	}
 
 }
