@@ -1,10 +1,10 @@
-// Copyright 1999-2018 - Universit de Strasbourg/CNRS
+// Copyright 1999-2020 - Universit de Strasbourg/CNRS
 // The Aladin Desktop program is developped by the Centre de Donnes
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
 //
-//This file is part of Aladin.
+//This file is part of Aladin Desktop.
 //
 //    Aladin Desktop is free software: you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 //    GNU General Public License for more details.
 //
 //    The GNU General Public License is available in COPYING file
-//    along with Aladin.
+//    along with Aladin Desktop.
 //
 
 package cds.aladin;
@@ -328,7 +328,26 @@ public final class Slide {
          g.setColor( !button?Color.black:c);
          g.drawPolygon(bx,by,bx.length);
       }
-
+   }
+   
+   // dessin d'une petite horloge en haut  gauche du logo
+   private void drawClock(Graphics g,int dx,int dy, Color c, Color bg) {
+      int x = dx+frX[2]-27;
+      int y = dy+frY[2]-12;
+      int r=4;
+      drawClock(g,x,y,r,c,bg);
+   }
+   
+   /** Dessin d'une petit horloge */
+   static public void drawClock(Graphics g,int x,int y,int r, Color c, Color bg) {
+      Color c1= g.getColor();
+      if( bg!=null ) g.setColor( bg );
+      g.fillOval(x-r, y-r, 2*r, 2*r);
+      if( c!=null ) g.setColor(c);
+      g.drawOval(x-r, y-r, 2*r, 2*r);
+      g.drawLine(x, y, x, y-r+2);
+      g.drawLine(x, y, x+r-3, y);
+      g.setColor(c1);
    }
    
    // Adaptation du logo pour un plan IMAGE
@@ -362,6 +381,19 @@ public final class Slide {
       g.drawRect(dx+frX[2]-15,dy+frY[2]-8,5,5);
    }
    
+   // Adaptation du logo pour un plan TMOC
+   static void drawLogoTMOC(Graphics g,int dx,int dy,Color c) {
+      int x = dx+frX[2]-24;
+      int y = dy+frY[2]-9;
+      int h=5;
+      g.setColor(c);
+      g.drawRect(x,y,1,h); x+=2+1;
+      g.drawRect(x,y,5,h); x+=2+5;
+      g.drawRect(x,y,2,h); x+=3+2;
+      g.drawRect(x,y,1,h);
+      
+   }
+   
    // Adaptation du logo pour un plan MOC
    static void drawLogoMOC(Graphics g,int dx,int dy,Color c) {
       int x = dx+gapL+10;//frX[2]/2;
@@ -656,6 +688,13 @@ public final class Slide {
             g.drawLine(xc[1],yc[1]+1,xc[2],yc[1]+1);
          }
          
+         // dtermination de la couleur de l'intrieur du logo pour les plans dpendants d'un PlanBgRgb
+         Color c = Color.black;
+         if( p instanceof PlanBG && ((PlanBG)p).hasRecutListener() ) {
+            PlanBGRgb p1 = ((PlanBG)p).getRecutListener();
+            c = p1.red==p ? Color.red : p1.green==p ? Color.GREEN : Color.blue;
+         }
+         
          // Le logo du plan en fonction de son type
          switch( p.type ) {
             case Plan.IMAGE:
@@ -664,9 +703,11 @@ public final class Slide {
             case Plan.IMAGECUBERGB:
             case Plan.IMAGERGB:    drawLogoImg(g,dx,dy,null);                                 break;
             case Plan.IMAGEHUGE:   drawLogoImgHuge(g,dx,dy,colorForeground);                  break;
-            case Plan.ALLSKYMOC:   drawLogoMOC(g,dx,dy,isViewable?p.c:colorFillFG);           break;
+            case Plan.ALLSKYTMOC:  drawLogoTMOC(g,dx,dy,isViewable?p.c:colorFillFG);          break;
+            case Plan.ALLSKYMOC:   
+            case Plan.ALLSKYSTMOC: drawLogoMOC(g,dx,dy,isViewable?p.c:colorFillFG);           break;
             case Plan.ALLSKYCAT:   drawLogoImgBG(g,dx,dy,isViewable?p.c:colorFillFG);         break;
-            case Plan.ALLSKYIMG:   drawLogoImgBG(g,dx,dy,isViewable?Color.black:colorFillFG); break;
+            case Plan.ALLSKYIMG:   drawLogoImgBG(g,dx,dy,isViewable?c:colorFillFG);           break;
             case Plan.ALLSKYPOL:   drawLogoPolarisation(g,dx,dy,isViewable?p.c:colorFillFG);  break;
             case Plan.IMAGEMOSAIC:
             case Plan.IMAGECUBE: 
@@ -682,6 +723,8 @@ public final class Slide {
          
          // On dessine 4 CCD cote  cote
          if( p instanceof PlanMultiCCD )  drawFolderExt(g, ((PlanMultiCCD)p).getSize(), dx, dy, Color.yellow );
+         
+         if( p.isTime() ) drawClock(g,dx,dy,Color.black,Color.white);
                      
          // Affichage de la checkbox
 //         if( p.type!=Plan.NO && mode!=DRAG && p.flagOk && !p.hasError()
