@@ -1,10 +1,10 @@
-// Copyright 1999-2018 - Universit de Strasbourg/CNRS
+// Copyright 1999-2020 - Universit de Strasbourg/CNRS
 // The Aladin Desktop program is developped by the Centre de Donnes
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
 //
-//This file is part of Aladin.
+//This file is part of Aladin Desktop.
 //
 //    Aladin Desktop is free software: you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 //    GNU General Public License for more details.
 //
 //    The GNU General Public License is available in COPYING file
-//    along with Aladin.
+//    along with Aladin Desktop.
 //
 
 package cds.aladin;
@@ -66,7 +66,7 @@ public class Source extends Position implements Comparator {
 
    protected byte sourceType=SQUARE;    //Type de representation de la source par dfaut (CARRE, ...)
    protected String info;       // Information supplementaire associee a la source (en plus de id)
-   protected Legende leg;       // La legende associee a la source
+   private Legende leg;       // La legende associee a la source
    private String oid=null;     // L'OID de la source s'il a ete defini
 
    /**** variables lis aux filtres ****/
@@ -105,8 +105,21 @@ public class Source extends Position implements Comparator {
       super(plan,null,0.,0.,raj,dej,RADE,id);
       this.info = info;
       sourceType=(byte)plan.sourceType;
+      
       fixInfo();
    }
+   
+   /** Creation d'un objet source
+    * @param plan plan d'appartenance de la ligne
+    * @param raj,dej  coordonnees de l'objet
+    * @param jdtime   date associe  l'objet
+    * @param id       identificateur de l'objet
+    * @param info     information supplementaire
+    */
+    protected Source(Plan plan, double raj, double dej, double jdtime, String id, String info) {
+       this(plan,raj,dej,id,info);
+       this.jdtime = jdtime;
+    }
 
   /** Creation d'un objet source
    * @param plan plan d'appartenance de la ligne
@@ -118,9 +131,9 @@ public class Source extends Position implements Comparator {
                     String info, Legende leg) {
       super(plan,null,0.,0.,raj,dej,RADE,id);
       this.info = info;
-      this.leg = leg;
+      this.leg=leg;
       sourceType=(byte)plan.sourceType;
-
+      
       fixInfo();
    }
 
@@ -137,13 +150,28 @@ public class Source extends Position implements Comparator {
                     String id, String info, Legende leg) {
       super(plan,null,x,y,raj,dej,methode,id);
       this.info = info;
-      this.leg = leg;
+      this.leg=leg;
       sourceType=(byte)plan.sourceType;
-
+      
       fixInfo();
 
    }
    
+   /** Retourne la lgende associe  la source */
+   public Legende getLeg() {
+      
+      // Dans le cas d'un plan HiPs catalogue, la lgende peut tre gnrique
+      if( plan!=null && plan instanceof PlanBGCat ) {
+         Legende l = plan.getFirstLegende();
+         if( l!=null ) return l;
+      }
+      
+      return leg;
+   }
+
+   /** Positionne la lgende associe  la source */
+   protected void setLeg(Legende leg) { this.leg = leg; }
+
    /** Accroit ou dcroit la taille du type de source */
    void increaseSourceSize(int sens) { 
       sourceType+=sens;
@@ -158,7 +186,7 @@ public class Source extends Position implements Comparator {
     */
     protected void projection(ViewSimple v) {
 
-       if( v.isPlotView() ) {
+       if( v.isPlot() ) {
           double [] xy = v.plot.getXY(this);
           xv[v.n] = xy[0];
           yv[v.n] = xy[1];
@@ -188,7 +216,7 @@ public class Source extends Position implements Comparator {
    final protected boolean isHighlighted() { return (flags & HIGHLIGHT) !=0; /* == HIGHLIGHT;*/ }
 
    /** Retourne true si l'objet contient des informations de photomtrie  */
-   public boolean hasPhot() { return leg.hasGroup(); }
+   public boolean hasPhot() { return getLeg().hasGroup(); }
 
    /** Retourne le nom de la forme en fonction de l'indice */
    static protected final String getShape(int i) { return TYPENAME[i]; }
@@ -201,12 +229,12 @@ public class Source extends Position implements Comparator {
 
    /** fix for the AVO demo : sometimes, info is shorter than leg ! */
    protected void fixInfo() {
-       if( leg==null || leg.field==null || info==null ) return;
+       if( getLeg()==null || getLeg().field==null || info==null ) return;
 
        StringTokenizer st = new StringTokenizer(this.info,"\t");
        int nbInfo = st.countTokens()-1; // skip du triangle
        
-       int nbFields = leg.field.length;
+       int nbFields = getLeg().field.length;
 
        while( nbInfo<nbFields ) {
            this.info = new String(this.info)+"\t ";
@@ -214,7 +242,7 @@ public class Source extends Position implements Comparator {
            if( Aladin.levelTrace>=3) System.err.println("Source.fixInfo() =>  pour "+id);
        }
    }
-
+   
    /**
     * Retourne l'identificateur unique de la source, fourni par une application
     * externe tel VOPlot. S'il ne s'agit pas d'un objet de ce type, retourne
@@ -282,12 +310,15 @@ public class Source extends Position implements Comparator {
   /** Modification de l'information associee a la source
    * @param info la nouvelle info supplementaire
    */
-   public void setInfo(String info) { this.info = info; oid=""; }
+   public void setInfo(String info) { 
+      this.info = info; 
+      oid="";
+   }
 
   /** Modification de la legende associee a la source
    * @param leg la nouvelle legende
    */
-   protected void setLegende(Legende leg) { this.leg = leg; oid=""; }
+   protected void setLegende(Legende leg) {  this.leg=leg; oid=""; }
 
    // Affichage de l'info lie a la source
    protected void info(Aladin aladin) { aladin.mesure.setInfo(this); }
@@ -654,8 +685,8 @@ public class Source extends Position implements Comparator {
 
    /** Retourne le premier link associe  la source si il existe, sinon retourne "-" */
    protected String getFirstLink() {
-      if( leg==null ) return "-";
-      int i = leg.getFirstLink();
+      if( getLeg()==null ) return "-";
+      int i = getLeg().getFirstLink();
       if( i<0 ) return "-";
       String s;
       try {
@@ -679,6 +710,8 @@ public class Source extends Position implements Comparator {
    * @param v reference  la vue o on dessine
    */
    protected boolean draw(Graphics g,ViewSimple v,int dx,int dy) {
+      if( !inTime( v )  ) return false;
+
       //System.out.println("On repaint");
       int L =getL();
       Point p = getViewCoord(v,L,L);
@@ -860,7 +893,9 @@ public class Source extends Position implements Comparator {
     	int curPos;
     	String curUCD = null;
 
-    	Field[] fields = leg.field;
+        if( getLeg()==null ) return -1;
+
+        Field[] fields = getLeg().field;
     	// ucd contient-elle des wildcards ?
     	boolean wildcard = useWildcard(ucd);
 
@@ -883,8 +918,10 @@ public class Source extends Position implements Comparator {
    protected int findUtype(String utype) {
        int curPos;
        String curUtype = null;
+       
+       if( getLeg()==null ) return -1;
 
-       Field[] fields = leg.field;
+       Field[] fields = getLeg().field;
        // utype contient-il des wildcards ?
        boolean wildcard = useWildcard(utype);
 
@@ -938,11 +975,12 @@ public class Source extends Position implements Comparator {
     	int curPos;
     	String curName = null;
 
+        if( getLeg()==null ) return -1;
+
         // replace ajout pour la dmo AVO
         name = MetaDataTree.replace(name, " ", "", -1);
 
-
-    	Field[] fields = leg.field;
+    	Field[] fields = getLeg().field;
     	// name contient-elle des wildcards ?
 		boolean wildcard = useWildcard(name);
 
@@ -1002,12 +1040,32 @@ public class Source extends Position implements Comparator {
      * tag GLU pour les liens
      */
     protected String getCodedValue(int index) throws NoSuchElementException {
-       StringTokenizer st = new StringTokenizer(this.info,"\t");
-       st.nextElement();     // skip the triangle
-       for(int i=0;i<index;i++) st.nextElement();
-       return st.nextElement().toString();
+       index++;   // skip du triangle  (il y a toujours une premire valeur)
+       int deb= -1;
+       int n=info.length();
+       int i=0;
+       for( int j=0; i<n && j<index+1; i++) {
+          if( info.charAt(i)=='\t') {
+             j++;
+             if( j==index && deb==-1 ) deb=i+1;
+          }
+       }
+       if( deb==-1 ) throw new NoSuchElementException();
+       
+       // BUG FIX 1/1/2020 PF
+       if( i<n ) i--;
+       
+       return info.substring(deb,i);
     }
 
+    // VERSION ORIGINALE BIEN PLUS LENTE
+//    protected String getCodedValue(int index) throws NoSuchElementException {
+//       StringTokenizer st = new StringTokenizer(this.info,"\t");
+//       st.nextElement();     // skip the triangle
+//       for(int i=0;i<index;i++) st.nextElement();
+//       return st.nextElement().toString();
+//    }
+
 
    /** Returns the value of the field at position index
     *	@param index - the position of the field one wants
@@ -1017,7 +1075,7 @@ public class Source extends Position implements Comparator {
     	String ret;
       	try {
       	   ret = getCodedValue(index);
-      	   if( leg.isNullValue(ret, index) ) ret="";
+      	   if( getLeg().isNullValue(ret, index) ) ret="";
 
            // Pierre: En cas de marques GLU
            if( ret.startsWith("<&") ) {
@@ -1043,7 +1101,7 @@ public class Source extends Position implements Comparator {
        // sinon on passe par un Vector temporaire et on recopie  la fin
        String [] v = null;
        Vector tmp=null;
-       if( leg.field.length>0 ) v = new String[ leg.field.length ];
+       if( getLeg().field.length>0 ) v = new String[ getLeg().field.length ];
        else tmp = new Vector();
 
        st.nextElement();  // Skip le triangle
@@ -1078,16 +1136,17 @@ public class Source extends Position implements Comparator {
      */
     public boolean setValue(int index,String value) {
        StringTokenizer st = new StringTokenizer(info,"\t");
-       StringBuffer nInfo=null;
+       StringBuilder nInfo=null;
        boolean encore;
        index++;
        for( int i=0; (encore=st.hasMoreTokens()) || i<=index; i++ ) {
           String s = encore ? st.nextToken() : "";
           if( i==index ) s=value;
-          if( i==0 ) nInfo = new StringBuffer(s);
+          if( i==0 ) nInfo = new StringBuilder(s);
           else nInfo.append("\t"+s);
        }
        info = nInfo.toString();
+
        return true;
     }
 
@@ -1117,9 +1176,9 @@ public class Source extends Position implements Comparator {
        setColumn(index,name,null,unit,ucd,width);
     }
     public void setColumn(int index, String name,String datatype,String unit,String ucd,int width) {
-       if( leg==null ) leg = new Legende();
-       int newCol = leg.setField(index,name,datatype,unit,ucd,width);
-       if( newCol>0 && plan!=null && plan.pcat!=null ) plan.pcat.fixInfo(leg);
+       if( leg==null ) leg=new Legende();
+       int newCol = getLeg().setField(index,name,datatype,unit,ucd,width);
+       if( newCol>0 && plan!=null && plan.pcat!=null ) plan.pcat.fixInfo(getLeg());
     }
 
     /** Return the index of a column (Source object). Proceed in 2 steps,
@@ -1130,19 +1189,19 @@ public class Source extends Position implements Comparator {
      * @return index of first column matching the key
      */
     public int indexOf(String key) {
-       if( leg==null ) return -1;
-       for( int i=0; i<leg.field.length; i++ ) {
-          if( Util.matchMask(key,leg.field[i].name) ) return i;
+       if( getLeg()==null ) return -1;
+       for( int i=0; i<getLeg().field.length; i++ ) {
+          if( Util.matchMask(key,getLeg().field[i].name) ) return i;
        }
-       for( int i=0; i<leg.field.length; i++ ) {
-          if( Util.matchMask(key,leg.field[i].ucd) ) return i;
+       for( int i=0; i<getLeg().field.length; i++ ) {
+          if( Util.matchMask(key,getLeg().field[i].ucd) ) return i;
        }
        return -1;
     }
 
     /** Return the number of columns associated to this object */
     public int getSize() {
-       if( leg!=null ) return leg.getSize();
+       if( getLeg()!=null ) return getLeg().getSize();
        if( info==null ) return 0;
        return new StringTokenizer(info,"\t").countTokens();
     }
@@ -1196,19 +1255,19 @@ public class Source extends Position implements Comparator {
      *  @param m 0:label, 1:unit,  2:ucd
      */
     private String [] getMeta(int m) {
-       if( leg==null ) return new String[0];
-       String [] u = new String[leg.getSize()];
+       if( getLeg()==null ) return new String[0];
+       String [] u = new String[getLeg().getSize()];
        for( int i=0; i<u.length; i++ ) {
           switch(m) {
-             case 0: u[i]=leg.field[i].name; break;
-             case 1: u[i]=leg.field[i].unit; break;
-             case 2: u[i]=leg.field[i].ucd;  break;
-             case 3: u[i]=leg.field[i].datatype;  break;
-             case 4: u[i]=leg.field[i].arraysize;  break;
-             case 5: u[i]=leg.field[i].width;  break;
-             case 6: u[i]=leg.field[i].precision;  break;
-             case 7: u[i]=leg.field[i].nullValue;  break;
-             case 8: u[i]=leg.field[i].description;  break;
+             case 0: u[i]=getLeg().field[i].name; break;
+             case 1: u[i]=getLeg().field[i].unit; break;
+             case 2: u[i]=getLeg().field[i].ucd;  break;
+             case 3: u[i]=getLeg().field[i].datatype;  break;
+             case 4: u[i]=getLeg().field[i].arraysize;  break;
+             case 5: u[i]=getLeg().field[i].width;  break;
+             case 6: u[i]=getLeg().field[i].precision;  break;
+             case 7: u[i]=getLeg().field[i].nullValue;  break;
+             case 8: u[i]=getLeg().field[i].description;  break;
           }
        }
        return u;
@@ -1218,13 +1277,13 @@ public class Source extends Position implements Comparator {
      * @return XML string, or null
      */
     public String getXMLMetaData() {
-       return leg.getGroup();
+       return getLeg().getGroup();
     }
 
    /** Returns the unit for the field at position pos */
    protected String getUnit(int pos) {
     	if(pos<0) return "";
-    	String u = leg.field[pos].unit;
+    	String u = getLeg().field[pos].unit;
     	if( u!=null ) u = u.replace("year","yr");   // pour faire plaisir  l'ESAC pour Gaia qui utilise des units non conformes ni  l'IVOA, ni  l'UAI
     	return u;
    }
@@ -1239,12 +1298,12 @@ public class Source extends Position implements Comparator {
     * @return une chaine STC-S 
     */
    public String createSIAFoV() {
-      if( leg==null ) return null;
+      if( getLeg()==null ) return null;
       try {
-         int iScale = leg.findUCD("VOX:Image_Scale");
-         int iNaxis = leg.findUCD("VOX:Image_Naxes");
-         int iSize  = leg.findUCD("VOX:Image_Naxis");
-         int iCD    = leg.findUCD("VOX:WCS_CDMatrix");
+         int iScale = getLeg().findUCD("VOX:Image_Scale");
+         int iNaxis = getLeg().findUCD("VOX:Image_Naxes");
+         int iSize  = getLeg().findUCD("VOX:Image_Naxis");
+         int iCD    = getLeg().findUCD("VOX:WCS_CDMatrix");
          
          // Nombre de dimensions ?
          String val[] = getValues();
@@ -1279,7 +1338,7 @@ public class Source extends Position implements Comparator {
                double cd01 = Double.parseDouble( st.nextToken() );
                st.nextToken();
                double cd11 = Double.parseDouble( st.nextToken() );
-               angle = Math.atan2(cd01, cd11)*180.0/Math.PI;
+               angle = 90 - Math.atan2(cd01, cd11)*180.0/Math.PI;
                if( Double.isNaN(angle) ) angle=0;
             } catch( Exception e ) { angle=0; }
          }
@@ -1401,8 +1460,8 @@ public class Source extends Position implements Comparator {
       }
 
       // Il s'agit d'une source non concerne, on met  la fin
-      if( a.leg!=sortSource.leg ) return 1;
-      if( b.leg!=sortSource.leg ) return -1;
+      if( a.getLeg()!=sortSource.getLeg() ) return 1;
+      if( b.getLeg()!=sortSource.getLeg() ) return -1;
 
       String aVal = a.getValue(sortNField);
       String bVal = b.getValue(sortNField);
@@ -1433,13 +1492,13 @@ public class Source extends Position implements Comparator {
     * @param sens 1 - ascendant, -1 descendant
     */
    static protected void setSort(Source s, int nField, int sens) {
-      sortNumeric = s.leg.isNumField(nField);
+      sortNumeric = s.getLeg().isNumField(nField);
       sortSource  = s;
       sortNField  = nField;
       sortSens    = sens;
       Aladin.trace(1,"Measurement "+(sortNumeric?"numerical ":"alphanumerical ")
             +(sens==1?"ascending":"descending")+" sort on "
-            +(s.leg==null||nField==-1?"field "+nField:"${"+s.leg.field[nField].name)+"}");
+            +(s.getLeg()==null||nField==-1?"field "+nField:"${"+s.getLeg().field[nField].name)+"}");
 
    }
 
