@@ -1,10 +1,10 @@
-// Copyright 1999-2018 - Universit de Strasbourg/CNRS
+// Copyright 1999-2020 - Universit de Strasbourg/CNRS
 // The Aladin Desktop program is developped by the Centre de Donnes
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
 //
-//This file is part of Aladin.
+//This file is part of Aladin Desktop.
 //
 //    Aladin Desktop is free software: you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -16,23 +16,23 @@
 //    GNU General Public License for more details.
 //
 //    The GNU General Public License is available in COPYING file
-//    along with Aladin.
+//    along with Aladin Desktop.
 //
 
 package cds.aladin;
 import static cds.aladin.Constants.ADQLVALUE_FORBETWEEN;
 import static cds.aladin.Constants.CHANGESERVER;
 import static cds.aladin.Constants.EMPTYSTRING;
-import static cds.aladin.Constants.TEMPLATES;
 import static cds.aladin.Constants.GENERIC;
 import static cds.aladin.Constants.GLU;
+import static cds.aladin.Constants.NODE;
 import static cds.aladin.Constants.OBSCORE;
 import static cds.aladin.Constants.REGEX_OPALPHANUM;
 import static cds.aladin.Constants.REGEX_OPNUM;
 import static cds.aladin.Constants.REGEX_RANGENUMBERINPUT;
 import static cds.aladin.Constants.REGEX_TAPSCHEMATABLES;
 import static cds.aladin.Constants.TAPFORM_STATUS_NOTLOADED;
-import static cds.aladin.Constants.NODE;
+import static cds.aladin.Constants.TEMPLATES;
 
 import java.awt.Color;
 import java.awt.FlowLayout;
@@ -82,7 +82,7 @@ public class TapClient{
 	public static String modesLabel = "Modes";
 	public static String modesToolTip;
 	public DefaultComboBoxModel model = null;
-	public static String RELOAD, TIPRELOAD, GENERICERROR, TARGETERROR, NOGLURECFOUND, CHANGESERVERTOOLTIP;
+	public static String RELOAD, TIPRELOAD, GENERICERROR, TARGETERROR, NOGLURECFOUND, CHANGESERVERTOOLTIP, INVALIDNUMBERINPUT;
 	
 	public TapManager tapManager;
 	public String tapLabel;
@@ -108,7 +108,7 @@ public class TapClient{
 	Color primaryColor = Aladin.BLUE;
     Color secondColor = new Color(198,218,239);
     private boolean editing = false;
-    public Map<String, TapTable> obscoreTables = new LinkedHashMap<String, TapTable>();
+    public Map<String, TapTable> obscoreTables = new LinkedHashMap<>();
     
     public TapClient() {
 		// TODO Auto-generated constructor stub
@@ -122,7 +122,7 @@ public class TapClient{
 		}
     	if (nodeTable != null) {
     		this.nodeName = nodeTable;
-        	this.nodeTableNames = new ArrayList<String>();
+        	this.nodeTableNames = new ArrayList<>();
     		this.nodeTableNames.add(nodeTable);
 		}
     	
@@ -136,18 +136,18 @@ public class TapClient{
 			String[] modesAllowed = null;
 			
 			if (this.nodeName != null) {
-				if (Aladin.PROTO) {//TODO:: tintinproto
+//				if (Aladin.PROTO) {
 					modesAllowed = new String []{ /*GLU, */nodeName, GENERIC, TEMPLATES, OBSCORE };
-				} else {
-					modesAllowed = new String []{ /*GLU, */nodeName, GENERIC};
-				}
+//				} else {
+//					modesAllowed = new String []{ /*GLU, */nodeName, GENERIC};
+//				}
 				
 			} else {
-				if (Aladin.PROTO) {//TODO:: tintinproto
+//				if (Aladin.PROTO) {
 					modesAllowed = new String []{ /*GLU, */GENERIC, TEMPLATES, OBSCORE };
-				} else {
-					modesAllowed = new String []{ /*GLU, */ GENERIC};
-				}
+//				} else {
+//					modesAllowed = new String []{ /*GLU, */ GENERIC};
+//				}
 				
 			}
 			model = new DefaultComboBoxModel(modesAllowed); 
@@ -188,7 +188,6 @@ public class TapClient{
 							} else {
 								TapClient.this.tapManager.showTapServerOnServerSelector(serverToDisplay);
 							}
-
 						}
 					} catch (Exception ex) {
 						if (Aladin.levelTrace >= 3)
@@ -206,20 +205,21 @@ public class TapClient{
 		}
 	}
 	
+	@Deprecated
 	public static TapClient getUploadTapClient(Aladin aladin, String tapLabel, String mainServerUrl) {
 		TapManager tapManager = TapManager.getInstance(aladin);
 		TapClient tapClient = new TapClient(TapClientMode.UPLOAD, tapManager, tapLabel, mainServerUrl, null);
 		tapClient.serverTap = new ServerTap(aladin);
 		tapClient.serverTap.tapClient = tapClient;
-		tapClient.primaryColor = new Color(198,218,239);
-		tapClient.secondColor = Aladin.COLOR_CONTROL_FOREGROUND;
-		tapClient.tablesMetaData = new HashMap<String, TapTable>();
+//		tapClient.primaryColor = new Color(198,218,239);
+//		tapClient.secondColor = Aladin.COLOR_CONTROL_FOREGROUND;
+		tapClient.tablesMetaData = new HashMap<>();
 		return tapClient;
 	}
 	
 	public JButton getChangeServerButton(Server server) {
 		JButton button = null;
-		if (this.mode != TapClientMode.UPLOAD && this.mode != TapClientMode.STANDALONE) {
+		if (this.mode != TapClientMode.STANDALONE) {
 			Image image = Aladin.aladin.getImagette("changeServerOptions.png");
 			if (image == null) {
 				button = new JButton("Change server");
@@ -238,7 +238,7 @@ public class TapClient{
 	
 	public JPanel getModes(Server server) {
 		JPanel optionsPanel = null;
-		if (this.mode != TapClientMode.UPLOAD && this.mode != TapClientMode.STANDALONE) {
+		if (this.mode != TapClientMode.STANDALONE) {
 			optionsPanel = new JPanel(new FlowLayout(FlowLayout.RIGHT, 0 , 0));
 			server.modeChoice = new JComboBox(model);
 			server.modeChoice.setRenderer(new TapClientModesRenderer(this));
@@ -320,7 +320,12 @@ public class TapClient{
 			} else {//preference is for what is already loaded
 				if (this.serverTapNode != null && this.serverTapNode.isLoaded()) {
 					dynamicTapForm = this.serverTapNode;
-					model.setSelectedItem(NODE);
+					if (nodeName != null) {
+						model.setSelectedItem(nodeName);
+					} else {
+						model.setSelectedItem(NODE);
+					}
+					
 				} else if (this.serverTap != null && this.serverTap.isLoaded()) {
 					dynamicTapForm = this.serverTap;
 					model.setSelectedItem(GENERIC);
@@ -349,10 +354,10 @@ public class TapClient{
 					tapManager.createAndLoadATapServer(this, dynamicTapForm);
 				} else {
 					dynamicTapForm.showloading();
-					if (this.mode == TapClientMode.TREEPANEL) {
-						this.primaryColor = Aladin.COLOR_FOREGROUND;
-						this.secondColor = Color.white;
-					}
+//					if (this.mode == TapClientMode.TREEPANEL) {
+//						this.primaryColor = Aladin.COLOR_FOREGROUND;
+//						this.secondColor = Color.white;
+//					}
 					tapManager.createGenericTapFormFromMetaData(dynamicTapForm);
 				}
 			}
@@ -366,9 +371,10 @@ public class TapClient{
 	/**
 	 * Method to update the frame info panel.
 	 * 	if the old frame is visible the method reloads it.
+	 * @param newServer 
 	 * @param newInfoPanel
 	 */
-	protected void tackleFrameInfoServerUpdate(Aladin aladin, Future<JPanel> newInfoPanel) {
+	protected void tackleFrameInfoServerUpdate(Aladin aladin, DynamicTapForm newServer, Future<JPanel> newInfoPanel) {
 		try {
 			FrameInfoServer frameInfoServer = null;
 			if (this.infoPanel!=null) {
@@ -388,9 +394,15 @@ public class TapClient{
 				} else {
 					this.infoPanel.cancel(true);
 				}
-			} else if (aladin.frameInfoServer != null && aladin.frameInfoServer.getServer().equals(this)) {
+			} else if (aladin.frameInfoServer != null && aladin.frameInfoServer.getServer() != null
+					&& aladin.frameInfoServer.getServer().equals(newServer)) {
 				//this else part is for a specific case where generic status report is displayed before table meta can be obtained
-				frameInfoServer = new FrameInfoServer(aladin, newInfoPanel);
+				if (newInfoPanel == null) {
+					frameInfoServer = new FrameInfoServer(aladin);
+				} else {
+					frameInfoServer = new FrameInfoServer(aladin, newInfoPanel);
+				}
+				
 				if (aladin.frameInfoServer.isVisible()) {
 					frameInfoServer.updateInfoPanel();
 					aladin.frameInfoServer.dispose();
@@ -415,17 +427,13 @@ public class TapClient{
 
 	}
 	
-	public void	showOnUploadFrame() {
-		this.tapManager.showOnUploadFrame(this);
-	}
-	
 	/**
 	 * Method sets the database metadata to the parser
 	 * and sorts tap tables into the order where there is understanding 
 	 */
 	public void preprocessTapClient() {
 		Map<String, TapTable> backUpTablesMetaData = this.tablesMetaData;
-		List<Map.Entry<String, TapTable>> sorter = new LinkedList<Map.Entry<String,TapTable>>();
+		List<Map.Entry<String, TapTable>> sorter = new LinkedList<>();
 		sorter.addAll(this.tablesMetaData.entrySet());
 		
 		Collections.sort(sorter, new Comparator<Map.Entry<String,TapTable>>() {
@@ -451,15 +459,15 @@ public class TapClient{
 		});
 		
 		if (this.queryCheckerTables == null) {//initialise for the first time
-			this.queryCheckerTables = new ArrayList<DefaultDBTable>();
+			this.queryCheckerTables = new ArrayList<>();
 		} else {
 			this.queryCheckerTables.clear();
 		}
 		//sory as per size.
 		//make obscore second choice- if it exixts
 		//place tap schema at the last
-		Map<String, TapTable> sorted = new LinkedHashMap<String, TapTable>();
-		Map<String, TapTable> tapSchemaMeta = new LinkedHashMap<String, TapTable>();
+		Map<String, TapTable> sorted = new LinkedHashMap<>();
+		Map<String, TapTable> tapSchemaMeta = new LinkedHashMap<>();
 		for (Entry<String, TapTable> metaDataEntry : sorter) {
 			DefaultDBTable parserTable = new DefaultDBTable(metaDataEntry.getKey());
 			Vector<TapTableColumn> columnsMeta = backUpTablesMetaData.get(metaDataEntry.getKey()).getColumns();
@@ -477,7 +485,7 @@ public class TapClient{
 		this.tapManager.setTargetDimensions(this);
 	}
 
-	public boolean isSchemaTable(String tableName) {
+	public static boolean isSchemaTable(String tableName) {
 		// TODO Auto-generated method stub
 		boolean result = false;
 		Pattern tapSchemaPattern = Pattern.compile(REGEX_TAPSCHEMATABLES, Pattern.CASE_INSENSITIVE);
@@ -531,7 +539,7 @@ public class TapClient{
 		}
 	}
 	
-	//TODO::: tintin refactor may be move this to obstap server constructor
+	//TODO::: refactor may be move this to obstap server constructor
 	public boolean parseForObscore(String tableName, TapTable tapTable) {
 		String priRaColumnName = tapTable.getRaColumnName();
 		String priDecColumnName = tapTable.getDecColumnName();
@@ -709,7 +717,7 @@ public class TapClient{
 				inputInProgress = input;
 			}
 		} else if (showError) {
-			Aladin.error(s, input + " is incorrect! Please rectify. Valid examples are: >3, 10..12, <=-788 etc..");
+			Aladin.error(s, s.getNumberIncorrectMessage(input));
 			s.ball.setMode(Ball.NOK);
 		}
 		return inputInProgress;
@@ -771,6 +779,7 @@ public class TapClient{
 		
 		if (this.serverTap != null) {
 			this.serverTap.removeAll();
+			this.serverTap.resetJoin();
 			this.serverTap.formLoadStatus = TAPFORM_STATUS_NOTLOADED;
 		} 
 		if (this.serverExamples != null) {
@@ -783,6 +792,7 @@ public class TapClient{
 			this.serverObsTap.removeAll();
 			this.serverObsTap.formLoadStatus = TAPFORM_STATUS_NOTLOADED;
 		}
+		this.infoPanel = null;
 		tapManager.createAndLoadATapServer(this, dynamicTapForm);
 		if (this.mode == TapClientMode.TREEPANEL) {
 			tapManager.showTapPanelFromTree(tapLabel, dynamicTapForm);
@@ -795,7 +805,7 @@ public class TapClient{
 	
 	public boolean isUploadAllowed() {
 		boolean result = false;
-		if (this.capabilities != null) {
+		if (this.capabilities != null /* && Aladin.PROTO */) {
 			VOSICapabilitiesReader meta;
 			try {
 				meta = this.capabilities.get();
@@ -811,25 +821,97 @@ public class TapClient{
 		return result;
 	}
 	
-	public void updateUploadedTablesToParser(DynamicTapForm server) {
+	/**
+	 * Method trying to update upload meta needed to validate query and submit it.
+	 * Parsing and other issues are not tackled here
+	 * @param server
+	 * @param requestParams
+	 * @throws Exception 
+	 */
+	public void updateUploadedTablesToADQLParser(DynamicTapForm server, Map<String, Object> requestParams) throws Exception {
 		// TODO Auto-generated method stub
-		if ((this.mode != TapClientMode.UPLOAD)) {
-			List<DefaultDBTable> queryCheckerTables = new ArrayList<DefaultDBTable>();
-			if (this.queryCheckerTables != null && !this.queryCheckerTables.isEmpty()) {
-				queryCheckerTables.addAll(this.queryCheckerTables);
-			}
-			if (tapManager.uploadFrame != null) {
-				List<String> uploadedTables = new ArrayList<String>();
-				uploadedTables.addAll(tapManager.uploadFrame.uploadClient.tablesMetaData.keySet());
-				if (!uploadedTables.isEmpty()) {
-					for (String uploadedTable : uploadedTables) {
-						server.updateQueryChecker(true, uploadedTable, tapManager.uploadFrame.uploadClient.tablesMetaData, queryCheckerTables);
+		/*Map<String, Object> requestParams = null; //added off in  updateUploadedTablesToParser. keeping for ref
+		if (secondaryTable != null) {
+			TapManager tapManager = TapManager.getInstance(aladin);
+			if (!this.tapClient.tablesMetaData.containsKey(secondaryTable)) {
+				if (tapManager.uploadFrame.uploadedTableFiles.containsKey(secondaryTable)) {
+					if (tap.getText().contains(secondaryTable)) {
+						requestParams = new HashMap<String, Object>();
+						FrameUploadServer uploadFrame = tapManager.uploadFrame;
+						if (uploadFrame.uploadedTableFiles.get(secondaryTable) == null) {
+							Aladin.error(this, "Unable to submit " + secondaryTable + " data!");
+							return;
+						}
+						String uploadFileName = FrameUploadServer.UPLOADFILEPREFIX+secondaryTable;
+						requestParams.put("upload", uploadFrame.getUploadParam(secondaryTable, uploadFileName));
+						requestParams.put(uploadFileName, uploadFrame.uploadedTableFiles.get(secondaryTable));
 					}
 				}
 			}
-			QueryChecker checker = new DBChecker(queryCheckerTables);
-			server.adqlParser.setQueryChecker(checker);
+		}*/
+		//if there are no uploaded table?
+		//check if query has any of those references
+		//if yes update
+		
+		List<DefaultDBTable> queryCheckerTablesUpdate = new ArrayList<>();
+		if (this.queryCheckerTables != null && !this.queryCheckerTables.isEmpty()) {
+			queryCheckerTablesUpdate.addAll(this.queryCheckerTables);
 		}
+		
+		Map<String, TapTable> uploadedTables = tapManager.getUploadedTables();
+		List<String> uploadTablesReferenced = new ArrayList<>();
+		if (uploadedTables != null) {
+			for (String uploadtable : uploadedTables.keySet()) {
+				if (server.tap.getText().toUpperCase().contains(uploadtable.toUpperCase())) {
+					if (requestParams != null
+							&& !tapManager.uploadFacade.uploadTableNameDict.containsValue(uploadtable)) {
+						Aladin.error(server, "Unable to submit " + uploadtable + " data!");
+						Aladin.trace(3, "No file found for "+uploadtable);
+					} else{
+						uploadTablesReferenced.add(uploadtable);
+					}
+					server.updateQueryChecker(true, uploadtable, tapManager.uploadFacade.uploadTablesMetaData, queryCheckerTablesUpdate);
+				}
+			}
+			
+			tapManager.uploadFacade.addUploadToSubmitParams(uploadTablesReferenced, requestParams);
+			
+			QueryChecker checker = new DBChecker(queryCheckerTablesUpdate);
+			server.adqlParser.setQueryChecker(checker);
+			
+		}
+		
+		
+		/*try {
+			ADQLQuery query =  server.adqlParser.parseQuery(server.tap.getText());
+		} catch (UnresolvedIdentifiersException e) {
+			// TODO Auto-generated catch block
+			List<String> tableNames = server.getTableNamesofNoMetadataInQuery(server.tap.getText());
+			if (requestParams != null && tableNames != null && !tableNames.isEmpty()) {
+				this.tapManager.getUploadSubmitParams(tableNames, requestParams);
+			}
+			
+			if ((this.mode != TapClientMode.UPLOAD)) {
+				List<DefaultDBTable> queryCheckerTables = new ArrayList<DefaultDBTable>();
+				if (this.queryCheckerTables != null && !this.queryCheckerTables.isEmpty()) {
+					queryCheckerTables.addAll(this.queryCheckerTables);
+				}
+				if (tapManager.uploadFrame != null) {
+					List<String> uploadedTables = new ArrayList<String>();
+					uploadedTables.addAll(tapManager.uploadFrame.uploadClient.tablesMetaData.keySet());
+					if (!uploadedTables.isEmpty()) {
+						for (String uploadedTable : uploadedTables) {
+							server.updateQueryChecker(true, uploadedTable, tapManager.uploadFrame.uploadClient.tablesMetaData, queryCheckerTables);
+						}
+					}
+				}
+				QueryChecker checker = new DBChecker(queryCheckerTables);
+				server.adqlParser.setQueryChecker(checker);
+			}
+		} catch (ParseException e) {
+			// TODO Auto-generated catch block
+			e.printStackTrace();
+		}*/
 	}
 	
 	public void activateWaitMode(ServerTap serverTap) {
@@ -868,7 +950,5 @@ public class TapClient{
 	    NOGLURECFOUND = Aladin.chaine.getString("NOGLURECFOUND");
 	    CHANGESERVERTOOLTIP = Aladin.chaine.getString("CHANGESERVERTOOLTIP");
 	}
-
-	
 	
 }
