@@ -1,10 +1,10 @@
-// Copyright 1999-2018 - Universit de Strasbourg/CNRS
+// Copyright 1999-2020 - Universit de Strasbourg/CNRS
 // The Aladin Desktop program is developped by the Centre de Donnes
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
 //
-//This file is part of Aladin.
+//This file is part of Aladin Desktop.
 //
 //    Aladin Desktop is free software: you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 //    GNU General Public License for more details.
 //
 //    The GNU General Public License is available in COPYING file
-//    along with Aladin.
+//    along with Aladin Desktop.
 //
 
 package cds.aladin;
@@ -51,8 +51,10 @@ import static cds.aladin.Constants.UCD_RA_PATTERN2;
 import static cds.aladin.Constants.UCD_RA_PATTERN3;
 import static cds.aladin.Constants.REGEX_VALIDTABLENAME;
 import static cds.aladin.Constants.REGEX_VALIDTABLEPREFIX;
+import static cds.aladin.Constants.REGEX_TABLENAMEALREADYQUOTED;
 
 import java.util.HashMap;
+import java.util.List;
 import java.util.Map;
 import java.util.Vector;
 import java.util.regex.Matcher;
@@ -72,10 +74,13 @@ public class TapTable {
 	private String table_type;
 	private String description;
 	private String utype;
+//	private String table_index; v1.1
 	private Vector<TapTableColumn> columns;
 	public Map<String, TapTableColumn> flaggedColumns;
 	public Map<String, String> obsCoreColumns;
 	public static final int MAXOBSCORECOLSCOUNTED = 6;
+	public List<ForeignKeyColumn> foreignKeyColumns;
+	public String alias;
 	
 	/**
 	 * Identifies certain columns
@@ -196,6 +201,7 @@ public class TapTable {
 	}
 	
 	/**
+	 * Unused. For reference
 	 * Method only used for cases of table names with special chars.
 	 * Adds double quote to the names
 	 * @param tapTable 
@@ -204,7 +210,7 @@ public class TapTable {
 	 */
 	public static String getQueryPart1(String queryPartInput) {//keeping for reference
 		if (queryPartInput != null) {
-//			queryPartInput = tapTable.getAdqlName(); //TODO:: tintin : when we add schema name
+//			queryPartInput = tapTable.getAdqlName(); //TODO::  : when we add schema name
 			Pattern regex = Pattern.compile(REGEX_TABLENAME_SPECIALCHAR);
 			/*String[] tableName = queryPartInput.split("\\."); nope. Vizier can have dot inside a table adql name: J/other/BAJ/24.62/table5 
 			if (tableName.length > 1) {
@@ -243,8 +249,10 @@ public class TapTable {
 					}
 					matcher = regex.matcher(potentialTableName);
 					if (!matcher.find()){
-						queryPartInput = Glu.doubleQuote(potentialTableName);
-						queryPartInput = prefix+queryPartInput;
+						if (isUnQuotedPattern(potentialTableName)) {
+							queryPartInput = Glu.doubleQuote(potentialTableName);
+							queryPartInput = prefix+queryPartInput;
+						}
 					}
 				}
 			}
@@ -252,6 +260,16 @@ public class TapTable {
 			 return queryPartInput;
 	}
 	
+	public static boolean isUnQuotedPattern(String input) {
+		boolean result = true;
+		Pattern isQuotedPattern = Pattern.compile(REGEX_TABLENAMEALREADYQUOTED);
+		Matcher isQuotedMatcher =isQuotedPattern.matcher(input);
+		if (isQuotedMatcher.find()) {
+			result = false;
+		}
+		return result;
+	}
+	
 	public synchronized void parseForObscore(boolean isUpload, TapTableColumn columnMeta) {
 		if (isUpload || hasObscoreInTheName()) {
 			initObsCoreColumns();
@@ -387,10 +405,14 @@ public class TapTable {
 		}
 	}
 	
-	public String getFlaggedColumnName(String key) {
+	public String getFlaggedColumnName(String key, boolean withAlias) {
 		String result = null;
-		if (this.flaggedColumns != null && this.flaggedColumns.containsKey(key)) {
-			result = this.flaggedColumns.get(key).getColumn_name();
+		TapTableColumn column = getFlaggedColumn(key);
+		if (column != null) {
+			result = this.flaggedColumns.get(key).getColumnNameForQuery();
+			if (withAlias && this.alias != null) {
+				result = this.alias + DOT_CHAR + result;
+			}
 		}
 		return result;
 	}
@@ -419,12 +441,12 @@ public class TapTable {
 		return result;
 	}
 	
-	public String getRaColumnName() {
-		return getFlaggedColumnName(RA);
+	public String getRaColumnName(boolean withAlias) {
+		return getFlaggedColumnName(RA, withAlias);
 	}
 
-	public String getDecColumnName() {
-		return getFlaggedColumnName(DEC);
+	public String getDecColumnName(boolean withAlias) {
+		return getFlaggedColumnName(DEC, withAlias);
 	}
 	
 	/**
@@ -520,5 +542,13 @@ public class TapTable {
 	public void setObsCoreColumns(Map<String, String> obsCoreColumns) {
 		this.obsCoreColumns = obsCoreColumns;
 	}
-	
+
+	public String getAlias() {
+		return alias;
+	}
+
+	public void setAlias(String alias) {
+		this.alias = alias;
+	}
+
 }
