@@ -1,10 +1,10 @@
-// Copyright 1999-2018 - Universit de Strasbourg/CNRS
+// Copyright 1999-2020 - Universit de Strasbourg/CNRS
 // The Aladin Desktop program is developped by the Centre de Donnes
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
 //
-//This file is part of Aladin.
+//This file is part of Aladin Desktop.
 //
 //    Aladin Desktop is free software: you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 //    GNU General Public License for more details.
 //
 //    The GNU General Public License is available in COPYING file
-//    along with Aladin.
+//    along with Aladin Desktop.
 //
 
 package cds.aladin;
@@ -52,9 +52,11 @@ import cds.aladin.prop.Propable;
 import cds.allsky.Constante;
 import cds.allsky.Context;
 import cds.moc.Healpix;
-import cds.moc.HealpixMoc;
+import cds.moc.SMoc;
 import cds.mocmulti.MocItem2;
+import cds.mocmulti.MultiMoc;
 import cds.tools.Util;
+import cds.tools.pixtools.CDSHealpix;
 
 /**
  * Gre les noeuds de l'arbre des HiPS
@@ -96,11 +98,73 @@ public class TreeObjDir extends TreeObj implements Propable {
    public double radius=-1;   // Field size for starting display
    public int nside=-1;          // Max NSIDE
    public boolean local=false;   // Il s'agit d'un survey sur disque local
-   public Image imPreview=null;  // Image preview
+   
+   public Image previewImg=null;  // Image preview courante
+   public Image previewImgHips2Fits=null;  // Image de la vignette courante
    public boolean previewError=false; // true s'il y a eu un problme lors du chargement du preview
+   public String previewUrlHips2Fits=null; // URL vers l'image preview rcupre par Hips2fits
+   public boolean previewErrorHips2Fits=false; // true s'il y a eu un problme  l'accs  Hips2fits
+   public boolean previewLoading=false; // true si on est en train de charger un preview ou une vignette
    
    protected MyProperties prop=null; // Ensemble des proprits associes au HiPS (via son fichier de properties ou MocServer)
 
+   public final static String DIRECT = "DIRECT/"; // prfixe ajout  l'ID dans le cas d'un accs direct par URL explicite
+   
+   
+   /** Retourne le titre long de la ressource, et si c'est une table VizieR, un nom compos du titre du catalogue,
+    * suivi du titre de la table et suffix par le label de la table */
+   protected String getVizieRLongTitre() {
+     String titre = description;
+     
+     // Est-ce bien un enregistrement VizieR ? nouvelle formule ?
+     if( !internalId.startsWith("CDS/") ) return titre;
+     if( internalId.startsWith("CDS/P") || internalId.startsWith("CDS/C") || internalId.startsWith("CDS/Simbad") ) return titre;
+     if( prop.getFirst("client_category")==null ) return titre;
+     
+     String tableTitle = prop.getFirst("obs_title");
+     String tableLabel = prop.getFirst("obs_label");
+     String catalogTitle = prop.getFirst("obs_collection");
+     String prefix = prop.getFirst("obs_collection_label");
+    
+     return aladin.directory.addLabelPrefix( prefix, (catalogTitle!=null ? catalogTitle+" - "+tableTitle : tableTitle) 
+                                                   + (tableLabel!=null? " ("+tableLabel+")":"") );
+   }
+   
+   /** Ajustement des labels qui s'affiche dans l'arbre pour les ressources VizieR */
+   private void adjustVizieR() {
+      
+      // Est-ce bien un enregistrement VizieR ?
+      if( !internalId.startsWith("CDS/") ) return;
+      if( internalId.startsWith("CDS/P") || internalId.startsWith("CDS/C") || internalId.startsWith("CDS/Simbad") ) return;
+      
+      if( prop.getFirst("client_category")==null ) return;
+      
+      
+      // Incorpartion en prfixe de la description du label du catalogue
+      Directory dir = aladin.directory;
+      
+      // Une table unique => remplacement par le nom du catalogue et prfixage avec le label
+      if( !dir.hasMultiple( dir.getCatParent(internalId)) ) {
+         String catalogTitle = prop.getFirst("obs_collection");
+         if(  catalogTitle!=null ) {
+            // Ce n'est qu'un accronyme, donc il y a dj eu traitement
+            if( catalogTitle.indexOf(' ')<0 ) return;
+            label = catalogTitle;
+         }
+         
+         String prefix = prop.getFirst("obs_collection_label");
+         label = dir.addLabelPrefix( prefix ,label);
+         
+      } else {
+         String suffixe = prop.getFirst("obs_label");
+         if( suffixe!=null && label.indexOf("("+suffixe+")")<0 ) label = label+" ("+suffixe+")";
+      }
+
+      label = dir.cleanLatexMacro( label );
+      aladinLabel=label;
+
+   }
+   
    /** Construction d'un TreeObjHips  partir des infos qu'il est possible de glaner
     *  l'endroit indiqu, soit par exploration du rpertoire, soit par le fichier Properties */
    public TreeObjDir(Aladin aladin,String pathOrUrl) throws Exception {
@@ -138,7 +202,8 @@ public class TreeObjDir extends TreeObj implements Propable {
          if( offset==end-1 && offset>0 ) { end=offset; offset = pathOrUrl.lastIndexOf(c,end-1); }
          label = pathOrUrl.substring(offset+1,end);
       }
-      id="__"+label;
+//      id="__"+label;
+      id= internalId = DIRECT+System.currentTimeMillis()/1000+"/"+MultiMoc.getID(prop);
 
       s = prop.getProperty(Constante.OLD_VERSION);
       if( s!=null ) version=s;
@@ -154,6 +219,9 @@ public class TreeObjDir extends TreeObj implements Propable {
       if( copyrightUrl==null ) copyrightUrl = prop.getProperty(Constante.OLD_OBS_COPYRIGHT_URL);
       useCache = !local && Boolean.parseBoolean( prop.getProperty(Constante.OLD_USECACHE,"True") );
       skyFraction = prop.getProperty(Constante.KEY_MOC_SKY_FRACTION);
+      
+      // Petit peaufinage pour VizieR
+      adjustVizieR();
 
       s = prop.getProperty(Constante.KEY_HIPS_INITIAL_RA);
       if( s!=null) {
@@ -168,7 +236,7 @@ public class TreeObjDir extends TreeObj implements Propable {
       if( s==null ) target=null;
       else {
          try { target = new Coord(s); }
-         catch( Exception e) { aladin.trace(3,"target error!"); target=null; }
+         catch( Exception e) { aladin.trace(3,"HiPS target error for "+id+" ["+s+"]!"); target=null; }
       }
       double div2=2;
       s = prop.getProperty(Constante.KEY_HIPS_INITIAL_FOV);
@@ -176,7 +244,7 @@ public class TreeObjDir extends TreeObj implements Propable {
       if( s==null ) radius=-1;
       else {
          try { radius=(Server.getAngleInArcmin(s, Server.RADIUSd)/60.)/div2; }
-         catch( Exception e) { aladin.trace(3,"radius error!"); radius=-1; }
+         catch( Exception e) { aladin.trace(3,"HiPS radius error for "+id+" ["+s+"]!"); radius=-1; }
       }
 
       s = prop.getProperty(Constante.KEY_HIPS_TILE_WIDTH);
@@ -192,7 +260,7 @@ public class TreeObjDir extends TreeObj implements Propable {
       catch( Exception e ) {
          maxOrder = getMaxOrderByPath(pathOrUrl,local);
          
-         if( maxOrder==-1 ) throw new Exception("Not an HiPS");
+//         if( maxOrder==-1 ) throw new Exception("Not an HiPS");
          if( maxOrder==-1 ) {
             aladin.trace(3,"No maxOrder found (even with scanning dir.) => assuming 11");
             maxOrder=11;
@@ -250,6 +318,7 @@ public class TreeObjDir extends TreeObj implements Propable {
       else {
          keyColor = prop.getProperty(Constante.OLD_ISCOLOR);
          if( keyColor==null ) keyColor = prop.getProperty("isColor");
+         if( keyColor==null ) keyColor = prop.getProperty("isColored");
          if( keyColor!=null ) color = new Boolean(keyColor);
       }
 
@@ -326,6 +395,9 @@ public class TreeObjDir extends TreeObj implements Propable {
       if( s==null ) s=prop.getProperty( Constante.KEY_OBS_COLLECTION );
       aladinLabel = label = s!=null ? s : createLabel(id,cat);      
 
+      // Petit peaufinage des labels pour VizieR
+      adjustVizieR();
+      
       // Initialisation de la cl de tri et du path
       setTri();
       setPath();
@@ -337,6 +409,7 @@ public class TreeObjDir extends TreeObj implements Propable {
       copyright = prop.getProperty(Constante.KEY_OBS_COPYRIGHT);
       copyrightUrl = prop.getProperty(Constante.KEY_OBS_COPYRIGHT_URL);
       skyFraction = prop.getProperty(Constante.KEY_MOC_SKY_FRACTION);
+      
 
       // Le champ initial
       s = prop.getProperty(Constante.KEY_HIPS_INITIAL_RA);
@@ -348,7 +421,7 @@ public class TreeObjDir extends TreeObj implements Propable {
       if( s==null ) target=null;
       else {
          try { target = new Coord(s); }
-         catch( Exception e) { aladin.trace(3,"target error!"); target=null; }
+         catch( Exception e) { aladin.trace(3,"HiPS target error for "+id+" ["+s+"]!"); target=null; }
       }
       double div2=2;
       s = prop.getProperty(Constante.KEY_HIPS_INITIAL_FOV);
@@ -357,7 +430,7 @@ public class TreeObjDir extends TreeObj implements Propable {
          try {
             radius=(Server.getAngleInArcmin(s, Server.RADIUSd)/60.)/div2;
             if( radius>180 ) radius=180;
-         } catch( Exception e) { aladin.trace(3,"radius error!"); radius=-1; }
+         } catch( Exception e) { aladin.trace(3,"HiPS radius error for "+id+" ["+s+"]!"); radius=-1; }
       }
 
       // La taille de la tuile Hips
@@ -522,21 +595,41 @@ public class TreeObjDir extends TreeObj implements Propable {
       return prop.getProperty(Constante.KEY_HIPS_PROGENITOR_URL);
    }
    
+//   /** Retourne l'URL d'accs au MOC, null sinon */
+//   protected String getMocUrl() {
+//      String u=null;
+//      
+//      // On dispose du MOC directement dans le MOC server qui a fourni les proprits ?
+//      if( hasMocByMocServer() ) {
+//         String params = internalId+"&get=moc";
+//         u = aladin.glu.getURL("MocServer",params,true).toString();
+//         
+//      // Serait-il explicitement mentionn ?
+//      } else if( prop!=null ) u = prop.getProperty("moc_access_url");
+//      
+//      // Pour un HiPS on peut y accder directement
+//      else if( isHiPS() ) return getUrl()+"/Moc.fits";
+//      
+//      return u;
+//   }
+//   
    /** Retourne l'URL d'accs au MOC, null sinon */
-   protected String getMocUrl() {
+   protected String getTMocUrl() {
       String u=null;
       
       // On dispose du MOC directement dans le MOC server qui a fourni les proprits ?
-      if( hasMocByMocServer() ) {
-         String params = internalId+"&get=moc";
+      if( hasTMocByMocServer() ) {
+         String params = internalId+"&get=tmoc";
          u = aladin.glu.getURL("MocServer",params,true).toString();
          
       // Serait-il explicitement mentionn ?
-      } else if( prop!=null ) u = prop.getProperty("moc_access_url");
-      
-      // Pour un HiPS on peut y accder directement
-      else if( isHiPS() ) return getUrl()+"/Moc.fits";
-      
+      } else  {
+         
+         if( prop!=null ) u = prop.getProperty("tmoc_access_url");
+
+         // Pour un HiPS on peut y accder directement
+         if( u==null && isHiPS() ) return getUrl()+"/TMoc.fits";
+      }
       return u;
    }
    
@@ -678,6 +771,13 @@ public class TreeObjDir extends TreeObj implements Propable {
    protected boolean hasHips() { return prop!=null && 
          (prop.get("hips_service_url")!=null || prop.get("hips_service_path")!=null); }
    
+   /** Retourne true si la collection is est un HiPS catalogue */
+   protected boolean isHipsCat() {
+      if( !hasHips() ) return false;
+      String s = prop.get("dataproduct_type");
+      return s!=null && s.indexOf("catalog")>=0; 
+   }
+   
    /** Retourne true si la collection dispose d'un accs TAP */
    protected boolean hasTAP() {
       if( prop==null ) return false;
@@ -703,6 +803,16 @@ public class TreeObjDir extends TreeObj implements Propable {
       return prop!=null && prop.get("access_url")!=null;
    }
    
+   /** retourne true si on peut faire un chargement global */
+   protected boolean hasAllAccess() {
+      if( !isCDSCatalog() ) return false;
+      try {
+         long nbRows = Long.parseLong(getProperty(Constante.KEY_NB_ROWS));
+         return nbRows>-1 && nbRows<20000;
+      } catch( Exception e ) { }
+      return false;
+   }
+   
    /** Retourne true si la collection dispose d'une URL donnant accs  un preview */
    protected boolean hasPreview() {
       return (isCDSCatalog() || isHiPS());
@@ -740,7 +850,7 @@ public class TreeObjDir extends TreeObj implements Propable {
    
    /** Retourne le type de donnes auquel on va accder */
    protected String getDataType() {
-      if( hasCS() || hasTAP() ) return "Tabular";
+      if( hasCS() || hasTAP() || isHipsCat() ) return "Catalog";
       if( hasHips() || hasSIA() ) return "Image";
       if( hasSSA() ) return "Spectrum";
       return "";
@@ -776,10 +886,28 @@ public class TreeObjDir extends TreeObj implements Propable {
       return hasMocByMocServer() || prop!=null && prop.getProperty("moc_access_url")!=null || isHiPS();
    }
    
+   /** Retourne true si la collection dispose d'un TMOC */
+   protected  boolean hasTMoc() {
+      return hasTMocByMocServer() || prop!=null && prop.getProperty("tmoc_access_url")!=null
+            ;
+            // || hasHiPSTMocFile();   // CA PREND TROP DE TEMPS
+   }
+   
+   /** En attendant que ce soit dans le MocServer, je regarde l'existence du fichier HpxFinder/TMoc.fits */
+   private boolean hasHiPSTMocFile() {
+      if( !isHiPS() ) return false;
+      String url = getTMocUrl();
+      return Util.isUrlResponding( url );
+   }
+   
    /** Retourne true si la collection dispose d'un MOC via le MocServer
     * (=> celui-ci ayant ajout le mot cl moc_sky_fraction) */
    private boolean hasMocByMocServer() { return prop!=null && prop.get("moc_sky_fraction")!=null; }
    
+   /** Retourne true si la collection dispose d'un TMOC via le MocServer
+    * (=> celui-ci ayant ajout le mot cl tmoc_total_time) */
+   private boolean hasTMocByMocServer() { return prop!=null && prop.get("tmoc_total_time")!=null; }
+   
    /** Retourne l'URL d'un Cone search ou null si aucun */
    protected String getCSUrl() {
       if( prop==null ) return null;
@@ -864,6 +992,9 @@ public class TreeObjDir extends TreeObj implements Propable {
    /** retourne l'URL de base pour accder au serveur HTTP */
    protected String getUrl() {
       
+      // Accs direct ? (sans passer par le GLU)
+      if( id!=null && id.startsWith(TreeObjDir.DIRECT) && url!=null ) return url;
+      
       // HiPS local ?
       if( prop!=null ) {
          String path = prop.getProperty("hips_service_path");
@@ -914,10 +1045,11 @@ public class TreeObjDir extends TreeObj implements Propable {
    
    /** Chargement par dfaut  effectuer (suite  un double-clic sur le noeud de l'arbre) */
    protected void load() {
-      if( hasSIA() ) loadSIA();
+      if( hasGlobalAccess() ) loadGlobalAccess();
+      else if( hasAllAccess() ) loadAll();
+      else if( hasSIA() ) loadSIA();
       else if( hasSSA() ) loadSSA();
       else if( getUrl()==null && isCatalog() ) loadCS();
-      else if( hasGlobalAccess() ) loadGlobalAccess();
       else loadHips();
    }
    
@@ -986,7 +1118,7 @@ public class TreeObjDir extends TreeObj implements Propable {
     	  aladin.error("Error! No TAP form configured for "+id);
       }
       
-      //TODO:: tintin remove the below just for demo
+      //TODO:: below just for demo
      /* String url1 = "http://tapvizier.u-strasbg.fr/TAPVizieR/tap";
       String id1 = "GAIA_VIZIER";
       url1 = "http://www.cadc-ccda.hia-iha.nrc-cnrc.gc.ca/tap2";
@@ -1034,16 +1166,18 @@ public class TreeObjDir extends TreeObj implements Propable {
      exec( addBrowse( getSIACmd()+" "+cone) );
    }
    protected String getSIABkm() { return addBrowse( getSIACmd()+" $TARGET $RADIUS"); }
-   private String getSIACmd() {
-      // Glu tags spcifiques ?
-      String gluTag = prop.get("sia_glutag");
-      if( gluTag==null ) gluTag = prop.get("sia2_glutag");
+   protected String getSIACmd() {
+      String gluTag=null;
       
       // URL de base ?
       // On passe tout de mme par le ID afin d'avoir une commande script clean
       // => voir GluServer pour la rsolution internalId => Url de base
       if( prop.get("sia2_service_url")!=null ) gluTag = "SIA2("+internalId+")";
-      if( prop.get("sia_service_url")!=null )  gluTag = "SIA("+internalId+")";
+      else if( prop.get("sia_service_url")!=null ) gluTag = "SIA("+internalId+")";
+      
+      // Glu tags spcifiques ?
+      if( gluTag==null ) gluTag = prop.get("sia2_glutag");
+      if( gluTag==null ) gluTag = prop.get("sia_glutag");
       
       return "get "+gluTag;
    }
@@ -1052,7 +1186,7 @@ public class TreeObjDir extends TreeObj implements Propable {
    /** Gnration et excution de la requte script correspondant  un accs global */
    protected void loadGlobalAccess() {exec( addBrowse( getGlobalAccessCmd(), false ) ); }
    protected String getGlobalAccessBkm() { return addBrowse( getGlobalAccessCmd(), false ); }
-   private String getGlobalAccessCmd() {
+   protected String getGlobalAccessCmd() {
       String cmd = null;
       
       String id = Tok.quote(internalId);
@@ -1092,18 +1226,27 @@ public class TreeObjDir extends TreeObj implements Propable {
    }
    protected void loadCS(String cone) {
       if( cone==null ) { loadCS(); return; }
-     exec( addBrowse(  getCSCmd()+" "+cone));
+      exec( addBrowse(  getCSCmd()+" "+cone));
    }
-   protected String getCSBkm() { return addBrowse( getCSCmd()+" $TARGET $RADIUS" ); }
-   private String getCSCmd() {
+   protected String getCSBkm() { return addBrowse( getCSCmd(false)+" $TARGET $RADIUS" ); }
+   protected String getCSCmd() { return getCSCmd(true); }
+   protected String getCSCmd(boolean flagReplace) {
       String cmd = null;
       String allcolumns = aladin.directory.getParam("QueryCatColumns");
+      String label="";
+      
+      // Pour viter de charger 2x le mme plan CS si ce n'est pas souhait
+      if( !Aladin.NOGUI && flagReplace && aladin.calque.isCSAlreadyLoaded(internalId) ) {
+         if( aladin.confirmation(aladin, aladin.chaine.getString("CSALREADYLOADED")) ) {
+            label=Tok.quote( internalId)+"=";
+         }
+      }
 
       // On passe par VizieR/Simbad via la commande script adapte
       if( isCDSCatalog() ) {
          int i = internalId.indexOf('/');
          String cat = internalId.substring(i+1);
-         if( internalId.startsWith("CDS/Simbad") ) cmd = /* Tok.quote(internalId)+"=*/ "get Simbad";
+         if( internalId.startsWith("CDS/Simbad") ) cmd = "get Simbad";
          else {
             String s = allcolumns.equals("all") ? ",allcolumns":"";
             cmd = "get VizieR("+cat+s+")";
@@ -1115,7 +1258,7 @@ public class TreeObjDir extends TreeObj implements Propable {
          cmd = "get CS("+internalId+s+")";
       }
       
-      return cmd;
+      return label+cmd;
    }
    
    protected void loadCustom() {
@@ -1137,6 +1280,15 @@ public class TreeObjDir extends TreeObj implements Propable {
       return "get "+glutag+"(....) $TARGET $RADIUS";
    }
    
+   /** Gnration et excution de la requte script correspondant au protocole TMOC */
+   protected void loadTMoc( ) {exec( getTMocCmd() ); }
+   protected String getTMocBkm() { return getTMocCmd(); }
+//   private String getTMocCmd() { return "get TMOC("+Tok.quote(internalId)+")"; }
+   private String getTMocCmd() {
+      String id = Tok.quote(internalId!=null?internalId:label);
+      return "TMoc_"+id+"=load "+getTMocUrl();
+   }
+
    /** Gnration et excution de la requte script correspondant au protocole MOC */
    protected void loadMoc( ) {exec( getMocCmd() ); }
    protected String getMocBkm() { return getMocCmd(); }
@@ -1257,9 +1409,10 @@ public class TreeObjDir extends TreeObj implements Propable {
       }
       
       if( url==null ) return;
+//      System.out.println("ULR scan = "+url);
       
       // Mmorisation du rsultat
-      HealpixMoc moc;
+      SMoc moc;
       try { 
          moc = scan(url); 
          if( mo.moc==null ) mo.moc=moc;
@@ -1269,15 +1422,20 @@ public class TreeObjDir extends TreeObj implements Propable {
       // Mmorisation de la surface couverte
       try {
          int order=11;
-         Healpix hpx = new Healpix();
-         moc = new HealpixMoc(order);
-         int i=0;
-         moc.setCheckConsistencyFlag(false);
-         for( long n : hpx.queryDisc(order, c.al, c.del,  rad) ) {
-            moc.add(order, n);
-            if( (++i)%1000==0 )  moc.checkAndFix();
-         }
-         moc.setCheckConsistencyFlag(true);
+//         moc = new SMoc(order);
+//         int i=0;
+//         moc.setCheckConsistencyFlag(false);
+//         for( long n : CDSHealpix.query_disc(order, c.al, c.del,  Math.toRadians(rad), false) ) {
+//            moc.add(order, n);
+//            if( (++i)%1000==0 )  moc.checkAndFix();
+//         }
+//         moc.setCheckConsistencyFlag(true);
+         moc = CDSHealpix.getMocByCircle(order, c.al, c.del,  Math.toRadians(rad), false);
+         
+//         Iterator<Long> it  = moc.pixelIterator();
+//         System.out.print("TreeObjDir.scan1():\ndraw circle("+c.al+","+c.del+","+radius+")\ndraw moc "+order+"/");
+//         while( it.hasNext() ) System.out.print(" "+it.next());
+//         System.out.println();
          
          if( mo.mocRef==null ) mo.mocRef=moc;
          else mo.mocRef.add(moc);
@@ -1296,16 +1454,16 @@ public class TreeObjDir extends TreeObj implements Propable {
    
    /**  Gnration d'un Moc  partir du catalogue retourne par l'URL
     * passe en paramtre */
-   private HealpixMoc scan( String url) {
+   private SMoc scan( String url) {
       Pcat pcat = new Pcat(aladin);
       pcat.plan = new PlanCatalog(aladin);
       pcat.plan.label="test";
-      HealpixMoc moc=null;
+      SMoc moc=null;
       int order=11;
       
       try {
-         moc = new HealpixMoc(order);
-         inScan=new MyInputStream( Util.openStream(url,false,30000) );
+         moc = new SMoc(order);
+         inScan=new MyInputStream( Util.openStream(url,false,true,30000) );
          pcat.tableParsing(inScan,null);
          
          Iterator<Obj> it = pcat.iterator();
@@ -1317,7 +1475,10 @@ public class TreeObjDir extends TreeObj implements Propable {
             if( !(o instanceof Position) ) continue;
             if( Double.isNaN( ((Position)o).raj ) ) continue;
             try {
-               long pix = hpx.ang2pix(order, ((Position)o).raj, ((Position)o).dej);
+               
+               double [] c = CDSHealpix.normalizeRaDec( ((Position)o).raj, ((Position)o).dej);
+               long pix = hpx.ang2pix(order, c[0], c[1] );
+//               long pix = hpx.ang2pix(order, ((Position)o).raj, ((Position)o).dej);
                moc.add(order,pix);
                n++;
                if( n>10000 ) { moc.checkAndFix(); n=0; }
@@ -1438,7 +1599,7 @@ public class TreeObjDir extends TreeObj implements Propable {
    /**********************  Gestion des proprits associes aux diffrents modes d'interrogation ************************/
    
    static public HashMap<String, String> paramsFactory() {
-      HashMap<String, String> params = new HashMap<String, String>();
+      HashMap<String, String> params = new HashMap<>();
       params.put(XMATCH_RADIUS_KEY, "5");
       params.put(XMATCH_SELECTION_KEY, XMATCH_SELECTION[0]);
       params.put(HIPS_FORMAT_KEY, HIPS_FORMAT[0]);
@@ -1461,7 +1622,7 @@ public class TreeObjDir extends TreeObj implements Propable {
 
    @Override
    public Vector<Prop> getProp() {
-      Vector<Prop> v = new Vector<Prop>();
+      Vector<Prop> v = new Vector<>();
       if( hasHipsFmt() ) v.add( getHipsFmtProp() );
       if( isCatalog() ) {
          v.add( getCatLimitProp() );
@@ -1496,7 +1657,7 @@ public class TreeObjDir extends TreeObj implements Propable {
    static public String    BROWSING_HELP;
 
    public Prop getBrowsingProp() {
-      final JComboBox<String> combo =  new JComboBox<String>( BROWSING );
+      final JComboBox<String> combo =  new JComboBox<>( BROWSING );
       final PropAction update = new PropAction() {
          public int action() { combo.setSelectedItem( aladin.directory.getParam(BROWSING_KEY)); return PropAction.SUCCESS; }
       };
@@ -1546,7 +1707,7 @@ public class TreeObjDir extends TreeObj implements Propable {
    static public String    XMATCH_SELECTION_HELP;
 
    public Prop getXmatchSelectionProp() {
-      final JComboBox<String> combo =  new JComboBox<String>( XMATCH_SELECTION );
+      final JComboBox<String> combo =  new JComboBox<>( XMATCH_SELECTION );
       final PropAction update = new PropAction() {
          public int action() { combo.setSelectedItem( aladin.directory.getParam(XMATCH_SELECTION_KEY)); return PropAction.SUCCESS; }
       };
@@ -1568,7 +1729,7 @@ public class TreeObjDir extends TreeObj implements Propable {
    static public String    HIPS_FORMAT_HELP;
 
    public Prop getHipsFmtProp() {
-      final JComboBox<String> combo =  new JComboBox<String>( HIPS_FORMAT );
+      final JComboBox<String> combo =  new JComboBox<>( HIPS_FORMAT );
       final PropAction update = new PropAction() {
          public int action() { combo.setSelectedItem( aladin.directory.getParam(HIPS_FORMAT_KEY)); return PropAction.SUCCESS; }
       };
@@ -1589,7 +1750,7 @@ public class TreeObjDir extends TreeObj implements Propable {
    static public String    CAT_LIMIT_HELP;
 
    public Prop getCatLimitProp() {
-      final JComboBox<String> combo =  new JComboBox<String>( CAT_LIMIT );
+      final JComboBox<String> combo =  new JComboBox<>( CAT_LIMIT );
       final PropAction update = new PropAction() {
          public int action() { combo.setSelectedItem( aladin.directory.getParam(CAT_LIMIT_KEY)); return PropAction.SUCCESS; }
       };
@@ -1610,7 +1771,7 @@ public class TreeObjDir extends TreeObj implements Propable {
    static public String    CAT_COLUMNS_HELP;
 
    public Prop getCatColumnProp() {
-      final JComboBox<String> combo =  new JComboBox<String>( CAT_COLUMNS );
+      final JComboBox<String> combo =  new JComboBox<>( CAT_COLUMNS );
       final PropAction update = new PropAction() {
          public int action() { combo.setSelectedItem( aladin.directory.getParam(CAT_COLUMNS_KEY)); return PropAction.SUCCESS; }
       };
