@@ -1,10 +1,10 @@
-// Copyright 1999-2018 - Universit de Strasbourg/CNRS
+// Copyright 1999-2020 - Universit de Strasbourg/CNRS
 // The Aladin Desktop program is developped by the Centre de Donnes
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
 //
-//This file is part of Aladin.
+//This file is part of Aladin Desktop.
 //
 //    Aladin Desktop is free software: you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 //    GNU General Public License for more details.
 //
 //    The GNU General Public License is available in COPYING file
-//    along with Aladin.
+//    along with Aladin Desktop.
 //
 
 /**
@@ -31,10 +31,8 @@ import static cds.aladin.Constants.EMPTYSTRING;
 import static cds.aladin.Constants.GETPREVIOUSSESSIONJOB;
 import static cds.aladin.Constants.LOADDEFAULTTAPRESULT;
 import static cds.aladin.Constants.NEWLINE_CHAR;
-import static cds.aladin.Constants.PATHASYNC;
 import static cds.aladin.Constants.PATHPHASE;
 
-
 import java.awt.Dimension;
 import java.awt.GridBagConstraints;
 import java.awt.GridBagLayout;
@@ -72,7 +70,6 @@ import javax.swing.JScrollPane;
 import javax.swing.JSplitPane;
 import javax.swing.JTextField;
 
-import adql.query.ADQLQuery;
 import cds.tools.MultiPartPostOutputStream;
 import cds.tools.Util;
 import cds.xml.UWSReader;
@@ -98,9 +95,12 @@ public class UWSFacade implements ActionListener{
 	
 	public static String JOBNOTFOUNDMESSAGE, JOBERRORTOOLTIP, UWSNOJOBMESSAGE, CANTSTARTJOB, GENERICERROR1LINE,
 			STANDARDRESULTSLOAD, STANDARDRESULTSLOADTIP, UWSASKLOADDEFAULTRESULTS, CANTABORTJOB, UWSJOBRADIOTOOLTIP,
-			JOBCONTROLLERTITLE;
+			JOBCONTROLLERTITLE, UWSPANELCURRECTSESSIONTITLE, UWSPANELPREVIOUSSESSIONTITLE, JOBNOTSELECTED, JOBNOTFOUNDGIVENURL,
+			NOJOBURLMESSAGE, JOBDELETEERRORMESSAGE, DELETEONCLOSEBUTTONLABEL, UWSMULTIJOBLOADMESSAGE, UWSASKTOREMOVENOTFOUNDJOBS, 
+			UWSASKTOREMOVENOTFOUNDJOBSTITLE ;
 	public static String ERROR_INCORRECTPROTOCOL = "IOException. Job url not http protocol!";
-	public static final int POLLINGDELAY = 1000;
+	public static final int POLLINGDELAY = 4000; //increasing the polling delay to 4secs after consulting Markus Demleitner and Mark Taylor (at Asterics TechForum4)
+	public static int DELETETIMEOUTTIME = 7000;
 	
 	static {
 		JOBNOTFOUNDMESSAGE = Aladin.chaine.getString("JOBNOTFOUNDMESSAGE");
@@ -114,6 +114,16 @@ public class UWSFacade implements ActionListener{
 		CANTABORTJOB= Aladin.getChaine().getString("CANTABORTJOB");
 		UWSJOBRADIOTOOLTIP = Aladin.getChaine().getString("UWSJOBRADIOTOOLTIP");
 		JOBCONTROLLERTITLE = Aladin.getChaine().getString("JOBCONTROLLERTITLE");
+		UWSPANELCURRECTSESSIONTITLE = Aladin.getChaine().getString("UWSPANELCURRECTSESSIONTITLE");
+		UWSPANELPREVIOUSSESSIONTITLE = Aladin.getChaine().getString("UWSPANELPREVIOUSSESSIONTITLE");
+		JOBNOTSELECTED = Aladin.getChaine().getString("JOBNOTSELECTED");
+		JOBNOTFOUNDGIVENURL = Aladin.getChaine().getString("JOBNOTFOUNDGIVENURL");
+		NOJOBURLMESSAGE = Aladin.getChaine().getString("NOJOBURLMESSAGE");
+		JOBDELETEERRORMESSAGE = Aladin.getChaine().getString("JOBDELETEERRORMESSAGE");
+		DELETEONCLOSEBUTTONLABEL = Aladin.getChaine().getString("DELETEONCLOSEBUTTONLABEL");
+		UWSMULTIJOBLOADMESSAGE = Aladin.getChaine().getString("UWSMULTIJOBLOADMESSAGE");
+		UWSASKTOREMOVENOTFOUNDJOBS = Aladin.getChaine().getString("UWSASKTOREMOVENOTFOUNDJOBS");
+		UWSASKTOREMOVENOTFOUNDJOBSTITLE = Aladin.getChaine().getString("UWSASKTOREMOVENOTFOUNDJOBSTITLE");
 	}
 	
 	public UWSFacade() {
@@ -133,25 +143,48 @@ public class UWSFacade implements ActionListener{
 		return instance;
 	}
 	
+	/**
+	 * Handles async request for server panels
+	 * @param aladin
+	 * @param server
+	 * @param label
+	 * @param url
+	 * @param requestParams
+	 * @param requestNumber
+	 */
+	public static synchronized void fireAsync(Aladin aladin, Server server, String label, String url,
+			Map<String, Object> requestParams, int requestNumber) {
+		UWSFacade instance = getInstance(aladin);
+		instance.instantiateGui();
+		try {
+			URL requestUrl = TapManager.getUrl(url, null, null);
+			instance.handleJob(server, label, requestUrl, null, requestParams, false, requestNumber); //gavo gives error with PHASE = RUN on create job
+		} catch (MalformedURLException e) {
+			// TODO Auto-generated catch block
+			e.printStackTrace();
+		} catch (URISyntaxException e) {
+			// TODO Auto-generated catch block
+			e.printStackTrace();
+		}
+	}
+	
 	/**
 	 * Method where an asyn job is created and handled 
 	 * @param server 
 	 * @param query 
 	 * @param queryString 
-	 * @param postParams 
+	 * @param requestParams 
 	 * @param requestNumber 
 	 */
-	public void handleJob(Server server, String url, String queryString, ADQLQuery adqlQueryObj, Map<String, Object> postParams, int requestNumber) {
+	public void handleJob(Server server, String label, URL url, String queryString,
+			Map<String, Object> requestParams, boolean doRun, int requestNumber) {
 		UWSJob job = null;
 		try {
-			job = createJob(server, url, queryString, adqlQueryObj, postParams, requestNumber);
-//			printStringFromInputStream(job);
+			job = createJob(server, label, url, queryString, requestParams, doRun, requestNumber);
 			addNewJobToGui(job);
 			refreshGui();
 			job.run();
-//			printStringFromInputStream(job);
 			job.pollForCompletion(server, true, this, requestNumber);
-//			printStringFromInputStream(job);
 			
 		} catch (IOException e) {
 			// TODO Auto-generated catch block
@@ -159,7 +192,11 @@ public class UWSFacade implements ActionListener{
 			if (job == null || job.gui == null) {
 				errorMessageToUser.append("\n Unable to create job");
 			}
-			errorMessageToUser.append("\n For query: ").append(queryString).append(NEWLINE_CHAR).append(e.getMessage());
+			if (queryString != null) {
+				errorMessageToUser.append("\n For query: ").append(queryString).append(NEWLINE_CHAR).append(e.getMessage());
+			} else {
+				errorMessageToUser.append("\n at: ").append(url.toString()).append(NEWLINE_CHAR).append(e.getMessage());
+			}
 			
 			if (job != null && UWSJob.JOBNOTFOUND.equals(job.getCurrentPhase())) {//specific case of job not found
 				if (checkIfJobInCache(job)) {
@@ -180,37 +217,41 @@ public class UWSFacade implements ActionListener{
 					job.showAsErroneous();
 				} 
 			}
+			if (server != null) {
+				server.setStatusForCurrentRequest(requestNumber, Ball.NOK);
+			}
 		} catch (Exception e) {
 			// TODO Auto-generated catch block
 			if (job != null) {
 				job.showAsErroneous();
 			}
-			server.setStatusForCurrentRequest(requestNumber, Ball.NOK);
+			if (server != null) {
+				server.setStatusForCurrentRequest(requestNumber, Ball.NOK);
+			}
 			Aladin.error(asyncPanel, "Error with async job! "+e.getMessage());
 		}
-		
 	}
 	
 	/**
-	 * Method creates an sync job
+	 * Method creates an async job
 	 * @param query
 	 * @param queryString 
 	 * @param newJobCreationUrl
-	 * @param adqlQueryObj
 	 * @param postParams 
 	 * @param requestNumber 
 	 * @return
 	 * @throws Exception
 	 */
-	public UWSJob createJob(Server server, String url, String queryString, ADQLQuery adqlQueryObj, Map<String, Object> postParams, int requestNumber) throws Exception {
+	public UWSJob createJob(Server server, String jobLabel, URL requestUrl, String queryString,
+			Map<String, Object> postParams, boolean doRun, int requestNumber) throws Exception {
 		UWSJob job = null;
+		HttpURLConnection httpClient = null;
 		try {
-			URL tapServerRequestUrl = TapManager.getUrl(url, null, PATHASYNC);
-			Aladin.trace(3,"trying to createJob() uws for:: "+tapServerRequestUrl.toString());
+			Aladin.trace(3,"trying to createJob() uws for:: "+requestUrl.toString());
 //			URL tapServerRequestUrl = new URL("http://cdsportal.u-strasbg.fr/uwstuto/basic/timers");
 			MultiPartPostOutputStream.setTmpDir(Aladin.CACHEDIR);
 			String boundary = MultiPartPostOutputStream.createBoundary();
-			URLConnection urlConn = MultiPartPostOutputStream.createConnection(tapServerRequestUrl);
+			URLConnection urlConn = MultiPartPostOutputStream.createConnection(requestUrl);
 			urlConn.setRequestProperty("Accept", "*/*");
 			urlConn.setRequestProperty("Content-Type", MultiPartPostOutputStream.getContentType(boundary));
 			// set some other request headers...
@@ -225,27 +266,15 @@ public class UWSFacade implements ActionListener{
 //			out.writeField("format", "votable");
 			
 			
-			out.writeField( "REQUEST", "doQuery" );
-			out.writeField( "LANG", "ADQL" );
+//			out.writeField( "REQUEST", "doQuery" );
+//			out.writeField( "LANG", "ADQL" );
+//			
+//			Aladin.trace(3,"createJob() REQUEST :: doQuery");
+//			Aladin.trace(3,"createJob() LANG :: ADQL");
+//			
+//			out.writeField("QUERY", queryString);
+//			Aladin.trace(3,"createJob() QUERY :: "+queryString);
 			
-			Aladin.trace(3,"createJob() REQUEST :: doQuery");
-			Aladin.trace(3,"createJob() LANG :: ADQL");
-			
-			if (adqlQueryObj != null) {
-				int limit = adqlQueryObj.getSelect().getLimit();
-				if (limit > 0) {
-					out.writeField("MAXREC", String.valueOf(limit));
-					Aladin.trace(3,"createJob() MAXREC :: "+String.valueOf(limit));
-				}
-			}
-			
-			out.writeField("QUERY", queryString);
-			Aladin.trace(3,"createJob() QUERY :: "+queryString);
-			
-			out.writeField("PHASE", "RUN"); // remove this if we start comparing quotes
-			Aladin.trace(3,"createJob() PHASE :: "+"RUN");
-//			out.writeField("time", "10");
-//			out.writeField("name", "ti");
 			
 			if (postParams != null) {//this part only for upload as of now
 				for (Entry<String, Object> postParam : postParams.entrySet()) {
@@ -259,37 +288,55 @@ public class UWSFacade implements ActionListener{
 				}
 			}
 			
+			if (doRun) { //gavo won't allow in case of async soda service.
+				out.writeField("PHASE", "RUN"); // remove this if we start comparing quotes
+				Aladin.trace(3,"createJob() PHASE :: "+"RUN");
+			}
+
+//			out.writeField("time", "10");
+//			out.writeField("name", "ti");
+			
 			out.close();
 			if (!(urlConn instanceof HttpURLConnection)) {
 				throw new Exception("Error url is not http!");
 			}
 			
-			HttpURLConnection httpClient = (HttpURLConnection) urlConn;
+			httpClient = (HttpURLConnection) urlConn;
 			httpClient.setInstanceFollowRedirects(false);
 			
 			if (httpClient.getResponseCode() == HttpURLConnection.HTTP_SEE_OTHER) {// is accepted
 				String location = httpClient.getHeaderField("Location");
-				job = new UWSJob(this, server.tapClient.tapLabel, new URL(location));
+				job = new UWSJob(this, jobLabel, new URL(location));
 				populateJob(job.getLocation().openStream(), job);
-				job.setQuery(adqlQueryObj.toADQL());
+				if (postParams != null && postParams.containsKey("QUERY")) {
+					job.setQuery((String) postParams.get("QUERY"));
+				}
+				
 				job.setDeleteOnExit(true);
 //				getsetPhase(job);
 				job.setInitialGui();
 			} else {
-				Aladin.trace(3,"createJob() ERROR !! did not get a url redirect. reponse code "+httpClient.getResponseCode());
-				throw new Exception("Error in calling tap server: "+tapServerRequestUrl+"\n"+httpClient.getResponseMessage());
+				String errorMessage = Util.handleErrorResponseForTapAndDL(requestUrl, httpClient);
+				//also for erroneous scenarios where error code is <400
+				Aladin.trace(3,"createJob() ERROR !! did not get a url redirect. \n"+errorMessage);
+				throw new Exception(errorMessage);
+				
 			}
-			httpClient.disconnect();
 		} catch (IOException e) {
 			e.printStackTrace();
 			throw e;
+		} finally {
+			if (httpClient != null) {
+				httpClient.disconnect();
+			}
 		}
-		Aladin.trace(3,"In createJob. Jon phase is:"+job.getCurrentPhase());
+		Aladin.trace(3,"In createJob. Job phase is:"+job.getCurrentPhase());
 		job.server = server;
 		job.requestNumber = requestNumber;
 		return job;
 	}
 	
+	
 	/**
 	 * Adds newly created job into front-end and model
 	 * @param job
@@ -411,6 +458,7 @@ public class UWSFacade implements ActionListener{
 	
 	/**
 	 * Method updates job based on the new uws status
+	 * UWS spec 1.1, Section 2.2.2: the default behavior is to return XML, "accept" not set explicitly.
 	 * @param inputStream
 	 * @param job
 	 * @throws Exception 
@@ -444,9 +492,9 @@ public class UWSFacade implements ActionListener{
 	/**
 	 * Method deletes UWS job.
 	 * @param job
-	 * @throws IOException 
+	 * @throws IOException, Exception
 	 */
-	public void deleteJob(UWSJob job, boolean updateModelAndGui) throws IOException {
+	public void deleteJob(UWSJob job, boolean updateModelAndGui) throws IOException, Exception {
 		HttpURLConnection httpConn = null;
 		try {
 			URLConnection conn = job.getLocation().openConnection();
@@ -454,26 +502,42 @@ public class UWSFacade implements ActionListener{
 				httpConn = (HttpURLConnection) conn;
 				httpConn.setInstanceFollowRedirects(false);
 				httpConn.setRequestMethod("DELETE");
+				httpConn.setConnectTimeout(DELETETIMEOUTTIME);
 				httpConn.connect();
 				if (httpConn.getResponseCode() == HttpURLConnection.HTTP_SEE_OTHER) {
 					if (updateModelAndGui) {
 						removeJobUpdateGui(job);
 					}
 				} else {
-					Aladin.trace(3, "Error when deleting job! Http response is: "+httpConn.getResponseCode());
-					throw new IOException("Error when deleting job!");
+					if (httpConn.getResponseCode() == HttpURLConnection.HTTP_NOT_FOUND) {
+						int option = JOptionPane.showConfirmDialog(asyncPanel, UWSASKTOREMOVENOTFOUNDJOBS,
+								UWSASKTOREMOVENOTFOUNDJOBSTITLE, JOptionPane.OK_CANCEL_OPTION);
+						if (option == JOptionPane.OK_OPTION) {
+							removeJobUpdateGui(job);
+							return;
+						}
+					}
+					StringBuffer errorMessage = new StringBuffer(JOBDELETEERRORMESSAGE);
+					errorMessage.append("\n")
+					.append(Util.handleErrorResponseForTapAndDL(job.getLocation(), httpConn));
+					Aladin.trace(3, "Error when deleting job!"+errorMessage.toString());
+					throw new IOException(errorMessage.toString());
 				}
 			}
 		} catch (ProtocolException e) {
 			// TODO Auto-generated catch block
-			e.printStackTrace();
+			if(Aladin.levelTrace >=3 ) e.printStackTrace();
 			throw e;
 		} catch (IOException e) {
 			// TODO Auto-generated catch block
-			e.printStackTrace();
+			if(Aladin.levelTrace >=3 ) e.printStackTrace();
+			throw e;
+		} catch (Exception e) {
+			// TODO Auto-generated catch block
+			if(Aladin.levelTrace >=3 ) e.printStackTrace();
 			throw e;
 		} finally {
-			if (httpConn!=null) {
+			if (httpConn != null) {
 				httpConn.disconnect();
 			}
 		}
@@ -484,6 +548,9 @@ public class UWSFacade implements ActionListener{
 	 * @return
 	 */
 	public MySplitPane instantiateGui() {
+		if (this.jobControllerGui == null) {
+			this.jobControllerGui = new FrameSimple(aladin);
+		}
 		if (asyncPanel == null) {
 			JPanel topPanel = new JPanel(new GridBagLayout());
 			
@@ -512,7 +579,7 @@ public class UWSFacade implements ActionListener{
 //			c.anchor = GridBagConstraints.NONE;
 			c.fill = GridBagConstraints.BOTH;
 			c.insets = new Insets(4, 7, 0, 4);
-			jobsScroll.setBorder(BorderFactory.createTitledBorder("Asynchronous jobs of current session:"));
+			jobsScroll.setBorder(BorderFactory.createTitledBorder(UWSPANELCURRECTSESSIONTITLE));
 			topPanel.add(jobsScroll,c);
 			
 			JPanel searchPanel = new JPanel();
@@ -534,7 +601,7 @@ public class UWSFacade implements ActionListener{
 			c.insets = new Insets(7, 7, 0, 4);
 			c.fill = GridBagConstraints.HORIZONTAL;
 //			c.anchor = GridBagConstraints.BASELINE;
-			searchPanel.setBorder(BorderFactory.createTitledBorder("Or choose an already submitted job:"));
+			searchPanel.setBorder(BorderFactory.createTitledBorder(UWSPANELPREVIOUSSESSIONTITLE));
 			topPanel.add(searchPanel,c);
 			
 			JPanel actionPanel = new JPanel();
@@ -558,7 +625,7 @@ public class UWSFacade implements ActionListener{
 			button.setActionCommand(DELETEJOB);
 			button.addActionListener(this);
 			actionPanel.add(button);
-			deleteOnExit = new JCheckBox("Delete on closing Aladin");
+			deleteOnExit = new JCheckBox(DELETEONCLOSEBUTTONLABEL);
 			deleteOnExit.setActionCommand(DELETEONEXIT);
 			deleteOnExit.addActionListener(this);
 			deleteOnExit.setSelected(true);
@@ -641,7 +708,7 @@ public class UWSFacade implements ActionListener{
 				if (uwsJob.isDeleteOnExit()) {
 					try {
 						deleteJob(uwsJob, false);
-					} catch (IOException e) {
+					} catch (Exception e) {
 						// We do nothing on quit
 					}
 				}
@@ -670,10 +737,25 @@ public class UWSFacade implements ActionListener{
 					}
 				}
 			} else {
-				resultsUrl = uwsJob.getDefaultResultsUrl();
+				//try to load a single result 
+				//if multiple ask user to choose..show message
+				resultsUrl = uwsJob.getIfSingleResult();
+				if (resultsUrl == null) {
+					showAsyncPanel();
+					Aladin.info(asyncPanel, UWSMULTIJOBLOADMESSAGE);
+					return;
+				} else {
+					resultsUrl = getResultsPath(uwsJob, resultsUrl);
+				}
 			}
 			URL urlToLoad = TapManager.getUrl(resultsUrl, null, null);
-			TapManager.handleResponse(aladin, urlToLoad, null, uwsJob.getServerLabel(), server, uwsJob.getQuery(), requestNumber);
+			try {
+				TapManager.handleResponse(aladin, urlToLoad, null, uwsJob.getServerLabel(), server, uwsJob.getQuery(), requestNumber);
+			} catch (IOException e) {
+				// TODO: handle exception
+				showAsyncPanel();
+				throw e;
+			}
 //			aladin.calque.newPlan(resultsUrl, uwsJob.getServerLabel(), null);
 		}
 	}
@@ -683,41 +765,58 @@ public class UWSFacade implements ActionListener{
 		URL resultsUrl = null;
 		if (resultToLoad.startsWith("http") || resultToLoad.startsWith("https")) {
 			resultsUrl = validateUrlSimple(resultToLoad);
-			if (resultsUrl != null) {
-				resultsUrlString = resultsUrl.toString();
-			}
-		} else {//its path to be constructed from original url. we will assume located on the same server
+		} else {//if just path is provided and not full url: its path to be constructed from original url. we will assume located on the same server
 			try {
-				resultsUrl =  new URL(uwsJob.getLocation().getProtocol(), uwsJob.getLocation().getHost(), uwsJob.getLocation().getPort(), URLDecoder.decode(resultToLoad, Constants.UTF8));
-				resultsUrl.toURI();
+				resultsUrl = new URL(uwsJob.getLocation().getProtocol(), uwsJob.getLocation().getHost(), uwsJob.getLocation().getPort(), URLDecoder.decode(resultToLoad, Constants.UTF8));
+				resultsUrl = validateUrlSimple(resultsUrl);
+				if (resultsUrl == null) {
+					resultsUrl = new URL(uwsJob.getLocation().getProtocol(), uwsJob.getLocation().getHost(), uwsJob.getLocation().getPort(), resultToLoad);
+					resultsUrl = validateUrlSimple(resultsUrl);
+				}
 			} catch (MalformedURLException e) {
 				// TODO Auto-generated catch block
-				e.printStackTrace();
+				Aladin.trace(3, e.getMessage());
 			} catch (UnsupportedEncodingException e) {
 				// TODO Auto-generated catch block
-				e.printStackTrace();
-			} catch (URISyntaxException e) {
-				// TODO Auto-generated catch block
-				e.printStackTrace();
-			}
-			
-			if (resultsUrl != null) {
-				resultsUrlString = resultsUrl.toString();
+				Aladin.trace(3, e.getMessage());
 			}
 			//parse if relative path ::
 			//server : http://heasarc.gsfc.nasa.gov/xamin/vo/tap/async/1479376281691_5
 			//results path : /xamin/vo/tap/async/1479376281691_5/results/result
 			//server base url : http://heasarc.gsfc.nasa.gov/xamin/vo/tap/
+			
+			//some server results arrive un encoded example:
+			//result0=/skynode/do/tap/spcam/async/181112212721+0900_1184_107_9922775895291997506/results/result
+		}
+		if (resultsUrl != null) {
+			resultsUrlString = resultsUrl.toString();
 		}
 		return resultsUrlString;
 	}
 	
 	public URL validateUrlSimple(String resultToLoad) {
+		URL resultsUrl = validateUrlSimple(resultToLoad, true);
+		if (resultsUrl == null) {
+			resultsUrl = validateUrlSimple(resultToLoad, false);
+		}
+		return resultsUrl;
+	}
+	
+	public URL validateUrlSimple(String resultToLoad, boolean decode) {
 		URL resultsUrl = null;
 		try {
-			resultsUrl = new URL(URLDecoder.decode(resultToLoad, Constants.UTF8));
+			if (decode) {
+				resultsUrl = new URL(URLDecoder.decode(resultToLoad, Constants.UTF8));
+			} else {
+				resultsUrl = new URL(resultToLoad);
+			}
 			//if in application/x-www-form-urlencoded form
 			// as in case of csirohttp%3A%2F%2Fatoavo.atnf.csiro.au%2Ftap%2Fasync%2F1488450523217A%2Fresults%2Fresult
+			resultsUrl = validateUrlSimple(resultsUrl);
+			if (resultsUrl != null && resultsUrl.getAuthority() == null) {
+				Aladin.trace(3, "no authority " + resultsUrl);
+				resultsUrl = null;
+			}
 		} catch (MalformedURLException e) {
 			// TODO Auto-generated catch block
 			e.printStackTrace();
@@ -725,19 +824,21 @@ public class UWSFacade implements ActionListener{
 			// TODO Auto-generated catch block
 			e.printStackTrace();
 		}
-		
-		if (resultsUrl != null) {
-			try {
+		return resultsUrl;
+	}
+	
+	public URL validateUrlSimple(URL resultsUrl) {
+		try {
+			if (resultsUrl != null) {
 				resultsUrl.toURI();
-			} catch (URISyntaxException e) {
-				Aladin.trace(3, "URISyntaxException "+e.getMessage());
-				resultsUrl = null;
-			}
-			
-			if (resultsUrl.getAuthority() == null) {
-				Aladin.trace(3, "no authority " + resultsUrl);
-				resultsUrl = null;
+				if (resultsUrl.getAuthority() == null) {
+					Aladin.trace(3, "no authority " + resultsUrl);
+					resultsUrl = null;
+				}
 			}
+		} catch (URISyntaxException e) {
+			Aladin.trace(3, "URISyntaxException when trying to decode"+e.getMessage());
+			resultsUrl = null;
 		}
 		return resultsUrl;
 	}
@@ -748,13 +849,13 @@ public class UWSFacade implements ActionListener{
 		try {
 			if (prevJobRadio.isSelected()) {
 				if (previousSessionJob.getText().isEmpty()) {
-					throw new Exception("Please enter the job url!");
+					throw new Exception(NOJOBURLMESSAGE);
 				} else {
 					URL jobUrl = new URL(previousSessionJob.getText());
 					selectedJob = getJobFromCache(jobUrl.toString());
 					if (selectedJob == null) {
 						selectedJob = new UWSJob(this, EMPTYSTRING, jobUrl);
-						populateJob(selectedJob.getLocation().openStream(), selectedJob);
+						populateJob(Util.openStreamForTapAndDL(selectedJob.getLocation(), null, true, 10000), selectedJob);
 					}
 					if (loadJobSummary) {
 						selectedJob.setJobDetailsPanel();
@@ -768,7 +869,7 @@ public class UWSFacade implements ActionListener{
 				selectedJob = getSelectedInSessionJob();
 			}
 			if (selectedJob == null) {
-				throw new Exception("Cannot process results for selection! Please select again.");
+				throw new Exception(JOBNOTSELECTED);
 			}
 		} catch (MalformedURLException e1) {
 			// TODO Auto-generated catch block
@@ -777,7 +878,7 @@ public class UWSFacade implements ActionListener{
 		} catch (IOException e1) {
 			// TODO Auto-generated catch block
 			if (Aladin.levelTrace >= 3) e1.printStackTrace();
-			throw new IOException("No job found for the given url \n"+e1.getMessage());
+			throw new IOException(JOBNOTFOUNDGIVENURL+ " \n"+e1.getMessage());
 		}
 		return selectedJob;
 	}
@@ -837,7 +938,8 @@ public class UWSFacade implements ActionListener{
 			} else if (action.equals(LOADDEFAULTTAPRESULT)) {
 				try {
 					UWSJob selectedJob = processJobSelection(true);
-					loadResults(selectedJob, null, -1, null);
+					String resultsUrl = selectedJob.getDefaultResultsUrl();
+					loadResults(selectedJob, resultsUrl, -1, null);
 				} catch (MalformedURLException e1) {
 					// TODO Auto-generated catch block
 					Aladin.error(asyncPanel, "Error in processing results url!");
@@ -892,13 +994,11 @@ public class UWSFacade implements ActionListener{
 					}
 				} catch (IOException e1) {
 					// TODO Auto-generated catch block
-					e1.printStackTrace();
 					if (selectedJob != null) {
-						Aladin.error(asyncPanel, "Please try again. Error when aborting job: "+selectedJob.getJobId());
 						selectedJob.showErrorOnServer();
-					} else {
-						Aladin.error(asyncPanel, e1.getMessage());
-					}
+					} 
+					Aladin.trace(3, e1.getMessage());
+					Aladin.error(asyncPanel, e1.getMessage());
 				} catch (Exception e1) {
 					// TODO Auto-generated catch block
 					Aladin.error(asyncPanel, e1.getMessage());
@@ -946,4 +1046,16 @@ public class UWSFacade implements ActionListener{
 		}
 	}
 	
+	/**
+	 * Method shows the async panel
+	 * @throws Exception 
+	 */
+	public void showAsyncPanel(){
+		MySplitPane asyncPanel = this.instantiateGui();
+		this.jobControllerGui.show(asyncPanel, JOBCONTROLLERTITLE);
+//		tapFrameServer.tabbedTapThings.setSelectedIndex(1);
+//		tapFrameServer.setVisible(true);
+		this.jobControllerGui.toFront();
+	}
+	
 }
