@@ -1,10 +1,10 @@
-// Copyright 1999-2018 - Universit de Strasbourg/CNRS
+// Copyright 1999-2020 - Universit de Strasbourg/CNRS
 // The Aladin Desktop program is developped by the Centre de Donnes
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
 //
-//This file is part of Aladin.
+//This file is part of Aladin Desktop.
 //
 //    Aladin Desktop is free software: you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -16,17 +16,17 @@
 //    GNU General Public License for more details.
 //
 //    The GNU General Public License is available in COPYING file
-//    along with Aladin.
+//    along with Aladin Desktop.
 //
 
 package cds.aladin;
 
-import static cds.aladin.Constants.LOADDEFAULTTAPRESULT;
 import static cds.aladin.Constants.LOADJOBRESULT;
 import static cds.aladin.Constants.PATHPHASE;
 import static cds.aladin.Constants.PATHRESULTS;
 import static cds.aladin.Constants.UTF8;
 
+import java.awt.Component;
 import java.awt.Dimension;
 import java.awt.FlowLayout;
 import java.awt.Font;
@@ -164,6 +164,7 @@ public class UWSJob implements ActionListener{
 		HttpURLConnection httpClient = null;
 		try {
 			httpClient = createWritePostData(this.location.toString(), PATHPHASE, "PHASE", PHASEACTION_ABORT);
+			httpClient.setConnectTimeout(7000);
 			String previousPhase = this.currentPhase;
 			if (httpClient.getResponseCode() == HttpURLConnection.HTTP_SEE_OTHER) {
 				String locationString = httpClient.getHeaderField("Location");
@@ -171,9 +172,15 @@ public class UWSJob implements ActionListener{
 				httpClient.disconnect();
 				UWSFacade.populateJob(this.location.openStream(), this);
 			} else {
+				StringBuffer errorMessage = new StringBuffer("Error when aborting job:\n");
+				errorMessage.append("phase: ").append(this.currentPhase);
+				if (httpClient.getResponseCode() >= 400) {
+					errorMessage.append("\n")
+							.append(Util.handleErrorResponseForTapAndDL(this.location, httpClient));
+		        }
 				httpClient.disconnect();
 				notificationText = UWSFacade.CANTABORTJOB;
-				throw new IOException("Error when aborting job:\n"+this.location.toString()+"\n httpcode:"+httpClient.getResponseCode()+"\n phase: "+this.currentPhase);
+				throw new IOException(errorMessage.toString());
 			}
 			this.updateGui(previousPhase);
 		} catch (IOException e) {
@@ -204,10 +211,16 @@ public class UWSJob implements ActionListener{
 					httpClient.disconnect();
 					UWSFacade.populateJob(this.location.openStream(), this);
 				} else {
-					httpClient.disconnect();
 					this.currentPhase = UNEXPECTEDERROR+ "-Phase: "+this.currentPhase;//Unexpected Error(phase)
 					notificationText = UWSFacade.CANTSTARTJOB;
-					throw new IOException("Error when starting job:\n"+this.location.toString()+"\n httpcode:"+httpClient.getResponseCode()+"\n phase: "+this.currentPhase);
+					StringBuffer errorMessage = new StringBuffer("Error when starting job:\n");
+					errorMessage.append("\n phase: ").append(this.currentPhase);
+					if (httpClient.getResponseCode() >= 400) {
+						errorMessage.append("\n ")
+								.append(Util.handleErrorResponseForTapAndDL(this.location, httpClient));
+			        }
+					httpClient.disconnect();
+					throw new IOException(errorMessage.toString());
 				}
 				this.updateGui(previousPhase);
 			} catch (IOException e) {
@@ -256,7 +269,7 @@ public class UWSJob implements ActionListener{
 	 */
 	public void pollForCompletion(Server server, boolean useBlocking, UWSFacade uwsFacade, int requestNumber) throws IOException, InterruptedException, Exception {
 		try {
-			if (Aladin.levelTrace >= 3) System.out.println("pollForCompletion. Jon phase is: "+this.currentPhase);
+			if (Aladin.levelTrace >= 3) System.out.println("pollForCompletion. Job phase is: "+this.currentPhase);
 			URL jobInProgressUrl = this.location;
 			String previousPhase = this.currentPhase;
 			while (true) {
@@ -275,7 +288,11 @@ public class UWSJob implements ActionListener{
 					break;
 				} else if (this.currentPhase.equals(ERROR)) {
 					this.showAsErroneous();
-					server.setStatusForCurrentRequest(requestNumber, Ball.NOK);
+					this.uwsFacade.showAsyncPanel();
+					Aladin.error(uwsFacade.asyncPanel, UWSFacade.JOBERRORTOOLTIP);
+					if (server != null) {
+						server.setStatusForCurrentRequest(requestNumber, Ball.NOK);
+					}
 					break;
 				} else if (this.currentPhase.equals(PENDING) || this.currentPhase.equals(HELD)) {
 					this.run();
@@ -329,7 +346,14 @@ public class UWSJob implements ActionListener{
 				} else {
 					notificationText = genericErrorMessage+this.location.toString()+"\n phase: "+this.currentPhase;
 					this.currentPhase = UNEXPECTEDERROR+ "-Phase: "+this.currentPhase;//Unexpected Error(phase)
-					throw new IOException(notificationText);
+					StringBuffer errorMessage = new StringBuffer(genericErrorMessage);
+					errorMessage.append("\n phase: ").append(this.currentPhase);
+					if (httpConn.getResponseCode() >= 400) {
+						errorMessage.append("\n")
+								.append(Util.handleErrorResponseForTapAndDL(this.location, httpConn));
+			        }
+					httpConn.disconnect();
+					throw new IOException(errorMessage.toString());
 				}
 				httpConn.disconnect();
 			} else {
@@ -373,6 +397,14 @@ public class UWSJob implements ActionListener{
 		return result;
 	}
 	
+	public String getIfSingleResult() {
+		String singleResultUrl = null;
+		if (this.results != null && this.results.size() == 1) {
+			singleResultUrl = results.values().iterator().next();
+		}
+		return singleResultUrl;
+	}
+	
 	public void setInitialGui() {
 //		this.gui = new JRadioButton(this.serverLabel+", Job: "+this.location+"     "+this.currentPhase);
 		this.gui = new JRadioButton(getJobLabel());
@@ -387,8 +419,11 @@ public class UWSJob implements ActionListener{
 	 */
 	public String getJobLabel() {
 		StringBuffer radioLabel =  new StringBuffer("<html><p width=\"1600\">").append(this.currentPhase);
-		radioLabel.append(" , Start time: ").append(this.startTime)
-		.append(" , Query: ").append(this.query).append(" ( server: ").append(this.serverLabel).append(")</p></html>");
+		radioLabel.append(" , Start time: ").append(this.startTime);
+		if (this.query != null) {
+			radioLabel.append(" , Query: ").append(this.query.replaceAll("&", "&amp;").replaceAll(">", "&gt;").replaceAll("<", "&lt;"));
+		}
+		radioLabel.append(" ( server: ").append(this.serverLabel).append(")</p></html>");
 		return radioLabel.toString();
 	}
 	
@@ -462,22 +497,20 @@ public class UWSJob implements ActionListener{
 		jobDetails.add(new JLabel("Destruction: "+this.destructionTime));
 		jobDetails.add(new JLabel("Parameters: "+this.parameters));*/
 		
-		JTextPane summary = new JTextPane();
-		summary.setContentType("text/html");
-		summary.setText(this.getResponsetoDisplay());
-		jobDetails.add(summary);
-		
 		if (this.results!=null && !this.results.isEmpty()) {
 			JPanel resultsPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
 			resultsPanel.add(new JLabel("Load on Aladin: "));
 			displayResults = new JComboBox(this.results.values().toArray());
 			resultsPanel.add(displayResults);
+			resultsPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
+			jobDetails.add(resultsPanel);
 
 			JButton loadbutton = new JButton("LOAD");
 			loadbutton.setActionCommand(LOADJOBRESULT);
 			loadbutton.addActionListener(this);
-			resultsPanel.add(loadbutton);
-			jobDetails.add(resultsPanel);
+			resultsPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
+//			resultsPanel.add(loadbutton);
+			jobDetails.add(loadbutton);
 			
 			/*JButton loadDefault = new JButton(UWSFacade.STANDARDRESULTSLOAD);
 			loadDefault.setToolTipText(UWSFacade.STANDARDRESULTSLOADTIP);
@@ -501,6 +534,14 @@ public class UWSJob implements ActionListener{
 			});
 			jobDetails.add(button);
 		}
+		
+		JTextPane summary = new JTextPane();
+		summary.setContentType("text/html");
+		summary.setText(this.getResponsetoDisplay());
+		summary.setAlignmentX(Component.LEFT_ALIGNMENT);
+		summary.setCaretPosition(0);
+		jobDetails.add(summary);
+		
 		if (!jobDetails.isVisible()) {
 			jobDetails.setVisible(true);
 		}
@@ -596,20 +637,6 @@ public class UWSJob implements ActionListener{
 					// TODO Auto-generated catch block
 					Aladin.error(uwsFacade.asyncPanel, e1.getMessage());
 				}
-				
-			} else if (action.equals(LOADDEFAULTTAPRESULT)) {
-				try {
-					uwsFacade.loadResults(this, null, -1, null);
-				} catch (MalformedURLException e1) {
-					// TODO Auto-generated catch block
-					Aladin.error(uwsFacade.asyncPanel, "Error in processing results url!");
-				} catch (IOException e1) {
-					// TODO Auto-generated catch block
-					Aladin.error(uwsFacade.asyncPanel, "Unable to get the job information, please try again!");
-				} catch (Exception e1) {
-					// TODO Auto-generated catch block
-					Aladin.error(uwsFacade.asyncPanel, e1.getMessage());
-				}
 			}
 		}
 	}
@@ -617,7 +644,9 @@ public class UWSJob implements ActionListener{
 	public String getResponsetoDisplay() {
 		// TODO Auto-generated method stub
 		responseBody = new StringBuffer("<html><p>");
-		responseBody.append("Job created to execute query: <b>").append(query);
+		if (query != null) {
+			responseBody.append("Job created to execute query: <b>").append(query);
+		}
 		responseBody.append("</b><br>Job ID: ").append(this.jobId).append("<br>Run ID: ").append(this.runId)
 				.append("<br>URL: <b>").append(this.location.toString()).append("<br></b>Owner ID: ")
 				.append(this.ownerId).append("<br>Phase: ").append(this.currentPhase).append("<br>Quote: ")
@@ -628,11 +657,11 @@ public class UWSJob implements ActionListener{
 			responseBody.append("<br>Parameters: ").append(this.parameters);
 		}
 		if (this.results != null && !this.results.isEmpty()) {
-			responseBody.append("<br>Results: ").append(this.results);
+			responseBody.append("<br>Results: ").append(this.results.toString().replaceAll("&", "&amp;").replaceAll(">", "&gt;").replaceAll("<", "&lt;"));
 		}
 		if (errorType != null) {
 			responseBody.append("<br>Error type: ").append(this.errorType).append("<br>Error message: ")
-					.append(this.errorMessage);
+					.append(this.errorMessage.replaceAll("&", "&amp;").replaceAll(">", "&gt;").replaceAll("<", "&lt;"));
 		}
 		if (jobInfoXml != null) {
 			responseBody.append("<br>Job info: ").append(this.jobInfoXml);
