@@ -1,10 +1,10 @@
-// Copyright 1999-2018 - Universit de Strasbourg/CNRS
+// Copyright 1999-2020 - Universit de Strasbourg/CNRS
 // The Aladin Desktop program is developped by the Centre de Donnes
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
 //
-//This file is part of Aladin.
+//This file is part of Aladin Desktop.
 //
 //    Aladin Desktop is free software: you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 //    GNU General Public License for more details.
 //
 //    The GNU General Public License is available in COPYING file
-//    along with Aladin.
+//    along with Aladin Desktop.
 //
 
 package cds.allsky;
@@ -25,9 +25,16 @@ import java.io.BufferedInputStream;
 import java.io.File;
 import java.io.FileInputStream;
 import java.io.InputStreamReader;
+import java.nio.file.Files;
+import java.util.HashMap;
+import java.util.Iterator;
 
 import cds.aladin.MyProperties;
+import cds.aladin.Tok;
 import cds.fits.Fits;
+import cds.moc.SMoc;
+import cds.mocmulti.MultiMoc;
+import cds.tools.pixtools.CDSHealpix;
 import cds.tools.pixtools.Util;
 
 /**
@@ -46,6 +53,7 @@ public abstract class Builder {
    static public Builder createBuilder(Context context,Action action) throws Exception {
       switch(action) {
          case INDEX:     return new BuilderIndex(context);
+//         case TINDEX:    return new BuilderTIndex(context);
          case TILES:     return new BuilderTiles(context);
          case ALLSKY:    return new BuilderAllsky(context);
          case JPEG:      return new BuilderJpg(context);
@@ -54,7 +62,9 @@ public abstract class Builder {
          case MOCERROR:  return new BuilderMocError(context);
          case MOCINDEX:  return new BuilderMocIndex(context);
          case CLEAN:     return new BuilderClean(context);
+         case CLEANALL:  return new BuilderCleanAll(context);
          case CLEANINDEX:return new BuilderCleanIndex(context);
+         case CLEANTINDEX:return new BuilderCleanTIndex(context);
          case CLEANDETAILS:return new BuilderCleanDetails(context);
          case CLEANTILES:return new BuilderCleanTiles(context);
          case CLEANFITS: return new BuilderCleanFits(context);
@@ -63,7 +73,7 @@ public abstract class Builder {
          case CLEANDATE: return new BuilderCleanDate(context);
          case CLEANWEIGHT:return new BuilderCleanWeight(context);
          case LINT:      return new BuilderLint(context);
-         case GZIP:      return new BuilderGzip(context);
+//         case GZIP:      return new BuilderGzip(context);
          case GUNZIP:    return new BuilderGunzip(context);
          case RGB:       return new BuilderRgb(context);
          case TREE:      return new BuilderTree(context);
@@ -76,6 +86,9 @@ public abstract class Builder {
          case PROP:      return new BuilderProp(context);
          case MIRROR:    return new BuilderMirror(context);
          case MAP:       return new BuilderMap(context);
+         case TMOC:      return new BuilderTMoc(context);
+         case STMOC:     return new BuilderSTMoc(context);
+//         case ZIP:       return new BuilderZip(context);
          default: break;
       }
       throw new Exception("No builder associated to this action");
@@ -90,6 +103,9 @@ public abstract class Builder {
    /** Excute la tche */
    public abstract void run() throws Exception;
 
+   /** Excute la tche sans info technique */
+   protected void build() throws Exception {}
+      
    /** Retourne l'identificateur de l'action */
    public abstract Action getAction();
 
@@ -121,49 +137,86 @@ public abstract class Builder {
       }
       context.setValidateInput(true);
    }
-
+   
+   
+   static private String FS = cds.tools.Util.FS;
+   
+   
    // Vrifie que le rpertoire Output a t pass en paramtre, sinon essaye de le dduire
    // du rpertoire Input en ajoutant le suffixe HiPS
    // S'il existe dj, vrifie qu'il s'agit bien d'un rpertoire utilisable
    protected void validateOutput() throws Exception { 
       String output = context.getOutputPath();
-      if( output==null ) {
-         output = context.getInputPath();
-         if( output!=null && (output.startsWith("http://") || output.startsWith("https://"))) {
-            output = context.getInputPath();
-            int n = output.length();
-            if( output.charAt(n-1)=='/' ) n--;
-            int offset = output.lastIndexOf('/',n);
-            output = output.substring(offset+1,n);
-         } else {
-            
-            String id = context.getHipsId();
-            
-            // Pas d'id => On ajoute simplement le suffixe HiPS au rpertoire d'origine
-            if( id==null ) {
-               output = (output==null?"":output) + Constante.HIPS;
-               
-            // Un Id => on l'utilise comme nom de rpertoire cible (avec des _  la place des / et ?)
-            } else {
-               output = context.getInputPath();
-               output = output.replace('\\', '/');
-               int n = output.length();
-               if( output.charAt(n-1)=='/' ) n--;
-               int offset = output.lastIndexOf('/',n);
-               output = output.substring(0,offset+1);
-               id = id.substring(6);
-               id = id.replace('/','_');
-               id = id.replace('?','_');
-               output = output+id;
-            }
-            
-         }
-         context.setOutputPath(output);
+      
+
+      String name = null;
+      String path = null;
+      int i = output==null ? -1 : output.lastIndexOf('/');
+      if( i==-1 && output!=null ) i=output.lastIndexOf('\\');
+
+      // Path indiqu spcifiquement ?
+      if( i>=0 ) path = output.substring(0,i);
+
+      // non ! donc par dfaut le rpertoire contenant l'image ou le rpertoire des images originales
+      else {
+         String input = context.getInputPath();
+         if( input!=null && !input.startsWith("http://") && !input.startsWith("https://") ) {
+            int j = input.lastIndexOf(FS);
+            if( j>=0 ) path = input.substring(0, j);
+            else path = ".";
+
+            // Sauf en cas de mirroir => le rpertoire courant   
+         } else path = ".";
       }
+
+      // Nom indiqu spcifiquement ?
+      if( output!=null && i<output.length()-1 ) name = output.substring(i+1);
+
+      // non ! alors on le fabrique depuis l'ID
+      else {
+         String id = context.getHipsId();
+
+         // Pas d'id => On ajoute simplement le suffixe HiPS au rpertoire d'origine
+         if( id==null ) {
+            String input = context.getInputPath();
+            if( input!=null && !input.startsWith("http://") && !input.startsWith("https://") ) {
+               int j = input.lastIndexOf(FS);
+               name = input.substring(j+1)+Constante.HIPS;
+
+               // sauf s'il s'agit d'un MIRROR, ou alors on le prend tel que
+            } else {
+               int j = input.lastIndexOf('/');
+               name = input.substring(j+1);
+            }
+
+            // Un Id => on l'utilise comme nom de rpertoire cible (avec des _  la place des / et ?)
+         } else {
+            id = id.substring(6);
+            id = id.replace('/','_');
+            id = id.replace('?','_');
+            name = id;
+         }
+      }
+
+      output = path+FS+name;
+      context.setOutputPath( output );
+
       File f = new File(output);
       if( f.exists() && (!f.isDirectory()  || !f.canRead())) throw new Exception("Ouput directory not available ["+output+"]");
       context.info("the output directory will be "+output);
+      
+      
       context.setValidateOutput(true);
+      
+//      if( true ) System.exit(0);
+   }
+   
+   /** Vrifie que le rpertoire HpxIndex existe et peut tre utilis */
+   protected void validateIndex() throws Exception {
+      String path = context.getHpxFinderPath();
+      if( path==null ) throw new Exception("HEALPix index directory [HpxFinder] not defined => specify the output (or input) directory");
+      File f = new File(path);
+      if( !f.exists() || !f.isDirectory() || !f.canRead() ) throw new Exception("HEALPix index directory not available ["+path+"]");
    }
 
    // Rcupre l'ordre en fonction d'un rpertoire. Si un order particulier a t pass en paramtre,
@@ -176,6 +229,17 @@ public abstract class Builder {
          context.setOrder(orderIndex);
       } else if( orderIndex!=-1 && order!=orderIndex ) throw new Exception("Detected order ["+orderIndex+"] does not correspond to the param order ["+order+"]");
    }
+   
+   // Rcupre le tile side en fonction d'une tuile pr-existante. 
+   protected void validateTileSide(String path) throws Exception {
+      int tileSide = context.getTileSide();
+      if( tileSide==-1 ) return;
+      int tileSideByNpixFile = context.getTileWidthByNpixFile(path);
+      if( tileSideByNpixFile!=-1 && tileSide!=tileSideByNpixFile ) {
+         context.warning("Size of pre-existing tiles incoherent with declarative tileSize ("+tileSide+"). Will assume "+tileSideByNpixFile);
+         context.setTileOrder( (int)CDSHealpix.log2(tileSideByNpixFile ));
+      }
+   }
 
    /** Rcupration de la profondeur (cube) */
    protected void validateDepth() throws Exception {
@@ -346,7 +410,7 @@ public abstract class Builder {
    }
 
    protected void validateLabel() {
-      if( context.getLabel()!=null ) return;
+      if( context.label!=null ) return;
       String label = getALabel(context.getOutputPath(),context.getInputPath());
       if( label!=null && label.length()>0 ) context.label=label;
    }
@@ -363,9 +427,16 @@ public abstract class Builder {
             InputStreamReader in = new InputStreamReader( new BufferedInputStream( new FileInputStream(propFile) ), "UTF-8");
             prop.load(in);
             in.close();
-            String s = prop.getProperty(Constante.KEY_OBS_TITLE);
+            
+            String s =  MultiMoc.getID(prop);
+            s=context.getLabelFromHipsId(s);
+            if( s==null ) {
+               s = prop.getProperty(Constante.KEY_OBS_TITLE);
+            }
 //            String s = prop.getProperty(Constante.KEY_OBS_COLLECTION);
-            if( s==null ) s = prop.getProperty(Constante.OLD_OBS_COLLECTION);
+            if( s==null ) {
+               s = prop.getProperty(Constante.OLD_OBS_COLLECTION);
+            }
             if( s!=null && s.length()>0 ) label=s;
          }
       } catch( Exception e ) { }
@@ -455,4 +526,140 @@ public abstract class Builder {
 
       }
    }
+   
+   
+   // Gestion des paramtres pour le stockage dans une partition alternative
+   private class Part {
+      double size;   // taille courante en Ko
+      double max;    // taille max de la partition en Ko ou <=0 si non utilis
+      String dir;    // Nom du rpertoire root de la partition
+   }
+   
+   static final int MAXPART = 10;   // Nombre max de partitions d'un HiPS
+   protected Part part [];            // Gestion des partitions alternatives (mais pas la partition originale)
+   
+   // Retourne le nombre de Ko correspondant  une expression du genre 4.2g
+   private double getMem( String s ) throws Exception {
+      char u = Character.toLowerCase( s.charAt( s.length()-1 ) );
+      double fct = u=='k' ? 1 : u=='m' ? 1024 : u=='g' ? 1024*1024 : u=='t' ? 1024*1024*1024 : -1;
+      if( fct==-1 ) throw new Exception("Invalid mem unit");
+      return Double.parseDouble( s.substring( 0,s.length()-1 )) * fct;
+   }
+   
+   // Retourne la taille moyenne d'une tuile (tous les formats additionns) - en bytes
+   private long getTileSize( int bitpix, int width, int depth, String fmt ) {
+      long nbpix = width*width;
+      
+      // Cas couleur
+      if( bitpix==0 ) {
+         long nbBytesJpg = fmt.toLowerCase().indexOf("jpeg")>=0 ? (nbpix*4)/10L : 0L;
+         long nbBytesPng = fmt.toLowerCase().indexOf("png")>=0  ? (nbpix*4)/8L : 0L;
+         long nbBytes = (nbBytesJpg + nbBytesPng ) * depth;
+         return nbBytes;
+      } 
+      
+      // Cas classique
+      long nbBytesFits = fmt.toLowerCase().indexOf("fits")>=0 ? 2880 + Math.abs(bitpix)/8 * nbpix : 0L;
+      long nbBytesJpg = fmt.toLowerCase().indexOf("jpeg")>=0  ? nbBytesFits/10L : 0L;
+      long nbBytesPng = fmt.toLowerCase().indexOf("png")>=0   ? nbBytesFits/8L : 0L;
+      long nbBytes = (nbBytesFits + nbBytesJpg + nbBytesPng ) * depth;
+      return nbBytes;
+   }
+   
+   /** Gnration des liens symboliques et des rpertoires Dirnn  l'order le plus profond afin de
+     * pouvoir par la suite rpartir le HiPS sur plusieurs partitions
+     */
+   protected void validateSplit(String outputPath, String split, SMoc moc, int order, int bitpix, int tileWidth, int depth, String fmt) throws Exception {
+      
+      // Dtermination de la taille totale requise (en Ko)
+      moc.setMocOrder(order);
+      long numberOfTiles = moc.getNbCells();
+      long tileSize = getTileSize( bitpix, tileWidth, depth, fmt);
+      if( tileSize==0 ) throw new Exception("No remote tile found");
+      long fullSize = (long)( ( 1.3 * tileSize * numberOfTiles + 8 ) / 1024. );
+      context.stat("Max order="+order+", Number of tiles="+numberOfTiles+", tile size("+fmt+")="+cds.tools.Util.getUnitDisk( tileSize )+", HiPS size estimation="+ cds.tools.Util.getUnitDisk( fullSize*1024L ) );
+      
+      // Scan du paramtre de split
+      part = new Part[ MAXPART ];
+      String s = split;
+      Tok tok = new Tok(s,";");
+      double firstPartSize = getMem( tok.nextToken() );
+//      System.out.println("First part => "+Util.getUnitDisk( (long)( firstPartSize*1024L) ));
+      if( fullSize<=firstPartSize ) {
+         context.info("Enough space in master partition quota => no split required");
+         return;   // tout tient dans la premire partition
+      }
+      for( int n=0; tok.hasMoreTokens(); n++ ) {
+         if( n==MAXPART ) throw new Exception("Too many slitting partitions");
+         part[n] = new Part();
+         String s1 = tok.nextToken();
+         int i = s1.lastIndexOf(' ');
+         part[n].max = i<0 ? -1 : getMem( s1.substring(i+1) );
+         part[n].dir = i<0 ? s1 : s1.substring(0,i);
+         
+         if( !( new File(part[n].dir).isAbsolute() ) ) throw new Exception("alternative target partitions must use an absolute path");
+//         System.out.println(part[n].dir+" => "+Util.getUnitDisk( (long)( part[n].max)*1024L ));
+      }
+      
+      // Dtermination de la taille de chaque Dirnn de niveau le plus profond (en Ko)
+      HashMap<Long, Double> hashDir = new HashMap<>();
+      Iterator<Long> it = moc.pixelIterator();
+      while( it.hasNext() ) {
+         long ndirLink = it.next()/10000;
+         Double mem = hashDir.get(ndirLink);
+         if( mem==null ) hashDir.put(ndirLink, tileSize/1024. );
+         else hashDir.put( ndirLink, tileSize/1024. + mem);
+      }
+      
+      // Gnration des rpertoires et liens symboliques vers les partitions alternatives
+      String mainPart = outputPath;   // Path de la partition principale
+      int nAltPart = 0;   // indice de la partition alternative courante
+      int nlinks=0;       // Nombre de links pour la partition altenative courante
+      int totNlinks=0;    // Total du nombre de links vers les partitions alternatives
+      boolean first = true;
+      
+      for( Long norderLink : hashDir.keySet() ) {
+         
+         // Que va-t-on gagner comme place ?
+         double sizeLink = hashDir.get(norderLink);
+         fullSize -= sizeLink;
+         
+         // Plus de place sur la partition alternative courante ? on passe  la partition suivante
+         if( part[nAltPart].size > part[nAltPart].max && part[nAltPart].max>0 ) { 
+            context.stat(part[nAltPart].dir+" will content "+cds.tools.Util.getUnitDisk( (long)( part[nAltPart].size*1024L))+" thanks to "+nlinks+" links");
+            nAltPart++;
+            nlinks=0;
+         }
+         if( nAltPart>=part.length ) throw new Exception("Not enough space on alternative target partitions");
+         part[nAltPart].size += sizeLink;
+         nlinks++;
+         totNlinks++;
+         
+         // Cration du rpertoire Ndir sur la partition alternative, ainsi que du lien symbolique associ
+         if( !context.fake ) {
+            String sOrig = cds.tools.pixtools.Util.getFileDir( mainPart, order, norderLink*10000);
+            String sTarg = cds.tools.pixtools.Util.getFileDir( part[nAltPart].dir, order, norderLink*10000);
+
+            File fTarg = new File( sTarg );
+            File fOrig = new File( sOrig );
+
+            if( first ) cds.tools.Util.createPath(sOrig);
+            fTarg.mkdirs();
+            Files.createSymbolicLink(fOrig.toPath(), fTarg.toPath() );
+         }
+         first=false;
+         
+         // si on a fait assez de liens symboliques, pas la peine d'aller plus loin
+         if( fullSize<firstPartSize ) break;
+
+      }
+      
+      if( part[nAltPart].size!=0 ) context.stat(part[nAltPart].dir+" will content "+cds.tools.Util.getUnitDisk( (long)( part[nAltPart].size*1024L))+" thanks to "+nlinks+" links");
+      int nbdir = (hashDir.size()-totNlinks);
+      context.stat(mainPart+" will content "+cds.tools.Util.getUnitDisk( fullSize*1024L )+" (HiPS hierarchy"+(nbdir>0 ?" and "+nbdir+" directories)":")"));
+
+   }
+   
+
+   
 }
