@@ -1,10 +1,10 @@
-// Copyright 1999-2018 - Universit de Strasbourg/CNRS
+// Copyright 1999-2020 - Universit de Strasbourg/CNRS
 // The Aladin Desktop program is developped by the Centre de Donnes
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
 //
-//This file is part of Aladin.
+//This file is part of Aladin Desktop.
 //
 //    Aladin Desktop is free software: you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 //    GNU General Public License for more details.
 //
 //    The GNU General Public License is available in COPYING file
-//    along with Aladin.
+//    along with Aladin Desktop.
 //
 
 package cds.allsky;
@@ -31,7 +31,7 @@ import cds.aladin.HealpixProgen;
 import cds.aladin.MyProperties;
 import cds.aladin.PlanBG;
 import cds.fits.Fits;
-import cds.moc.HealpixMoc;
+import cds.moc.SMoc;
 import cds.tools.pixtools.Util;
 
 /** Fusion de 2 HiPS, puis reconstruction de l'arborescence, du allsky et du MOC
@@ -41,7 +41,7 @@ public class BuilderConcat extends BuilderTiles {
    //   private int statNbFile;
    //   private long statSize;
    //   private long startTime,totalTime;
-   private HealpixMoc inputMoc,outputMoc;
+   private SMoc inputMoc,outputMoc;
    private String outputPath;
    private String inputPath;
    private String outputPathIndex;
@@ -85,11 +85,11 @@ public class BuilderConcat extends BuilderTiles {
 
       // Mise  jour ou generation du MOC final
       String outputPath = context.getOutputPath();
-      outputMoc = new HealpixMoc();
+      outputMoc = new SMoc();
       File f = new File(outputPath+Util.FS+Constante.FILE_MOC);
       if( f.exists() ) {
          outputMoc.read( f.getCanonicalPath() );
-         outputMoc = outputMoc.union(inputMoc);
+         outputMoc = (SMoc)outputMoc.union(inputMoc);
          outputMoc.write( context.getOutputPath()+Util.FS+Constante.FILE_MOC);
          context.info("MOC updated");
       } else {
@@ -171,7 +171,7 @@ public class BuilderConcat extends BuilderTiles {
       doHpxFinder = (new File(inputPathIndex)).isDirectory() && (new File(outputPathIndex)).isDirectory();
       if( doHpxFinder ) context.info("HpxFinder will be also concatenated (mode="+mode+")");
 
-      inputMoc = new HealpixMoc();
+      inputMoc = new SMoc();
       f = new File(inputPath+Util.FS+Constante.FILE_MOC);
       if( f.exists() ) inputMoc.read( f.getCanonicalPath() );
       else {
@@ -183,7 +183,7 @@ public class BuilderConcat extends BuilderTiles {
          inputMoc.read( f.getCanonicalPath() );
       }
 
-      if( context.mocArea!=null ) inputMoc = inputMoc.intersection(context.mocArea);
+      if( context.mocArea!=null ) inputMoc = (SMoc)inputMoc.intersection(context.mocArea);
       context.moc = inputMoc;
 
       // Dans le cas de rngration des allsky.png il faut connatre les cuts
