@@ -1,10 +1,10 @@
-// Copyright 1999-2018 - Universit de Strasbourg/CNRS
+// Copyright 1999-2020 - Universit de Strasbourg/CNRS
 // The Aladin Desktop program is developped by the Centre de Donnes
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
 //
-//This file is part of Aladin.
+//This file is part of Aladin Desktop.
 //
 //    Aladin Desktop is free software: you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 //    GNU General Public License for more details.
 //
 //    The GNU General Public License is available in COPYING file
-//    along with Aladin.
+//    along with Aladin Desktop.
 //
 
 package cds.allsky;
@@ -26,6 +26,7 @@ import java.io.FileOutputStream;
 import java.io.RandomAccessFile;
 
 import cds.aladin.HealpixProgen;
+import cds.fits.Fits;
 import cds.tools.pixtools.Util;
 
 /** Construction de la hirarchie des tuiles d'index  partir des tuiles de plus bas
@@ -106,14 +107,6 @@ public class BuilderDetails extends Builder {
       context.initRegion();
    }
 
-   // Vrifie que le rpertoire HpxIndex existe et peut tre utilis
-   private void validateIndex() throws Exception {
-      String path = context.getHpxFinderPath();
-      if( path==null ) throw new Exception("HEALPix index directory [HpxFinder] not defined => specify the output (or input) directory");
-      File f = new File(path);
-      if( !f.exists() || !f.isDirectory() || !f.canRead() ) throw new Exception("HEALPix index directory not available ["+path+"]");
-   }
-
    private void validateImgWidth() throws Exception {
       String img = context.getImgEtalon();
       if( img==null && context.getInputPath()!=null) {
@@ -264,7 +257,7 @@ public class BuilderDetails extends Builder {
       return out;
    }
 
-   private static final String METADATA =
+   private static final String METADATA1 =
          "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
                "\n" +
                "<!-- VOTable HiPS hpxfinder mapping file.\n" +
@@ -307,6 +300,8 @@ public class BuilderDetails extends Builder {
                "          <DESCRIPTION>Instrument</DESCRIPTION>\n" +
                "       </FIELD \n" +
                "     -->\n" +
+               "";
+   private static final String METADATA2 =
                "<DATA>\n" +
                "   <TABLEDATA> \n" +
                "      <TR>\n" +
@@ -315,6 +310,8 @@ public class BuilderDetails extends Builder {
                "      <TD>$[name]</TD>\n" +
                "      <TD>$[path:([^\\[]*).*]</TD>\n" +
                "      <TD>$[stc]</TD>\n" +
+               "";
+    private static final String METADATA3 =
                "      <!-- Extended example via prefix and regular expression mapping\n" +
                "           (here, the instrument name is coded in the original path after \"data\" directory)\n" +
                "           <TD>Instrument: $[path:.*/data/(.+)/.*]</TD> \n" +
@@ -336,8 +333,12 @@ public class BuilderDetails extends Builder {
          context.info("Pre-existing "+Constante.FILE_METADATAXML+" file => keep it");
       } else {
          RandomAccessFile f = new RandomAccessFile(metadata ,"rw");
-         String s = METADATA.replace("YOUR_SURVEY_LABEL",context.getLabel()+" details");
+         String s = METADATA1.replace("YOUR_SURVEY_LABEL",context.getLabel()+" details");
          f.write(s.getBytes());
+         f.write( getAdditionalMetaFileFields().getBytes() );
+         f.write(METADATA2.getBytes());
+         f.write( getAdditionalMetaFileData().getBytes() );
+         f.write(METADATA3.getBytes());
          f.close();
          context.info("Mapping hpxFinder/"+Constante.FILE_METADATAXML+" file has been generated");
       }
@@ -346,6 +347,51 @@ public class BuilderDetails extends Builder {
       context.writeHpxFinderProperties();
       context.writeIndexHtml();
    }
+   
+   // Gnration des descriptions FIELD des champs additionnels indiqus via le paramtre
+   // -fitskeys=, ou automatiquement par le HEADER dans l'image talon
+   private String getAdditionalMetaFileFields() {
+      if( context.getFitsKeys()==null ) return "";
+      
+      // chargement de l'entte de l'image talon (si possible)
+      Fits fits = null;
+      String path=null;
+      try {
+         path = context.getImgEtalon();
+         fits = new Fits();
+         fits.loadHeaderFITS(path);
+      } catch( Exception e ) { 
+         context.warning("Reference image not available ["+path+"] => no metafile.xml FIELD description"); 
+         fits=null; 
+      }
+      
+      StringBuilder s = new StringBuilder();
+      for( String k : context.getFitsKeys() ) {
+         boolean flagNumeric = false;
+         try {
+            fits.headerFits.getDoubleFromHeader(k);
+            flagNumeric=true;
+         } catch( Exception e ) {}
+         String type = flagNumeric ? "datatype=\"double\"" : "datatype=\"char\" arraysize=\"*\"";
+         
+         String desc = fits==null ? null : fits.headerFits.getDescriptionFromHeader(k);
+         desc = desc==null ? "/>\n" : ">\n      <DESCRIPTION>"+desc+"</DESCRIPTION>\n   </FIELD>\n";
+         
+         s.append("   <FIELD name=\""+k+"\" "+type+desc);
+      }
+      return s.toString();
+   }
+
+   // Gnration des descriptions DATA des champs additionnels indiqus via le paramtre
+   // -fitskeys=, ou automatiquement par le HEADER dans l'image talon
+   private String getAdditionalMetaFileData() {
+      if( context.getFitsKeys()==null ) return "";
+      StringBuilder s = new StringBuilder();;
+      for( String k : context.getFitsKeys() ) {
+         s.append("      <TD>$["+k+"]</TD>\n");
+      }
+      return s.toString();
+   }
 
    //   // On crit le fichier des proprits
    //   private void writeProperties() throws Exception {
