@@ -1,10 +1,10 @@
-// Copyright 1999-2018 - Universit de Strasbourg/CNRS
+// Copyright 1999-2020 - Universit de Strasbourg/CNRS
 // The Aladin Desktop program is developped by the Centre de Donnes
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
 //
-//This file is part of Aladin.
+//This file is part of Aladin Desktop.
 //
 //    Aladin Desktop is free software: you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 //    GNU General Public License for more details.
 //
 //    The GNU General Public License is available in COPYING file
-//    along with Aladin.
+//    along with Aladin Desktop.
 //
 
 package cds.allsky;
@@ -32,7 +32,7 @@ import cds.aladin.Calib;
 import cds.aladin.Coord;
 import cds.aladin.Localisation;
 import cds.fits.Fits;
-import cds.moc.HealpixMoc;
+import cds.moc.SMoc;
 import cds.tools.pixtools.CDSHealpix;
 import cds.tools.pixtools.Util;
 
@@ -48,7 +48,7 @@ public class BuilderIndex extends Builder {
    private boolean partitioning;
    private double maxRatio;
    private int [] hdu = null;
-   private HealpixMoc area;                // rgion de travail
+   private SMoc area;                  // rgion de travail
    private boolean flagAppend;             // true => inutile de vrifier les doublons
    private int maxOverlays;                // Estimation du nombre max d'overlays 
                                            // (en dcomptant le nombre d'entres dans chaque tuile de HpxFinder=
@@ -84,7 +84,7 @@ public class BuilderIndex extends Builder {
    /** Ajout d'un fichier  la liste des fichiers images sources carts lors de l'indexation
     * suivi de la raison de son viction. */
    private void addBadFile(String file, String error) {
-      if( badFiles==null ) badFiles = new ArrayList<String>();
+      if( badFiles==null ) badFiles = new ArrayList<>();
       badFiles.add( file+(error!=null && error.length()>0 ? " => "+error:"") );
    }
 
@@ -126,18 +126,17 @@ public class BuilderIndex extends Builder {
       partitioning = context.partitioning;
       if( partitioning ) context.info("Partitioning large original image files in blocks of "+Constante.ORIGCELLWIDTH+"x"+Constante.ORIGCELLWIDTH+" pixels");
 
-      
       validateInput();
       validateOutput();
       validateLabel();
-
+      
       // S'il existe dj un hpxFinder, on va vrifier qu'on insre pas de doublons
       flagAppend = !context.isExistingIndexDir();
       if( !flagAppend ) context.info("Pre-existing HpxFinder index => will add new images only...");
       
       // Tests order
       int order = context.getOrder();
-      if( order==-1 ) {
+      if( order==-1 || context.getFitsKeys()==null ) {
          String img = context.getImgEtalon();
          if( img==null ) {
             img = context.justFindImgEtalon( context.getInputPath() );
@@ -145,11 +144,18 @@ public class BuilderIndex extends Builder {
          }
          if( img==null ) throw new Exception("No source image found in "+context.getInputPath());
          try {
-            Fits file = new Fits();
+            Fits file = new Fits( context.getHDU() );
             file.loadHeaderFITS(img);
-            long nside = calculateNSide(file.getCalib().GetResol()[0] * 3600.);
-            order = (Util.order((int) nside) - context.getTileOrder() );
-            if( order<3 ) order=3;
+            if( file.getCalib()==null ) throw new Exception("null calib");
+
+            // Recherche des fitsKey  garder par dfaut
+            context.defaultFitsKey = context.scanDefaultFitsKey( file.headerFits );
+
+            if( order==-1 ) {
+               long nside = calculateNSide(file.getCalib().GetResol()[0] * 3600.);
+               order = (Util.order((int) nside) - context.getTileOrder() );
+               if( order<3 ) order=3;
+            }
             context.setOrder(order);
          } catch (Exception e) {
             e.printStackTrace();
@@ -162,7 +168,7 @@ public class BuilderIndex extends Builder {
       } else if( order>context.getOrder() ) {
          context.warning("The provided order ["+order+"] is greater than the optimal order ["+context.getOrder()+"] => SUB-sample will be applied");
       } else context.info("Order="+context.getOrder()+" => PixelAngularRes="
-            +Coord.getUnit( CDSHealpix.pixRes( CDSHealpix.pow2(context.getOrder()+context.getTileOrder()))/3600. ) );
+            +Coord.getUnit( CDSHealpix.pixRes( context.getOrder()+context.getTileOrder())/3600. ) );
 
       int w = context.getTileSide();
       context.info("TileOrder="+context.getTileOrder()+" => tileSize="+w+"x"+w+" pixels");
@@ -180,9 +186,9 @@ public class BuilderIndex extends Builder {
 
       // Pour indiquer les listes des mots cls fits dont les valeurs vont tre retenues
       // dans les fichiers d'index JSON afin d'tre utiliser dans l'accs  la "Table des dtails" (progniteurs)
-      if( context.fitsKeys!=null ) {
+      if( context.getFitsKeys()!=null ) {
          StringBuilder res = null;
-         for( String key : context.fitsKeys ) {
+         for( String key : context.getFitsKeys() ) {
             if( res==null ) res = new StringBuilder();
             else res.append(", ");
             res.append(key);
@@ -214,7 +220,7 @@ public class BuilderIndex extends Builder {
 
 
    // Gnration de l'index
-   private boolean build() throws Exception {
+   protected void build() throws Exception {
       initStat();
       String input = context.getInputPath();
       String output = context.getOutputPath();
@@ -228,8 +234,6 @@ public class BuilderIndex extends Builder {
       String pathDest = context.getHpxFinderPath();
 
       create(input, pathDest, order);
-
-      return true;
    }
 
    // Initialisation des statistiques
@@ -302,18 +306,41 @@ public class BuilderIndex extends Builder {
 
          if( flagAppend ) out.seek( out.length() );
          else {
-            String s;
-            while( (s=out.readLine())!=null ) {
-               if( (s+"\n").equals(line) ) {
-//                  System.out.println("Bloc dj prsent dans la tuile => "+line);
-                  return;       // dj prsent dans la tuile d'index
-               }
+            if( checkIn(out,line) ) {
+//             System.out.print("Ligne dj prsente dans la tuile => "+line);
+             return;
+               
             }
+//            String s;
+//            while( (s=out.readLine())!=null ) {
+//               if( (s+"\n").equals(line) ) {
+////                  System.out.print("Ligne dj prsente dans la tuile => "+line);
+//                  return;       // dj prsent dans la tuile d'index
+//               }
+//            }
          }
          out.write( line.getBytes() );
+         out.close();
+         out=null;
       } finally { if( out!=null ) out.close(); }
    }
    
+   // Recherche rapide d'une chaine dans le fichier out
+   // Charge tout d'abord, puis compare ensuite.
+   private boolean checkIn(RandomAccessFile out, String line) throws IOException {
+      long size = out.length();
+      byte [] buf = new byte[(int)size];
+      out.readFully(buf);
+      int start=0;
+      for( int i=0; i<buf.length; i++ ) {
+         if( (char)buf[i]=='\n' ) {
+            if( (new String(buf,start,i-start+1)).equals(line) ) return true;
+            start=i+1;
+         }
+      }
+      return false;
+   }
+         
    // Pour chaque fichiers FITS, cherche la liste des losanges couvrant la
    // zone. Cr (ou complte) un fichier texte "d'index" contenant le chemin vers
    // les fichiers FITS
@@ -322,7 +349,7 @@ public class BuilderIndex extends Builder {
       // pour chaque fichier dans le sous rpertoire
       File main = new File(pathSource);
 
-      ArrayList<File> dir = new ArrayList<File>();
+      ArrayList<File> dir = new ArrayList<>();
       File[] list = context.isInputFile ? new File[]{ main } : main.listFiles();
       if (list == null) return true;
       
@@ -330,11 +357,13 @@ public class BuilderIndex extends Builder {
       int nbPilot = context.nbPilot;
 
       int i=0;
+      int nbFiles=0;
+      
       context.setProgress(0,list.length-1);
       for( File file : list ) {
          
          // S'agit-il d'un Pilot ?
-         if( nbPilot>=0 && i>nbPilot ) {
+         if( nbPilot>=0 && nbFiles>nbPilot ) {
             context.warning("Test Pilot limited to "+nbPilot+" images => partial HiPS");
             return false;
          }
@@ -365,6 +394,8 @@ public class BuilderIndex extends Builder {
                   if( flagDefaultHDU ) break;
                   else continue;
                }
+               
+              nbFiles++;
 
                if( firstDepth==0 ) firstDepth=fitsfile.depth;
                else if( fitsfile.depth!=firstDepth ) continue;
@@ -450,7 +481,7 @@ public class BuilderIndex extends Builder {
       // Recherche les 4 coins de l'image (cellule)
       Calib c = fitsfile.getCalib();
       boolean isCAR = c.getProj()==Calib.CAR;
-      ArrayList<double[]> cooList = new ArrayList<double[]>(4);
+      ArrayList<double[]> cooList = new ArrayList<>(4);
       Coord coo = new Coord();
       Coord corner[] = new Coord[4];
       Coord cornerCell[] = new Coord[4];
@@ -467,6 +498,9 @@ public class BuilderIndex extends Builder {
 //         System.out.print(" "+coo.al+" "+coo.del);
 
          cooList.add( context.ICRS2galIfRequired(coo.al, coo.del) );
+//         double [] cx = CDSHealpix.normalizeRaDec(coo.al, coo.del);
+//         cooList.add( context.ICRS2galIfRequired(cx[0], cx[1]) );
+         
          cornerCell[i] = new Coord(coo.al,coo.del);
 
          // S'il s'agit d'une cellule, il faut galement calcul le STC pour l'observation complte
@@ -500,9 +534,9 @@ public class BuilderIndex extends Builder {
       c.GetCoord(center);
 
       // Faut-il rcuprer des infos dans l'entte fits, ou dans la premire HDU
-      if( context.fitsKeys!=null ) {
+      if( context.getFitsKeys()!=null ) {
          StringBuilder res=null;
-         for( String key : context.fitsKeys ) {
+         for( String key : context.getFitsKeys() ) {
             String val;
             if( (val=fitsfile.headerFits.getStringFromHeader(key))==null ) {
                if( fitsfile.headerFits0==null
@@ -519,7 +553,7 @@ public class BuilderIndex extends Builder {
             * (fitsfile.bitpix==0 ?32 : Math.abs( fitsfile.bitpix ) /8);
 
       long[] npixs=null;
-      long nside = CDSHealpix.pow2(order);
+//      long nside = CDSHealpix.pow2(order);
       
       double maxRadius=0;
       for( Coord c2 : corner ) {
@@ -531,7 +565,16 @@ public class BuilderIndex extends Builder {
       // on vite les rayons trop grands pour ne pas tomber sur le cas d'un polygone concave
       if( radius<30 && !isCAR ) {
          try {
-            npixs = CDSHealpix.query_polygon(nside, cooList);
+//            System.out.print("draw stc polygon");
+//            for( double [] c1 : cooList ) System.out.print(" "+c1[0]+" "+c1[1]);
+//            System.out.println();
+//            System.out.print("draw moc "+order+"/");
+            
+            npixs = CDSHealpix.query_polygon(order, cooList, true);
+            
+//            for( long a : npixs ) System.out.print(" "+a);
+//            System.out.println();
+            
          } catch( Exception e ) { }
       }
          
@@ -566,7 +609,7 @@ public class BuilderIndex extends Builder {
             }
 
             double cent[] = context.ICRS2galIfRequired(center.al, center.del);
-            npixs = CDSHealpix.query_disc(nside, cent[0], cent[1], Math.toRadians(radius));
+            npixs = CDSHealpix.query_disc(order, cent[0], cent[1], Math.toRadians(radius),true);
          } catch( Exception e ) {
           throw new Exception("BuilderIndex error in CDSHealpix.query_disc() order="+order+" center="+center+" radius="+radius+"deg file="+fitsfile.getFilename()+" => ignored");
          }
