@@ -1,10 +1,10 @@
-// Copyright 1999-2018 - Universit de Strasbourg/CNRS
+// Copyright 1999-2020 - Universit de Strasbourg/CNRS
 // The Aladin Desktop program is developped by the Centre de Donnes
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
 //
-//This file is part of Aladin.
+//This file is part of Aladin Desktop.
 //
 //    Aladin Desktop is free software: you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 //    GNU General Public License for more details.
 //
 //    The GNU General Public License is available in COPYING file
-//    along with Aladin.
+//    along with Aladin Desktop.
 //
 
 package cds.allsky;
@@ -76,13 +76,17 @@ public class BuilderMapTiles extends Builder {
 
       if( !context.isTaskAborting() ) {
 
-         if( context.getOrder() <= 3 ) {
-            (new BuilderAllsky(context)).run();
-            context.info("ALLSKY file done");
-         } else {
-            (new BuilderTree(context)).run();
-            context.info("HiPS TREE done");
-         }
+//         if( context.getOrder() <= 3 ) {
+//            (new BuilderAllsky(context)).run();
+//            context.info("ALLSKY file done");
+//         } else {
+//            (new BuilderTree(context)).run();
+//            context.info("HiPS TREE done");
+//         }
+         
+         (new BuilderTree(context)).run();
+         context.info("HiPS TREE done");
+
 
          context.setProgressLastNorder3(1);
          
@@ -442,6 +446,11 @@ public class BuilderMapTiles extends Builder {
                if( context.isTaskAborting() ) throw new Exception("Task abort !");
                if( step == nbStep - 1 ) updateStat(n);
 
+               
+               // ON DEVRAIT PROBABLEMENT POUVOIR UTILISER UNE AUTRE VARIABLE QUI CONTIENT DEJA o
+               // MAIS JE PREFERE ASSURER SANS TROP ME CASSER LA TETE
+               int o = (int)CDSHealpix.log2(nside);
+               
                for( int i = 0; i < nbValPerSegment; i++ ) {
 
                   // Dtermination du numro de pixel HEALPix
@@ -451,7 +460,7 @@ public class BuilderMapTiles extends Builder {
                   else npix = count;
                   count++;
 
-                  if( !isNested ) npix = CDSHealpix.ring2nest(nside, npix);
+                  if( !isNested ) npix = CDSHealpix.ring2nest(o, npix);
 
                   // Rcupration de la valeur associe
                   double val = getVal(buf, bitpixOrig, offsetVal + cRecordInBuf * sizeRecord + i * sizeFieldVal);
