@@ -1,10 +1,10 @@
-// Copyright 1999-2018 - Universit de Strasbourg/CNRS
+// Copyright 1999-2020 - Universit de Strasbourg/CNRS
 // The Aladin Desktop program is developped by the Centre de Donnes
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
 //
-//This file is part of Aladin.
+//This file is part of Aladin Desktop.
 //
 //    Aladin Desktop is free software: you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 //    GNU General Public License for more details.
 //
 //    The GNU General Public License is available in COPYING file
-//    along with Aladin.
+//    along with Aladin Desktop.
 //
 
 package cds.allsky;
@@ -30,10 +30,11 @@ import java.io.InputStreamReader;
 import cds.aladin.CanvasColorMap;
 import cds.aladin.KernelList;
 import cds.aladin.MyProperties;
+import cds.aladin.PlanBG;
 import cds.aladin.PlanImage;
 import cds.aladin.Tok;
 import cds.fits.Fits;
-import cds.moc.HealpixMoc;
+import cds.moc.SMoc;
 import cds.tools.pixtools.Util;
 
 public class BuilderRgb extends BuilderTiles {
@@ -41,7 +42,7 @@ public class BuilderRgb extends BuilderTiles {
 
    private String [] inputs;         // Les paths des HiPS red, green, blue
    private int [] cubeIndex;         // Les indices des frames des composantes red, green, blue en cas de cubes d'origine (0 par dfaut)
-   private HealpixMoc [] moc;        // Les Mocs de chaque composante
+   private SMoc [] moc;        // Les Mocs de chaque composante
    private MyProperties [] prop;     // Les proprits de chaque composante
    private String [] labels;         // Les labels des HiPS red, green blue
    private String [] transfertFcts;  // Fonction de transfert  appliquer  chaque composante
@@ -60,6 +61,11 @@ public class BuilderRgb extends BuilderTiles {
    private int missing=-1;
    private boolean flagGauss;
    
+   private boolean flagLupton;
+   private double luptonS[];
+   private double luptonM[];
+   private double luptonQ;
+   
    private int statNbFile;
 
    private Mode coaddMode=Mode.REPLACETILE;
@@ -77,14 +83,14 @@ public class BuilderRgb extends BuilderTiles {
       context.setPropriete(Constante.KEY_HIPS_PROCESS_HIERARCHY, context.getJpegMethod().toString().toLowerCase());
 
       if( !context.isTaskAborting() ) (new BuilderMoc(context)).createMoc(output);
-      if( !context.isTaskAborting() ) { (new BuilderAllsky(context)).run(); context.info("ALLSKY file done"); }
+      if( !context.isTaskAborting() ) { (new BuilderAllsky(context)).run(); context.done("ALLSKY file done"); }
    }
    
    protected void activateCache(long size,long sizeCache) { }
    
    /** Demande d'affichage des stats via Task() */
    public void showStatistics() {
-      context.showRGBStat(statNbFile, totalTime,statNbThread,statNbThreadRunning);
+      context.showRGBStat(statNbFile, totalTime,getNbThreads(), getNbThreadRunning());
 //      if( !(context instanceof ContextGui ) ) super.showStatistics();
    }
    
@@ -139,6 +145,7 @@ public class BuilderRgb extends BuilderTiles {
    }
    
    public void validateContext() throws Exception {
+      
 
       String path = context.getRgbOutput();
       
@@ -156,7 +163,7 @@ public class BuilderRgb extends BuilderTiles {
       inputs = new String[3];
       cubeIndex = new int[3];
       labels = new String[3];
-      moc = new HealpixMoc[3];
+      moc = new SMoc[3];
       prop = new MyProperties[3];
       pixelMin = new double[3];
       pixelMiddle = new double[3];
@@ -180,11 +187,11 @@ public class BuilderRgb extends BuilderTiles {
             if( missing!=-1 ) throw new Exception("HiPS RGB generation required at least 2 original components");
             missing=c;
          } else {
-            
+
             if( !context.isExistingAllskyDir( inputs[c] ) ) {
                throw new Exception("Input HiPS missing ["+inputs[c]+"]");
             }
-                  
+
             if( pathRef==null ) pathRef=inputs[c];
 
             // Rcupration des proprits
@@ -206,18 +213,18 @@ public class BuilderRgb extends BuilderTiles {
             if( o>maxOrder ) maxOrder=o;
 
             // Ajustement de la rgion qu'il faudra calculer
-            HealpixMoc m = moc[c] = loadMoc( inputs[c] );
+            SMoc m = moc[c] = loadMoc( inputs[c] );
             if( context.moc==null ) context.moc = m;
-            else context.moc = context.moc.union(m);
-            
-           // Vrification de la cohrence des systmes de coordonnes
+            else context.moc = (SMoc)context.moc.union(m);
+
+            // Vrification de la cohrence des systmes de coordonnes
             String f = getFrameFromProp( prop[c] );
             if( frame==null ) frame=f;
             else if( !frame.equals(f) ) throw new Exception("Uncompatible coordsys for "+labels[c]);
 
          }
       }
-      
+
       // Si le rpertoire de destination est absent, je donne une valeur par dfaut
       if( path==null ) {
          int n = pathRef.length()-1;
@@ -227,11 +234,13 @@ public class BuilderRgb extends BuilderTiles {
       }
       this.output = path;
 
-      if( context instanceof ContextGui ) {
-         HealpixMoc m = context.moc;
-         ((ContextGui)context).mainPanel.clearForms();
-         context.moc = m;
-      }
+      if( context instanceof ContextGui ) context.resetProgressParam();
+      
+//      if( context instanceof ContextGui ) {
+//         SMoc m = context.moc;
+//         ((ContextGui)context).mainPanel.clearForms();
+//         context.moc = m;
+//      }
 
       // Pas d'ordre indiqu => on va utiliser l'ordre de la composante la plus profonde
       if( context.getOrder()==-1 ) {
@@ -248,18 +257,22 @@ public class BuilderRgb extends BuilderTiles {
 
       context.setBitpixOrig(0);
 
+      // Est-ce un traitement  la Lupton ?
+      if( context.flagLupton ) initLuptonParam();
 
       // mmorisation des informations de colormaps
       for( int c=0; c<3; c++) {
          if( c==missing ) continue;
-         String info =  labels[c]+" ["+pixelMin[c]+" "+pixelMiddle[c]+" "+pixelMax[c]+" "+transfertFcts[c]+"]";
+         String info;
+         if( flagLupton ) info = labels[c]+" [lupton min="+R(luptonM[c])+" scale="+R(luptonS[c])+" Q="+luptonQ+"]";
+         else info =  labels[c]+" ["+pixelMin[c]+" "+pixelMiddle[c]+" "+pixelMax[c]+" "+transfertFcts[c]+"]";
          if( c==0 ) context.redInfo=info;
          else if( c==1 ) context.greenInfo=info;
          else context.blueInfo=info;
       }
 
       // dtermination de la zone  calculer
-      if( context.mocArea!=null ) context.moc = context.moc.intersection( context.mocArea );
+      if( context.mocArea!=null ) context.moc = (SMoc)context.moc.intersection( context.mocArea );
       
       // Faut-il un filtre gaussien
       if( context.gaussFilter ) context.info("Gauss filter activated...");
@@ -267,12 +280,46 @@ public class BuilderRgb extends BuilderTiles {
 
       context.writeMetaFile();
    }
+   
+   
+   /** Estimation du paramtre S lupton (Scale) */
+   static public double estimateLuptonS( PlanBG p ) {
+      return 10/( p.getCutCtrlMax()- p.getCutCtrlMin() );
+   }
+
+   /** Estimation du paramtre M lupton (minimum) */
+   static public double estimateLuptonM( PlanBG p ) {
+      return -(( (p.getCutCtrlMin()+( p.getCutCtrlMax() - p.getCutCtrlMin())/20) * p.bScale + p.bZero) *  estimateLuptonS(p) ); 
+   }
+
+   // Initialisation des paramtres de l'algo Lupton en fonction de cutmin et cutmax
+   // (uniquement pour les valeurs non dtermines explicitement)
+   private void initLuptonParam() {
+      flagLupton=true;
+      luptonQ = !Double.isNaN(context.luptonQ) ? context.luptonQ : 20;
+      luptonS = new double[3];
+      luptonM = new double[3];
+      for(int c=0; c<3; c++ ) {
+         double z = 10/((pixelMax[c]*bscale[c]+bzero[c]) - (pixelMin[c]*bscale[c]+bzero[c]));
+         luptonS[c] = !Double.isNaN(context.luptonS[c]) ? context.luptonS[c] : z;
+         luptonM[c] = !Double.isNaN(context.luptonM[c]) ? context.luptonM[c]
+               : -(( (pixelMin[c]+(pixelMax[c]-pixelMin[c])/20)*bscale[c]+bzero[c]) * z); 
+      }
+      
+      String sm = (luptonM[0]==luptonM[1] && luptonM[1]==luptonM[2]) ? R(luptonM[0])
+            : R(luptonM[0])+"/"+R(luptonM[1])+"/"+R(luptonM[2]);
+      String ss = (luptonS[0]==luptonS[1] && luptonS[1]==luptonS[2]) ? R(luptonS[0])
+            : R(luptonS[0])+"/"+R(luptonS[1])+"/"+R(luptonS[2]);
+      context.info("Lupton algo: Q="+luptonQ+" m="+sm+" scale="+ss);
+   }
+   
+   private String R(double x) { return cds.tools.Util.myRound(x); }
 
    // Chargement d'un MOC, et par dfaut, d'un MOC couvrant tout le ciel
-   private HealpixMoc loadMoc( String path ) throws Exception {
+   private SMoc loadMoc( String path ) throws Exception {
       String s = path+Util.FS+"Moc.fits";
-      if( !(new File(s)).canRead() ) return new HealpixMoc("0/0-11");
-      HealpixMoc m = new HealpixMoc();
+      if( !(new File(s)).canRead() ) return new SMoc("0/0-11");
+      SMoc m = new SMoc();
       m.read(s);
       return m;
    }
@@ -350,7 +397,17 @@ public class BuilderRgb extends BuilderTiles {
          min = Double.parseDouble(tok.nextToken());
          max = Double.parseDouble(tok.nextToken());
       } catch( Exception e1 ) {
-         throw new Exception("Colormap parameter error ["+s+"] => usage: min [middle] max [fct]");
+         
+         // Pas fournis ? On va rcuprer les cuts depuis les properties
+         String s1 = getCmParamFromProp( prop[c] );
+         Tok tok1 = new Tok(s1);
+         try {
+            min = Double.parseDouble(tok1.nextToken());
+            max = Double.parseDouble(tok1.nextToken());
+            tok = new Tok(s);
+         } catch( Exception e2 ) {
+            throw new Exception("Colormap parameter error ["+s+"] => usage: min [middle] max [fct]");
+         }
       }
       int transfertFct;
 
@@ -389,6 +446,9 @@ public class BuilderRgb extends BuilderTiles {
       try { bzero = f.headerFits.getDoubleFromHeader("BZERO"); } catch( Exception e ) { }
       try { bscale = f.headerFits.getDoubleFromHeader("BSCALE"); } catch( Exception e ) { }
 
+      this.bscale[c] = bscale;
+      this.bzero[c] = bzero;
+      
       pixelMin[c]    = (min-bzero)/bscale;
       pixelMiddle[c] = (med-bzero)/bscale;
       pixelMax[c]    = (max-bzero)/bscale;
@@ -399,7 +459,7 @@ public class BuilderRgb extends BuilderTiles {
          tr1 = (int)( ( (med-min)/(max-min) ) *255 );
       }
 
-      context.info("Using pixelCut "+L(c)+" = "+min+(Double.isNaN(med)?"":" "+med)+" "+max+
+      if( !context.flagLupton ) context.info("Using pixelCut "+L(c)+" = "+min+(Double.isNaN(med)?"":" "+med)+" "+max+
             (transfertFct!=PlanImage.LINEAR ?" "+PlanImage.getTransfertFctInfo(transfertFct):""));
 
       IndexColorModel cm = CanvasColorMap.getCM(0, tr1, 255, false, PlanImage.CMGRAY, transfertFct, true );
@@ -408,22 +468,6 @@ public class BuilderRgb extends BuilderTiles {
 
    private String L(int c) { return c==0?"red":c==1?"green":"blue"; }
 
-
-//   public void build() throws Exception  {
-//      initStat();
-//      
-//      for( int i=0; i<768; i++ ) {
-//         if( context.isTaskAborting() ) new Exception("Task abort !");
-//         // Si le fichier existe dj on ne l'crase pas
-//         String rgbfile = Util.getFilePath(output,3, i)+Constante.TILE_EXTENSION[format];
-//         if( !context.isInMocTree(3, i) ) continue;
-//         if ((new File(rgbfile)).exists()) continue;
-//         createRGB(3,i);
-//         context.setProgressLastNorder3(i);
-//      }
-//   }
-   
-   
    protected Fits createLeaveHpx(ThreadBuilderTile hpx, String file,String path,int order,long npix, int z) throws Exception {
       long t = System.currentTimeMillis();
       Fits [] in = getLeaves(order, npix);
@@ -442,19 +486,101 @@ public class BuilderRgb extends BuilderTiles {
       return rgb;
    }
    
-//   protected void write(String file, Fits out) throws Exception {
-//      String filename = file+context.getTileExt();
-//      out.writeCompressed(filename,0,0,null, Constante.TILE_MODE[ context.targetColorMode ]);
-//   }
-   
-   // gnration du RGB  partir des composantes
+   // gnration du RGB
    private Fits createLeaveRGB(Fits [] in) throws Exception {
       
+      // Quel traitement ?
+      return flagLupton ? createLeaveRGBLupton(in,width,format,flagGauss,missing,luptonQ,luptonM,luptonS)
+            : createLeaveRGBClassic(in,width,format,flagGauss,missing,pixelMin,pixelMax,tcm);
+   } 
+   
+   static double arcsinh( double v ) {
+      return Math.log(v + Math.sqrt(Math.pow(v, 2.)+1));
+   }
+   
+
+   // Gnration du RGB  partir des composantes (mthode Lupton)
+   // Une des composantes peut tre manquante (remplace par la moyenne des deux autres)
+   // r = np.maximum(0, r + m)
+   // g = np.maximum(0, g + m)
+   // b = np.maximum(0, b + m)
+   // I = (r+g+b)/3.
+   // fI = np.arcsinh(Q * I) / np.sqrt(Q)
+   // I += (I == 0.) * 1e-6
+   // R = fI * r / I
+   // G = fI * g / I
+   // B = fI * b / I
+   public static Fits createLeaveRGBLupton(Fits [] in,int width,int format, boolean flagGauss, int missing,
+         double luptonQ, double [] luptonM, double [] luptonS) throws Exception {
+      
       // Application d'un filtre ?
       if( flagGauss ) {
          for( int i=0; i<3; i++ ) gaussian(in[i]);
       }
+
+      double Q2 = Math.sqrt(luptonQ);
       
+      double gap=0;
+      double range=256;
+      if( Fits.isTransparent(format==Constante.TILE_PNG ? Fits.PIX_255 : Fits.PIX_256) ) {
+         range = 255;
+         gap = 1;
+      }
+
+      Fits rgb = new Fits(width,width,0);
+      double [] pix = new double[3];
+      int i=0;
+      
+      for( int y=0; y<width; y++ ) {
+         for( int x=0; x<width; x++,i++ ) {
+            
+            double I=0;
+            int tot=0;
+            boolean allBlank=true;
+            for( int c=0; c<3; c++ ) {
+               if( c==missing || in[c]==null ) continue;
+               double v = in[c].getPixelFull(x,y);   // getPixelDouble(x,y);
+               if( in[c].isBlankPixel(v) ) pix[c] = 0;
+               else {
+                  allBlank=false;
+                  pix[c] = v * luptonS[c] + luptonM[c];
+                  if( pix[c]<0 ) pix[c]=0;
+               }
+               I += pix[c];
+               tot++;
+            }
+            if( allBlank ) { rgb.rgb[i]=0; continue; }
+            I /= tot;
+            if( I==0 ) I=1e-6;
+            double fI = arcsinh(luptonQ * I) / Q2;
+            if( missing!=-1 ) pix[missing] = I;
+            
+            int pixel = 0xFF;
+            for( int c=0; c<3; c++ ) {
+               double v = fI * pix[c] / I;
+               if( v>1-1e-6 ) v=1-1e-6;
+               v = v*range + gap;
+               v = v<gap ? gap : v>range ? range : v;
+               pixel = (pixel<<8) | ((int)v & 0xFF);
+            }
+
+            rgb.rgb[i]=pixel;
+         }
+      }
+      
+      return rgb;
+   }
+   
+   // Gnration du RGB  partir des composantes (mthode classique -> base sur la colormap)
+   // Une des composantes peut tre manquante (remplace par la moyenne des deux autres)
+   static public Fits createLeaveRGBClassic(Fits [] in,int width,int format,boolean flagGauss, int missing,
+         double [] pixelMin, double [] pixelMax, byte [][] tcm) throws Exception {
+
+      // Application d'un filtre ?
+      if( flagGauss ) {
+         for( int i=0; i<3; i++ ) gaussian(in[i]);
+      }
+
       double gap=0;
       double range=256;
       if( Fits.isTransparent(format==Constante.TILE_PNG ? Fits.PIX_255 : Fits.PIX_256) ) {
@@ -471,7 +597,7 @@ public class BuilderRgb extends BuilderTiles {
             double tot = 0;  // Pour faire la moyenne en cas d'une composante manquante
             for( int c=0; c<3; c++ ) {
                if( c==missing || in[c]==null ) continue;
-               double v = in[c].getPixelDouble(x,y); //width-y-1);
+               double v = in[c].getPixelDouble(x,y); 
                if( in[c].isBlankPixel(v) ) pix[c] = 0;
                else {
                   pix[c] = gap+ ( v< pixelMin[c] ? 0 : v> pixelMax[c] ? range-gap-1
@@ -515,6 +641,7 @@ public class BuilderRgb extends BuilderTiles {
    
    static final double [] kernel = KernelList.createFastGaussienMatrix(2, 0.8);
    
+   /** Convolution par une gaussienne */
    static public void gaussian(Fits in) {
       if( in==null ) return;
       double [] inPixels = new double[in.width*in.height];
@@ -556,153 +683,6 @@ public class BuilderRgb extends BuilderTiles {
    }
 
 
-
-
-   // Gnration des RGB rcursivement en repartant du niveau de meilleure rsolution
-   // afin de pouvoir utiliser un calcul de mdiane sur chacune des composantes R,G et B
-//   private Fits [] createRGB(int order, long npix) throws Exception {
-//
-//      if( context.isTaskAborting() ) new Exception("Task abort !");
-//
-//      // si on n'est pas dans le Moc, il faut retourner les 4 fits de son niveau
-//      // pour la construction de l'arborescence...
-//      if( !context.isInMocTree(order,npix) ) return createLeaveRGB(order, npix);
-//
-//      // si le losange a dj t calcul on renvoie les 4 fits de son niveau
-//      if( coaddMode==Mode.KEEPTILE ) {
-//         if( findLeaf(order, npix) ) {
-//            Fits [] oldOut = createLeaveRGB(order, npix);
-//            HealpixMoc moc = context.getRegion();
-//            moc = moc.intersection(new HealpixMoc(order+"/"+npix));
-//            int nbTiles = (int)moc.getUsedArea();
-//            updateStat(nbTiles);
-//            return oldOut;
-//         }
-//      }
-//
-//      // S'il n'existe pas au-moins 1 tuile de la composante  cette position, c'est une branche morte
-//      boolean trouve=false;
-//      for( int c=0; !trouve && c<3; c++ ) {
-//         if( c==missing ) continue;
-//         trouve = moc[c].isIntersecting(order, npix);
-//      }
-//      if( !trouve ) return null;
-//
-//      Fits [] out = null;
-//
-//      // Branche terminale
-//      boolean leaf=false;
-//      if( order==maxOrder ) {
-//         out = createLeaveRGB(order, npix);
-//         leaf=true;
-//      }
-//
-//      // Noeud intermdiaire
-//      else {
-//         Fits [][] fils = new Fits[4][3];
-//         boolean found = false;
-//         for( int i=0; i<4; i++ ) {
-//            fils[i] = createRGB(order+1,npix*4+i);
-//            if( fils[i] != null && !found ) found = true;
-//         }
-//         if( found ) out = createNodeRGB(fils);
-//      }
-//      if( out!=null ) generateRGB(out, order, npix, leaf);
-//      return out;
-//   }
-
-//   private boolean findLeaf(int order, long npix) throws Exception {
-//      String filename = Util.getFilePath(output,order, npix)+Constante.TILE_EXTENSION[format];
-//      File f = new File(filename);
-//      return f.exists();
-//   }
-
-   // Gnration d'un noeud par la mdiane pour les 3 composantes
-   // (on ne gnre pas encore le RGB (voir generateRGB(...))
-//   private Fits [] createNodeRGB(Fits [][] fils) throws Exception {
-//
-//      Fits [] out = new Fits[3];
-//      if( context.isTaskAborting() ) throw new Exception("Task abort !");
-//
-//      for( int c=0; c<3; c++ ) {
-//         out[c] = new Fits(width,width,bitpix[c]);
-//         out[c].setBlank(blank[c]);
-//         out[c].setBzero(bzero[c]);
-//         out[c].setBscale(bscale[c]);
-//      }
-//
-//      for( int dg=0; dg<2; dg++ ) {
-//         for( int hb=0; hb<2; hb++ ) {
-//            int quad = dg<<1 | hb;
-//            int offX = (dg*width)/2;
-//            int offY = ((1-hb)*width)/2;
-//
-//            for( int c=0; c<3; c++ ) {
-//               if( c==missing ) continue;
-//               Fits in = fils[quad]==null ? null : fils[quad][c];
-//               double p[] = new double[4];
-//               double coef[] = new double[4];
-//
-//               for( int y=0; y<width; y+=2 ) {
-//                  for( int x=0; x<width; x+=2 ) {
-//
-//                     double pix=blank[c];
-//                     if( in!=null ) {
-//
-//                        // On prend la moyenne (sans prendre en compte les BLANK)
-//                        if( method==Context.JpegMethod.MEAN ) {
-//                           double totalCoef=0;
-//                           for( int i=0; i<4; i++ ) {
-//                              int dx = i==1 || i==3 ? 1 : 0;
-//                              int dy = i>=2 ? 1 : 0;
-//                              p[i] = in.getPixelDouble(x+dx,y+dy);
-//                              if( in.isBlankPixel(p[i]) ) coef[i]=0;
-//                              else coef[i]=1;
-//                              totalCoef+=coef[i];
-//                           }
-//                           if( totalCoef!=0 ) {
-//                              pix = 0;
-//                              for( int i=0; i<4; i++ ) {
-//                                 if( coef[i]!=0 ) pix += p[i]*(coef[i]/totalCoef);
-//                              }
-//                           }
-//
-//                           // On garde la valeur mdiane (les BLANK seront automatiquement non retenus)
-//                        } else {
-//
-//                           double p1 = in.getPixelDouble(x,y);
-//                           if( in.isBlankPixel(p1) ) p1=Double.NaN;
-//                           double p2 = in.getPixelDouble(x+1,y);
-//                           if( in.isBlankPixel(p2) ) p1=Double.NaN;
-//                           double p3 = in.getPixelDouble(x,y+1);
-//                           if( in.isBlankPixel(p3) ) p1=Double.NaN;
-//                           double p4 = in.getPixelDouble(x+1,y+1);
-//                           if( in.isBlankPixel(p4) ) p1=Double.NaN;
-//
-//                           if( p1>p2 && (p1<p3 || p1<p4) || p1<p2 && (p1>p3 || p1>p4) ) pix=p1;
-//                           else if( p2>p1 && (p2<p3 || p2<p4) || p2<p1 && (p2>p3 || p2>p4) ) pix=p2;
-//                           else if( p3>p1 && (p3<p2 || p3<p4) || p3<p1 && (p3>p2 || p3>p4) ) pix=p3;
-//                           else pix=p4;
-//                        }
-//                     }
-//
-//                     out[c].setPixelDouble(offX+(x/2), offY+(y/2), pix);
-//                  }
-//               }
-//            }
-//         }
-//      }
-//
-//      for( int j=0; j<4; j++ ) {
-//         for( int c=0; c<3; c++ ) {
-//            if( c==missing ) continue;
-//            if( fils[j]!=null && fils[j][c]!=null ) fils[j][c].free();
-//         }
-//      }
-//
-//      return out;
-//   }
-
    // Gnration d'une feuille terminale (=> simple chargement des composantes)
    private Fits [] getLeaves(int order, long npix) throws Exception {
       if( context.isTaskAborting() ) new Exception("Task abort !");
@@ -802,120 +782,8 @@ public class BuilderRgb extends BuilderTiles {
       return fits;
    }
    
-   // gnration du RGB  partir des composantes
-//   private void generateRGB(Fits [] out, int order, long npix, boolean leaf) throws Exception {
-//      
-//      double gap=0;
-//      double range=256;
-//      if( Fits.isTransparent(format==Constante.TILE_PNG ? Fits.PIX_255 : Fits.PIX_256) ) {
-//         range = 255;
-//         gap = 1;
-//      }
-//
-//      Fits rgb = new Fits(width,width,0);
-//      double [] pix = new double[3];
-//      int i=0;
-//      for( int y=0; y<width; y++ ) {
-//         for( int x=0; x<width; x++,i++ ) {
-//
-//            double tot = 0;  // Pour faire la moyenne en cas d'une composante manquante
-//            for( int c=0; c<3; c++ ) {
-//               if( c==missing || out[c]==null ) continue;
-//               double v = out[c].getPixelDouble(x,width-y-1);
-//               if( out[c].isBlankPixel(v) ) pix[c] = 0;
-//               else {
-//                  pix[c] = gap+ ( v< pixelMin[c] ? 0 : v> pixelMax[c] ? range-gap-1
-//                        : range *( (v-pixelMin[c]) / (pixelMax[c] - pixelMin[c]) ) );
-//                  tot += pix[c];
-//               }
-//            }
-//            if( missing!=-1 ) pix[missing] = tot/2;
-//            
-//            int pixel;
-//            double[] pix8 = new double[3];
-//            if( tot==0 ) pixel=0x00;
-//            else {
-//               pixel = 0xFF;
-//               for( int c=0; c<3; c++ ) { 
-//                  int itcm =  (int)Math.floor( pix[c] );
-//                  if( tcm[c]==null ) pix8[c] = itcm;
-//                  else if( itcm>=255 ) pix8[c] = tcm[c][255];
-//                  else if( itcm<=gap ) pix8[c] = tcm[c][(int)gap];
-//                  else {
-//                     double d1 = pix[c] - itcm;
-//                     if( d1==0 ) pix8[c] = tcm[c][ itcm ] & 0xFF;
-//                     else {
-//                        double d2 = 1-d1;
-//                        double v1 = tcm[c][ itcm ] & 0xFF;
-//                        double v2 = tcm[c][ itcm+1 ] & 0xFF;
-//                        pix8[c] = (int)Math.round( v1*d2  + v2*d1 );
-//                     }
-//                  }
-//                  pixel = (pixel<<8) | ((int)pix8[c] & 0xFF);
-//               }
-//            }
-//
-//            rgb.rgb[i]=pixel;
-//         }
-//      }
-//      String file="";
-//
-//      file = Util.getFilePath(output,order, npix)+Constante.TILE_EXTENSION[format];
-//      rgb.writeRGBPreview(file,Constante.TILE_MODE[format]);
-//      rgb.free();
-//
-//      if( leaf ) {
-//         File f = new File(file);
-//         updateStat(f);
-//      }
-//   }
    
    
-   // gnration du RGB  partir des composantes
-//   private void generateRGB(Fits [] out, int order, long npix, boolean leaf) throws Exception {
-//      byte [][] pixx8 = new byte [3][];
-//
-//      // Passage en 8 bits pour chaque composante
-//      for( int c=0; c<3; c++ ) {
-//         if( c==missing || out[c]==null ) continue;
-//         pixx8[c] = out[c].toPix8( pixelMin[c],pixelMax[c], tcm[c],
-//               format==Constante.TILE_PNG ? Fits.PIX_255 : Fits.PIX_256);
-//      }
-//
-//      Fits rgb = new Fits(width,width,0);
-//      int [] pix8 = new int[3];
-//      for( int i=width*width-1; i>=0; i-- ) {
-//         
-//         int tot = 0;  // Pour faire la moyenne en cas d'une composante manquante
-//         for( int c=0; c<3; c++ ) {
-//            if( c==missing ) continue;
-//            if( out[c]==null ) pix8[c]=0;
-//            else {
-//               pix8[c] = 0xFF & pixx8[c][i];
-//               tot += pix8[c];
-//            }
-//         }
-//         if( missing!=-1 ) pix8[missing] = tot/2;
-//         int pix;
-//         if( tot==0 ) pix=0x00;
-//         else {
-//            pix = 0xFF;
-//            for( int c=0; c<3; c++ ) pix = (pix<<8) | pix8[c];
-//         }
-//         rgb.rgb[i]=pix;
-//      }
-//      String file="";
-//
-//      file = Util.getFilePath(output,order, npix)+Constante.TILE_EXTENSION[format];
-//      rgb.writeRGBPreview(file,Constante.TILE_MODE[format]);
-//      rgb.free();
-//
-//      if( leaf ) {
-//         File f = new File(file);
-//         updateStat(f);
-//      }
-//   }
-
 }
 
 
