@@ -1,10 +1,10 @@
-// Copyright 1999-2018 - Universit de Strasbourg/CNRS
+// Copyright 1999-2020 - Universit de Strasbourg/CNRS
 // The Aladin Desktop program is developped by the Centre de Donnes
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
 //
-//This file is part of Aladin.
+//This file is part of Aladin Desktop.
 //
 //    Aladin Desktop is free software: you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -16,12 +16,11 @@
 //    GNU General Public License for more details.
 //
 //    The GNU General Public License is available in COPYING file
-//    along with Aladin.
+//    along with Aladin Desktop.
 //
 
 package cds.allsky;
 
-import java.awt.Shape;
 import java.io.File;
 import java.io.FileInputStream;
 import java.util.ArrayList;
@@ -34,7 +33,7 @@ import cds.aladin.Aladin;
 import cds.aladin.MyInputStream;
 import cds.fits.CacheFits;
 import cds.fits.Fits;
-import cds.moc.HealpixMoc;
+import cds.moc.SMoc;
 import cds.tools.pixtools.Util;
 
 /** Permet la gnration du survey HEALPix  partir d'un index pralablement gnr
@@ -51,7 +50,7 @@ public class BuilderTiles extends Builder {
    private Context.JpegMethod method;
 
    // Liste des Threads de calcul
-   protected ArrayList<ThreadBuilder> threadList = new ArrayList<ThreadBuilder>();
+   protected ArrayList<ThreadBuilder> threadList = new ArrayList<>();
    private Mode coaddMode=Mode.REPLACETILE;
 
    protected int ordermin = 3;
@@ -70,8 +69,8 @@ public class BuilderTiles extends Builder {
    static { FS = System.getProperty("file.separator"); }
 
    // Pour les stat
-   protected int statNbThreadRunning=-1;     // Nombre de Thread qui sont en train de calculer
-   protected int statNbThread;               // Nombre max de Thread de calcul
+//   protected int statNbThreadRunning=-1;     // Nombre de Thread qui sont en train de calculer
+//   protected int statNbThread;               // Nombre max de Thread de calcul
    protected int statNbTile;                 // Nombre de tuiles terminales dj calculs
    protected long statMinTime,statMaxTime,statTotalTime,statAvgTime;
    protected int statEmptyTile;               // Nombre de tuiles terminales de fait vides
@@ -99,9 +98,9 @@ public class BuilderTiles extends Builder {
             context.info("Map original raw pixel range ["+cutOrig[2]+" .. "+cutOrig[3]+"] to ["+cut[2]+" .. "+cut[3]+"]");
          }
          else context.info("BITPIX = "+b1+" (no conversion)");
-         if( context.getDiskMem()!=-1 ) {
-            context.info("Disk requirement (upper approximation) : "+cds.tools.Util.getUnitDisk((long)(context.getDiskMem()*1.25)));
-         }
+//         if( context.getDiskMem()!=-1 ) {
+//            context.info("Disk requirement (upper approximation) : "+cds.tools.Util.getUnitDisk((long)(context.getDiskMem()*1.25)));
+//         }
          double bs=context.getBScale(), bz=context.getBZero();
          if( bs!=1 || bz!=0 ) { context.info("BSCALE="+bs+" BZERO="+bz); }
          double bl0 = context.getBlankOrig();
@@ -137,8 +136,10 @@ public class BuilderTiles extends Builder {
                context.fading ? "border_fading" : context.mixing ? "mean" : "first");
       context.setPropriete(Constante.KEY_HIPS_PROCESS_HIERARCHY, context.getJpegMethod().toString().toLowerCase());
       
-      if( !context.isTaskAborting() ) { (new BuilderAllsky(context)).run(); context.info("ALLSKY file done"); }
+      if( !context.isTaskAborting() ) { (new BuilderAllsky(context)).run(); context.done("ALLSKY file done"); }
       if( !context.isTaskAborting() ) { (b=new BuilderMoc(context)).run(); b=null; }
+      
+      context.removeListReport();
    }
 
    //   public boolean isAlreadyDone() {
@@ -235,7 +236,49 @@ public class BuilderTiles extends Builder {
       
       // Info sur le coordinate frame
       context.info("HiPS coordinate frame => "+context.getFrameName());
+      
+      validateSplit();
    }
+   
+   
+   private void validateSplit() throws Exception {
+      String splitCmd = context.getSplit();
+      if( splitCmd==null ) return;
+      
+      int bitpix, tileWidth, depth, order;
+      String format, outputPath;
+      SMoc moc;
+      
+      outputPath   = context.getOutputPath();
+      bitpix       = context.getBitpix();
+      tileWidth    = context.getTileSide();
+      order        = context.getOrder();
+      depth        = context.getDepth();
+      format       = context.isColor() ? "jpeg" : "fits png";  // on suppose qu'on va crer des tuiles fits et preview (png)
+      moc          = (SMoc)( context.mocIndex.clone() );
+      if( moc==null ) throw new Exception("No MOC available => splitting action not possible");
+      if( !outputIsFree( outputPath, order ) ) {
+         context.warning("HiPS output dir not empty => split function ignored");
+         return;
+      }
+
+      validateSplit( outputPath, splitCmd, moc, order, bitpix, tileWidth, depth, format );
+   }
+   
+   private boolean outputIsFree(String outputPath, int order) {
+      String path = cds.tools.Util.concatDir(outputPath, "Norder" + order);
+      File dir = new File(path);
+      File [] fs = dir.listFiles();
+      if( fs==null ) return true;
+      for( File f : fs ) {
+         if( f.isDirectory() && f.getName().startsWith("Dir") ) {
+            try { Integer.parseInt( f.getName().substring(3) ); return false; }
+            catch( Exception e ) {}
+         }
+      }
+      return false;
+   }
+   
 
    long lastTime = 0L;
    long lastNbTile = 0L;
@@ -243,7 +286,8 @@ public class BuilderTiles extends Builder {
    /** Demande d'affichage des statistiques (via Task()) */
    public void showStatistics() {
       if( b!=null ) { b.showStatistics(); return; }
-      statNbThreadRunning = nbThreadRunning();
+      int statNbThreadRunning = getNbThreadRunning();
+      int statNbThread = getNbThreads();
       if( statNbThreadRunning==0 || statNbTile==0 ) return;
       long now = System.currentTimeMillis();
       totalTime = now-startTime;
@@ -256,7 +300,9 @@ public class BuilderTiles extends Builder {
       if( Aladin.levelTrace>=3 ) {
          String s = showMem();
          if( s.length()>0 ) context.stat(s);
+         showDebugInfo();
       }
+     
       //      if( context.cacheFits!=null && context.cacheFits.getStatNbOpen()>0 ) context.stat(context.cacheFits+"");
    }
 
@@ -264,12 +310,12 @@ public class BuilderTiles extends Builder {
 
    // Initialisation des statistiques
    private void initStat(int nbThread) {
-      statNbThread=nbThread; statNbThreadRunning=0;
+//      statNbThread=nbThread; statNbThreadRunning=0;
       statNbTile=statNodeTile=0;
       statTotalTime=statNodeTotalTime=0L;
       startTime = System.currentTimeMillis();
       totalTime=0L;
-      memPerThread = new Hashtable<Thread,ArrayList<Fits>>();
+      memPerThread = new Hashtable<>();
    }
 
    //   private long maxMem=0;
@@ -307,6 +353,7 @@ public class BuilderTiles extends Builder {
          ArrayList<Fits> m = memPerThread.get(key);
          if( m!=null ) for( Fits f : m ) f.free();
          memPerThread.remove(key);
+//         ThreadBuilderTile.nbThreads--;
       }
    }
 
@@ -321,7 +368,7 @@ public class BuilderTiles extends Builder {
       synchronized( memPerThread ) {
          ArrayList<Fits> m = memPerThread.get(t);
          if( m==null ) {
-            m=new  ArrayList<Fits>();
+            m=new  ArrayList<>();
             memPerThread.put(t,m);
          }
          m.add(f);
@@ -394,7 +441,7 @@ public class BuilderTiles extends Builder {
 
    // Mise  jour des stats
    protected void updateStat(int deltaNbThread,int deltaTile,int deltaEmptyTile,long timeTile,int deltaNodeTile,long timeNodeTile) {
-      statNbThreadRunning+=deltaNbThread;
+//      statNbThreadRunning+=deltaNbThread;
       statNbTile+=deltaTile;
       statNodeTile+=deltaNodeTile;
       statEmptyTile+=deltaEmptyTile;
@@ -421,54 +468,60 @@ public class BuilderTiles extends Builder {
       Fits fits;
       boolean ready;
       Thread th;
+      boolean suspendable;
       
-      Item(int order, long npix, int z,Thread th ) {
+      Item() { ready=true; order=-1; }
+      
+      Item(int order, long npix, int z,Thread th, boolean suspendable ) {
          this.order=order;
          this.npix=npix;
          this.z=z;
          this.th=th;
          ready=false;
+         this.suspendable=suspendable;
       }
       
-      private boolean isReady() {
-         synchronized( objItem ) { 
-            return ready; 
-         }
+      protected boolean hasBeenUsed() { 
+         synchronized( fifo ) { return order!=-1; }
       }
       
-      private Fits getFits() {
-         synchronized( objItem ) { return fits; }
+      private boolean isReady() { 
+         synchronized( fifo ) { return ready; }
+      }
+      
+      private Fits getFits() { 
+         synchronized( fifo ) { return fits; }
       }
       
       private void setFits(Fits fits) throws Exception {
-         if( th==null ) return;
-         synchronized( objItem ) { 
-            this.fits=fits;
+         synchronized( fifo ) { 
             ready=true;
+            this.fits=fits;
          }
+         if( th==null ) return;
          th.interrupt();
       }
       
-      public String toString() { return order+"/"+npix+(z>0?"-"+z:"")+(isReady()?"R":""); }
+      public String toString() { return order+"/"+npix+(z>0?"-"+z:"")+(isReady()?"R":"")+(suspendable?"s":""); }
       
    }
    
-   static final private Object objItem = new Object();
-   
-   
    protected int getBitpix0() { return context.getBitpix(); }
    
    protected void build() throws Exception {
       this.ordermax = context.getOrder();
       long t = System.currentTimeMillis();
 
-      HealpixMoc moc = new HealpixMoc();
-      HealpixMoc m = context.getRegion();
-      if( m==null ) m = new HealpixMoc("0/1-11");
+      SMoc moc = new SMoc();
+      SMoc m = context.getRegion();
+      if( m==null ) m = new SMoc("0/1-11");
       moc.add( m );
-      moc.setMocOrder(3);
+      int minorder=0; //3;
+      context.setMinOrder( minorder );
+      
+      moc.setMocOrder(minorder);
       int depth = context.getDepth();
-      fifo = new LinkedList<Item>();
+      fifo = new LinkedList<>();
       
 //      Iterator<Long> it = moc.pixelIterator();
 //      while( it.hasNext() ) {
@@ -482,7 +535,7 @@ public class BuilderTiles extends Builder {
          Iterator<Long> it = moc.pixelIterator();
          while( it.hasNext() ) {
             long npix=it.next();
-            fifo.add( new Item(3, npix,z,null ) );
+            fifo.add( new Item(minorder, npix,z,null,true ) );
          }
       }
       
@@ -509,9 +562,8 @@ public class BuilderTiles extends Builder {
       long maxMemPerThread = 4*oneRhomb + bufMem;
       if( isColor )  maxMemPerThread += oneRhomb*(ordermax-ordermin);
       //      context.info("Minimal RAM required per thread (upper estimation): "+cds.tools.Util.getUnitDisk(maxMemPerThread));
-      int nbThread = (int) (size / maxMemPerThread);
+      int nbThread = this instanceof BuilderMirror ? 16 : (int) (size / maxMemPerThread);
 
-      if( this instanceof BuilderMirror ) nbThread=6;
 
       int maxNbThread = context.getMaxNbThread();
       if( maxNbThread>0 && nbThread>maxNbThread ) nbThread=maxNbThread;
@@ -519,7 +571,7 @@ public class BuilderTiles extends Builder {
       if( nbThread>nbProc && !(this instanceof BuilderMirror) ) nbThread=nbProc;
 
       Aladin.trace(4,"BuildController.build(): Found "+nbProc+" processor(s) for "+size/(1024*1024)+"MB RAM => Launch "+nbThread+" thread(s)");
-      context.info("Will use "+nbThread+" thread"+(nbThread>1?"s":""));
+      context.info("Starts with "+nbThread+" thread"+(nbThread>1?"s":""));
 
       activateCache(size,2L*size/3L);
 
@@ -538,7 +590,7 @@ public class BuilderTiles extends Builder {
       if( !context.isTaskAborting() ) {
 
          if( ThreadBuilderTile.statMaxOverlays>0 )
-            context.info("Tile overlay stats : max overlays="+ThreadBuilderTile.statMaxOverlays+", " +
+            context.stat("Tile overlay stats : max overlays="+ThreadBuilderTile.statMaxOverlays+", " +
                   ThreadBuilderTile.statOnePass+" in one step, "+
                   ThreadBuilderTile.statMultiPass+" in multi steps");
          if( context.cacheFits!=null ) Aladin.trace(3,"Cache FITS status: "+ context.cacheFits);
@@ -558,7 +610,7 @@ public class BuilderTiles extends Builder {
       if( t==-1 ) { lastCheckTime=t; return; }
       
       // On attend un certain temps avant de tester la progression
-      if( t-lastCheckTime<MAXCHECKTIME ) return;
+      if( t-lastCheckTime<MAXCHECKTIME || context.getVerbose()>=3 && t-lastCheckTime<20*1000L ) return;
       lastCheckTime=t;
 //      System.out.println("Check 10s progress="+context.progress);
       
@@ -568,18 +620,24 @@ public class BuilderTiles extends Builder {
       // Il y a eu du progres => c'est bon
       if( context.progress!=lastProgress ) { lastProgress=context.progress; return; }
       
-      context.warning("ALERT !!!");
       context.warning("Nothing done since a while. Here a short report to understand the problem:");
+      showDebugInfo();
+   }
+   
+   protected void showDebugInfo() {
+      context.warning("DEBUG REPORT !!! (fifosize="+fifo.size()+")");
+      int nbDied=0;
       Iterator<ThreadBuilder> it = threadList.iterator();
       while( it.hasNext() ) {
          ThreadBuilder tb = it.next();
          String info = tb.getInfo();
-         context.warning(".thread "+tb.getName()+": "+tb.getMode()+ (info!=null?" => "+info:""));
+         if( tb.isDied() ) nbDied++;
+         context.warning(".thread "+tb.getName()+": "+tb.getMode()+(info!=null?" => "+info:""));
       }
+      if( nbDied>0 ) context.warning(".and "+nbDied+" threads DIED");
       CacheFits cache = context.getCache();
       if( cache!=null ) context.warning(cache.toString());
       else context.warning("No cache. FreeRAM="+cds.tools.Util.getUnitDisk(CacheFits.getFreeMem()));
-      
    }
    
    protected void activateCache(long size,long sizeCache) {
@@ -611,14 +669,14 @@ public class BuilderTiles extends Builder {
       // si on n'est pas dans le Moc, il faut retourner le fichier
       // pour la construction de l'arborescence...
       if( !context.isInMocTree(order,npix) ) return findLeaf(file);
-
+      
       // si le losange a dj t calcul on le renvoie
       if( coaddMode==Mode.KEEPTILE ) {
          Fits oldOut = findLeaf(file);
          if( oldOut!=null ) {
-            HealpixMoc moc = context.getRegion();
-            moc = moc.intersection(new HealpixMoc(order+"/"+npix));
-            int nbTiles = (int)moc.getUsedArea();
+            SMoc moc = context.getRegion();
+            moc = (SMoc)moc.intersection(new SMoc(order+"/"+npix));
+            int nbTiles = (int)moc.getNbCells();
             updateStat(0,0,nbTiles,0,nbTiles/4,0);
             return oldOut;
          }
@@ -628,8 +686,10 @@ public class BuilderTiles extends Builder {
 
       // Cration d'un losange terminal
       if( order==ordermax )  {
+         hpx.threadBuilder.setInfo("createLeavveHpx "+file+"...");
          try { f = createLeaveHpx(hpx,file,path,order,npix,z); }
          catch( Exception e ) {
+            hpx.threadBuilder.setInfo("createLeavveHpx error "+file+"...");
             System.err.println("BuilderTiles.createLeave error: "+file);
             e.printStackTrace();
             return null;
@@ -639,18 +699,41 @@ public class BuilderTiles extends Builder {
       } else {
 
          Fits fils[] = new Fits[4];
-         
-         if( fifo.isEmpty() && oneWaiting() ) {
-            multiThreadCreateHpx(hpx, fils, path,order,npix, z);
-            
-         } else {
-         
-            for( int i =0; /* !stopped && */ i<4; i++ ) {
-               if( context.isTaskAborting() ) throw new Exception("Task abort !");
+         Item item[] = new Item[4];
+         for( int i=0; i<4; i++ ) item[i]=new Item();
+
+         int nbDelegate=0;  // Nombre de fils dlgus  d'autres threads
+         for( int i =0; i<4; i++ ) {
+            if( context.isTaskAborting() ) throw new Exception("Task abort !");
+
+            // Si je peux dlguer des calculs  d'autres threads libres, je le fais
+            // sauf pour au-moins une tuile fille
+            if( nbDelegate<3 && fifo.isEmpty() && oneWaiting() ) {
+               item[i] = addNpix(order+1,npix*4+i, z,Thread.currentThread());
+               nbDelegate++;
+               wakeUp();
+
+            } else {
+               hpx.threadBuilder.setInfo("CreateHpx go to next order => "+(order+1)+"/"+(npix*4+i)+"...");
                fils[i] = createHpx(hpx, path,order+1,npix*4+i, z);
             }
          }
 
+         // Attente et rcupration des fils si besoin
+         while( !(item[0].isReady() && item[1].isReady() && item[2].isReady() && item[3].isReady()) ) {
+            hpx.threadBuilder.setWaitingChildren(true);
+            StringBuilder t = new StringBuilder();
+            for( int i=0; i<item.length; i++ ) t.append( item[i].isReady()?".": (item[i].hasBeenUsed()?"x":"o"));
+            hpx.threadBuilder.setInfo("CreateHpx still waiting children of "+order+"/"+npix+" ["+t+"]...");
+            try { Thread.currentThread().sleep(300); } catch( Exception e ) { }
+            if( context.isTaskAborting() ) throw new Exception("Task abort !");
+         }
+         hpx.threadBuilder.setWaitingChildren(false);
+         for( int i=0; i<4; i++ ) {
+            if( item[i].hasBeenUsed() ) fils[i] = item[i].getFits();
+         }
+
+         hpx.threadBuilder.setInfo("createNodeHpx "+file+"...");
          try { f = createNodeHpx(file,path,order,npix,fils,z); }
          catch( Exception e ) {
             System.err.println("BuilderTiles.createNodeHpx error: "+file);
@@ -665,32 +748,40 @@ public class BuilderTiles extends Builder {
       return f;
    }
    
-   // Gnration des 4 fils en parallle
-   private void multiThreadCreateHpx(ThreadBuilderTile hpx, Fits [] fils, String path,int order,long npix, int z) throws Exception {
-      Item [] item = new Item[3];
-      
-      // Ajout des travaux dans la file d'attente des threads de travail
-      for( int i=0; i<3; i++ ) {
-         item[i] = addNpix(order+1,npix*4+i, z,Thread.currentThread());
-      }
-      
-      // Il faudrait ici rveiller les threads de calculs en attente
-      wakeUp();
-            
-      // Un des fils est calcul par le thread courant lui-mme
-      fils[3] = createHpx(hpx, path,order+1,npix*4+3, z);
-      
-      // Attente et rcupration des fils
-      while( !(item[0].isReady() && item[1].isReady() && item[2].isReady()) ) {
-         try { Thread.currentThread().sleep(500); } catch( Exception e ) { }
-      }
-      for( int i=0; i<3; i++ ) fils[i] = item[i].getFits();
-   }
+//   // Gnration des 4 fils en parallle
+//   private void multiThreadCreateHpx(ThreadBuilderTile hpx, Fits [] fils, String path,int order,long npix, int z) throws Exception {
+//      Item [] item = new Item[3];
+//      
+//      // Ajout des travaux dans la file d'attente des threads de travail
+//      for( int i=0; i<3; i++ ) {
+//         item[i] = addNpix(order+1,npix*4+i, z,Thread.currentThread());
+//      }
+//      
+//      // Il faudrait ici rveiller les threads de calculs en attente
+//      wakeUp();
+//            
+//      // Un des fils est calcul par le thread courant lui-mme
+//      fils[3] = createHpx(hpx, path,order+1,npix*4+3, z);
+//      
+//      // Attente et rcupration des fils
+//      while( !(item[0].isReady() && item[1].isReady() && item[2].isReady()) ) {
+//         hpx.threadBuilder.setWaitingChildren(true);
+//         StringBuilder t = new StringBuilder();
+//         for( int i=0; i<item.length; i++ ) t.append( item[i].isReady()?".":"x");
+//         hpx.threadBuilder.setInfo("multiThreadCreateHpx still waiting children of "+order+"/"+npix+" ["+t+"]...");
+//         try { Thread.currentThread().sleep(100); } catch( Exception e ) { }
+//         if( context.isTaskAborting() ) throw new Exception("Task abort !");
+//      }
+//      hpx.threadBuilder.setWaitingChildren(false);
+//      for( int i=0; i<3; i++ ) fils[i] = item[i].getFits();
+//   }
    
    static final String [] MODE =  { "START","WAIT","EXEC","DIED","SUSPEND" };
    
+   protected Object lockStop = new Object();
+   
    // Classe des threads de calcul
-   private class ThreadBuilder extends Thread {
+   public class ThreadBuilder extends Thread {
       static final int START =0;
       static final int WAIT  =1;
       static final int EXEC  =2;
@@ -701,10 +792,12 @@ public class BuilderTiles extends Builder {
       private int mode=START;
       private String info;      // Message expliquant l'tat courant
       private boolean encore=true;
+      protected Item item=null;   // Item en cours de calcul
 
       public ThreadBuilder(String name,ThreadBuilderTile threadBuilderTile) {
          super(name);
          this.threadBuilderTile = threadBuilderTile;
+         threadBuilderTile.threadBuilder = this;   // pour faire des remontes de debug
          Aladin.trace(3,"Creating "+getName());
       }
       
@@ -722,6 +815,12 @@ public class BuilderTiles extends Builder {
 
       /** Le thread est suspendu */
       public boolean isSuspend() { return mode==SUSPEND; }
+      
+      
+      /** Les thread est en attente du rsultat d'un ou plusieurs fils */
+      private boolean waitingChildren=false;
+      public void setWaitingChildren( boolean flag ) { waitingChildren = flag; }
+      public boolean isWaitingChildren() { return waitingChildren; }
 
       /** Le thread est en attente pour un nouveau boulot */
       public boolean isWait() { return mode==WAIT; }
@@ -732,12 +831,14 @@ public class BuilderTiles extends Builder {
          if( mode!=WAIT ) return false;
          if( checkMem ) {
             try {
-               return !threadBuilderTile.requiredMem(Constante.MAXOVERLAY,3 ); 
+               return !threadBuilderTile.requiredMem(Constante.MAXOVERLAY,1 ); 
             } catch( Exception e ) {}
          }
          return true;
       }
       
+      public void setInfo(String info) { this.info = info; }
+      
       public boolean arret(String info) {
          if( !suspendable ) return false;
          mode=SUSPEND;
@@ -757,11 +858,22 @@ public class BuilderTiles extends Builder {
       
       /** Demande la mort du thread */
       public void tue() { encore=false; }
-
+      
       public void run() {
 
          while( encore ) {
-            Item item=null;
+            item=null;
+            
+            // arrt du thread de calcul
+            if( ThreadBuilderTile.nbThreadsToStop>0 ) {
+               synchronized( lockStop ) {
+                  if( ThreadBuilderTile.nbThreadsToStop>0 ) {
+                     ThreadBuilderTile.nbThreadsToStop--;
+                     break;
+                  }
+               }
+            }
+            
             while( encore && mode==SUSPEND ) {
                try { Thread.currentThread().sleep(100); } catch( Exception e) { };
             }
@@ -769,20 +881,22 @@ public class BuilderTiles extends Builder {
                mode=WAIT;
                info="No more HEALPix cell branch to compute => thread by waiting another task";
                try { Thread.currentThread().sleep(100); } catch( Exception e) { };
+               if( context.isTaskAborting() ) { encore=false; break; }
             }
             if( encore ) {
-               suspendable= item.order==3;
+//               suspendable= item.order==0; //  3;
+               suspendable= item.suspendable;
                mode=EXEC;
                info=null;
-               updateStat(+1,0,0,0,0,0);
                try {
-                  Aladin.trace(4,Thread.currentThread().getName()+" process HEALPix cell branch "+item.npix+"...");
+                  Aladin.trace(4,Thread.currentThread().getName()+" process HEALPix cell branch "+item.order+"/"+item.npix+"...");
 
                   // si le process a t arrt on essaie de ressortir au plus vite
 //                  if (stopped) break;
                   if( context.isTaskAborting() ) break;
 
                   Fits fits = createHpx(threadBuilderTile, context.getOutputPath(), item.order, item.npix, item.z);
+                  setInfo("Tile ready");
                   item.setFits(fits);
                   
                   rmFits(Thread.currentThread(), fits);
@@ -790,11 +904,11 @@ public class BuilderTiles extends Builder {
                   if( item.order==3 && item.z==0 ) setProgressBar((int)item.npix);
 
                } catch( Throwable e ) {
+                  try { item.setFits(null); } catch( Exception e1 ) {}
                   Aladin.trace(1,"*** "+Thread.currentThread().getName()+" exception !!! ("+e.getMessage()+")");
                   e.printStackTrace();
                   context.taskAbort();
                }
-               updateStat(-1,0,0,0,0,0);
             }
          }
 
@@ -804,7 +918,8 @@ public class BuilderTiles extends Builder {
          //         Aladin.trace(3,Thread.currentThread().getName()+" died !");
       }
    }
-
+   
+   
    /** Mise  jour de la barre de progression en mode GUI */
    protected void setProgressBar(int npix) { context.setProgressLastNorder3(npix); }
 
@@ -813,37 +928,98 @@ public class BuilderTiles extends Builder {
       Item pix=null;
       synchronized( lockObj ) {
          if( fifo.isEmpty() ) return null;
-         pix = fifo.removeFirst();
+         pix = fifo.removeLast();
       }
       return pix;
    }
    
    private Item addNpix(int order, long npix,int z,Thread th) {
-      Item item=new Item(order,npix,z,th);
+      Item item=new Item(order,npix,z,th,false);
       synchronized( lockObj ) { fifo.add(item); }
       return item;
    }
    
    private Object lockObj = new Object();
-
+   
+   private int threadId = 0;
 
    // Cre une srie de threads de calcul
    private void launchThreadBuilderHpx(int nbThreads) throws Exception {
 
       initStat(nbThreads);
       context.createHealpixOrder(context.getTileOrder());
-      ThreadBuilderTile.nbThreads=nbThreads;
-      ThreadBuilderTile.hasShape = new HashMap<File, Shape>();
+//      ThreadBuilderTile.nbThreads=nbThreads;
+      ThreadBuilderTile.hasShape = new HashMap<>();
 
       for( int i=0; i<nbThreads; i++ ) {
          if( context.isTaskAborting() ) throw new Exception("Task abort !");
          ThreadBuilderTile threadBuilderTile = new ThreadBuilderTile(context,this);
-         ThreadBuilder t = new ThreadBuilder("Builder"+i,threadBuilderTile);
+         ThreadBuilder t = new ThreadBuilder("Builder"+threadId++,threadBuilderTile);
          threadList.add( t );
          t.start();
+         if( i==0 ) cds.tools.Util.pause(100);   // Juste pour que la fifo se remplisse plus efficacement (profondeur d'abord)
       }
    }
+   
+   /** Augmente le nombre de threads de calcul (notamment pour BuilderMirror) */
+   protected boolean addThreadBuilderHpx(int nbThreads) throws Exception {
+      
+      // On ne peut  la fois ralentir et acclerer
+      if( ThreadBuilderTile.nbThreadsToStop>0 ) return false;
+      
+      context.info("Probably not enough threads => ask to launch "+nbThreads+" thread(s) asap...");
+      
+      synchronized( lockObj ) {
+         
+         // On rcupre d'ventuelles places libres ?
+         for( int i=0; nbThreads>0 && i<threadList.size(); i++ ) {
+            ThreadBuilder tb = threadList.get(i);
+            if( tb.isDied() ) {
+               ThreadBuilderTile threadBuilderTile = new ThreadBuilderTile(context,this);
+               ThreadBuilder t = new ThreadBuilder(tb.getName(),threadBuilderTile);
+               threadList.set(i, t);
+               t.start();
+               nbThreads--;
+            }
+         }
+         if( nbThreads==0 ) return true;
+         
+         // On augmente la liste
+//         int lastnbThreads = ThreadBuilderTile.nbThreads;
+//         ThreadBuilderTile.nbThreads+=nbThreads;
 
+         for( int i=0; i<nbThreads; i++ ) {
+            if( context.isTaskAborting() ) throw new Exception("Task abort !");
+            ThreadBuilderTile threadBuilderTile = new ThreadBuilderTile(context,this);
+            ThreadBuilder t = new ThreadBuilder("Builder"+threadId++,threadBuilderTile);
+            threadList.add( t );
+            t.start();
+         }
+      }
+      return true;
+   }
+   
+   /** Retourne le nombre de threads actifs */
+   protected int getNbThreads() {
+      int nb=0;
+      try {
+         for( ThreadBuilder tb : threadList ) {
+            if( !tb.isDied() ) nb++;
+         }
+      } catch( Exception e ) { return -1; }
+      return nb;
+   }
+   
+   /** Demande d'arrt de threads de calcul (notamment pour BuilderMirror) */
+   protected boolean removeThreadBuilderHpx(int nbThreads) {
+      if( ThreadBuilderTile.nbThreadsToStop>0 ) return false;
+      synchronized( lockObj ) {
+         ThreadBuilderTile.nbThreadsToStop=nbThreads;
+         context.info("Probably too many threads => ask to stop "+nbThreads+" thread(s) asap...");
+      }
+      return true;
+   }
+   
    // Demande l'arrt de tous les threads de calcul
    void destroyThreadBuilderHpx() {
       Iterator<ThreadBuilder> it = threadList.iterator();
@@ -871,7 +1047,7 @@ public class BuilderTiles extends Builder {
       return false;
    }
 
-   int nbThreadRunning() {
+   int getNbThreadRunning() {
       int n=0;
       Iterator<ThreadBuilder> it = threadList.iterator();
       while( it.hasNext() ) {
@@ -1150,6 +1326,7 @@ public class BuilderTiles extends Builder {
       if (out!=null) {
          context.updateHeader(out,order,npix);
          write(file,out);
+         hpx.threadBuilder.setInfo("createLeavveHpx write done "+file+"...");
          duree = System.currentTimeMillis()-t;
          //         if( npix%10 == 0 || DEBUG ) Aladin.trace(4,Thread.currentThread().getName()+".createLeaveHpx("+order+"/"+npix+") "+coaddMode+" in "+duree+"ms");
          updateStat(0,1,0,duree,0,0);
