@@ -1,10 +1,10 @@
-// Copyright 1999-2018 - Universit de Strasbourg/CNRS
+// Copyright 1999-2020 - Universit de Strasbourg/CNRS
 // The Aladin Desktop program is developped by the Centre de Donnes
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
 //
-//This file is part of Aladin.
+//This file is part of Aladin Desktop.
 //
 //    Aladin Desktop is free software: you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 //    GNU General Public License for more details.
 //
 //    The GNU General Public License is available in COPYING file
-//    along with Aladin.
+//    along with Aladin Desktop.
 //
 
 package cds.allsky;
@@ -30,7 +30,7 @@ import cds.aladin.Calib;
 import cds.aladin.Coord;
 import cds.fits.Fits;
 import cds.moc.Healpix;
-import cds.moc.HealpixMoc;
+import cds.moc.SMoc;
 import cds.tools.Util;
 import cds.tools.pixtools.CDSHealpix;
 
@@ -47,7 +47,7 @@ public class MocGen {
    private String previous=null;    // Previous MOC file if need
    private int order=10;            // Default order
    private double blank=Double.NaN; // Alternative blank value
-   private int fmt=HealpixMoc.FITS;
+   private int fmt=SMoc.FITS;
    public  boolean verbose=false;
    public boolean debug=false;
    private boolean strict=false;
@@ -55,7 +55,7 @@ public class MocGen {
    private boolean multWrite=false; // true for updating output MOC continously
    private int hdu[] = null;
    
-   private HealpixMoc moc;      // Le MOC en cour de calcul
+   private SMoc moc;      // Le MOC en cour de calcul
    private boolean ready;       // true si le MOC est prt
    private boolean error;       // true si le MOC n'a pas pu tre gnr
    private String serror;       // Message d'erreur
@@ -91,7 +91,7 @@ public class MocGen {
    public String getError() { return serror; }
    
    /** Retourne le MOC gnr */
-   public HealpixMoc getMoc() throws Exception {
+   public SMoc getMoc() throws Exception {
       if( error ) throw new Exception("MOC error => "+serror);
       if( !ready ) throw new Exception("MOC not yet ready");
       return moc;
@@ -114,7 +114,7 @@ public class MocGen {
                serror=null;
                error=abort=ready=false;
                nbImg=0;
-               moc = new HealpixMoc();
+               moc = new SMoc();
                moc.setMocOrder(order);
                moc.setCheckConsistencyFlag(false);
                scanAndDo(moc,new File(in),order);
@@ -132,7 +132,7 @@ public class MocGen {
    }
    
    // Ajout dans le MOC du fichier pass en paramtre avec scan des pixels
-   private boolean addInMocPixel(HealpixMoc moc,File file,int order) throws Exception {
+   private boolean addInMocPixel(SMoc moc,File file,int order) throws Exception {
       boolean rep=false;
       
       String currentfile = file.getPath();
@@ -197,7 +197,7 @@ public class MocGen {
       return rep;
    }
    
-   private boolean addInMocPixel1(Fits f, HealpixMoc moc, String currentfile, String currentCell, int order) throws Exception {
+   private boolean addInMocPixel1(Fits f, SMoc moc, String currentfile, String currentCell, int order) throws Exception {
       boolean rep=false;
       
       int localOrder=0;
@@ -214,7 +214,7 @@ public class MocGen {
       double gap=1;
       try { 
          double resImage = Math.min(c.GetResol()[0],c.GetResol()[1]);
-         for( localOrder=order; CDSHealpix.pixRes( CDSHealpix.pow2(localOrder) )/3600. <= resImage; localOrder--);
+         for( localOrder=order; CDSHealpix.pixRes( localOrder )/3600. <= resImage; localOrder--);
          if( gap<1 ) gap=1;
       } catch( Exception e1 ) {
          e1.printStackTrace();
@@ -264,7 +264,7 @@ public class MocGen {
    
    // Ajout dans le MOC du losange indiqu  la condition que le pixel central correspondant soit dans l'image
    // et ne soit pas BLANK, sinon appel rcursif sur les 4 fils jusqu' atteindre la rsolution de l'image
-//   private boolean addInMocNpix(Fits f,HealpixMoc moc,int order, long npix) throws Exception {
+//   private boolean addInMocNpix(Fits f,SMoc moc,int order, long npix) throws Exception {
 //      
 //      boolean rep=true;
 //      Calib c = f.getCalib();
@@ -296,10 +296,10 @@ public class MocGen {
 //   }
    
    // Ajout dans le MOC de la Calib pass en paramtre
-   private boolean addInMocBox(Fits f,HealpixMoc moc,int order) throws Exception {
+   private boolean addInMocBox(Fits f,SMoc moc,int order) throws Exception {
       boolean res=true;
       Coord coo = new Coord();
-      ArrayList<double[]> cooList = new ArrayList<double[]>(10);
+      ArrayList<double[]> cooList = new ArrayList<>(10);
       Calib c = f.getCalib();
       Dimension dim = c.getImgSize();
       for( int i=0; i<4; i++ ) {
@@ -308,8 +308,11 @@ public class MocGen {
          c.GetCoord(coo);
          cooList.add(new double[]{coo.al,coo.del});
       }
-      long [] npixs = CDSHealpix.query_polygon(CDSHealpix.pow2(order), cooList);
-      for( long npix : npixs ) moc.add(order,npix) ;
+//      long [] npixs = CDSHealpix.query_polygon( order, cooList,true);
+//      for( long npix : npixs ) moc.add(order,npix) ;
+      
+      SMoc m1 = CDSHealpix.createSMoc(cooList, order);
+      moc.add( m1 );
       return res;
    }
    
@@ -317,7 +320,7 @@ public class MocGen {
    // 1) Recherche d'une calibration astromtrique, 2) calcul des 4 coins en ra,dec
    // 3) extraction des pixels HEALPix correspondants, 4) ajout dans le MOC
    // Rq : les fichiers qui n'ont pas de calibration sont simplement ignors
-   private boolean addInMocBox(HealpixMoc moc, File file,int order) throws Exception {
+   private boolean addInMocBox(SMoc moc, File file,int order) throws Exception {
       boolean rep=false;
       boolean flagFirstHdu = hdu==null;
       boolean flagAllHdu = hdu!=null && hdu.length>0 && hdu[0]==-1;
@@ -340,7 +343,7 @@ public class MocGen {
      return rep;
    }
    
-   private boolean addInMoc(HealpixMoc moc, File file, int order,boolean strict) throws Exception {
+   private boolean addInMoc(SMoc moc, File file, int order,boolean strict) throws Exception {
       if( !strict ) return addInMocBox(moc,file,order);
       return addInMocPixel(moc,file,order);
    }
@@ -350,7 +353,7 @@ public class MocGen {
    // trouvs dans le rpertoire
    // Rq: mthode rcursive en parcours en largeur d'abord.
    // @return : le nombre de fichiers trait
-   private void scanAndDo(HealpixMoc moc,File rep,int order) throws Exception {
+   private void scanAndDo(SMoc moc,File rep,int order) throws Exception {
       File [] list;
       
       scanningDir=rep.getCanonicalPath();
@@ -385,7 +388,7 @@ public class MocGen {
    private boolean needNL=false;
    
    // Met  jour si ncessaire le Moc sur le disque (mode maj continue)
-   private boolean updateMoc(HealpixMoc moc, String out, int fmt) throws Exception {
+   private boolean updateMoc(SMoc moc, String out, int fmt) throws Exception {
       long t = System.currentTimeMillis();
       if( (t-t1)<6000 ) return false;
       t1=t;
@@ -410,7 +413,7 @@ public class MocGen {
    }
          
    
-//   private int scanStdin(HealpixMoc moc,int order) throws Exception {
+//   private int scanStdin(SMoc moc,int order) throws Exception {
 //      int n=0;
 //      String s;
 //      BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
@@ -485,9 +488,9 @@ public class MocGen {
             
          } else if( s.startsWith("mocfmt=") ) {
             String a=s.substring(x);
-            if( a.equalsIgnoreCase("fits") ) fmt=HealpixMoc.FITS;
+            if( a.equalsIgnoreCase("fits") ) fmt=SMoc.FITS;
             else if( a.equalsIgnoreCase("json") 
-                  || a.equalsIgnoreCase("ascii") ) fmt=HealpixMoc.JSON;
+                  || a.equalsIgnoreCase("ascii") ) fmt=SMoc.JSON;
             else {
                System.out.println("Unkown MOC format ["+a+"]");
                return false;
@@ -531,7 +534,7 @@ public class MocGen {
             System.out.println();
          }
       }
-      System.out.println(".mocfmt="+(fmt==HealpixMoc.FITS?"fits":"ascii"));
+      System.out.println(".mocfmt="+(fmt==SMoc.FITS?"fits":"ascii"));
       if( strict ) System.out.println(".blank=NaN"+(Double.isNaN(blank)?"":"|"+blank));
 
       return true;
@@ -578,7 +581,7 @@ public class MocGen {
          serror=null;
          error=abort=ready=false;
          nbImg=0;
-         moc = new HealpixMoc();
+         moc = new SMoc();
          if( previous!=null ) moc.read(previous);
          moc.setMocOrder(order);
          moc.setCheckConsistencyFlag(false);
