@@ -1,10 +1,10 @@
-// Copyright 1999-2018 - Universit de Strasbourg/CNRS
+// Copyright 1999-2020 - Universit de Strasbourg/CNRS
 // The Aladin Desktop program is developped by the Centre de Donnes
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
 //
-//This file is part of Aladin.
+//This file is part of Aladin Desktop.
 //
 //    Aladin Desktop is free software: you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 //    GNU General Public License for more details.
 //
 //    The GNU General Public License is available in COPYING file
-//    along with Aladin.
+//    along with Aladin Desktop.
 //
 
 package cds.allsky;
@@ -46,6 +46,7 @@ import javax.swing.JPanel;
 import javax.swing.JProgressBar;
 import javax.swing.JRadioButton;
 import javax.swing.JScrollPane;
+
 import cds.aladin.Aladin;
 import cds.aladin.Coord;
 import cds.tools.Util;
@@ -471,7 +472,7 @@ public class TabBuild extends JPanel implements ActionListener {
 
    private void perform() {
       try {
-         Vector<Action> actions = new Vector<Action>();
+         Vector<Action> actions = new Vector<>();
 
          boolean isMap = context.isMap();
 
@@ -485,7 +486,6 @@ public class TabBuild extends JPanel implements ActionListener {
          else actions.add(Action.TILES);
 
          if( !context.isColor() ) {
-            actions.add(Action.GZIP);
             if( !isMap ) actions.add(Action.DETAILS);
          }
 
