@@ -1,10 +1,10 @@
-// Copyright 1999-2018 - Universit de Strasbourg/CNRS
+// Copyright 1999-2020 - Universit de Strasbourg/CNRS
 // The Aladin Desktop program is developped by the Centre de Donnes
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
 //
-//This file is part of Aladin.
+//This file is part of Aladin Desktop.
 //
 //    Aladin Desktop is free software: you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 //    GNU General Public License for more details.
 //
 //    The GNU General Public License is available in COPYING file
-//    along with Aladin.
+//    along with Aladin Desktop.
 //
 
 package cds.allsky;
@@ -41,15 +41,15 @@ import cds.aladin.Coord;
 import cds.aladin.Localisation;
 import cds.fits.CacheFits;
 import cds.fits.Fits;
-import cds.tools.pixtools.CDSHealpix;
+import cds.healpix.Healpix;
+import cds.healpix.HealpixNestedFast;
 import cds.tools.pixtools.Util;
-import healpix.essentials.HealpixBase;
-import healpix.essentials.Pointing;
 
 final public class ThreadBuilderTile {
 
    private Context context;
    protected BuilderTiles builderTiles;
+   protected BuilderTiles.ThreadBuilder threadBuilder;   // just pour faire des remontes de debug sur setInfo(...)
    private int bitpix;
    private Mode coaddMode;
    private double max;
@@ -71,6 +71,8 @@ final public class ThreadBuilderTile {
    private boolean mixing;
    private int tileSide;
    
+   static protected int nbThreadsToStop=0;
+   
 
    static protected HashMap<File, Shape> hasShape=null;   // Polygones associs  chaque fichier ou rpertoire
 
@@ -106,7 +108,7 @@ final public class ThreadBuilderTile {
       hpxFinderPath = context.getHpxFinderPath();
       tileSide = context.getTileSide();
 
-      downFiles = new ArrayList<SrcFile>(Constante.MAXOVERLAY);
+      downFiles = new ArrayList<>(Constante.MAXOVERLAY);
    }
 
    public long getMem() {
@@ -122,7 +124,7 @@ final public class ThreadBuilderTile {
       return mem-40*1024L*1024L<rqMem;
    }
    
-   static public int nbThreads;
+//   static public int nbThreads;
 
    protected boolean requiredMem(long nbProgen,int nbThreads ) throws Exception {
       long rqMem = 4 * nbProgen * Constante.ORIGCELLWIDTH*Constante.ORIGCELLWIDTH*context.getNpixOrig();
@@ -150,8 +152,8 @@ final public class ThreadBuilderTile {
             if( context.getVerbose()>3 ) context.info("Need more RAM: output Fits bitmap release => "+cds.tools.Util.getUnitDisk(sizeReleaseBitmap));
             if( !needMem(rqMem) ) return;
          }
-
-         if( builderTiles.nbThreadRunning()<=1 ) {
+         
+         if( builderTiles.getNbThreadRunning()<=1 ) {
             context.cacheFits.forceClean();
             if( context.getVerbose()>3 ) context.warning(Thread.currentThread().getName()+" needs "+
                   cds.tools.Util.getUnitDisk(rqMem)+" but can not stop (last thread running) !");
@@ -171,7 +173,7 @@ final public class ThreadBuilderTile {
 
                   cds.tools.Util.pause((int)( 1000*(1+Math.random()*5)));
                   context.cacheFits.forceClean();
-                  if( builderTiles.nbThreadRunning()<=0 ) {
+                  if( builderTiles.getNbThreadRunning()<=0 ) {
                      if( context.getVerbose()>3 ) context.warning(Thread.currentThread().getName()+" resumes (last thread)");
                      break;
                   }
@@ -208,7 +210,7 @@ final public class ThreadBuilderTile {
       
       try {
          // initialisation de la liste des fichiers originaux pour ce losange
-         downFiles = new ArrayList<SrcFile>(Constante.MAXOVERLAY*2);
+         downFiles = new ArrayList<>(Constante.MAXOVERLAY*2);
          if( !askLocalFinder(bt,downFiles,hpxFinderPath, order, npix_file, blank)) {
             return null;
          }
@@ -219,11 +221,13 @@ final public class ThreadBuilderTile {
 
          // Pas trop de progniteurs => on peut tout faire d'un coup
          // Pour les cubes, on va pour le moment travailler en 1 seule passe (A VOIR PAR LA SUITE S'IL FAUT AMELIORER)
-         if( !context.live && (!mixing || n<Constante.MAXOVERLAY  || !requiredMem(mixing ? n : 1, nbThreads)) ) {
+         if( !context.live && (!mixing || n<Constante.MAXOVERLAY  || !requiredMem(mixing ? n : 1, 1 /*nbThreads */)) ) {
 
             statOnePass++;
             long mem = getReqMem(downFiles, 0, n);
             checkMem(mixing ? n : 1, mem, true);
+            
+            threadBuilder.setInfo("createLeavveHpx onepass memOk "+order+"/"+npix_file+"...");
             out = buildHealpix1(bt,order,npix_file,z,downFiles,0,n,null);
 
             // Trop de progniteurs, on va travailler en plusieurs couches de peinture
@@ -245,6 +249,7 @@ final public class ThreadBuilderTile {
                long mem = getReqMem(downFiles,deb,fin); //n);
                checkMem( fin-deb, mem, false);
                
+               threadBuilder.setInfo("createLeavveHpx multipass memOk "+order+"/"+npix_file+"...");
                f = buildHealpix1(bt,order,npix_file,z,downFiles,deb,fin,fWeight);
                if( f!=null ) {
                   if( out==null ) {
@@ -306,6 +311,7 @@ final public class ThreadBuilderTile {
 
       if( context.isTaskAborting() ) throw new Exception("Task abort !");
       
+      threadBuilder.setInfo("createLeavveHpx done "+path+"...");
       return out;
    }
    
@@ -359,8 +365,10 @@ final public class ThreadBuilderTile {
    static long statMultiPass=0L;
    static int statMaxOverlays=0;
 
-   static private double toRad = 180./Math.PI;
-   static private double PI2 = Math.PI/2.;
+   final static private double toDeg = 180./Math.PI;
+   final static private double PI2 = Math.PI/2.;
+   
+   static public int I = 0;
 
    /**
     * Rempli le tableau de pixels correspondant au fichier (losange) Healpix
@@ -423,20 +431,41 @@ final public class ThreadBuilderTile {
          double [] pixcoef = new double[overlay];
          if( flagColor ) { pixvalG = new double[overlay]; pixvalB = new double[overlay]; }
 
-         HealpixBase hpx = CDSHealpix.getHealpixBase(order+context.getTileOrder());
+         // METHODE REINECKE
+//         healpix.essentials.HealpixBase hpx = CDSHealpix.getHealpixBase(order+context.getTileOrder());
+         
+//         long nside = CDSHealpix.pow2( order+context.getTileOrder() );
+         int orderPix = order+context.getTileOrder();
 
          boolean gal2ICRS = context.getFrame()!=Localisation.ICRS;
+         
+         final HealpixNestedFast hn = Healpix.getNestedFast(orderPix);
+//         final VerticesAndPathComputer vpc = hn.newVerticesAndPathComputer(); // For thread safety issues
 
          for (int y = 0; y < out.height; y++) {
             for (int x = 0; x < out.width; x++) {
                index = min + context.xy2hpx(y * out.width + x);
-               // recherche les coordonnes du pixels HPX
-               Pointing pt = hpx.pix2ang(index);
-               radec[1] = (PI2 - pt.theta)*toRad;
-               radec[0] = pt.phi*toRad;
-
-               if( gal2ICRS ) radec = context.gal2ICRSIfRequired(radec);
-               coo.al = radec[0]; coo.del = radec[1];
+               
+               // recherche les coordonnes du pixels HPX (METHODE REINECKE)
+//               healpix.essentials.Pointing pt = hpx.pix2ang(index);
+//               radec[1] = (PI2 - pt.theta)*toRad;
+//               radec[0] = pt.phi*toRad;
+               
+               // ON TRAVAILLE DIRECTEMENT AVEC vpc POUR EVITER LES ALLOCATIONS INUTILES
+//               radec = CDSHealpix.pix2ang_nest(orderPix, index);
+//               CDSHealpix.polarToRadec( radec, radec );
+               
+               hn.center(index,radec);
+               if( gal2ICRS ) {
+                  radec[0] *= toDeg;
+                  radec[1] *= toDeg;
+                  radec = context.gal2ICRSIfRequired(radec);
+                  coo.al = radec[0]; 
+                  coo.del = radec[1];
+               } else {
+                  coo.al = radec[0]*toDeg;
+                  coo.del = radec[1]*toDeg;
+               }
 
                int nbPix=0;
                double totalCoef=0;
@@ -445,16 +474,14 @@ final public class ThreadBuilderTile {
                for( int i=deb; i<fin; i++ ) {
                   try {
                      file = downFiles.get(i);
-                     if( file.flagRemoved ) {
-                        continue;
-                     }
+                     if( file.flagRemoved ) continue;
                      try {
                         file.open(z, flagGauss);
 
                      } catch( Exception e ) {
-                        if( context.getVerbose()>=3 ) {
-                           context.warning("Problem on open => file retired: "+file.name+" => exception "+e.getMessage());
-                        }
+                        System.err.println("Error on open:");
+                        /* if( context.getVerbose()>=3 ) */ e.printStackTrace();
+                        context.addFileRemoveList(file.name);
                         
                         // Cas complexe o l'image originale est un JPEG ou un PNG et que JAVA
                         // tente de l'ouvrir sur une portion uniquement. Il va en interne
@@ -499,6 +526,7 @@ final public class ThreadBuilderTile {
                         // Cas normal
                      } else {
                         double pix = getBilinearPixel(file,coo,z,file.blank);
+
                         if( Double.isNaN(pix) ) continue;
                         pixval[nbPix]=pix;
                      }
@@ -588,7 +616,7 @@ final public class ThreadBuilderTile {
                            : pixelFinal<=cutOrig[2] ? cut[2]
                                  : pixelFinal>=cutOrig[3] ? cut[3]
                                        : (pixelFinal-cutOrig[2])*context.coef + cut[2];
-                                 if( bitpix>0 && (long)pixelFinal==blank && pixelFinal!=blank ) pixelFinal+=0.5;
+                       if( bitpix>0 && (long)pixelFinal==blank && pixelFinal!=blank ) pixelFinal+=0.5;
                   } else if( Double.isNaN(pixelFinal) ) pixelFinal = blank;
 
                   out.setPixelDouble(x,y,pixelFinal);
@@ -711,9 +739,13 @@ final public class ThreadBuilderTile {
 
       int x1 = (int)x;
       int y1 = (int)y;
+
+//      if( x-x1<0.5 ) x1--;
+//      if( y-y1<0.5 ) y1--;
+      
       int x2=x1+1;
       int y2=y1+1;
-
+ 
       int ox1= x1;
       int oy1= y1;
       int ox2= x2;
@@ -1108,7 +1140,11 @@ final public class ThreadBuilderTile {
          return fitsfile.getFilename();
       }
 
-      /** Ouverture effective du fichier FITS */
+      /** Ouverture effective du fichier FITS
+       * @param frame
+       * @return true si ouverture effective, false si dj ouvert
+       * @throws Exception
+       */
       protected void open(int frame) throws Exception { open(frame,false); }
       protected void open(int frame, boolean gauss) throws Exception {
          if( isOpened==frame ) return;
