@@ -1,10 +1,10 @@
-// Copyright 1999-2018 - Universit de Strasbourg/CNRS
+// Copyright 1999-2020 - Universit de Strasbourg/CNRS
 // The Aladin Desktop program is developped by the Centre de Donnes
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
 //
-//This file is part of Aladin.
+//This file is part of Aladin Desktop.
 //
 //    Aladin Desktop is free software: you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 //    GNU General Public License for more details.
 //
 //    The GNU General Public License is available in COPYING file
-//    along with Aladin.
+//    along with Aladin Desktop.
 //
 
 package cds.fits;
@@ -35,6 +35,7 @@ import cds.aladin.Save;
  * Classe dediee a la gestion d'un header FITS.
  *
  * @author Pierre Fernique [CDS]
+ * @version 1.7 : Jul 2018 : Prise en compte des descriptions des champs
  * @version 1.6 : Dc 2007 : Possibilit de surcharger les mots cls
  * @version 1.5 : 20 aout 2002 methode readFreeHeader
  * @version 1.4 : 19 juin 00 Utilisation du PushbackInputStream et
@@ -46,10 +47,11 @@ import cds.aladin.Save;
  */
 public final class HeaderFits {
 
-   private StringBuffer   memoHeaderFits = null;  // Memorisation de l'entete FITS telle quelle (en Strings)
+   private StringBuilder   memoHeaderFits = null;  // Memorisation de l'entete FITS telle quelle (en Strings)
    
   /** Les elements de l'entete */
-   protected Hashtable header;
+   protected Hashtable header;     // Valeur pour chaque cl
+   protected Hashtable headDescr;  // Description de chaque cl
    protected Vector<String> keysOrder;
 
    /** La taille de l'entete FITS (en octets) */
@@ -82,14 +84,14 @@ public final class HeaderFits {
     public String getOriginalHeaderFits() { return memoHeaderFits.toString(); }
     
     /** Mmorise le header FITS original (en Strings) */
-    public void setOriginalHeaderFits(String s) { memoHeaderFits= new StringBuffer(s); }
+    public void setOriginalHeaderFits(String s) { memoHeaderFits= new StringBuilder(s); }
 
    /** Ajoute la ligne courante a la memorisation du header FITS
     * en supprimant les blancs en fin de ligne
     * @param s la chaine a ajouter
     */
     public void appendMHF(String s) {
-       if( memoHeaderFits==null ) memoHeaderFits=new StringBuffer();
+       if( memoHeaderFits==null ) memoHeaderFits=new StringBuilder();
        memoHeaderFits.append(s.trim()+"\n");
     }
 
@@ -124,27 +126,107 @@ public final class HeaderFits {
     * @param buffer La ligne
     * @return La valeur
     */
+//   static public String getValue(byte [] buffer) {
+//       int i;
+//       boolean quote = false;
+//       boolean blanc=true;
+//       int offset = 9;
+//
+//       for( i=offset ; i<80; i++ ) {
+//          if( !quote ) {
+//             if( buffer[i]==(byte)'/' ) break;   // on a atteint le commentaire
+//          } else {
+//             if( buffer[i]==(byte)'\'') break;   // on a atteint la prochaine quote
+//          }
+//
+//          if( blanc ) {
+//             if( buffer[i]!=(byte)' ' ) blanc=false;
+//             if( buffer[i]==(byte)'\'' ) { quote=true; offset=i+1; }
+//          }
+//       }
+//       return (new String(buffer, 0, offset, i-offset)).trim();
+//   }
+   
+   /** Extraction de la valeur d'un champ FITS. va jusqu'au commentaire, ou sinon la fin de la ligne
+    * Vire les quotes si ncessaires. Vire les blancs aprs.
+    * @param buffer La ligne
+    * @return La valeur
+    */
    static public String getValue(byte [] buffer) {
-       int i;
-       boolean quote = false;
-       boolean blanc=true;
-       int offset = 9;
-
-       for( i=offset ; i<80; i++ ) {
-          if( !quote ) {
-             if( buffer[i]==(byte)'/' ) break;   // on a atteint le commentaire
-          } else {
-             if( buffer[i]==(byte)'\'') break;   // on a atteint la prochaine quote
-          }
-
-          if( blanc ) {
-             if( buffer[i]!=(byte)' ' ) blanc=false;
-             if( buffer[i]==(byte)'\'' ) { quote=true; offset=i+1; }
-          }
-       }
-       return (new String(buffer, 0, offset, i-offset)).trim();
+      int mode=0;
+      int tailSpace=0;
+      boolean end=false;
+      boolean oQuote=false;
+      
+      StringBuilder s1 = new StringBuilder(80);
+      
+      for( int i=9; i<buffer.length && i<80 && !end; i++ ) {
+         char ch= (char) buffer[i];
+         switch(mode) {
+            case 0: // Avant la valeur
+               if( ch!=' ' ) {
+                  mode=1;
+                  if( ch=='\'' ) mode=1;
+                  else { mode=2; s1.append(ch); }
+               }
+               break;
+            case 1: // Dans la valeur quote
+               if( ch=='\'' ) {
+                  
+                  // Premire quote ?
+                  if( !oQuote ) {
+                     
+                     // Pas de quote aprs => c'est termin
+                     char nCh = (char)( i>=buffer.length-1 || i>=79 ? ' ' : buffer[i+1] );
+                     if( nCh!='\'' ) { mode=2; break; }
+                     
+                  // Deuxime quote
+                  } else { oQuote=false; break; }
+               }
+               oQuote = ch=='\'';
+               s1.append(ch);
+               break;
+            case 2: // Dans la valeur non quot (ou fin de valeur quote)
+               if( ch=='/' ) { end=true; break;  } // je suis arriv au commentaire
+               if( ch==' ' ) tailSpace++;
+               else tailSpace=0;
+               s1.append(ch);
+               break;
+         }
+      }
+      return s1.substring(0, s1.length()-tailSpace);
    }
 
+
+
+   /** Extraction de la description d'un champ FITS.
+    * @return La description, ou null si aucune
+    */
+   static public String getDescription(byte [] buffer) {
+      int mode=0;
+      int deb=-1;
+      char och=' ';
+      
+      for( int i=9; i<buffer.length && i<80 && deb<0; i++ ) {
+         char ch= (char) buffer[i];
+         switch(mode) {
+            case 0: // Dans le champ valeur
+               if( ch=='/' ) deb=i+1;
+               else if( ch=='\'' ) mode=1;
+               break;
+            case 1: // Dans une valeur quote
+               if( ch=='\'' && och!='\\' ) mode=0;
+               break;
+         }
+         och=ch;
+      }
+      if( deb<0 ) return null;
+      String s = new String(buffer,deb,buffer.length-deb).trim();
+      if( s.length()==0 ) return null;
+      return s;
+   }
+
+
    /** retourne la cl d'une ligne d'entete FITS */
    static public  String getKey(byte [] buffer) {
       return new String(buffer, 0, 0, 8).trim();
@@ -192,10 +274,10 @@ public final class HeaderFits {
    public boolean readHeader(MyInputStream dis,FrameHeaderFits frameHeaderFits) throws Exception {
       int blocksize = 2880;
       int fieldsize = 80;
-      String key, value;
+      String key, value, desc;
       int linesRead = 0;
       sizeHeader=0;
-      boolean firstLine=true;
+//      boolean firstLine=true;
 
 //Aladin.trace(3,"Reading FITS header");
       byte[] buffer = new byte[fieldsize];
@@ -214,10 +296,36 @@ public final class HeaderFits {
             linesRead++;
             if( key.equals("END" ) ) break;
             appendMHF(new String(buffer,0));
-            if( buffer[8] != '=' ) continue;
+            
             value=getValue(buffer);
+            desc = null;
+            
+            // Pour supporter la convention CONTINUE
+            if( key.equals("CONTINUE") && buffer[8] != '=') {
+               int n = keysOrder.size();
+               if( n==0 ) {
+                  //                  throw new Exception("FITS CONTINUE convention error: no previous keyword");
+                  System.err.println("FITS CONTINUE convention error: no previous keyword => ignored");
+               } else {
+                  String lastKey = keysOrder.get( n-1 );
+                  String lastValue = (String ) header.get( lastKey );
+                  n = lastValue.length();
+                  if( n==0 || lastValue.charAt(n - 1)!='&' ) {
+                     //                  throw new Exception("FITS CONTINUE convention error: & missing");
+                     System.err.println("FITS CONTINUE convention error: & missing => ignored");
+                  } else {
+                     value = lastValue.substring(0,n-1)+value;
+                     key=lastKey;
+                  }
+               }
+               
+            } else {
+               if( buffer[8] != '=' ) continue;
+               desc=getDescription(buffer);
+            }
 //Aladin.trace(3,key+" ["+value+"]");
             header.put(key, value);
+            if( desc!=null ) headDescr.put(key, desc);
             keysOrder.addElement(key);
          }
 
@@ -290,7 +398,8 @@ public final class HeaderFits {
          // o l'on sauvegarde en FITS par la suite une image PNG ou JPEG avec entte par .hhh
          if( first ) {
             first=false;
-            if( buf.length>i+7 && !(new String(buf,i,6)).equals("SIMPLE") ) {
+            if( buf.length>i+7 && !(new String(buf,i,6)).equals("SIMPLE") 
+                  && !(new String(buf,i,6)).equals("XTENSI")) {
                appendMHF((new String(Save.getFitsLine("SIMPLE","T",null))).trim());
             }
          }
@@ -333,7 +442,9 @@ public final class HeaderFits {
                // Dans le cas d'une entte DSS dans un fichier ".hhh" il ne faut pas retenir les mots
                // cls concernant l'astromtrie de la plaque entire
                //            if( !( specialDSS && (key.startsWith("AMD") || key.startsWith("PLT"))) ) {
+               
                header.put(key, value);
+               if( com.length()>0 ) headDescr.put(key, com);
                keysOrder.addElement(key);
                //            }
                appendMHF((new String(Save.getFitsLine(key, value, com))).trim());
@@ -416,6 +527,12 @@ public final class HeaderFits {
       header.put(key,value);
    }
 
+   /** Surcharge ou ajout d'un mot cl */
+   public void setKeyword(String key,String value,String description) {
+      header.put(key,value);
+      headDescr.put(key,description);
+   }
+
   /** Recherche d'un element double par son mot cle
    * @param key le mot cle (inutile de l'aligner en 8 caractres)
    * @return la valeur recherchee
@@ -429,19 +546,59 @@ public final class HeaderFits {
       result = Double.valueOf(trimDouble(s)).doubleValue();
       return result;
    }
-
-  /** Recherche d'une chaine par son mot cle
-   * @param key le mot cle  (inutile de l'aligner en 8 caractres)
-   * @return la valeur recherchee
-   */
-   public String getStringFromHeader(String key)
-                 throws NullPointerException {
-      String s = (String) header.get(key.trim());
-      if( s==null || s.length()==0 ) return s;
-      if( s.charAt(0)=='\'' ) return s.substring(1,s.length()-1).trim();
-      return s;
-//      return (String) header.get(key.trim());
+   
+   /** Enlve les quotes d'une valeur String FITS (si ncessaire) */
+   static public String unquoteFits( String s ) {
+      int n;
+      if( s==null || (n=s.length())<2 ) return s;
+      char c = s.charAt(0);
+      if( c!='\'' || c!=s.charAt(n-1) ) return s;
+      
+      char [] a = s.toCharArray();
+      StringBuilder s1 = new StringBuilder(a.length);
+      boolean quote=false;
+      for( int i=1; i<n-1; i++ ) {
+         c=a[i];
+         if( quote && c=='\'' ) { quote=false; continue; }
+         else s1.append(c);
+         quote= c=='\'';
+      }
+      return s1.toString();
    }
+   
+   /** Ajoute les quotes pour mettre en forme une chaine */
+   static public String quoteFits( String s ) {
+      if( s==null ) return s;
+      StringBuilder s1 = new StringBuilder( s.length()+10 );
+      s1.append('\'');
+      for( char a : s.toCharArray() ) {
+         if( a=='\'' ) s1.append('\'');
+         s1.append(a);
+      }
+      s1.append('\'');
+      return s1.toString();
+   }
+
+   /** Recherche d'une chaine par son mot cle
+    * @param key le mot cle  (inutile de l'aligner en 8 caractres)
+    * @return la valeur recherchee
+    */
+    public String getStringFromHeader(String key) throws NullPointerException {
+       String s = (String) header.get(key.trim());
+       if( s==null || s.length()==0 ) return s;
+//       if( s.charAt(0)=='\'' ) return s.substring(1,s.length()-1).trim();
+//       return s;
+       return unquoteFits(s).trim();
+    }
+
+    /** Recherche de la description d'une entre par son mot cle
+     * @param key le mot cle  (inutile de l'aligner en 8 caractres)
+     * @return la valeur recherchee
+     */
+     public String getDescriptionFromHeader(String key) throws NullPointerException {
+        String s = (String) headDescr.get(key.trim());
+        return s;
+     }
 
    /** Ajout, surcharge ou suppression d'un mot cle
     * @param key le mot cl (inutile de l'aligner en 8 caractres)
@@ -471,7 +628,7 @@ public final class HeaderFits {
    }
 
    /** Ecriture de l'entte FITS des mots cls mmoriss. L'ordre est conserv
-    * comme  l'origine - les commentaires ne sont pas restitus 
+    * comme  l'origine - les commentaires sont restitus 
     * @return le nombre d'octets crits */
    public int writeHeader(OutputStream os ) throws Exception {
       int n=keysOrder.size()*80;
@@ -484,7 +641,8 @@ public final class HeaderFits {
          String key = (String)e.nextElement();
          String value = (String) header.get(key);
          if( value==null ) continue;
-         System.arraycopy(getFitsLine(key,value),0,buf,m,80 );
+         String desc = (String) headDescr.get(key);
+         System.arraycopy(getFitsLine(key,value,desc),0,buf,m,80 );
          m+=80;
       }
       System.arraycopy(b,0,buf,m,b.length);
@@ -619,14 +777,26 @@ public final class HeaderFits {
 
      return s.toString().toCharArray();
   }
+  
+  /** Retourne true si l'entte ne contient encore rien */
+  public boolean isEmpty() { return header.size()==0; }
 
   /** Allocation ou rallocation des structures de mmorisation */
   protected void alloc() {
 //     if( header!=null && keysOrder!=null ) return;
      header = new Hashtable(200);
+     headDescr = new Hashtable(200);
      keysOrder = new Vector(200);
   }
   
+  /** Copie des donnes du Header dans le Header pass en paramtre (crasement des donnes prcdentes ventuelles) */
+  protected void copyTo( HeaderFits out ) {
+     out.header = (Hashtable) header.clone();
+     out.headDescr = (Hashtable) headDescr.clone();
+     out.keysOrder = (Vector) keysOrder.clone();
+     
+  }
+  
   /** Retourne la taille mmoire approximative */
   public long getMem() { return 16+(keysOrder==null?0:keysOrder.size()*50); }
 
