@@ -1,10 +1,10 @@
-// Copyright 1999-2018 - Universit de Strasbourg/CNRS
+// Copyright 1999-2020 - Universit de Strasbourg/CNRS
 // The Aladin Desktop program is developped by the Centre de Donnes
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
 //
-//This file is part of Aladin.
+//This file is part of Aladin Desktop.
 //
 //    Aladin Desktop is free software: you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 //    GNU General Public License for more details.
 //
 //    The GNU General Public License is available in COPYING file
-//    along with Aladin.
+//    along with Aladin Desktop.
 //
 
 package cds.moc;
@@ -38,23 +38,22 @@ import java.util.StringTokenizer;
 /** HEALPix Multi Order Coverage Map (MOC) IO routines
  * Compliante with IVOA MOC Rec 1.0 June 2014
  *
- * Example : HealpixMoc moc = new HealpixMoc();
+ * Example : SMoc moc = new SMoc();
  *           (new IO(moc).read(String filename);
  *
  * @author Pierre Fernique [CDS]
+ * @version 1.5 Mar 2018 - TMOC patch
  * @version 1.4 Nov 2015 - JSON #MOCORDER patching
  * @version 1.3 Sep 2013 - WD 1.0 10/9/2013 compliante
  * @version 1.2 Avr 2012 - JSON ASCII support
  * @version 1.1 Mar 2012 - Healpix FITS header adjustements
- * @version 1.0 Oct 2011 - Dedicated class (removed from HealpixMoc)
+ * @version 1.0 Oct 2011 - Dedicated class (removed from SMoc)
  */
 public final class MocIO {
 
-   static public final int FITS = HealpixMoc.FITS;         // Standard format
-   static public final int JSON = HealpixMoc.JSON;         // JSON format (suggested in IVOA REC)
-   static public final int ASCII = HealpixMoc.ASCII;       // ASCII format (suggested in IVOA REC)
-   
-   static public final int JSON0 = HealpixMoc.JSON0;       // JSON obsolete format (only reading supported for compatibility)
+   static public final int FITS = SMoc.FITS;         // Standard format
+   static public final int ASCII = SMoc.ASCII;       // ASCII format
+   static public final int JSON = SMoc.JSON;         // JSON format (suggested in IVOA REC)
 
    static private String CR = System.getProperty("line.separator");
 
@@ -73,10 +72,10 @@ public final class MocIO {
    static final public String OLDSIGNATURE = "HPXMOC";   // FITS keywords used as signature (obsoleted syntax)
    static final public String SIGNATURE = "MOCORDER";    // FITS keywords used as signature
 
-   private HealpixMoc moc;
+   private Moc moc;
    private byte firstChar=0; // Use for storing first character (mode detection)
 
-   public MocIO(HealpixMoc m) { moc=m; }
+   public MocIO(Moc m) { moc=m; }
 
    /** Read HEALPix MOC from a file.
     * Support standard FITS format and ASCII non standard alternatives (JSON, ASCII)
@@ -166,6 +165,10 @@ public final class MocIO {
     *     ...
     *     "MAXORDER":[]
     *    }
+    *    
+    * TMOC (in JD ranges)
+    *    #TMOC
+    *    jdmin jdmax jdmin jdmax...
     *
     * @param in input stream
     * @throws Exception
@@ -175,22 +178,56 @@ public final class MocIO {
       moc.clear();
       moc.setCheckConsistencyFlag(false); // We assume that the input MOC is well formed
       String s;
+      int mocOrder=SMoc.MAXORDER;
+      boolean flagTMoc = false;
       boolean flagMocOrder=false;
       for( int line=0; (s=dis.readLine())!=null; line++ ) {
          if( firstChar!=0 ) { s=(char)firstChar+s; firstChar=0; }
          if( s.length()==0 ) continue;
-         if( line==0 && s.startsWith("#MOCORDER ") ) {
-            String v = s.substring(10);
-            if( v.length()>0 ) { flagMocOrder=true; moc.setProperty("MOCORDER", v); }
-
-         }
-         parseASCIILine(s);
+         if( line==0 ) {
+            if( s.startsWith("#MOCORDER ") ) {
+               String v = s.substring(10);
+               if( v.length()>0 ) { flagMocOrder=true; moc.setProperty("MOCORDER", v); }
+            } else if( s.startsWith("#TMOC") ) {
+               String v = s.substring(5);
+               try {
+                  mocOrder = Integer.parseInt(v.trim());
+                  if( mocOrder<29 ) flagMocOrder=true;
+               } catch( Exception e ) {}
+               flagTMoc=true;
+               continue;
+            }
+         } 
+         if( flagTMoc ) parseTMocASCIILine(s);
+         else parseASCIILine(s);
+      }
+      
+      // Pour conclure (parsing STMOC notamment)
+      moc.addHpix(null);
+      
+      // If the MocOrder is found by the content
+      if( !flagMocOrder ) {
+//         moc.setProperty("MORORDER", moc.getMaxOrder()+"" );
+         moc.setMocOrder(moc.getMaxUsedOrder());
+      }
+      
+      if( flagTMoc ) {
+         moc.toMocSet();
+         if( mocOrder!=29 ) moc.setMocOrder(mocOrder);
       }
       
-      // If the MocOrder is found by the containts
-      if( !flagMocOrder ) moc.setProperty("MORORDER", moc.getMaxOrder()+"" );
    }
-
+  
+   // Parse une ligne d'un flux TMOC par intervalles JD en notation floattante
+   private void parseTMocASCIILine(String s) throws Exception {
+      StringTokenizer st = new StringTokenizer(s," ;,\n\r\t");
+      while( st.hasMoreTokens() ) {
+         String s1 = st.nextToken();
+         String s2 = st.nextToken();
+         ((TMoc)moc).add( Double.parseDouble(s1), Double.parseDouble(s2));
+      }
+   }
+   
    /** Read HEALPix MOC from an Binary FITS stream */
    public void readFits(InputStream in) throws Exception {
       moc.clear();
@@ -219,10 +256,9 @@ public final class MocIO {
          int naxis2 = header.getIntFromHeader("NAXIS2");
          String tform = header.getStringFromHeader("TFORM1");
          int nbyte= tform.indexOf('K')>=0 ? 8 : tform.indexOf('J')>=0 ? 4 : -1;   // entier 64 bits, sinon 32
-         if( nbyte<=0 ) throw new Exception("HEALPix Multi Order Coverage Map only requieres integers (32bits or 64bits)");
-         byte [] buf = new byte[naxis1*naxis2];
-         readFully(in,buf);
-         createUniq((naxis1*naxis2)/nbyte,nbyte,buf);
+         if( nbyte<=0 ) throw new Exception("Multi Order Coverage Map only requieres integers (32bits or 64bits)");
+
+         moc.readSpecificData( in, naxis1,naxis2, nbyte, header);
       } catch( EOFException e ) { }
    }
 
@@ -248,8 +284,8 @@ public final class MocIO {
          fo = new FileOutputStream(f);
          fb = new BufferedOutputStream(fo);
          if( mode==FITS ) writeFits(fb);
-         else if( mode==JSON  || mode==ASCII) writeJSON(fb);
-//         else if( mode==ASCII ) writeASCII(fb);
+         else if( mode==JSON  ) writeJSON(fb);
+         else if( mode==ASCII ) writeASCII(fb);
       } finally {
          if( fb!=null ) fb.close();
          else if( fo!=null ) fo.close();
@@ -266,66 +302,31 @@ public final class MocIO {
    /** Write HEALPix MOC to an output stream
     * At the end, the stream is not closed
     * @param out output stream
-    * @param mode encoded format (FITS or JSON)
+    * @param mode encoded format (FITS, JSON or ASCII)
     */
    public void write(OutputStream out,int mode) throws Exception {
       if( mode!=FITS && mode!=ASCII && mode!=JSON ) throw new Exception("Unknown MOC format !");
       if( mode==FITS ) writeFits(out);
-      else if( mode==JSON || mode==ASCII ) writeJSON(out);
-//      else if( mode==ASCII ) writeASCII(out);
+      else if( mode==JSON ) writeJSON(out);
+      else if( mode==ASCII ) writeASCII(out);
    }
 
    private void testMocNotNull() throws Exception {
-      if( moc==null ) throw new Exception("No MOC assigned (use setMoc(HealpixMoc))");
+      if( moc==null ) throw new Exception("No MOC assigned (use setMoc(SMoc))");
    }
 
-   private static final int MAXWORD=20;
-   private static final int MAXSIZE=80;
-
-   /** Write HEALPix MOC to an output stream IN ASCII encoded format
-    * @param out output stream
-    * @deprecated use FITS format
-    */
    public void writeASCII(OutputStream out) throws Exception {
       testMocNotNull();
-      out.write(("#"+SIGNATURE+" "+moc.getMocOrder()+CR).getBytes());
-      StringBuilder res= new StringBuilder(moc.getSize()*8);
-      int order=-1;
-      boolean flagNL = moc.getSize()>MAXWORD;
-      int sizeLine=0;
-      int j=0;
-      for( MocCell c : moc ) {
-         if( res.length()>0 ) {
-            if( c.order!=order ) {
-               if( flagNL ) { res.append("\n"); sizeLine=0; j++; }
-               else res.append(" ");
-            } else {
-               int n=(c.npix+"").length();
-               if( flagNL && n+sizeLine>MAXSIZE ) { res.append(",\n "); sizeLine=3; j++; }
-               else { res.append(','); sizeLine++; }
-            }
-            if( j>15) { writeASCIIFlush(out,res,false); j=0; }
-         }
-         String s = c.order!=order ?  c.order+"/"+c.npix : c.npix+"";
-         res.append(s);
-         sizeLine+=s.length();
-         order=c.order;
-      }
-      int n = res.length();
-
-      if( n>0 && res.charAt(n-1)==',' ) res.replace(n-1, n-1, (flagNL?"\n":" "));
-      else res.append((flagNL?"\n":" "));
-
-      writeASCIIFlush(out,res);
+      moc.writeASCII(out);
    }
 
    /** Write HEALPix MOC to an output stream IN JSON encoded format
     * @param out output stream
-    * @deprecated use FITS format
     */
    public void writeJSON(OutputStream out) throws Exception {
       testMocNotNull();
       
+      if( moc instanceof STMoc ) throw new Exception("JSON format not supported for STMoc");
       
       // Nouvelle version JSON
       // On remplace tout de mme l'ancienne signature non compatible JSON par une ligne vide
@@ -333,14 +334,14 @@ public final class MocIO {
       out.write(CR.getBytes());
       
       StringBuilder s = new StringBuilder(2048);
-      int nOrder = moc.getMaxOrder()+1;
+      int nOrder = moc.getMaxUsedOrder()+1;
       s.append("{");
       boolean first=true;
       int order;
       for( order=0; order<nOrder; order++) {
-         int n = moc.getSize(order);
+         int n = ((SMoc)moc).getSize(order);
          if( n==0 && order<nOrder-1 ) continue;
-         Array a = moc.getArray(order);
+         Array a = ((SMoc)moc).getArray(order);
          if( !first ) s.append("],"+CR);
          first = false;
          s.append("\""+order+"\":[");
@@ -348,13 +349,13 @@ public final class MocIO {
          for( int i=0; i<n; i++ ) {
             s.append( a.get(i)+(i==n-1?"":",") );
             j++;
-            if( j==15 ) { writeASCIIFlush(out,s); j=0; }
+            if( j==15 ) { moc.writeASCIIFlush(out,s); j=0; }
          }
       }
       
       if( !first ) s.append("]");
       s.append("}");
-      writeASCIIFlush(out,s);
+      moc.writeASCIIFlush(out,s);
    }
 
    /** Write HEALPix MOC to an output stream in FITS encoded format
@@ -363,10 +364,8 @@ public final class MocIO {
    public void writeFits(OutputStream out) throws Exception {
       testMocNotNull();
       writeHeader0(out);
-      int maxOrder = moc.getMocOrder();
-      int nbytes=moc.getType(maxOrder)==HealpixMoc.LONG ? 8 : 4;  // Codage sur des integers ou des longs
-      writeHeader1(out,nbytes);
-      writeData(out,nbytes);
+      writeHeader1(out);
+      writeData(out);
    }
 
    /*********************************************** Private methods  *****************************************/
@@ -375,7 +374,7 @@ public final class MocIO {
    private void setCurrentParseOrder(String s) throws Exception {
       int i = s.indexOf('=');
       try {
-         moc.setCurrentOrder( (int)HealpixMoc.log2(Long.parseLong(s.substring(i+1))) );
+         moc.setCurrentOrder( (int)SMoc.log2(Long.parseLong(s.substring(i+1))) );
       } catch( Exception e ) {
          throw new Exception("HpixList.setNside: syntax error ["+s+"]");
       }
@@ -402,7 +401,7 @@ public final class MocIO {
    // Parsing de la ligne COORDSYS=G|C
    private void setCoord(String s) throws Exception {
       int i = s.indexOf('=');
-      moc.setCoordSys(s.substring(i+1));
+      ((SMoc)moc).setSys(s.substring(i+1));
    }
 
    // Parse une ligne d'un flux (reconnait JSON et basic ASCII)
@@ -422,46 +421,20 @@ public final class MocIO {
       }
    }
 
-   public void createUniq(int nval,int nbyte,byte [] t) throws Exception {
-      int i=0;
-      long [] hpix = null;
-      long oval=-1;
-      for( int k=0; k<nval; k++ ) {
-         long val=0;
-
-         int a =   ((t[i])<<24) | (((t[i+1])&0xFF)<<16) | (((t[i+2])&0xFF)<<8) | (t[i+3])&0xFF;
-         if( nbyte==4 ) val = a;
-         else {
-            int b = ((t[i+4])<<24) | (((t[i+5])&0xFF)<<16) | (((t[i+6])&0xFF)<<8) | (t[i+7])&0xFF;
-            val = (((long)a)<<32) | ((b)& 0xFFFFFFFFL);
-         }
-         i+=nbyte;
-
-         long min = val;
-         if( val<0 ) { min = oval+1; val=-val; }
-         for( long v = min ; v<=val; v++) {
-            hpix = HealpixMoc.uniq2hpix(v,hpix);
-            int order = (int)hpix[0];
-            moc.add( order, hpix[1]);
-         }
-         oval=val;
-      }
-   }
-
-   private void writeASCIIFlush(OutputStream out, StringBuilder s) throws Exception {
-      writeASCIIFlush(out, s, true);
-   }
-   
-   private void writeASCIIFlush(OutputStream out, StringBuilder s,boolean nl) throws Exception {
-      if( nl ) s.append(CR);
-      out.write(s.toString().getBytes());
-      s.delete(0,s.length());
-   }
+//   private void writeASCIIFlush(OutputStream out, StringBuilder s) throws Exception {
+//      writeASCIIFlush(out, s, true);
+//   }
+//   
+//   private void writeASCIIFlush(OutputStream out, StringBuilder s,boolean nl) throws Exception {
+//      if( nl ) s.append(CR);
+//      out.write(s.toString().getBytes());
+//      s.delete(0,s.length());
+//   }
 
    // Write the primary FITS Header
    private void writeHeader0(OutputStream out) throws Exception {
       int n=0;
-      out.write( getFitsLine("SIMPLE","T","Written by MOC java API "+HealpixMoc.VERSION) ); n+=80;
+      out.write( getFitsLine("SIMPLE","T","Written by MOC java API "+SMoc.VERSION) ); n+=80;
       out.write( getFitsLine("BITPIX","8") ); n+=80;
       out.write( getFitsLine("NAXIS","0") );  n+=80;
       out.write( getFitsLine("EXTEND","T") ); n+=80;
@@ -469,10 +442,11 @@ public final class MocIO {
    }
    
    // Write the FITS HDU Header for the UNIQ binary table
-   private void writeHeader1(OutputStream out,int nbytes) throws Exception {
+   private void writeHeader1(OutputStream out) throws Exception {
       int n=0;
+      int nbytes = moc.getType()==SMoc.LONG ? 8 : 4;
       int naxis2 = moc.getSize();
-      out.write( getFitsLine("XTENSION","BINTABLE","HEALPix Multi Order Coverage map") ); n+=80;
+      out.write( getFitsLine("XTENSION","BINTABLE","Multi Order Coverage map") ); n+=80;
       out.write( getFitsLine("BITPIX","8") ); n+=80;
       out.write( getFitsLine("NAXIS","2") );  n+=80;
       out.write( getFitsLine("NAXIS1",nbytes+"") );  n+=80;
@@ -481,12 +455,9 @@ public final class MocIO {
       out.write( getFitsLine("GCOUNT","1") ); n+=80;
       out.write( getFitsLine("TFIELDS","1") ); n+=80;
       out.write( getFitsLine("TFORM1",nbytes==4 ? "1J" : "1K") ); n+=80;
-      out.write( getFitsLine("TTYPE1","UNIQ","HEALPix UNIQ pixel number") ); n+=80;
-      out.write( getFitsLine("PIXTYPE","HEALPIX","HEALPix magic code") );    n+=80;
-      out.write( getFitsLine("ORDERING","NUNIQ","NUNIQ coding method") );    n+=80;      
-      out.write( getFitsLine("COORDSYS",""+moc.getCoordSys(),"reference frame (C=ICRS)") );    n+=80;      
-      out.write( getFitsLine("MOCORDER",""+moc.getMocOrder(),"MOC resolution (best order)") );    n+=80;      
-      out.write( getFitsLine("MOCTOOL","CDSjavaAPI-"+HealpixMoc.VERSION,"Name of the MOC generator") );    n+=80;      
+      
+      n+=moc.writeSpecificFitsProp( out );
+      out.write( getFitsLine("MOCTOOL","CDSjavaAPI-"+SMoc.VERSION,"Name of the MOC generator") );    n+=80;      
 
       for( int i=0; i<FITSKEY.length; i++ ) {
          String key = FITSKEY[i][0];
@@ -502,24 +473,13 @@ public final class MocIO {
    }
 
    // Write the UNIQ FITS HDU Data in basic mode
-   private void writeData(OutputStream out,int nbytes) throws Exception {
-      if( moc.getSize()<=0 ) return;
-      byte [] buf = new byte[nbytes];
-      int size = 0;
-      int nOrder = moc.getMaxOrder()+1;
-      for( int order=0; order<nOrder; order++ ) {
-         int n = moc.getSize(order);
-         if( n==0 ) continue;
-         Array a = moc.getArray(order);
-         for( int i=0; i<n; i++) {
-            long val = HealpixMoc.hpix2uniq(order, a.get(i) );
-            size+=writeVal(out,val,buf);
-         }
-      }
+   protected void writeData(OutputStream out) throws Exception {
+      int size = moc.writeSpecificData( out);
       out.write( getBourrage(size) );
+//      System.out.println("write size="+size+" getSize="+moc.getSize()*8);
    }
 
-   private int writeVal(OutputStream out,long val,byte []buf) throws Exception {
+   static protected int writeVal(OutputStream out,long val,byte []buf) throws Exception {
       for( int j=0,shift=(buf.length-1)*8; j<buf.length; j++, shift-=8 ) buf[j] = (byte)( 0xFF & (val>>shift) );
       out.write( buf );
       return buf.length;
@@ -540,7 +500,7 @@ public final class MocIO {
     * @param comment The commend, or null
     * @return the 80 character FITS line
     */
-   private byte [] getFitsLine(String key, String value, String comment) {
+   static protected byte [] getFitsLine(String key, String value, String comment) {
       int i=0,j;
       char [] a;
       byte [] b = new byte[80];
@@ -593,20 +553,20 @@ public final class MocIO {
    }
 
    /** Generate the end of a FITS block assuming a current block size of headSize bytes */
-   private byte [] getBourrage(int currentPos) {
+   static protected byte [] getBourrage(int currentPos) {
       int size = 2880 - currentPos%2880;
       byte [] b = new byte[size];
       return b;
    }
 
    /** Fully read buf.length bytes from in input stream */
-   private void readFully(InputStream in, byte buf[]) throws IOException {
+   static public void readFully(InputStream in, byte buf[]) throws IOException {
       readFully(in,buf,0,buf.length);
    }
 
    /** Fully read len bytes from in input stream and store the result in buf[]
     * from offset position. */
-   private void readFully(InputStream in,byte buf[],int offset, int len) throws IOException {
+   static public void readFully(InputStream in,byte buf[],int offset, int len) throws IOException {
       int m;
       for( int n=0; n<len; n+=m ) {
          m = in.read(buf,offset+n,(len-n)<512 ? len-n : 512);
@@ -620,7 +580,7 @@ public final class MocIO {
     * @return true si s est une chaine ni numrique, ni boolenne
     * ATTENTION: NE PREND PAS EN COMPTE LES NOMBRES IMAGINAIRES
     */
-   private boolean isFitsString(String s) {
+   static private boolean isFitsString(String s) {
       if( s.length()==0 ) return true;
       char c = s.charAt(0);
       if( s.length()==1 && (c=='T' || c=='F') ) return false;   // boolean
@@ -631,7 +591,7 @@ public final class MocIO {
       } catch( Exception e ) { return true; }
    }
 
-   private char [] formatFitsString(char [] a) {
+   static private char [] formatFitsString(char [] a) {
       if( a.length==0 ) return a;
       StringBuffer s = new StringBuffer();
       int i;
@@ -705,7 +665,7 @@ public final class MocIO {
          int linesRead = 0;
          sizeHeader=0;
 
-         header = new Hashtable<String,String>(200);
+         header = new Hashtable<>(200);
          byte[] buffer = new byte[fieldsize];
 
          while (true) {
@@ -742,7 +702,7 @@ public final class MocIO {
        * @param key FITs key (with or without trailing blanks)
        * @return corresponding integer value
        */
-      private int getIntFromHeader(String key) throws NumberFormatException,NullPointerException {
+      public int getIntFromHeader(String key) throws NumberFormatException,NullPointerException {
          String s = header.get(key.trim());
          return (int)Double.parseDouble(s.trim());
       }
@@ -752,7 +712,7 @@ public final class MocIO {
        * @param key FITs key (with or without trailing blanks)
        * @return corresponding string value without quotes (')
        */
-      private String getStringFromHeader(String key) throws NullPointerException {
+      public String getStringFromHeader(String key) throws NullPointerException {
          String s = header.get(key.trim());
          if( s==null || s.length()==0 ) return s;
          if( s.charAt(0)=='\'' ) return s.substring(1,s.length()-1).trim();
