@@ -1,10 +1,10 @@
-// Copyright 1999-2018 - Universit de Strasbourg/CNRS
+// Copyright 1999-2020 - Universit de Strasbourg/CNRS
 // The Aladin Desktop program is developped by the Centre de Donnes
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
 //
-//This file is part of Aladin.
+//This file is part of Aladin Desktop.
 //
 //    Aladin Desktop is free software: you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 //    GNU General Public License for more details.
 //
 //    The GNU General Public License is available in COPYING file
-//    along with Aladin.
+//    along with Aladin Desktop.
 //
 
 package cds.mocmulti;
@@ -64,7 +64,7 @@ import cds.astro.Ecliptic;
 import cds.astro.Galactic;
 import cds.astro.ICRS;
 import cds.moc.Healpix;
-import cds.moc.HealpixMoc;
+import cds.moc.SMoc;
 
 
 /**
@@ -99,8 +99,8 @@ public class MultiMoc implements Iterable<MocItem> {
    private MyProperties example = null;  // List of existing properties with examples
    
    public MultiMoc() {
-      map = new HashMap<String, MocItem>(30000);
-      tri = new ArrayList<String>(30000);
+      map = new HashMap<>(30000);
+      tri = new ArrayList<>(30000);
    }
    
    /** MultiMoc creation from a binary dump file
@@ -117,10 +117,10 @@ public class MultiMoc implements Iterable<MocItem> {
     * @param mocId  MOC identifier (unique)
     * @param moc MOC to memorize
     */
-   public void add(String mocId, HealpixMoc moc, MyProperties prop, long dateMoc, long dateProp) throws Exception {
+   public void add(String mocId, SMoc moc, MyProperties prop, long dateMoc, long dateProp) throws Exception {
       if( moc!=null ) {
          int o = moc.getMocOrder();
-         if( o==HealpixMoc.MAXORDER ) o = moc.getMaxOrder();  // A cause du bug
+         if( o==SMoc.MAXORDER ) o = moc.getMaxUsedOrder();  // A cause du bug
          if( mocOrder<o) mocOrder=o;
          moc.sort();
       }
@@ -162,7 +162,7 @@ public class MultiMoc implements Iterable<MocItem> {
    }
    
    /** Return the MOC associated to a mocId, null if not found */
-   public HealpixMoc getMoc(String mocId) {
+   public SMoc getMoc(String mocId) {
       MocItem mi = map.get(mocId);
       return mi==null ? null : mi.moc;
    }
@@ -177,7 +177,7 @@ public class MultiMoc implements Iterable<MocItem> {
    /** Return the list of current properties known by the MultiMoc, with one example by property */
    public MyProperties getProperties() {
       MyProperties prop = new MyProperties();
-      HashMap<String, Integer> count = new HashMap<String, Integer>();
+      HashMap<String, Integer> count = new HashMap<>();
       
       for( MocItem mi : this ) {
          if( mi.prop==null ) continue;
@@ -219,7 +219,7 @@ public class MultiMoc implements Iterable<MocItem> {
    
    static private Healpix hpx = new Healpix();
    
-   public String adjustProp(MyProperties prop, String id, HealpixMoc moc) {
+   public String adjustProp(MyProperties prop, String id, SMoc moc) {
       
       String s;
       String mocId = getID(prop,id);
@@ -235,8 +235,8 @@ public class MultiMoc implements Iterable<MocItem> {
 
          // Petit bricolage horrible pour contourner les MocOrder que l'on a oubli
          // d'indiquer
-         if( s==null || s.equals(""+HealpixMoc.MAXORDER) || s.equals("0") || s.equals("-1")) {
-            int maxOrder = moc.getMaxOrder();
+         if( s==null || s.equals(""+SMoc.MAXORDER) || s.equals("0") || s.equals("-1")) {
+            int maxOrder = moc.getMaxUsedOrder();
             if( maxOrder==0 ) {
                String s1 = prop.get("hips_order");
                if( s1!=null ) s=s1;
@@ -263,7 +263,7 @@ public class MultiMoc implements Iterable<MocItem> {
                // pour viter de rcuprer le bug sur le MocOrder
                try {
                   int n = Integer.parseInt( prop.get("moc_order"));
-                  HealpixMoc mm = new HealpixMoc();
+                  SMoc mm = new SMoc();
                   mm.setMocOrder(n);
                   fov =  mm.getAngularRes()+"";
                } catch( Exception e) {
@@ -271,7 +271,7 @@ public class MultiMoc implements Iterable<MocItem> {
                }
             }
             if( ra==null || dec==null ) {
-               if( moc.isAllSky() ) { ra="0"; dec="+0"; }
+               if( moc.isFull() ) { ra="0"; dec="+0"; }
                else {
                   try {
                      int order = moc.getMocOrder();
@@ -336,7 +336,7 @@ public class MultiMoc implements Iterable<MocItem> {
                if( (new File(mocFile)).exists()) continue;
             }
 
-            HealpixMoc moc;
+            SMoc moc;
             MyProperties prop;
 
             try {
@@ -362,7 +362,7 @@ public class MultiMoc implements Iterable<MocItem> {
                if( prop==null && moc==null ) continue;
                if( prop==null && moc!=null ) prop = new MyProperties();
                mocId = adjustProp(prop,mocId,moc);
-               if( moc!=null && !moc.getCoordSys().equals(COORDSYS) ) moc=null; // Incompatible MOC coordsys
+               if( moc!=null && !moc.getSys().equals(COORDSYS) ) moc=null; // Incompatible MOC coordsys
 
                if( !exceptProp(prop,mocId) ) { prop=null; moc=null; }
 
@@ -467,8 +467,7 @@ public class MultiMoc implements Iterable<MocItem> {
    public boolean exceptProp(MyProperties prop,String id) {
       if( except==null || prop==null ) return true;
       for( MyProperties p : except ) prop.exceptions(p,id);
-      return prop.get(KEY_ID)==null
-            || prop.get("MOCSERVER_REMOVE")==null;   // A virer plus tard
+      return prop.get(KEY_ID)==null || prop.get("MOCSERVER_REMOVE")==null;   // A virer plus tard
    }
    
    // Loading Exceptions as Array of property records
@@ -501,7 +500,7 @@ public class MultiMoc implements Iterable<MocItem> {
    //   #     
    private void loadException(String file) {
       if( file==null ) return;
-      ArrayList<MyProperties> p = new ArrayList<MyProperties>();
+      ArrayList<MyProperties> p = new ArrayList<>();
       BufferedReader br = null;
       try {
          br = new BufferedReader(new FileReader( file ));
@@ -572,35 +571,35 @@ public class MultiMoc implements Iterable<MocItem> {
    }
    
    /** Changement de rfrentiel d'un MOC */
-   static final public HealpixMoc convertToICRS(HealpixMoc moc) throws Exception {
-      char a = moc.getCoordSys().charAt(0);
+   static final public SMoc convertToICRS(SMoc moc) throws Exception {
+      char a = moc.getSys().charAt(0);
       
       // Dj en ICRS
       if( a!='G' && a!='E' ) return moc;   // dj en ICRS
       
       // Ca va prendre trop de temps si on garde la rsolution max
-      if( moc.getMaxOrder()>10 && moc.getCoverage()>0.99 && !moc.isAllSky() ) {
+      if( moc.getMaxUsedOrder()>10 && moc.getCoverage()>0.99 && !moc.isFull() ) {
          moc.setMocOrder(10);
          
       // Pour convertir, il faut avoir un cran de marge
 //      } else {
-//         if( moc.getMaxOrder()==HealpixMoc.MAXORDER ) {
+//         if( moc.getMaxOrder()==SMoc.MAXORDER ) {
 //            System.out.println("Changement de order de 29  28");
-//            moc.setMocOrder(HealpixMoc.MAXORDER-1);
+//            moc.setMocOrder(SMoc.MAXORDER-1);
 //            System.out.println("C'est fait");
 //         }
       }
       
       // Ciel complet => cas trivial
-      if( moc.isAllSky()) {
-         moc.setCoordSys("C");
+      if( moc.isFull()) {
+         moc.setSys("C");
          return moc;
       }
       
       Astroframe frameSrc = a=='G' ? new Galactic() : a=='E' ? new Ecliptic() : new ICRS();
       Healpix hpx = new Healpix();
-      int order = moc.getMaxOrder();
-      HealpixMoc moc1 = new HealpixMoc(moc.getMinLimitOrder(),moc.getMocOrder());
+      int order = moc.getMaxUsedOrder();
+      SMoc moc1 = new SMoc(moc.getMinOrder(),moc.getMocOrder());
       moc1.setCheckConsistencyFlag(false);
       long onpix1=-1;
       int n=0;
@@ -629,15 +628,15 @@ public class MultiMoc implements Iterable<MocItem> {
 
    
    // Moc Loading
-   private HealpixMoc loadMoc(String filename) {
-      HealpixMoc moc=null;
+   private SMoc loadMoc(String filename) {
+      SMoc moc=null;
       try {
-         moc = new HealpixMoc();
+         moc = new SMoc();
          moc.read(filename);
          if( moc.getSize()==0 ) moc=null;
          
          // Pas dans le bon systme de rfrence
-         String sys=moc.getCoordSys();
+         String sys=moc.getSys();
          if( !sys.equals(COORDSYS) ) {
             long t = System.currentTimeMillis();
             try {
@@ -704,7 +703,7 @@ public class MultiMoc implements Iterable<MocItem> {
    public ArrayList<String> scan(int order,long npix,String mask ) {
       boolean match=false;
       if( mask.charAt(0)=='!' ) { match=true; mask=mask.substring(1); }
-      ArrayList<String> res = new ArrayList<String>();
+      ArrayList<String> res = new ArrayList<>();
       for( MocItem mi : this ) {
          if( mask!=null && MyProperties.matchMask(mask, mi.mocId )==match ) continue;
          if( mi.moc!=null && mi.moc.isIntersecting(order, npix) ) res.add(mi.mocId);
@@ -717,7 +716,7 @@ public class MultiMoc implements Iterable<MocItem> {
     * @param moc MOC describing input sky region
     * @return list of MOC identifiers
     */
-   public ArrayList<String> scan(HealpixMoc moc) { return scan(moc, (HashMap<String,String[]>)null, true, -1, OVERLAPS); }
+   public ArrayList<String> scan(SMoc moc) { return scan(moc, (HashMap<String,String[]>)null, true, -1, OVERLAPS); }
    
    
    // Dtermination de l'ID, soit par le creator_did, ou le publisher_did sinon creator_id?obs_id ou publisher_id?obs_id
@@ -729,30 +728,37 @@ public class MultiMoc implements Iterable<MocItem> {
       
       // L'ID est donn explicitement (ex: CDS/P/DSS2/color)
       String id = prop.getProperty("ID");
-      if( id!=null ) return id;
-      
-      // l'ID doit tre construit  partir des diffrents champs possibles
-      id = prop.get("creator_did");
-      if( id==null ) id = prop.get("publisher_did");
       if( id==null ) {
-         String o =  prop.get("obs_id");
-         if( o==null && filename!=null ) id = "ivo://"+filename.replace("_","/");
-         else {
-            if( o==null ) o="id"+(System.currentTimeMillis()/1000);
-            String p = prop.get("creator_id");
-            if( p==null ) p = prop.get("publisher_id");
-            if( p==null ) {
-               return null;
-//               p = "ivo://UNK.AUT";
+
+         // l'ID doit tre construit  partir des diffrents champs possibles
+         id = prop.get("creator_did");
+         if( id==null ) id = prop.get("publisher_did");
+         if( id==null ) {
+            String o =  prop.get("obs_id");
+            if( o==null && filename!=null ) id = "ivo://"+filename.replace("_","/");
+            else {
+               if( o==null ) o="id"+(System.currentTimeMillis()/1000);
+               String p = prop.get("creator_id");
+               if( p==null ) p = prop.get("publisher_id");
+               if( p==null ) {
+                  return null;
+                  //               p = "ivo://UNK.AUT";
+               }
+               id = p+"/"+o;
             }
-            id = p+"/"+o;
          }
+         // On enlve le prfixe ivo://
+         if( id.startsWith("ivo://") ) id = id.substring(6);
+
+         // On remplace les ventuels ? par des / (merci Markus !)
+         id = id.replace('?', '/');
       }
-      // On enlve le prfixe ivo://
-      if( id.startsWith("ivo://") ) id = id.substring(6);
       
-      // On remplace les ventuels ? par des / (merci Markus !)
-      id = id.replace('?', '/');
+      // On met en minuscules l'authority
+//      int i = id.indexOf('/');
+//      if( i==-1 ) i=id.length();
+//      id = id.substring(0,i).toLowerCase()+id.substring(i);
+      
       return id;
    }
    
@@ -999,8 +1005,8 @@ public class MultiMoc implements Iterable<MocItem> {
     * @param casesensitive case sensitive (true by default) - never applied for ID field
     * @return list of MOC identifiers
     */
-   public ArrayList<String> scan(HealpixMoc moc,HashMap<String, String[]> mapFilter, boolean casesens, int top, int intersect ) {
-      ArrayList<String> res = new ArrayList<String>();
+   public ArrayList<String> scan(SMoc moc,HashMap<String, String[]> mapFilter, boolean casesens, int top, int intersect ) {
+      ArrayList<String> res = new ArrayList<>();
 
       int n=0;
       for( MocItem mi : this ) {
@@ -1028,7 +1034,7 @@ public class MultiMoc implements Iterable<MocItem> {
     * @return
     */
    public ArrayList<String> scan(HashMap<String,String[]> mapFilter) {
-      return scan( (HealpixMoc)null, mapFilter, true, -1, OVERLAPS);
+      return scan( (SMoc)null, mapFilter, true, -1, OVERLAPS);
    }
    
    /**
@@ -1039,8 +1045,8 @@ public class MultiMoc implements Iterable<MocItem> {
     * @param intersect true implies that all sky region must be inside
     * @return list of IDs (keep the original MultiMoc order)
     */
-   public ArrayList<String> scan( HealpixMoc moc, String expr, boolean casesens, int top, int intersect ) throws Exception {
-      ArrayList<String> res = new ArrayList<String>();
+   public ArrayList<String> scan( SMoc moc, String expr, boolean casesens, int top, int intersect ) throws Exception {
+      ArrayList<String> res = new ArrayList<>();
       
       // Dtermination des IDs candidats
       HashSet<String> candidateIds = scanExpr(expr,casesens);
@@ -1071,7 +1077,7 @@ public class MultiMoc implements Iterable<MocItem> {
     * @return list of IDs (keep the original MultiMoc order)
     */
    public ArrayList<String> scan() { 
-      ArrayList<String> res = new ArrayList<String> (this.size() );
+      ArrayList<String> res = new ArrayList<> (this.size() );
       for( MocItem mi : this ) res.add(mi.mocId);
       return res;
 //      IDENTIQUE A L'EXPRESSION SUIVANTE QUI EST PLUS LENTE
@@ -1087,7 +1093,7 @@ public class MultiMoc implements Iterable<MocItem> {
     */
    public ArrayList<String> scan(String mask) throws Exception { return scan( mask, true); }
    public ArrayList<String> scan(String mask, boolean casesensitive) throws Exception {
-      return scan( (HealpixMoc)null, mask, casesensitive, -1, OVERLAPS);
+      return scan( (SMoc)null, mask, casesensitive, -1, OVERLAPS);
    }
    
 
@@ -1439,7 +1445,7 @@ public class MultiMoc implements Iterable<MocItem> {
             && val.indexOf('*')<0 && val.indexOf('?')<0 && val.indexOf(',')<0) {
 
          // Est-ce qu'il n'y a que la contrainte sur l'ID ? alors je peux y accder directement
-         op.res = new HashSet<String>( 10 );   // <= je cre une Hashset petite car ce ne sera probablement 
+         op.res = new HashSet<>( 10 );   // <= je cre une Hashset petite car ce ne sera probablement 
                                                //    jamais elle qui sera conserv lors des oprations ensemblistes 
          MocItem mi1 = getItem(val);
 //         System.out.println("Accs direct pour "+val);
@@ -1448,12 +1454,12 @@ public class MultiMoc implements Iterable<MocItem> {
       }
       
      
-      HashMap<String, String[]> mapFilters = new HashMap<String, String[]>();
+      HashMap<String, String[]> mapFilters = new HashMap<>();
       mapFilters.put( key, new String[] { val } );
       
       // Scanning du multimoc et mmorisation des lments qui correspondent
-      ArrayList<String> res = scan( (HealpixMoc)null, mapFilters, casesens, -1, OVERLAPS);
-      op.res = new HashSet<String>( Math.max(2*res.size(), size()) );   // <= Je cre une HashSet assez grande, car elle est susceptible
+      ArrayList<String> res = scan( (SMoc)null, mapFilters, casesens, -1, OVERLAPS);
+      op.res = new HashSet<>( Math.max(2*res.size(), size()) );   // <= Je cre une HashSet assez grande, car elle est susceptible
                                                                         //    de contenir l'ensemble des rsultats. On pourrait affiner en
       op.res.addAll( res );
    }
