@@ -1,10 +1,10 @@
-// Copyright 1999-2018 - Universit de Strasbourg/CNRS
+// Copyright 1999-2020 - Universit de Strasbourg/CNRS
 // The Aladin Desktop program is developped by the Centre de Donnes
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
 //
-//This file is part of Aladin.
+//This file is part of Aladin Desktop.
 //
 //    Aladin Desktop is free software: you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 //    GNU General Public License for more details.
 //
 //    The GNU General Public License is available in COPYING file
-//    along with Aladin.
+//    along with Aladin Desktop.
 //
 
 package cds.tools.pixtools;
@@ -54,8 +54,8 @@ public class Util {
    }
 
    static public Coord [] getCorners(Coord [] corners,int order, long npix) throws Exception {
-      long nside = CDSHealpix.pow2(order);
-      double [][] x = CDSHealpix.corners(nside, npix);
+//      long nside = CDSHealpix.pow2(order);
+      double [][] x = CDSHealpix.corners(order, npix);
       if( corners==null ) corners = new Coord[4];
       for( int i=0; i<4; i++ ) {
          corners[i] = new Coord(x[i][0],x[i][1]);  
@@ -127,6 +127,20 @@ public class Util {
       "Npix" + npix
       + ( z<=0 ? "" : "_"+z);
    }
+   
+   /**
+    * Donne le rpertoire DIR d'un chemin HiPS
+    * ex: toto, 12, 130001 => toto/Norder12/Dir130000
+    * @param survey
+    * @param order
+    * @param npix
+    * @return
+    */
+   static public String getFileDir(String survey, int order, long npix) {
+      String suffix = "Norder" + order + "/" + "Dir" + ((npix / DIRSIZE)*DIRSIZE);
+      return cds.tools.Util.concatDir(survey, suffix);
+   }
+
 
    static public int getOrderFromPath(String filename) {
       int fromIndex = filename.indexOf("Norder");
@@ -171,8 +185,10 @@ public class Util {
             if( !sf[i].isDirectory() ) continue;
             String name = sf[i].getName();
             if( name.startsWith("Norder") ) {
-               int n = Integer.parseInt(name.substring(6));
-               if( n>maxOrder ) maxOrder=n;
+               try {
+                  int n = Integer.parseInt(name.substring(6));
+                  if( n>maxOrder ) maxOrder=n;
+               } catch( NumberFormatException e ) { }
             }
          }
       }
@@ -630,5 +646,16 @@ public class Util {
       return res.toString();
    }
    
+   public static final double pixRes(long nside) {
+      double res = 0.;
+      double degrad = Math.toDegrees(1.0);
+      double skyArea = 4.*Math.PI*degrad*degrad;
+      double arcSecArea = skyArea*3600.*3600.;
+      long npixels = 12*nside*nside;
+      res = arcSecArea/npixels;
+      res = Math.sqrt(res);
+      return res;
+   }
+   
 
 }
