@@ -1,10 +1,10 @@
-// Copyright 1999-2018 - Universit de Strasbourg/CNRS
+// Copyright 1999-2020 - Universit de Strasbourg/CNRS
 // The Aladin Desktop program is developped by the Centre de Donnes
 // astronomiques de Strasbourgs (CDS).
 // The Aladin Desktop program is distributed under the terms
 // of the GNU General Public License version 3.
 //
-//This file is part of Aladin.
+//This file is part of Aladin Desktop.
 //
 //    Aladin Desktop is free software: you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 //    GNU General Public License for more details.
 //
 //    The GNU General Public License is available in COPYING file
-//    along with Aladin.
+//    along with Aladin Desktop.
 //
 
 package cds.xml;
@@ -454,7 +454,12 @@ public final class XMLParser {
       // Memorization
       if( mode!=4 ) ch=curString.toString().toCharArray();
       start=0;
-      length=(mode==3)?ch.length-3:(mode==5 && xml4 && ol>=0 )?ol:ch.length-1;
+      
+      // Petite bidouille pour le CSV qui ne finirait pas par un CR
+      if( beforeXML && c1==EOF ) length = ch.length;
+      else 
+         length=(mode==3)?ch.length-3:(mode==5 && xml4 && ol>=0 )?ol: ch.length-1;
+      
       if( length<0 ) length=0;
 //      System.out.println("]");
       return c1;
